/* Generated by Cython 0.29.28 */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_28"
#define CYTHON_HEX_VERSION 0x001D1CF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyslow5
#define __PYX_HAVE_API__pyslow5
/* Early includes */
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include "pyslow5.h"
#include "slow5threads.h"
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "python/pyslow5.pyx",
  "stringsource",
  "__init__.pxd",
  ".eggs/Cython-0.29.28-py3.9-linux-x86_64.egg/Cython/Includes/cpython/type.pxd",
};

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":693
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":700
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":705
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":716
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":720
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":723
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":727
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_7pyslow5_Open;
struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__;
struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr;
struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches;
struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi;
struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads;
struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi;
struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list;
struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":731
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":733
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "pyslow5.pyx":25
 * #
 * 
 * cdef class Open:             # <<<<<<<<<<<<<<
 *     cdef pyslow5.slow5_file_t *s5
 *     cdef pyslow5.slow5_rec_t *rec
 */
struct __pyx_obj_7pyslow5_Open {
  PyObject_HEAD
  slow5_file_t *s5;
  slow5_rec_t *rec;
  slow5_rec_t *read;
  slow5_rec_t *write;
  slow5_rec_t **twrite;
  slow5_rec_t **trec;
  char **rid;
  char **rids;
  uint64_t rids_len;
  int index_state;
  int header_state;
  int header_add_attr_state;
  int close_state;
  uint64_t head_len;
  uint64_t aux_len;
  enum slow5_aux_type *s5_aux_type;
  int aux_get_err;
  uint64_t aux_get_len;
  npy_intp shape_get[1];
  npy_intp shape_seq[1];
  char *p;
  PyObject *path;
  char *m;
  PyObject *mode;
  PyObject *rec_press;
  PyObject *sig_press;
  int state;
  int V;
  PyObject *logger;
  PyObject *aux_names;
  PyObject *aux_types;
  PyObject *error_codes;
  PyObject *slow5_press_method;
  int8_t e0;
  int16_t e1;
  int32_t e2;
  int64_t e3;
  uint8_t e4;
  uint16_t e5;
  uint32_t e6;
  uint64_t e7;
  PyObject *e8;
  double e9;
  char e10;
  int e11;
  int8_t *e12;
  int16_t *e13;
  int32_t *e14;
  int64_t *e15;
  uint8_t *e16;
  uint16_t *e17;
  uint32_t *e18;
  uint64_t *e19;
  float *e20;
  double *e21;
  char *e22;
  char *channel_number;
  char *median_before;
  char *read_number;
  char *start_mux;
  char *start_time;
  char *end_reason;
  char *channel_number_val;
  double median_before_val;
  int32_t read_number_val;
  uint8_t start_mux_val;
  uint64_t start_time_val;
  char **channel_number_val_array;
  double *median_before_val_array;
  int32_t *read_number_val_array;
  uint8_t *start_mux_val_array;
  uint64_t *start_time_val_array;
  PyObject *total_time_slow5_get_next;
  PyObject *total_time_yield_reads;
  PyObject *total_single_write_time;
  PyObject *total_multi_write_signal_time;
  PyObject *total_multi_write_time;
};


/* "pyslow5.pyx":105
 *     cdef pyslow5.float total_multi_write_time
 * 
 *     def __cinit__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):             # <<<<<<<<<<<<<<
 *         # Set to default NULL type
 *         self.s5 = NULL
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ {
  PyObject_HEAD
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
};


/* "pyslow5.pyx":260
 *                 else:
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))
 *                     self.logger.error("Please use only the following: {}".format(",".join(press for press in self.slow5_press_method.keys())))             # <<<<<<<<<<<<<<
 *                     raise KeyError
 *             else:
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *__pyx_outer_scope;
  PyObject *__pyx_v_press;
};


/* "pyslow5.pyx":442
 *         return dic
 * 
 *     def _get_batches(self, read_list, size):             # <<<<<<<<<<<<<<
 *         """
 *         batchify readIDs
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches {
  PyObject_HEAD
  PyObject *__pyx_v_i;
  PyObject *__pyx_v_read_list;
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
  PyObject *__pyx_v_size;
  Py_ssize_t __pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "pyslow5.pyx":449
 *             yield read_list[i:i+size]
 * 
 *     def _get_read_multi(self, read_list, threads, batchsize, pA, aux):             # <<<<<<<<<<<<<<
 *         '''
 *         read_list = list of readIDs
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi {
  PyObject_HEAD
  PyObject *__pyx_v_aux;
  PyObject *__pyx_v_aux_dic;
  PyObject *__pyx_v_aux_set;
  PyObject *__pyx_v_batch;
  PyObject *__pyx_v_batch_len;
  PyObject *__pyx_v_batches;
  PyObject *__pyx_v_batchsize;
  PyObject *__pyx_v_dic;
  int __pyx_v_found_single_aux;
  PyObject *__pyx_8genexpr1__pyx_v_i;
  PyObject *__pyx_8genexpr2__pyx_v_i;
  PyObject *__pyx_v_i;
  PyObject *__pyx_v_n;
  PyObject *__pyx_v_n_list;
  PyObject *__pyx_v_n_set;
  Py_ssize_t __pyx_v_num_reads;
  PyObject *__pyx_v_pA;
  PyObject *__pyx_v_python_parse_read_start;
  PyObject *__pyx_v_read_list;
  int __pyx_v_ret;
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
  PyObject *__pyx_v_signal;
  PyObject *__pyx_v_t;
  PyObject *__pyx_v_t_list;
  PyObject *__pyx_v_threads;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
};


/* "pyslow5.pyx":1048
 * 
 * 
 *     def seq_reads(self, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for sequential reading of slow5 file
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads {
  PyObject_HEAD
  PyObject *__pyx_v_aux;
  PyObject *__pyx_v_aux_dic;
  PyObject *__pyx_v_aux_set;
  int __pyx_v_found_single_aux;
  PyObject *__pyx_v_i;
  PyObject *__pyx_v_n;
  PyObject *__pyx_v_n_list;
  PyObject *__pyx_v_n_set;
  PyObject *__pyx_v_pA;
  long __pyx_v_ret;
  PyObject *__pyx_v_row;
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
  PyObject *__pyx_v_signal;
  PyObject *__pyx_v_start_slow5_get_next;
  PyObject *__pyx_v_t;
  PyObject *__pyx_v_t_list;
};


/* "pyslow5.pyx":1139
 * 
 * 
 *     def seq_reads_multi(self, threads=4, batchsize=4096, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for sequential reading of slow5 file
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi {
  PyObject_HEAD
  PyObject *__pyx_v_aux;
  PyObject *__pyx_v_aux_dic;
  PyObject *__pyx_v_aux_set;
  PyObject *__pyx_v_aux_time_start;
  PyObject *__pyx_v_batchsize;
  int __pyx_v_found_single_aux;
  PyObject *__pyx_v_i;
  PyObject *__pyx_v_n;
  PyObject *__pyx_v_n_list;
  PyObject *__pyx_v_n_set;
  PyObject *__pyx_v_pA;
  PyObject *__pyx_v_pA_start_time;
  PyObject *__pyx_v_primary_start_time;
  PyObject *__pyx_v_python_parse_read_start;
  long __pyx_v_ret;
  PyObject *__pyx_v_row;
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
  PyObject *__pyx_v_signal;
  PyObject *__pyx_v_signal_start_time;
  PyObject *__pyx_v_start_slow5_get_next;
  PyObject *__pyx_v_t;
  PyObject *__pyx_v_t_list;
  PyObject *__pyx_v_threads;
  PyObject *__pyx_v_timedic;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "pyslow5.pyx":1253
 * 
 * 
 *     def get_read_list(self, read_list, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for random access of slow5 file
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list {
  PyObject_HEAD
  PyObject *__pyx_v_aux;
  PyObject *__pyx_v_pA;
  PyObject *__pyx_v_r;
  PyObject *__pyx_v_read_list;
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "pyslow5.pyx":1263
 * 
 * 
 *     def get_read_list_multi(self, read_list, threads=4, batchsize=100, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for random access of slow5 file
 */
struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi {
  PyObject_HEAD
  PyObject *__pyx_v_aux;
  PyObject *__pyx_v_batchsize;
  PyObject *__pyx_v_pA;
  PyObject *__pyx_v_r;
  PyObject *__pyx_v_read_list;
  struct __pyx_obj_7pyslow5_Open *__pyx_v_self;
  PyObject *__pyx_v_threads;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* Profile.proto */
#ifndef CYTHON_PROFILE
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
  #define CYTHON_PROFILE 0
#else
  #define CYTHON_PROFILE 1
#endif
#endif
#ifndef CYTHON_TRACE_NOGIL
  #define CYTHON_TRACE_NOGIL 0
#else
  #if CYTHON_TRACE_NOGIL && !defined(CYTHON_TRACE)
    #define CYTHON_TRACE 1
  #endif
#endif
#ifndef CYTHON_TRACE
  #define CYTHON_TRACE 0
#endif
#if CYTHON_TRACE
  #undef CYTHON_PROFILE_REUSE_FRAME
#endif
#ifndef CYTHON_PROFILE_REUSE_FRAME
  #define CYTHON_PROFILE_REUSE_FRAME 0
#endif
#if CYTHON_PROFILE || CYTHON_TRACE
  #include "compile.h"
  #include "frameobject.h"
  #include "traceback.h"
  #if CYTHON_PROFILE_REUSE_FRAME
    #define CYTHON_FRAME_MODIFIER static
    #define CYTHON_FRAME_DEL(frame)
  #else
    #define CYTHON_FRAME_MODIFIER
    #define CYTHON_FRAME_DEL(frame) Py_CLEAR(frame)
  #endif
  #define __Pyx_TraceDeclarations\
      static PyCodeObject *__pyx_frame_code = NULL;\
      CYTHON_FRAME_MODIFIER PyFrameObject *__pyx_frame = NULL;\
      int __Pyx_use_tracing = 0;
  #define __Pyx_TraceFrameInit(codeobj)\
      if (codeobj) __pyx_frame_code = (PyCodeObject*) codeobj;
#if PY_VERSION_HEX >= 0x030b00a2
  #define __Pyx_IsTracing(tstate, check_tracing, check_funcs)\
     (unlikely((tstate)->cframe->use_tracing) &&\
         (!(check_tracing) || !(tstate)->tracing) &&\
         (!(check_funcs) || (tstate)->c_profilefunc || (CYTHON_TRACE && (tstate)->c_tracefunc)))
  #define __Pyx_EnterTracing(tstate) PyThreadState_EnterTracing(tstate)
  #define __Pyx_LeaveTracing(tstate) PyThreadState_LeaveTracing(tstate)
#elif PY_VERSION_HEX >= 0x030a00b1
  #define __Pyx_IsTracing(tstate, check_tracing, check_funcs)\
     (unlikely((tstate)->cframe->use_tracing) &&\
         (!(check_tracing) || !(tstate)->tracing) &&\
         (!(check_funcs) || (tstate)->c_profilefunc || (CYTHON_TRACE && (tstate)->c_tracefunc)))
  #define __Pyx_EnterTracing(tstate)\
      do { tstate->tracing++; tstate->cframe->use_tracing = 0; } while (0)
  #define __Pyx_LeaveTracing(tstate)\
      do {\
          tstate->tracing--;\
          tstate->cframe->use_tracing = ((CYTHON_TRACE && tstate->c_tracefunc != NULL)\
                                 || tstate->c_profilefunc != NULL);\
      } while (0)
#else
  #define __Pyx_IsTracing(tstate, check_tracing, check_funcs)\
     (unlikely((tstate)->use_tracing) &&\
         (!(check_tracing) || !(tstate)->tracing) &&\
         (!(check_funcs) || (tstate)->c_profilefunc || (CYTHON_TRACE && (tstate)->c_tracefunc)))
  #define __Pyx_EnterTracing(tstate)\
      do { tstate->tracing++; tstate->use_tracing = 0; } while (0)
  #define __Pyx_LeaveTracing(tstate)\
      do {\
          tstate->tracing--;\
          tstate->use_tracing = ((CYTHON_TRACE && tstate->c_tracefunc != NULL)\
                                         || tstate->c_profilefunc != NULL);\
      } while (0)
#endif
  #ifdef WITH_THREAD
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)\
  if (nogil) {\
      if (CYTHON_TRACE_NOGIL) {\
          PyThreadState *tstate;\
          PyGILState_STATE state = PyGILState_Ensure();\
          tstate = __Pyx_PyThreadState_Current;\
          if (__Pyx_IsTracing(tstate, 1, 1)) {\
              __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          }\
          PyGILState_Release(state);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  } else {\
      PyThreadState* tstate = PyThreadState_GET();\
      if (__Pyx_IsTracing(tstate, 1, 1)) {\
          __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  }
  #else
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)\
  {   PyThreadState* tstate = PyThreadState_GET();\
      if (__Pyx_IsTracing(tstate, 1, 1)) {\
          __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  }
  #endif
  #define __Pyx_TraceException()\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (__Pyx_IsTracing(tstate, 0, 1)) {\
          __Pyx_EnterTracing(tstate);\
          PyObject *exc_info = __Pyx_GetExceptionTuple(tstate);\
          if (exc_info) {\
              if (CYTHON_TRACE && tstate->c_tracefunc)\
                  tstate->c_tracefunc(\
                      tstate->c_traceobj, __pyx_frame, PyTrace_EXCEPTION, exc_info);\
              tstate->c_profilefunc(\
                  tstate->c_profileobj, __pyx_frame, PyTrace_EXCEPTION, exc_info);\
              Py_DECREF(exc_info);\
          }\
          __Pyx_LeaveTracing(tstate);\
      }\
  }
  static void __Pyx_call_return_trace_func(PyThreadState *tstate, PyFrameObject *frame, PyObject *result) {
      PyObject *type, *value, *traceback;
      __Pyx_ErrFetchInState(tstate, &type, &value, &traceback);
      __Pyx_EnterTracing(tstate);
      if (CYTHON_TRACE && tstate->c_tracefunc)
          tstate->c_tracefunc(tstate->c_traceobj, frame, PyTrace_RETURN, result);
      if (tstate->c_profilefunc)
          tstate->c_profilefunc(tstate->c_profileobj, frame, PyTrace_RETURN, result);
      CYTHON_FRAME_DEL(frame);
      __Pyx_LeaveTracing(tstate);
      __Pyx_ErrRestoreInState(tstate, type, value, traceback);
  }
  #ifdef WITH_THREAD
  #define __Pyx_TraceReturn(result, nogil)\
  if (likely(!__Pyx_use_tracing)); else {\
      if (nogil) {\
          if (CYTHON_TRACE_NOGIL) {\
              PyThreadState *tstate;\
              PyGILState_STATE state = PyGILState_Ensure();\
              tstate = __Pyx_PyThreadState_Current;\
              if (__Pyx_IsTracing(tstate, 0, 0)) {\
                  __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
              }\
              PyGILState_Release(state);\
          }\
      } else {\
          PyThreadState* tstate = __Pyx_PyThreadState_Current;\
          if (__Pyx_IsTracing(tstate, 0, 0)) {\
              __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
          }\
      }\
  }
  #else
  #define __Pyx_TraceReturn(result, nogil)\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (__Pyx_IsTracing(tstate, 0, 0)) {\
          __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
      }\
  }
  #endif
  static PyCodeObject *__Pyx_createFrameCodeObject(const char *funcname, const char *srcfile, int firstlineno);
  static int __Pyx_TraceSetupAndCall(PyCodeObject** code, PyFrameObject** frame, PyThreadState* tstate, const char *funcname, const char *srcfile, int firstlineno);
#else
  #define __Pyx_TraceDeclarations
  #define __Pyx_TraceFrameInit(codeobj)
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)   if ((1)); else goto_error;
  #define __Pyx_TraceException()
  #define __Pyx_TraceReturn(result, nogil)
#endif
#if CYTHON_TRACE
  static int __Pyx_call_line_trace_func(PyThreadState *tstate, PyFrameObject *frame, int lineno) {
      int ret;
      PyObject *type, *value, *traceback;
      __Pyx_ErrFetchInState(tstate, &type, &value, &traceback);
      __Pyx_PyFrame_SetLineNumber(frame, lineno);
      __Pyx_EnterTracing(tstate);
      ret = tstate->c_tracefunc(tstate->c_traceobj, frame, PyTrace_LINE, NULL);
      __Pyx_LeaveTracing(tstate);
      if (likely(!ret)) {
          __Pyx_ErrRestoreInState(tstate, type, value, traceback);
      } else {
          Py_XDECREF(type);
          Py_XDECREF(value);
          Py_XDECREF(traceback);
      }
      return ret;
  }
  #ifdef WITH_THREAD
  #define __Pyx_TraceLine(lineno, nogil, goto_error)\
  if (likely(!__Pyx_use_tracing)); else {\
      if (nogil) {\
          if (CYTHON_TRACE_NOGIL) {\
              int ret = 0;\
              PyThreadState *tstate;\
              PyGILState_STATE state = PyGILState_Ensure();\
              tstate = __Pyx_PyThreadState_Current;\
              if (__Pyx_IsTracing(tstate, 0, 0) && tstate->c_tracefunc && __pyx_frame->f_trace) {\
                  ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
              }\
              PyGILState_Release(state);\
              if (unlikely(ret)) goto_error;\
          }\
      } else {\
          PyThreadState* tstate = __Pyx_PyThreadState_Current;\
          if (__Pyx_IsTracing(tstate, 0, 0) && tstate->c_tracefunc && __pyx_frame->f_trace) {\
              int ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
              if (unlikely(ret)) goto_error;\
          }\
      }\
  }
  #else
  #define __Pyx_TraceLine(lineno, nogil, goto_error)\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (__Pyx_IsTracing(tstate, 0, 0) && tstate->c_tracefunc && __pyx_frame->f_trace) {\
          int ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
          if (unlikely(ret)) goto_error;\
      }\
  }
  #endif
#else
  #define __Pyx_TraceLine(lineno, nogil, goto_error)   if ((1)); else goto_error;
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* py_dict_keys.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int8_t(int8_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int16_t(int16_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32_t(int32_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64_t(int64_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__slow5_aux_type(enum slow5_aux_type value);

/* CIntFromPy.proto */
static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE enum slow5_aux_type __Pyx_PyInt_As_enum__slow5_aux_type(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int16_t __Pyx_PyInt_As_int16_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'pyslow5' */
static PyTypeObject *__pyx_ptype_7pyslow5_Open = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct____cinit__ = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_1_genexpr = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_2__get_batches = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_3__get_read_multi = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_4_seq_reads = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_5_seq_reads_multi = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_6_get_read_list = 0;
static PyTypeObject *__pyx_ptype_7pyslow5___pyx_scope_struct_7_get_read_list_multi = 0;
#define __Pyx_MODULE_NAME "pyslow5"
extern int __pyx_module_is_main_pyslow5;
int __pyx_module_is_main_pyslow5 = 0;

/* Implementation of 'pyslow5' */
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_chr;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_[] = "";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_r[] = "r";
static const char __pyx_k_w[] = "w";
static const char __pyx_k__2[] = ".";
static const char __pyx_k__4[] = ",";
static const char __pyx_k__5[] = "{}: {}";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_pA[] = "pA";
static const char __pyx_k_all[] = "all";
static const char __pyx_k_aux[] = "aux";
static const char __pyx_k_chr[] = "chr";
static const char __pyx_k_num[] = "num";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_zip[] = "zip";
static const char __pyx_k_Open[] = "Open";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_attr[] = "attr";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_none[] = "none";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_zlib[] = "zlib";
static const char __pyx_k_zstd[] = "zstd";
static const char __pyx_k_Batch[] = "Batch: {}";
static const char __pyx_k_DEBUG[] = "DEBUG";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_blow5[] = "blow5";
static const char __pyx_k_chain[] = "chain";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_int16[] = "int16";
static const char __pyx_k_level[] = "level";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_astype[] = "astype";
static const char __pyx_k_closed[] = "{} closed";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_header[] = "header";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_record[] = "record";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_run_id[] = "run_id";
static const char __pyx_k_signal[] = "signal";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_svb_zd[] = "svb_zd";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_WARNING[] = "WARNING";
static const char __pyx_k_asic_id[] = "asic_id";
static const char __pyx_k_datefmt[] = "datefmt";
static const char __pyx_k_float32[] = "float32";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_logging[] = "logging";
static const char __pyx_k_package[] = "package";
static const char __pyx_k_pyslow5[] = "pyslow5";
static const char __pyx_k_read_id[] = "read_id";
static const char __pyx_k_records[] = "records";
static const char __pyx_k_threads[] = "threads";
static const char __pyx_k_version[] = "version";
static const char __pyx_k_warning[] = "warning";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_deepcopy[] = "deepcopy";
static const char __pyx_k_get_read[] = "_get_read";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_hostname[] = "hostname";
static const char __pyx_k_pathname[] = "pathname";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_FILE_mode[] = "FILE: {}, mode: {}";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_asic_temp[] = "asic_temp";
static const char __pyx_k_aux_names[] = "aux_names";
static const char __pyx_k_aux_types[] = "aux_types";
static const char __pyx_k_batchsize[] = "batchsize";
static const char __pyx_k_device_id[] = "device_id";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_getLogger[] = "getLogger";
static const char __pyx_k_itertools[] = "itertools";
static const char __pyx_k_read_list[] = "read_list";
static const char __pyx_k_rec_press[] = "rec_press";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_sample_id[] = "sample_id";
static const char __pyx_k_seq_reads[] = "seq_reads";
static const char __pyx_k_sig_press[] = "sig_press";
static const char __pyx_k_start_mux[] = "start_mux";
static const char __pyx_k_difference[] = "difference";
static const char __pyx_k_end_reason[] = "end_reason";
static const char __pyx_k_memoryview[] = "memoryview";
static const char __pyx_k_pathname_2[] = "pathname: {}";
static const char __pyx_k_readID_bin[] = "readID: {}, bin: {}";
static const char __pyx_k_read_group[] = "read_group";
static const char __pyx_k_start_time[] = "start_time";
static const char __pyx_k_usb_config[] = "usb_config";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_auto_update[] = "auto_update";
static const char __pyx_k_basicConfig[] = "basicConfig";
static const char __pyx_k_d_b_y_H_M_S[] = "%d-%b-%y %H:%M:%S";
static const char __pyx_k_device_type[] = "device_type";
static const char __pyx_k_get_batches[] = "_get_batches";
static const char __pyx_k_read_list_2[] = "read_list: {}";
static const char __pyx_k_read_number[] = "read_number";
static const char __pyx_k_user_record[] = "user_record";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_SLOW5_ERR_IO[] = "SLOW5_ERR_IO";
static const char __pyx_k_asic_version[] = "asic_version";
static const char __pyx_k_digitisation[] = "digitisation";
static const char __pyx_k_flow_cell_id[] = "flow_cell_id";
static const char __pyx_k_get_read_aux[] = "_get_read_aux";
static const char __pyx_k_No_more_reads[] = "No more reads: {}";
static const char __pyx_k_SLOW5_ERR_ARG[] = "SLOW5_ERR_ARG";
static const char __pyx_k_SLOW5_ERR_EOF[] = "SLOW5_ERR_EOF";
static const char __pyx_k_SLOW5_ERR_MEM[] = "SLOW5_ERR_MEM";
static const char __pyx_k_SLOW5_ERR_OTH[] = "SLOW5_ERR_OTH";
static const char __pyx_k_SLOW5_ERR_UNK[] = "SLOW5_ERR_UNK";
static const char __pyx_k_convert_to_pA[] = "_convert_to_pA";
static const char __pyx_k_get_aux_names[] = "get_aux_names";
static const char __pyx_k_get_aux_types[] = "get_aux_types";
static const char __pyx_k_get_read_list[] = "get_read_list";
static const char __pyx_k_guppy_version[] = "guppy_version";
static const char __pyx_k_heatsink_temp[] = "heatsink_temp";
static const char __pyx_k_median_before[] = "median_before";
static const char __pyx_k_pA_total_time[] = "pA_total_time";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_sampling_rate[] = "sampling_rate";
static const char __pyx_k_Open_seq_reads[] = "Open.seq_reads";
static const char __pyx_k_SLOW5_ERR_TYPE[] = "SLOW5_ERR_TYPE";
static const char __pyx_k_already_closed[] = "{} already closed";
static const char __pyx_k_asic_id_eeprom[] = "asic_id_eeprom";
static const char __pyx_k_aux_total_time[] = "aux_total_time";
static const char __pyx_k_channel_number[] = "channel_number";
static const char __pyx_k_exp_start_time[] = "exp_start_time";
static const char __pyx_k_file_truncated[] = "file truncated";
static const char __pyx_k_get_read_multi[] = "_get_read_multi";
static const char __pyx_k_len_raw_signal[] = "len_raw_signal";
static const char __pyx_k_sequencing_kit[] = "sequencing_kit";
static const char __pyx_k_SLOW5_ERR_MAGIC[] = "SLOW5_ERR_MAGIC";
static const char __pyx_k_SLOW5_ERR_NOAUX[] = "SLOW5_ERR_NOAUX";
static const char __pyx_k_SLOW5_ERR_NOFLD[] = "SLOW5_ERR_NOFLD";
static const char __pyx_k_SLOW5_ERR_NOIDX[] = "SLOW5_ERR_NOIDX";
static const char __pyx_k_SLOW5_ERR_PRESS[] = "SLOW5_ERR_PRESS";
static const char __pyx_k_SLOW5_ERR_TRUNC[] = "SLOW5_ERR_TRUNC";
static const char __pyx_k_exp_script_name[] = "exp_script_name";
static const char __pyx_k_field_not_found[] = "field not found";
static const char __pyx_k_protocol_run_id[] = "protocol_run_id";
static const char __pyx_k_seq_reads_multi[] = "seq_reads_multi";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_get_header_names[] = "get_header_names";
static const char __pyx_k_get_header_value[] = "get_header_value";
static const char __pyx_k_get_seq_read_aux[] = "_get_seq_read_aux";
static const char __pyx_k_hublett_board_id[] = "hublett_board_id";
static const char __pyx_k_index_not_loaded[] = "index not loaded";
static const char __pyx_k_no_auxiliary_map[] = "no auxiliary map";
static const char __pyx_k_operating_system[] = "operating_system";
static const char __pyx_k_sample_frequency[] = "sample_frequency";
static const char __pyx_k_Open__get_batches[] = "Open._get_batches";
static const char __pyx_k_SLOW5_ERR_VERSION[] = "SLOW5_ERR_VERSION";
static const char __pyx_k_bad_argument_NULL[] = "bad argument (NULL)";
static const char __pyx_k_barcoding_enabled[] = "barcoding_enabled";
static const char __pyx_k_bream_is_standard[] = "bream_is_standard";
static const char __pyx_k_installation_type[] = "installation_type";
static const char __pyx_k_local_basecalling[] = "local_basecalling";
static const char __pyx_k_protocol_group_id[] = "protocol_group_id";
static const char __pyx_k_protocols_version[] = "protocols_version";
static const char __pyx_k_read_id_not_found[] = "read id not found";
static const char __pyx_k_signal_total_time[] = "signal_total_time";
static const char __pyx_k_Open_get_read_list[] = "Open.get_read_list";
static const char __pyx_k_SLOW5_ERR_HDRPARSE[] = "SLOW5_ERR_HDRPARSE";
static const char __pyx_k_SLOW5_ERR_NOTFOUND[] = "SLOW5_ERR_NOTFOUND";
static const char __pyx_k_SLOW5_ERR_RECPARSE[] = "SLOW5_ERR_RECPARSE";
static const char __pyx_k_auto_update_source[] = "auto_update_source";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_exp_script_purpose[] = "exp_script_purpose";
static const char __pyx_k_primary_total_time[] = "primary_total_time";
static const char __pyx_k_End_Of_File_reached[] = "End Of File reached";
static const char __pyx_k_Setting_up_batching[] = "Setting up batching...";
static const char __pyx_k_distribution_status[] = "distribution_status";
static const char __pyx_k_file_format_unknown[] = "file format unknown";
static const char __pyx_k_get_read_list_multi[] = "get_read_list_multi";
static const char __pyx_k_protocol_start_time[] = "protocol_start_time";
static const char __pyx_k_Open__get_read_multi[] = "Open._get_read_multi";
static const char __pyx_k_Open_seq_reads_multi[] = "Open.seq_reads_multi";
static const char __pyx_k_batch_for_loop_start[] = "batch for loop start";
static const char __pyx_k_distribution_version[] = "distribution_version";
static const char __pyx_k_header_parsing_error[] = "header parsing error";
static const char __pyx_k_magic_number_invalid[] = "magic number invalid";
static const char __pyx_k_other_file_I_O_error[] = "other file I/O error";
static const char __pyx_k_record_parsing_error[] = "record parsing error";
static const char __pyx_k_version_incompatible[] = "version incompatible";
static const char __pyx_k_Number_of_read_groups[] = "Number of read_groups: {}";
static const char __pyx_k_configuration_version[] = "configuration_version";
static const char __pyx_k_get_aux_names_aux_len[] = "get_aux_names aux_len: {}";
static const char __pyx_k_get_aux_types_aux_len[] = "get_aux_types aux_len: {}";
static const char __pyx_k_get_read_return_not_0[] = "get_read return not 0: {}";
static const char __pyx_k_slow5_get_next_return[] = "slow5_get_next return: {}";
static const char __pyx_k_Creating_loading_index[] = "Creating/loading index...";
static const char __pyx_k_cinit___locals_genexpr[] = "__cinit__.<locals>.genexpr";
static const char __pyx_k_de_compression_failure[] = "(de)compression failure";
static const char __pyx_k_flow_cell_product_code[] = "flow_cell_product_code";
static const char __pyx_k_get_aux_names_starting[] = "get_aux_names starting";
static const char __pyx_k_get_aux_types_starting[] = "get_aux_types starting";
static const char __pyx_k_header_type_validation[] = "_header_type_validation";
static const char __pyx_k_record_type_validation[] = "_record_type_validation";
static const char __pyx_k_write_record_aux_stuff[] = "write_record: aux stuff...";
static const char __pyx_k_Starting_rid_assignment[] = "Starting rid assignment";
static const char __pyx_k_experiment_duration_set[] = "experiment_duration_set";
static const char __pyx_k_rid_assignment_complete[] = "rid assignment complete";
static const char __pyx_k_seq_reads_multi_timings[] = "seq_reads_multi timings:";
static const char __pyx_k_write_record_batch_free[] = "write_record_batch: free()";
static const char __pyx_k_File_unknown_open_method[] = "File '{}' unknown open method: {}";
static const char __pyx_k_Open_get_read_list_multi[] = "Open.get_read_list_multi";
static const char __pyx_k_write_record_checked_aux[] = "write_record: checked_aux: {}: {}";
static const char __pyx_k_write_record_slow5_write[] = "write_record: slow5_write()";
static const char __pyx_k_Unable_to_set_compression[] = "Unable to set compression";
static const char __pyx_k_get_aux_names_ret_is_NULL[] = "get_aux_names ret is NULL";
static const char __pyx_k_get_read_unknown_aux_name[] = "get_read unknown aux name: {}";
static const char __pyx_k_slow5_get_batch_num_reads[] = "slow5_get_batch: num_reads: {}";
static const char __pyx_k_slow5_get_next_error_code[] = "slow5_get_next error code: {}: {}";
static const char __pyx_k_aux_header_type_validation[] = "_aux_header_type_validation";
static const char __pyx_k_memory_re_allocation_error[] = "memory (re)allocation error";
static const char __pyx_k_satellite_firmware_version[] = "satellite_firmware_version";
static const char __pyx_k_slow5_get_hdr_keys_head_len[] = "slow5_get_hdr_keys head_len: {}";
static const char __pyx_k_slow5_get_next_multi_return[] = "slow5_get_next_multi return: {}";
static const char __pyx_k_slow5_idx_load_return_not_0[] = "slow5_idx_load return not 0: {}: {}";
static const char __pyx_k_write_record_aux_stuff_done[] = "write_record: aux stuff done";
static const char __pyx_k_write_record_batch_free_aux[] = "write_record_batch: free() aux";
static const char __pyx_k_write_record_header_written[] = "write_record: header written";
static const char __pyx_k_write_record_slow5_rec_free[] = "write_record: slow5_rec_free()";
static const char __pyx_k_write_record_slow5_rec_init[] = "write_record: slow5_rec_init()";
static const char __pyx_k_error_relating_to_slow5_type[] = "error relating to slow5 type";
static const char __pyx_k_multi_record_type_validation[] = "_multi_record_type_validation";
static const char __pyx_k_slow5_set_press_return_not_0[] = "slow5_set_press return not 0: {}";
static const char __pyx_k_write_record_batch_aux_stuff[] = "write_record_batch: aux stuff...";
static const char __pyx_k_write_record_slow5_write_ret[] = "write_record: slow5_write() ret: {}";
static const char __pyx_k_write_record_writting_header[] = "write_record: writting header...";
static const char __pyx_k_Please_use_only_the_following[] = "Please use only the following: {}";
static const char __pyx_k_slow5_get_hdr_keys_ret_is_NULL[] = "slow5_get_hdr_keys ret is NULL";
static const char __pyx_k_total_multi_write_time_seconds[] = "total_multi_write_time: {} seconds";
static const char __pyx_k_total_time_yield_reads_seconds[] = "total_time_yield_reads: {} seconds";
static const char __pyx_k_write_record_batch_checked_aux[] = "write_record_batch: checked_aux: {}: {}";
static const char __pyx_k_asctime_s_levelname_s_message_s[] = "%(asctime)s - [%(levelname)s]: %(message)s";
static const char __pyx_k_aux_type_validation_user_aux_fi[] = "_aux_type_validation {}: {} user aux field not in pyslow5 aux_types";
static const char __pyx_k_get_read_ids_length_of_read_ids[] = "get_read_ids: length of read_ids is 0, something when wrong";
static const char __pyx_k_header_type_validation_conversi[] = "_header_type_validation {}: {} conversion successful";
static const char __pyx_k_header_type_validation_could_no[] = "_header_type_validation {}: {} could not convert value to string";
static const char __pyx_k_header_type_validation_is_not_a[] = "_header_type_validation {}: {} is not a string type, attempting to convert";
static const char __pyx_k_not_open_for_writing_remake_cla[] = "{} not open for writing, remake class object to refresh";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_other_error_big_endian_internal[] = "other error (big endian, internal error, etc.)";
static const char __pyx_k_record_type_validation_aux_pass[] = "_record_type_validation: aux passed tests...";
static const char __pyx_k_record_type_validation_aux_stuf[] = "_record_type_validation: aux stuff done";
static const char __pyx_k_record_type_validation_doing_au[] = "_record_type_validation: doing aux stuff...";
static const char __pyx_k_record_type_validation_user_aux[] = "_record_type_validation {}: {} user aux field not in pyslow5 aux_types";
static const char __pyx_k_record_type_validation_user_pri[] = "_record_type_validation {}: {} user primary field not in pyslow5 record_types";
static const char __pyx_k_slow5_get_next_multi_error_code[] = "slow5_get_next_multi error code: {}: {}";
static const char __pyx_k_total_single_write_time_seconds[] = "total_single_write_time: {} seconds";
static const char __pyx_k_write_header_errors_encountered[] = "write_header: errors encountered setting up header, aborting write";
static const char __pyx_k_write_header_slow5_hdr_add_attr[] = "write_header: slow5_hdr_add_attr {}: {} could not initialise to C s5.header struct";
static const char __pyx_k_write_record_batch__record_type[] = "write_record_batch: _record_type_validation running";
static const char __pyx_k_write_record_batch_write_failed[] = "write_record_batch: write failed";
static const char __pyx_k_write_record_failed_to_allocate[] = "write_record: failed to allocate space for slow5 record (self.write)";
static const char __pyx_k_Compression_type_rec_press_sig_p[] = "Compression type rec_press: {}, sig_press: {} could not be found.";
static const char __pyx_k_File_could_not_be_opened_for_wri[] = "File '{}' could not be opened for writing.";
static const char __pyx_k_NOT_IMPLEMENTED_YET_get_aux_type[] = "NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}";
static const char __pyx_k_Not_writing_blow5_skipping_compr[] = "Not writing blow5, skipping compression steps";
static const char __pyx_k_get_aux_types_self_aux_get_err_i[] = "get_aux_types {} self.aux_get_err is {}: {}";
static const char __pyx_k_get_aux_types_self_s5_aux_type_i[] = "get_aux_types self.s5_aux_type is NULL";
static const char __pyx_k_get_header_value_header_value_no[] = "get_header_value header value not found: {} - rg: {}";
static const char __pyx_k_get_read_aux_atype_not_known_ski[] = "get_read_aux atype not known, skipping: {}";
static const char __pyx_k_get_read_aux_type_unknown_accept[] = "get_read aux type unknown, accepts str or list: {}";
static const char __pyx_k_get_read_ids_returned_rids_is_NU[] = "get_read_ids: returned rids is NULL, something when wrong";
static const char __pyx_k_get_read_multi_slow5_get_batch_r[] = "get_read_multi slow5_get_batch ret: {}";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_slow5_get_next_multi_aux_type_un[] = "slow5_get_next_multi aux type unknown, accepts str or list: {}";
static const char __pyx_k_slow5_get_next_multi_has_no_more[] = "slow5_get_next_multi has no more batches - batchsize:{} ret:{}";
static const char __pyx_k_slow5_get_next_multi_reached_end[] = "slow5_get_next_multi reached end of file (EOF)(-1): {}: {}";
static const char __pyx_k_slow5_get_next_multi_unknown_aux[] = "slow5_get_next_multi unknown aux name: {}";
static const char __pyx_k_slow5_get_next_reached_end_of_fi[] = "slow5_get_next reached end of file (EOF)(-1): {}: {}";
static const char __pyx_k_total_multi_write_signal_time_se[] = "total_multi_write_signal_time: {} seconds";
static const char __pyx_k_total_time_slow5_get_next_second[] = "total_time_slow5_get_next: {} seconds";
static const char __pyx_k_write_header_You_must_set_read_g[] = "write_header: You must set read_group=0 first";
static const char __pyx_k_write_header_headers_failed_to_i[] = "write_header: headers failed to initialise.";
static const char __pyx_k_write_header_slow5_hdr_add_rg_fa[] = "write_header: slow5_hdr_add_rg failed for read_group: {}";
static const char __pyx_k_write_header_slow5_hdr_set_could[] = "write_header: slow5_hdr_set {}: {} could not set to C s5.header struct";
static const char __pyx_k_write_record__record_type_valida[] = "write_record: _record_type_validation running";
static const char __pyx_k_write_record_aux_meta_fields_fai[] = "write_record: aux_meta fields failed to initialise";
static const char __pyx_k_write_record_batch_Setting_up_ba[] = "write_record_batch: Setting up batching...";
static const char __pyx_k_write_record_batch_aux_meta_fiel[] = "write_record_batch: aux_meta fields failed to initialise";
static const char __pyx_k_write_record_batch_aux_stuff_don[] = "write_record_batch: aux stuff done";
static const char __pyx_k_write_record_batch_batch_for_loo[] = "write_record_batch: batch for loop start";
static const char __pyx_k_write_record_batch_batch_len_0_o[] = "write_record_batch: batch_len 0 or less";
static const char __pyx_k_write_record_batch_checked_aux_i[] = "write_record_batch: checked_aux is None";
static const char __pyx_k_write_record_batch_checking_head[] = "write_record_batch: checking header stuff...";
static const char __pyx_k_write_record_batch_failed_to_all[] = "write_record_batch: failed to allocate space for slow5 record (self.twrite[idx])";
static const char __pyx_k_write_record_batch_function_comp[] = "write_record_batch: function complete, returning 0";
static const char __pyx_k_write_record_batch_header_writte[] = "write_record_batch: header written";
static const char __pyx_k_write_record_batch_self_write_as[] = "write_record_batch: self.write assignments...";
static const char __pyx_k_write_record_batch_self_write_pr[] = "write_record_batch: self.write processing raw_signal";
static const char __pyx_k_write_record_batch_self_write_ra[] = "write_record_batch: self.write.raw_signal malloc...";
static const char __pyx_k_write_record_batch_slow5_aux_add[] = "write_record_batch: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct";
static const char __pyx_k_write_record_batch_slow5_aux_set[] = "write_record_batch: slow5_aux_set_string running...";
static const char __pyx_k_write_record_batch_slow5_hdr_wri[] = "write_record_batch: slow5_hdr_write could not write header";
static const char __pyx_k_write_record_batch_slow5_rec_ini[] = "write_record_batch: slow5_rec_init()";
static const char __pyx_k_write_record_batch_slow5_write_b[] = "write_record_batch: slow5_write_batch()";
static const char __pyx_k_write_record_batch_writting_head[] = "write_record_batch: writting header...";
static const char __pyx_k_write_record_checked_aux_is_None[] = "write_record: checked_aux is None";
static const char __pyx_k_write_record_checking_header_stu[] = "write_record: checking header stuff...";
static const char __pyx_k_write_record_function_complete_r[] = "write_record: function complete, returning 0";
static const char __pyx_k_write_record_self_write_assignme[] = "write_record: self.write assignments...";
static const char __pyx_k_write_record_self_write_processi[] = "write_record: self.write processing raw_signal";
static const char __pyx_k_write_record_self_write_raw_sign[] = "write_record: self.write.raw_signal malloc...";
static const char __pyx_k_write_record_slow5_aux_add_could[] = "write_record: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct";
static const char __pyx_k_write_record_slow5_aux_set_could[] = "write_record: slow5_aux_set could not write aux value {}: {}";
static const char __pyx_k_write_record_slow5_aux_set_strin[] = "write_record: slow5_aux_set_string running...";
static const char __pyx_k_write_record_slow5_hdr_write_cou[] = "write_record: slow5_hdr_write could not write header";
static const char __pyx_k_aux_type_validation_user_aux_fi_2[] = "_aux_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types";
static const char __pyx_k_record_type_validation_user_aux_2[] = "_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types";
static const char __pyx_k_record_type_validation_user_pri_2[] = "_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types";
static const char __pyx_k_write_record_batch__record_type_2[] = "write_record_batch: _record_type_validation done";
static const char __pyx_k_File_could_not_be_opened_for_wri_2[] = "File '{}' could not be opened for writing - appending.";
static const char __pyx_k_write_record__record_type_valida_2[] = "write_record: _record_type_validation done";
static const char __pyx_k_write_record_batch_self_write_as_2[] = "write_record_batch: self.write assignments done";
static const char __pyx_k_write_record_batch_self_write_ra_2[] = "write_record_batch: self.write.raw_signal malloc done";
static const char __pyx_k_write_record_batch_self_write_ra_3[] = "write_record_batch: self.write raw_signal done";
static const char __pyx_k_write_record_batch_slow5_aux_set_2[] = "write_record_batch: slow5_aux_set_string type: {}";
static const char __pyx_k_write_record_batch_slow5_aux_set_3[] = "write_record_batch: slow5_aux_set_string running done: ret = {}";
static const char __pyx_k_write_record_batch_slow5_aux_set_4[] = "write_record_batch: slow5_aux_set_string could not write aux value {}: {}";
static const char __pyx_k_write_record_batch_slow5_aux_set_5[] = "write_record_batch: slow5_aux_set could not write aux value {}: {}";
static const char __pyx_k_write_record_self_write_assignme_2[] = "write_record: self.write assignments done";
static const char __pyx_k_write_record_self_write_raw_sign_2[] = "write_record: self.write.raw_signal malloc done";
static const char __pyx_k_write_record_self_write_raw_sign_3[] = "write_record: self.write raw_signal done";
static const char __pyx_k_write_record_slow5_aux_set_strin_2[] = "write_record: slow5_aux_set_string type: {}";
static const char __pyx_k_write_record_slow5_aux_set_strin_3[] = "write_record: slow5_aux_set_string running done: ret = {}";
static const char __pyx_k_write_record_slow5_aux_set_strin_4[] = "write_record: slow5_aux_set_string could not write aux value {}: {}";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_kp_u_Batch;
static PyObject *__pyx_kp_u_Compression_type_rec_press_sig_p;
static PyObject *__pyx_kp_u_Creating_loading_index;
static PyObject *__pyx_n_s_DEBUG;
static PyObject *__pyx_kp_u_End_Of_File_reached;
static PyObject *__pyx_kp_u_FILE_mode;
static PyObject *__pyx_kp_u_File_could_not_be_opened_for_wri;
static PyObject *__pyx_kp_u_File_could_not_be_opened_for_wri_2;
static PyObject *__pyx_kp_u_File_unknown_open_method;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_u_NOT_IMPLEMENTED_YET_get_aux_type;
static PyObject *__pyx_kp_u_No_more_reads;
static PyObject *__pyx_kp_u_Not_writing_blow5_skipping_compr;
static PyObject *__pyx_kp_u_Number_of_read_groups;
static PyObject *__pyx_n_s_Open;
static PyObject *__pyx_n_s_Open__get_batches;
static PyObject *__pyx_n_s_Open__get_read_multi;
static PyObject *__pyx_n_s_Open_get_read_list;
static PyObject *__pyx_n_s_Open_get_read_list_multi;
static PyObject *__pyx_n_s_Open_seq_reads;
static PyObject *__pyx_n_s_Open_seq_reads_multi;
static PyObject *__pyx_kp_u_Please_use_only_the_following;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_u_SLOW5_ERR_ARG;
static PyObject *__pyx_n_u_SLOW5_ERR_EOF;
static PyObject *__pyx_n_u_SLOW5_ERR_HDRPARSE;
static PyObject *__pyx_n_u_SLOW5_ERR_IO;
static PyObject *__pyx_n_u_SLOW5_ERR_MAGIC;
static PyObject *__pyx_n_u_SLOW5_ERR_MEM;
static PyObject *__pyx_n_u_SLOW5_ERR_NOAUX;
static PyObject *__pyx_n_u_SLOW5_ERR_NOFLD;
static PyObject *__pyx_n_u_SLOW5_ERR_NOIDX;
static PyObject *__pyx_n_u_SLOW5_ERR_NOTFOUND;
static PyObject *__pyx_n_u_SLOW5_ERR_OTH;
static PyObject *__pyx_n_u_SLOW5_ERR_PRESS;
static PyObject *__pyx_n_u_SLOW5_ERR_RECPARSE;
static PyObject *__pyx_n_u_SLOW5_ERR_TRUNC;
static PyObject *__pyx_n_u_SLOW5_ERR_TYPE;
static PyObject *__pyx_n_u_SLOW5_ERR_UNK;
static PyObject *__pyx_n_u_SLOW5_ERR_VERSION;
static PyObject *__pyx_kp_u_Setting_up_batching;
static PyObject *__pyx_kp_u_Starting_rid_assignment;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_u_Unable_to_set_compression;
static PyObject *__pyx_n_s_WARNING;
static PyObject *__pyx_kp_u__2;
static PyObject *__pyx_kp_u__4;
static PyObject *__pyx_kp_u__5;
static PyObject *__pyx_n_u_a;
static PyObject *__pyx_n_u_all;
static PyObject *__pyx_kp_u_already_closed;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_kp_u_asctime_s_levelname_s_message_s;
static PyObject *__pyx_n_u_asic_id;
static PyObject *__pyx_n_u_asic_id_eeprom;
static PyObject *__pyx_n_u_asic_temp;
static PyObject *__pyx_n_u_asic_version;
static PyObject *__pyx_n_s_astype;
static PyObject *__pyx_n_s_attr;
static PyObject *__pyx_n_u_auto_update;
static PyObject *__pyx_n_u_auto_update_source;
static PyObject *__pyx_n_s_aux;
static PyObject *__pyx_n_s_aux_header_type_validation;
static PyObject *__pyx_n_s_aux_names;
static PyObject *__pyx_n_u_aux_total_time;
static PyObject *__pyx_kp_u_aux_type_validation_user_aux_fi;
static PyObject *__pyx_kp_u_aux_type_validation_user_aux_fi_2;
static PyObject *__pyx_n_s_aux_types;
static PyObject *__pyx_kp_u_bad_argument_NULL;
static PyObject *__pyx_n_u_barcoding_enabled;
static PyObject *__pyx_n_s_basicConfig;
static PyObject *__pyx_kp_u_batch_for_loop_start;
static PyObject *__pyx_n_s_batchsize;
static PyObject *__pyx_n_u_blow5;
static PyObject *__pyx_n_u_bream_is_standard;
static PyObject *__pyx_n_s_chain;
static PyObject *__pyx_n_u_channel_number;
static PyObject *__pyx_n_s_chr;
static PyObject *__pyx_n_s_cinit___locals_genexpr;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_kp_u_closed;
static PyObject *__pyx_n_u_configuration_version;
static PyObject *__pyx_n_s_convert_to_pA;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_kp_u_d_b_y_H_M_S;
static PyObject *__pyx_n_s_datefmt;
static PyObject *__pyx_kp_u_de_compression_failure;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_deepcopy;
static PyObject *__pyx_n_u_device_id;
static PyObject *__pyx_n_u_device_type;
static PyObject *__pyx_n_s_difference;
static PyObject *__pyx_n_u_digitisation;
static PyObject *__pyx_n_u_distribution_status;
static PyObject *__pyx_n_u_distribution_version;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_u_end_reason;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_kp_u_error_relating_to_slow5_type;
static PyObject *__pyx_n_u_exp_script_name;
static PyObject *__pyx_n_u_exp_script_purpose;
static PyObject *__pyx_n_u_exp_start_time;
static PyObject *__pyx_n_u_experiment_duration_set;
static PyObject *__pyx_kp_u_field_not_found;
static PyObject *__pyx_kp_u_file_format_unknown;
static PyObject *__pyx_kp_u_file_truncated;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_float32;
static PyObject *__pyx_n_u_flow_cell_id;
static PyObject *__pyx_n_u_flow_cell_product_code;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_getLogger;
static PyObject *__pyx_n_s_get_aux_names;
static PyObject *__pyx_kp_u_get_aux_names_aux_len;
static PyObject *__pyx_kp_u_get_aux_names_ret_is_NULL;
static PyObject *__pyx_kp_u_get_aux_names_starting;
static PyObject *__pyx_n_s_get_aux_types;
static PyObject *__pyx_kp_u_get_aux_types_aux_len;
static PyObject *__pyx_kp_u_get_aux_types_self_aux_get_err_i;
static PyObject *__pyx_kp_u_get_aux_types_self_s5_aux_type_i;
static PyObject *__pyx_kp_u_get_aux_types_starting;
static PyObject *__pyx_n_s_get_batches;
static PyObject *__pyx_n_s_get_header_names;
static PyObject *__pyx_n_s_get_header_value;
static PyObject *__pyx_kp_u_get_header_value_header_value_no;
static PyObject *__pyx_n_s_get_read;
static PyObject *__pyx_n_s_get_read_aux;
static PyObject *__pyx_kp_u_get_read_aux_atype_not_known_ski;
static PyObject *__pyx_kp_u_get_read_aux_type_unknown_accept;
static PyObject *__pyx_kp_u_get_read_ids_length_of_read_ids;
static PyObject *__pyx_kp_u_get_read_ids_returned_rids_is_NU;
static PyObject *__pyx_n_s_get_read_list;
static PyObject *__pyx_n_s_get_read_list_multi;
static PyObject *__pyx_n_s_get_read_multi;
static PyObject *__pyx_kp_u_get_read_multi_slow5_get_batch_r;
static PyObject *__pyx_kp_u_get_read_return_not_0;
static PyObject *__pyx_kp_u_get_read_unknown_aux_name;
static PyObject *__pyx_n_s_get_seq_read_aux;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_u_guppy_version;
static PyObject *__pyx_n_s_header;
static PyObject *__pyx_kp_u_header_parsing_error;
static PyObject *__pyx_n_s_header_type_validation;
static PyObject *__pyx_kp_u_header_type_validation_conversi;
static PyObject *__pyx_kp_u_header_type_validation_could_no;
static PyObject *__pyx_kp_u_header_type_validation_is_not_a;
static PyObject *__pyx_n_u_heatsink_temp;
static PyObject *__pyx_n_u_hostname;
static PyObject *__pyx_n_u_hublett_board_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_kp_u_index_not_loaded;
static PyObject *__pyx_n_u_installation_type;
static PyObject *__pyx_n_s_int16;
static PyObject *__pyx_n_s_itertools;
static PyObject *__pyx_n_s_keys;
static PyObject *__pyx_n_u_len_raw_signal;
static PyObject *__pyx_n_s_level;
static PyObject *__pyx_n_u_local_basecalling;
static PyObject *__pyx_n_s_logging;
static PyObject *__pyx_kp_u_magic_number_invalid;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_u_median_before;
static PyObject *__pyx_kp_u_memory_re_allocation_error;
static PyObject *__pyx_n_s_memoryview;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_multi_record_type_validation;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_u_no_auxiliary_map;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_u_none;
static PyObject *__pyx_kp_u_not_open_for_writing_remake_cla;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_num;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_u_offset;
static PyObject *__pyx_n_u_operating_system;
static PyObject *__pyx_kp_u_other_error_big_endian_internal;
static PyObject *__pyx_kp_u_other_file_I_O_error;
static PyObject *__pyx_n_s_pA;
static PyObject *__pyx_n_u_pA_total_time;
static PyObject *__pyx_n_u_package;
static PyObject *__pyx_n_s_pathname;
static PyObject *__pyx_kp_u_pathname_2;
static PyObject *__pyx_n_u_primary_total_time;
static PyObject *__pyx_n_u_protocol_group_id;
static PyObject *__pyx_n_u_protocol_run_id;
static PyObject *__pyx_n_u_protocol_start_time;
static PyObject *__pyx_n_u_protocols_version;
static PyObject *__pyx_n_s_pyslow5;
static PyObject *__pyx_n_u_r;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_u_range;
static PyObject *__pyx_kp_u_readID_bin;
static PyObject *__pyx_n_s_read_group;
static PyObject *__pyx_n_u_read_group;
static PyObject *__pyx_n_s_read_id;
static PyObject *__pyx_n_u_read_id;
static PyObject *__pyx_kp_u_read_id_not_found;
static PyObject *__pyx_n_s_read_list;
static PyObject *__pyx_kp_u_read_list_2;
static PyObject *__pyx_n_u_read_number;
static PyObject *__pyx_n_s_rec_press;
static PyObject *__pyx_n_s_record;
static PyObject *__pyx_kp_u_record_parsing_error;
static PyObject *__pyx_n_s_record_type_validation;
static PyObject *__pyx_kp_u_record_type_validation_aux_pass;
static PyObject *__pyx_kp_u_record_type_validation_aux_stuf;
static PyObject *__pyx_kp_u_record_type_validation_doing_au;
static PyObject *__pyx_kp_u_record_type_validation_user_aux;
static PyObject *__pyx_kp_u_record_type_validation_user_aux_2;
static PyObject *__pyx_kp_u_record_type_validation_user_pri;
static PyObject *__pyx_kp_u_record_type_validation_user_pri_2;
static PyObject *__pyx_n_s_records;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_kp_u_rid_assignment_complete;
static PyObject *__pyx_n_u_run_id;
static PyObject *__pyx_n_u_sample_frequency;
static PyObject *__pyx_n_u_sample_id;
static PyObject *__pyx_n_u_sampling_rate;
static PyObject *__pyx_n_u_satellite_firmware_version;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_seq_reads;
static PyObject *__pyx_n_s_seq_reads_multi;
static PyObject *__pyx_kp_u_seq_reads_multi_timings;
static PyObject *__pyx_n_u_sequencing_kit;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_sig_press;
static PyObject *__pyx_n_u_signal;
static PyObject *__pyx_n_u_signal_total_time;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_kp_u_slow5_get_batch_num_reads;
static PyObject *__pyx_kp_u_slow5_get_hdr_keys_head_len;
static PyObject *__pyx_kp_u_slow5_get_hdr_keys_ret_is_NULL;
static PyObject *__pyx_kp_u_slow5_get_next_error_code;
static PyObject *__pyx_kp_u_slow5_get_next_multi_aux_type_un;
static PyObject *__pyx_kp_u_slow5_get_next_multi_error_code;
static PyObject *__pyx_kp_u_slow5_get_next_multi_has_no_more;
static PyObject *__pyx_kp_u_slow5_get_next_multi_reached_end;
static PyObject *__pyx_kp_u_slow5_get_next_multi_return;
static PyObject *__pyx_kp_u_slow5_get_next_multi_unknown_aux;
static PyObject *__pyx_kp_u_slow5_get_next_reached_end_of_fi;
static PyObject *__pyx_kp_u_slow5_get_next_return;
static PyObject *__pyx_kp_u_slow5_idx_load_return_not_0;
static PyObject *__pyx_kp_u_slow5_set_press_return_not_0;
static PyObject *__pyx_n_u_start_mux;
static PyObject *__pyx_n_u_start_time;
static PyObject *__pyx_n_u_string;
static PyObject *__pyx_n_u_svb_zd;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_threads;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_kp_u_total_multi_write_signal_time_se;
static PyObject *__pyx_kp_u_total_multi_write_time_seconds;
static PyObject *__pyx_kp_u_total_single_write_time_seconds;
static PyObject *__pyx_kp_u_total_time_slow5_get_next_second;
static PyObject *__pyx_kp_u_total_time_yield_reads_seconds;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_u_usb_config;
static PyObject *__pyx_n_s_user_record;
static PyObject *__pyx_n_u_version;
static PyObject *__pyx_kp_u_version_incompatible;
static PyObject *__pyx_n_u_w;
static PyObject *__pyx_n_s_warning;
static PyObject *__pyx_kp_u_write_header_You_must_set_read_g;
static PyObject *__pyx_kp_u_write_header_errors_encountered;
static PyObject *__pyx_kp_u_write_header_headers_failed_to_i;
static PyObject *__pyx_kp_u_write_header_slow5_hdr_add_attr;
static PyObject *__pyx_kp_u_write_header_slow5_hdr_add_rg_fa;
static PyObject *__pyx_kp_u_write_header_slow5_hdr_set_could;
static PyObject *__pyx_kp_u_write_record__record_type_valida;
static PyObject *__pyx_kp_u_write_record__record_type_valida_2;
static PyObject *__pyx_kp_u_write_record_aux_meta_fields_fai;
static PyObject *__pyx_kp_u_write_record_aux_stuff;
static PyObject *__pyx_kp_u_write_record_aux_stuff_done;
static PyObject *__pyx_kp_u_write_record_batch_Setting_up_ba;
static PyObject *__pyx_kp_u_write_record_batch__record_type;
static PyObject *__pyx_kp_u_write_record_batch__record_type_2;
static PyObject *__pyx_kp_u_write_record_batch_aux_meta_fiel;
static PyObject *__pyx_kp_u_write_record_batch_aux_stuff;
static PyObject *__pyx_kp_u_write_record_batch_aux_stuff_don;
static PyObject *__pyx_kp_u_write_record_batch_batch_for_loo;
static PyObject *__pyx_kp_u_write_record_batch_batch_len_0_o;
static PyObject *__pyx_kp_u_write_record_batch_checked_aux;
static PyObject *__pyx_kp_u_write_record_batch_checked_aux_i;
static PyObject *__pyx_kp_u_write_record_batch_checking_head;
static PyObject *__pyx_kp_u_write_record_batch_failed_to_all;
static PyObject *__pyx_kp_u_write_record_batch_free;
static PyObject *__pyx_kp_u_write_record_batch_free_aux;
static PyObject *__pyx_kp_u_write_record_batch_function_comp;
static PyObject *__pyx_kp_u_write_record_batch_header_writte;
static PyObject *__pyx_kp_u_write_record_batch_self_write_as;
static PyObject *__pyx_kp_u_write_record_batch_self_write_as_2;
static PyObject *__pyx_kp_u_write_record_batch_self_write_pr;
static PyObject *__pyx_kp_u_write_record_batch_self_write_ra;
static PyObject *__pyx_kp_u_write_record_batch_self_write_ra_2;
static PyObject *__pyx_kp_u_write_record_batch_self_write_ra_3;
static PyObject *__pyx_kp_u_write_record_batch_slow5_aux_add;
static PyObject *__pyx_kp_u_write_record_batch_slow5_aux_set;
static PyObject *__pyx_kp_u_write_record_batch_slow5_aux_set_2;
static PyObject *__pyx_kp_u_write_record_batch_slow5_aux_set_3;
static PyObject *__pyx_kp_u_write_record_batch_slow5_aux_set_4;
static PyObject *__pyx_kp_u_write_record_batch_slow5_aux_set_5;
static PyObject *__pyx_kp_u_write_record_batch_slow5_hdr_wri;
static PyObject *__pyx_kp_u_write_record_batch_slow5_rec_ini;
static PyObject *__pyx_kp_u_write_record_batch_slow5_write_b;
static PyObject *__pyx_kp_u_write_record_batch_write_failed;
static PyObject *__pyx_kp_u_write_record_batch_writting_head;
static PyObject *__pyx_kp_u_write_record_checked_aux;
static PyObject *__pyx_kp_u_write_record_checked_aux_is_None;
static PyObject *__pyx_kp_u_write_record_checking_header_stu;
static PyObject *__pyx_kp_u_write_record_failed_to_allocate;
static PyObject *__pyx_kp_u_write_record_function_complete_r;
static PyObject *__pyx_kp_u_write_record_header_written;
static PyObject *__pyx_kp_u_write_record_self_write_assignme;
static PyObject *__pyx_kp_u_write_record_self_write_assignme_2;
static PyObject *__pyx_kp_u_write_record_self_write_processi;
static PyObject *__pyx_kp_u_write_record_self_write_raw_sign;
static PyObject *__pyx_kp_u_write_record_self_write_raw_sign_2;
static PyObject *__pyx_kp_u_write_record_self_write_raw_sign_3;
static PyObject *__pyx_kp_u_write_record_slow5_aux_add_could;
static PyObject *__pyx_kp_u_write_record_slow5_aux_set_could;
static PyObject *__pyx_kp_u_write_record_slow5_aux_set_strin;
static PyObject *__pyx_kp_u_write_record_slow5_aux_set_strin_2;
static PyObject *__pyx_kp_u_write_record_slow5_aux_set_strin_3;
static PyObject *__pyx_kp_u_write_record_slow5_aux_set_strin_4;
static PyObject *__pyx_kp_u_write_record_slow5_hdr_write_cou;
static PyObject *__pyx_kp_u_write_record_slow5_rec_free;
static PyObject *__pyx_kp_u_write_record_slow5_rec_init;
static PyObject *__pyx_kp_u_write_record_slow5_write;
static PyObject *__pyx_kp_u_write_record_slow5_write_ret;
static PyObject *__pyx_kp_u_write_record_writting_header;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_n_u_zlib;
static PyObject *__pyx_n_u_zstd;
static PyObject *__pyx_pf_7pyslow5_4Open_9__cinit___genexpr(PyObject *__pyx_self); /* proto */
static int __pyx_pf_7pyslow5_4Open___cinit__(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_pathname, PyObject *__pyx_v_mode, PyObject *__pyx_v_rec_press, PyObject *__pyx_v_sig_press, PyObject *__pyx_v_DEBUG); /* proto */
static int __pyx_pf_7pyslow5_4Open_2__init__(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_pathname, CYTHON_UNUSED PyObject *__pyx_v_mode, CYTHON_UNUSED PyObject *__pyx_v_rec_press, CYTHON_UNUSED PyObject *__pyx_v_sig_press, CYTHON_UNUSED PyObject *__pyx_v_DEBUG); /* proto */
static void __pyx_pf_7pyslow5_4Open_4__dealloc__(struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_6_convert_to_pA(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_d); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_8_get_read(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_id, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_10_get_batches(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_13_get_read_multi(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_16_get_read_aux(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_aux_names, PyObject *__pyx_v_aux_types); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_18_get_seq_read_aux(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_aux_names, PyObject *__pyx_v_aux_types); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_20get_read_ids(struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_22get_read(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_id, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_24seq_reads(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_27seq_reads_multi(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_30get_read_list(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_33get_read_list_multi(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_36get_header_names(struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_38get_header_value(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_attr, PyObject *__pyx_v_read_group); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_40get_all_headers(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_group); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_42get_aux_names(struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_44get_aux_types(struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_46get_empty_header(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_48get_empty_record(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_50_header_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_header); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_52_aux_header_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_user_aux_types); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_54_record_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_user_record, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_56_multi_record_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_user_record, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_58write_header(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_header, PyObject *__pyx_v_read_group); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_60write_record(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_record, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_62write_record_batch(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_records, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_aux); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_64close(struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_66__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7pyslow5_4Open_68__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_7pyslow5_Open(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct____cinit__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_2__get_batches(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_3__get_read_multi(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_4_seq_reads(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_5_seq_reads_multi(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_6_get_read_list(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_7_get_read_list_multi(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_keys = {0, &__pyx_n_s_keys, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_update = {0, &__pyx_n_s_update, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PySet_Type_difference = {0, &__pyx_n_s_difference, 0, 0, 0};
static PyObject *__pyx_float_0_0;
static PyObject *__pyx_float_1_0;
static PyObject *__pyx_float_neg_1_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_7;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_9;
static PyObject *__pyx_int_10;
static PyObject *__pyx_int_11;
static PyObject *__pyx_int_12;
static PyObject *__pyx_int_13;
static PyObject *__pyx_int_14;
static PyObject *__pyx_int_15;
static PyObject *__pyx_int_16;
static PyObject *__pyx_int_17;
static PyObject *__pyx_int_18;
static PyObject *__pyx_int_19;
static PyObject *__pyx_int_20;
static PyObject *__pyx_int_21;
static PyObject *__pyx_int_22;
static PyObject *__pyx_int_100;
static PyObject *__pyx_int_4096;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_int_neg_2;
static PyObject *__pyx_int_neg_3;
static PyObject *__pyx_int_neg_4;
static PyObject *__pyx_int_neg_5;
static PyObject *__pyx_int_neg_6;
static PyObject *__pyx_int_neg_7;
static PyObject *__pyx_int_neg_8;
static PyObject *__pyx_int_neg_9;
static PyObject *__pyx_int_neg_10;
static PyObject *__pyx_int_neg_11;
static PyObject *__pyx_int_neg_12;
static PyObject *__pyx_int_neg_13;
static PyObject *__pyx_int_neg_14;
static PyObject *__pyx_int_neg_15;
static PyObject *__pyx_int_neg_16;
static PyObject *__pyx_int_neg_17;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
/* Late includes */

/* "pyslow5.pyx":105
 *     cdef pyslow5.float total_multi_write_time
 * 
 *     def __cinit__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):             # <<<<<<<<<<<<<<
 *         # Set to default NULL type
 *         self.s5 = NULL
 */

/* Python wrapper */
static int __pyx_pw_7pyslow5_4Open_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7pyslow5_4Open_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pathname = 0;
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_rec_press = 0;
  PyObject *__pyx_v_sig_press = 0;
  PyObject *__pyx_v_DEBUG = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pathname,&__pyx_n_s_mode,&__pyx_n_s_rec_press,&__pyx_n_s_sig_press,&__pyx_n_s_DEBUG,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[2] = ((PyObject *)__pyx_n_u_zlib);
    values[3] = ((PyObject *)__pyx_n_u_svb_zd);
    values[4] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pathname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 5, 1); __PYX_ERR(0, 105, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rec_press);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sig_press);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_DEBUG);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 105, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_pathname = values[0];
    __pyx_v_mode = values[1];
    __pyx_v_rec_press = values[2];
    __pyx_v_sig_press = values[3];
    __pyx_v_DEBUG = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 105, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open___cinit__(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_pathname, __pyx_v_mode, __pyx_v_rec_press, __pyx_v_sig_press, __pyx_v_DEBUG);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_9__cinit___2generator6(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":260
 *                 else:
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))
 *                     self.logger.error("Please use only the following: {}".format(",".join(press for press in self.slow5_press_method.keys())))             # <<<<<<<<<<<<<<
 *                     raise KeyError
 *             else:
 */

static PyObject *__pyx_pf_7pyslow5_4Open_9__cinit___genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *)__pyx_tp_new_7pyslow5___pyx_scope_struct_1_genexpr(__pyx_ptype_7pyslow5___pyx_scope_struct_1_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 260, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_9__cinit___2generator6, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_cinit___locals_genexpr, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open.__cinit__.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_9__cinit___2generator6(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  __Pyx_TraceCall("genexpr", __pyx_f[0], 260, 0, __PYX_ERR(0, 260, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 260, __pyx_L1_error)
  __pyx_r = PyList_New(0); if (unlikely(!__pyx_r)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_r);
  __pyx_t_2 = 0;
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 260, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->slow5_press_method == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "keys");
    __PYX_ERR(0, 260, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_dict_iterator(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->slow5_press_method, 1, __pyx_n_s_keys, (&__pyx_t_3), (&__pyx_t_4)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_5;
  __pyx_t_5 = 0;
  while (1) {
    __pyx_t_6 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_3, &__pyx_t_2, &__pyx_t_5, NULL, NULL, __pyx_t_4);
    if (unlikely(__pyx_t_6 == 0)) break;
    if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_press);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_press, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    __pyx_t_5 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_r, (PyObject*)__pyx_cur_scope->__pyx_v_press))) __PYX_ERR(0, 260, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":105
 *     cdef pyslow5.float total_multi_write_time
 * 
 *     def __cinit__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):             # <<<<<<<<<<<<<<
 *         # Set to default NULL type
 *         self.s5 = NULL
 */

static int __pyx_pf_7pyslow5_4Open___cinit__(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_pathname, PyObject *__pyx_v_mode, PyObject *__pyx_v_rec_press, PyObject *__pyx_v_sig_press, PyObject *__pyx_v_DEBUG) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *__pyx_cur_scope;
  CYTHON_UNUSED void *__pyx_v_channel_number_val;
  CYTHON_UNUSED double __pyx_v_median_before_val;
  CYTHON_UNUSED long __pyx_v_read_number_val;
  CYTHON_UNUSED long __pyx_v_start_mux_val;
  CYTHON_UNUSED long __pyx_v_start_time_val;
  CYTHON_UNUSED void *__pyx_v_channel_number_val_array;
  CYTHON_UNUSED void *__pyx_v_median_before_val_array;
  CYTHON_UNUSED void *__pyx_v_read_number_val_array;
  CYTHON_UNUSED void *__pyx_v_start_mux_val_array;
  CYTHON_UNUSED void *__pyx_v_start_time_val_array;
  PyObject *__pyx_v_lev = NULL;
  PyObject *__pyx_v_p = NULL;
  PyObject *__pyx_v_m = NULL;
  int __pyx_v_ret;
  PyObject *__pyx_gb_7pyslow5_4Open_9__cinit___2generator6 = 0;
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  char const *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *)__pyx_tp_new_7pyslow5___pyx_scope_struct____cinit__(__pyx_ptype_7pyslow5___pyx_scope_struct____cinit__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 105, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __Pyx_TraceCall("__cinit__", __pyx_f[0], 105, 0, __PYX_ERR(0, 105, __pyx_L1_error));
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "pyslow5.pyx":107
 *     def __cinit__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):
 *         # Set to default NULL type
 *         self.s5 = NULL             # <<<<<<<<<<<<<<
 *         self.rec = NULL
 *         self.read = NULL
 */
  __pyx_cur_scope->__pyx_v_self->s5 = NULL;

  /* "pyslow5.pyx":108
 *         # Set to default NULL type
 *         self.s5 = NULL
 *         self.rec = NULL             # <<<<<<<<<<<<<<
 *         self.read = NULL
 *         self.write = NULL
 */
  __pyx_cur_scope->__pyx_v_self->rec = NULL;

  /* "pyslow5.pyx":109
 *         self.s5 = NULL
 *         self.rec = NULL
 *         self.read = NULL             # <<<<<<<<<<<<<<
 *         self.write = NULL
 *         self.twrite = NULL
 */
  __pyx_cur_scope->__pyx_v_self->read = NULL;

  /* "pyslow5.pyx":110
 *         self.rec = NULL
 *         self.read = NULL
 *         self.write = NULL             # <<<<<<<<<<<<<<
 *         self.twrite = NULL
 *         self.path = ""
 */
  __pyx_cur_scope->__pyx_v_self->write = NULL;

  /* "pyslow5.pyx":111
 *         self.read = NULL
 *         self.write = NULL
 *         self.twrite = NULL             # <<<<<<<<<<<<<<
 *         self.path = ""
 *         self.p = ""
 */
  __pyx_cur_scope->__pyx_v_self->twrite = NULL;

  /* "pyslow5.pyx":112
 *         self.write = NULL
 *         self.twrite = NULL
 *         self.path = ""             # <<<<<<<<<<<<<<
 *         self.p = ""
 *         self.mode = ""
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __Pyx_GIVEREF(__pyx_kp_u_);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->path);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->path);
  __pyx_cur_scope->__pyx_v_self->path = __pyx_kp_u_;

  /* "pyslow5.pyx":113
 *         self.twrite = NULL
 *         self.path = ""
 *         self.p = ""             # <<<<<<<<<<<<<<
 *         self.mode = ""
 *         self.m = ""
 */
  __pyx_cur_scope->__pyx_v_self->p = ((char *)"");

  /* "pyslow5.pyx":114
 *         self.path = ""
 *         self.p = ""
 *         self.mode = ""             # <<<<<<<<<<<<<<
 *         self.m = ""
 *         self.rec_press = ""
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __Pyx_GIVEREF(__pyx_kp_u_);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->mode);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->mode);
  __pyx_cur_scope->__pyx_v_self->mode = __pyx_kp_u_;

  /* "pyslow5.pyx":115
 *         self.p = ""
 *         self.mode = ""
 *         self.m = ""             # <<<<<<<<<<<<<<
 *         self.rec_press = ""
 *         self.sig_press = ""
 */
  __pyx_cur_scope->__pyx_v_self->m = ((char *)"");

  /* "pyslow5.pyx":116
 *         self.mode = ""
 *         self.m = ""
 *         self.rec_press = ""             # <<<<<<<<<<<<<<
 *         self.sig_press = ""
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __Pyx_GIVEREF(__pyx_kp_u_);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->rec_press);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->rec_press);
  __pyx_cur_scope->__pyx_v_self->rec_press = __pyx_kp_u_;

  /* "pyslow5.pyx":117
 *         self.m = ""
 *         self.rec_press = ""
 *         self.sig_press = ""             # <<<<<<<<<<<<<<
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 *         self.state = -1
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __Pyx_GIVEREF(__pyx_kp_u_);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->sig_press);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->sig_press);
  __pyx_cur_scope->__pyx_v_self->sig_press = __pyx_kp_u_;

  /* "pyslow5.pyx":119
 *         self.sig_press = ""
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 *         self.state = -1             # <<<<<<<<<<<<<<
 *         self.trec = NULL
 *         self.rid = NULL
 */
  __pyx_cur_scope->__pyx_v_self->state = -1;

  /* "pyslow5.pyx":120
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 *         self.state = -1
 *         self.trec = NULL             # <<<<<<<<<<<<<<
 *         self.rid = NULL
 *         self.rids = NULL
 */
  __pyx_cur_scope->__pyx_v_self->trec = NULL;

  /* "pyslow5.pyx":121
 *         self.state = -1
 *         self.trec = NULL
 *         self.rid = NULL             # <<<<<<<<<<<<<<
 *         self.rids = NULL
 *         self.rids_len = 0
 */
  __pyx_cur_scope->__pyx_v_self->rid = NULL;

  /* "pyslow5.pyx":122
 *         self.trec = NULL
 *         self.rid = NULL
 *         self.rids = NULL             # <<<<<<<<<<<<<<
 *         self.rids_len = 0
 *         self.index_state = False
 */
  __pyx_cur_scope->__pyx_v_self->rids = NULL;

  /* "pyslow5.pyx":123
 *         self.rid = NULL
 *         self.rids = NULL
 *         self.rids_len = 0             # <<<<<<<<<<<<<<
 *         self.index_state = False
 *         self.header_state = False
 */
  __pyx_cur_scope->__pyx_v_self->rids_len = 0;

  /* "pyslow5.pyx":124
 *         self.rids = NULL
 *         self.rids_len = 0
 *         self.index_state = False             # <<<<<<<<<<<<<<
 *         self.header_state = False
 *         self.header_add_attr_state = False
 */
  __pyx_cur_scope->__pyx_v_self->index_state = 0;

  /* "pyslow5.pyx":125
 *         self.rids_len = 0
 *         self.index_state = False
 *         self.header_state = False             # <<<<<<<<<<<<<<
 *         self.header_add_attr_state = False
 *         self.close_state = False
 */
  __pyx_cur_scope->__pyx_v_self->header_state = 0;

  /* "pyslow5.pyx":126
 *         self.index_state = False
 *         self.header_state = False
 *         self.header_add_attr_state = False             # <<<<<<<<<<<<<<
 *         self.close_state = False
 *         self.s5_aux_type = NULL
 */
  __pyx_cur_scope->__pyx_v_self->header_add_attr_state = 0;

  /* "pyslow5.pyx":127
 *         self.header_state = False
 *         self.header_add_attr_state = False
 *         self.close_state = False             # <<<<<<<<<<<<<<
 *         self.s5_aux_type = NULL
 *         self.aux_get_err = 1
 */
  __pyx_cur_scope->__pyx_v_self->close_state = 0;

  /* "pyslow5.pyx":128
 *         self.header_add_attr_state = False
 *         self.close_state = False
 *         self.s5_aux_type = NULL             # <<<<<<<<<<<<<<
 *         self.aux_get_err = 1
 *         self.aux_get_len = 0
 */
  __pyx_cur_scope->__pyx_v_self->s5_aux_type = NULL;

  /* "pyslow5.pyx":129
 *         self.close_state = False
 *         self.s5_aux_type = NULL
 *         self.aux_get_err = 1             # <<<<<<<<<<<<<<
 *         self.aux_get_len = 0
 *         self.head_len = 0
 */
  __pyx_cur_scope->__pyx_v_self->aux_get_err = 1;

  /* "pyslow5.pyx":130
 *         self.s5_aux_type = NULL
 *         self.aux_get_err = 1
 *         self.aux_get_len = 0             # <<<<<<<<<<<<<<
 *         self.head_len = 0
 *         self.aux_len = 0
 */
  __pyx_cur_scope->__pyx_v_self->aux_get_len = 0;

  /* "pyslow5.pyx":131
 *         self.aux_get_err = 1
 *         self.aux_get_len = 0
 *         self.head_len = 0             # <<<<<<<<<<<<<<
 *         self.aux_len = 0
 *         self.shape_seq[0] = 0
 */
  __pyx_cur_scope->__pyx_v_self->head_len = 0;

  /* "pyslow5.pyx":132
 *         self.aux_get_len = 0
 *         self.head_len = 0
 *         self.aux_len = 0             # <<<<<<<<<<<<<<
 *         self.shape_seq[0] = 0
 *         self.shape_get[0] = 0
 */
  __pyx_cur_scope->__pyx_v_self->aux_len = 0;

  /* "pyslow5.pyx":133
 *         self.head_len = 0
 *         self.aux_len = 0
 *         self.shape_seq[0] = 0             # <<<<<<<<<<<<<<
 *         self.shape_get[0] = 0
 *         self.e0 = -1
 */
  (__pyx_cur_scope->__pyx_v_self->shape_seq[0]) = 0;

  /* "pyslow5.pyx":134
 *         self.aux_len = 0
 *         self.shape_seq[0] = 0
 *         self.shape_get[0] = 0             # <<<<<<<<<<<<<<
 *         self.e0 = -1
 *         self.e1 = -1
 */
  (__pyx_cur_scope->__pyx_v_self->shape_get[0]) = 0;

  /* "pyslow5.pyx":135
 *         self.shape_seq[0] = 0
 *         self.shape_get[0] = 0
 *         self.e0 = -1             # <<<<<<<<<<<<<<
 *         self.e1 = -1
 *         self.e2 = -1
 */
  __pyx_cur_scope->__pyx_v_self->e0 = -1;

  /* "pyslow5.pyx":136
 *         self.shape_get[0] = 0
 *         self.e0 = -1
 *         self.e1 = -1             # <<<<<<<<<<<<<<
 *         self.e2 = -1
 *         self.e3 = -1
 */
  __pyx_cur_scope->__pyx_v_self->e1 = -1;

  /* "pyslow5.pyx":137
 *         self.e0 = -1
 *         self.e1 = -1
 *         self.e2 = -1             # <<<<<<<<<<<<<<
 *         self.e3 = -1
 *         self.e4 = -1
 */
  __pyx_cur_scope->__pyx_v_self->e2 = -1;

  /* "pyslow5.pyx":138
 *         self.e1 = -1
 *         self.e2 = -1
 *         self.e3 = -1             # <<<<<<<<<<<<<<
 *         self.e4 = -1
 *         self.e5 = -1
 */
  __pyx_cur_scope->__pyx_v_self->e3 = -1L;

  /* "pyslow5.pyx":139
 *         self.e2 = -1
 *         self.e3 = -1
 *         self.e4 = -1             # <<<<<<<<<<<<<<
 *         self.e5 = -1
 *         self.e6 = -1
 */
  __pyx_cur_scope->__pyx_v_self->e4 = -1;

  /* "pyslow5.pyx":140
 *         self.e3 = -1
 *         self.e4 = -1
 *         self.e5 = -1             # <<<<<<<<<<<<<<
 *         self.e6 = -1
 *         self.e7 = -1
 */
  __pyx_cur_scope->__pyx_v_self->e5 = -1;

  /* "pyslow5.pyx":141
 *         self.e4 = -1
 *         self.e5 = -1
 *         self.e6 = -1             # <<<<<<<<<<<<<<
 *         self.e7 = -1
 *         self.e8 = -1.0
 */
  __pyx_cur_scope->__pyx_v_self->e6 = -1;

  /* "pyslow5.pyx":142
 *         self.e5 = -1
 *         self.e6 = -1
 *         self.e7 = -1             # <<<<<<<<<<<<<<
 *         self.e8 = -1.0
 *         self.e9 = -1.0
 */
  __pyx_cur_scope->__pyx_v_self->e7 = -1LL;

  /* "pyslow5.pyx":143
 *         self.e6 = -1
 *         self.e7 = -1
 *         self.e8 = -1.0             # <<<<<<<<<<<<<<
 *         self.e9 = -1.0
 *         self.e10 = 0
 */
  __Pyx_INCREF(__pyx_float_neg_1_0);
  __Pyx_GIVEREF(__pyx_float_neg_1_0);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->e8);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->e8);
  __pyx_cur_scope->__pyx_v_self->e8 = __pyx_float_neg_1_0;

  /* "pyslow5.pyx":144
 *         self.e7 = -1
 *         self.e8 = -1.0
 *         self.e9 = -1.0             # <<<<<<<<<<<<<<
 *         self.e10 = 0
 *         self.e11 = 0
 */
  __pyx_cur_scope->__pyx_v_self->e9 = -1.0;

  /* "pyslow5.pyx":145
 *         self.e8 = -1.0
 *         self.e9 = -1.0
 *         self.e10 = 0             # <<<<<<<<<<<<<<
 *         self.e11 = 0
 *         self.e12 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e10 = 0;

  /* "pyslow5.pyx":146
 *         self.e9 = -1.0
 *         self.e10 = 0
 *         self.e11 = 0             # <<<<<<<<<<<<<<
 *         self.e12 = NULL
 *         self.e13 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e11 = 0;

  /* "pyslow5.pyx":147
 *         self.e10 = 0
 *         self.e11 = 0
 *         self.e12 = NULL             # <<<<<<<<<<<<<<
 *         self.e13 = NULL
 *         self.e14 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e12 = NULL;

  /* "pyslow5.pyx":148
 *         self.e11 = 0
 *         self.e12 = NULL
 *         self.e13 = NULL             # <<<<<<<<<<<<<<
 *         self.e14 = NULL
 *         self.e15 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e13 = NULL;

  /* "pyslow5.pyx":149
 *         self.e12 = NULL
 *         self.e13 = NULL
 *         self.e14 = NULL             # <<<<<<<<<<<<<<
 *         self.e15 = NULL
 *         self.e16 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e14 = NULL;

  /* "pyslow5.pyx":150
 *         self.e13 = NULL
 *         self.e14 = NULL
 *         self.e15 = NULL             # <<<<<<<<<<<<<<
 *         self.e16 = NULL
 *         self.e17 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e15 = NULL;

  /* "pyslow5.pyx":151
 *         self.e14 = NULL
 *         self.e15 = NULL
 *         self.e16 = NULL             # <<<<<<<<<<<<<<
 *         self.e17 = NULL
 *         self.e18 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e16 = NULL;

  /* "pyslow5.pyx":152
 *         self.e15 = NULL
 *         self.e16 = NULL
 *         self.e17 = NULL             # <<<<<<<<<<<<<<
 *         self.e18 = NULL
 *         self.e19 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e17 = NULL;

  /* "pyslow5.pyx":153
 *         self.e16 = NULL
 *         self.e17 = NULL
 *         self.e18 = NULL             # <<<<<<<<<<<<<<
 *         self.e19 = NULL
 *         self.e20 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e18 = NULL;

  /* "pyslow5.pyx":154
 *         self.e17 = NULL
 *         self.e18 = NULL
 *         self.e19 = NULL             # <<<<<<<<<<<<<<
 *         self.e20 = NULL
 *         self.e21 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e19 = NULL;

  /* "pyslow5.pyx":155
 *         self.e18 = NULL
 *         self.e19 = NULL
 *         self.e20 = NULL             # <<<<<<<<<<<<<<
 *         self.e21 = NULL
 *         self.e22 = NULL
 */
  __pyx_cur_scope->__pyx_v_self->e20 = NULL;

  /* "pyslow5.pyx":156
 *         self.e19 = NULL
 *         self.e20 = NULL
 *         self.e21 = NULL             # <<<<<<<<<<<<<<
 *         self.e22 = NULL
 *         self.channel_number = strdup("channel_number")
 */
  __pyx_cur_scope->__pyx_v_self->e21 = NULL;

  /* "pyslow5.pyx":157
 *         self.e20 = NULL
 *         self.e21 = NULL
 *         self.e22 = NULL             # <<<<<<<<<<<<<<
 *         self.channel_number = strdup("channel_number")
 *         self.median_before = strdup("median_before")
 */
  __pyx_cur_scope->__pyx_v_self->e22 = NULL;

  /* "pyslow5.pyx":158
 *         self.e21 = NULL
 *         self.e22 = NULL
 *         self.channel_number = strdup("channel_number")             # <<<<<<<<<<<<<<
 *         self.median_before = strdup("median_before")
 *         self.read_number = strdup("read_number")
 */
  __pyx_cur_scope->__pyx_v_self->channel_number = strdup(((char const *)"channel_number"));

  /* "pyslow5.pyx":159
 *         self.e22 = NULL
 *         self.channel_number = strdup("channel_number")
 *         self.median_before = strdup("median_before")             # <<<<<<<<<<<<<<
 *         self.read_number = strdup("read_number")
 *         self.start_mux = strdup("start_mux")
 */
  __pyx_cur_scope->__pyx_v_self->median_before = strdup(((char const *)"median_before"));

  /* "pyslow5.pyx":160
 *         self.channel_number = strdup("channel_number")
 *         self.median_before = strdup("median_before")
 *         self.read_number = strdup("read_number")             # <<<<<<<<<<<<<<
 *         self.start_mux = strdup("start_mux")
 *         self.start_time = strdup("start_time")
 */
  __pyx_cur_scope->__pyx_v_self->read_number = strdup(((char const *)"read_number"));

  /* "pyslow5.pyx":161
 *         self.median_before = strdup("median_before")
 *         self.read_number = strdup("read_number")
 *         self.start_mux = strdup("start_mux")             # <<<<<<<<<<<<<<
 *         self.start_time = strdup("start_time")
 *         self.end_reason = strdup("end_reason")
 */
  __pyx_cur_scope->__pyx_v_self->start_mux = strdup(((char const *)"start_mux"));

  /* "pyslow5.pyx":162
 *         self.read_number = strdup("read_number")
 *         self.start_mux = strdup("start_mux")
 *         self.start_time = strdup("start_time")             # <<<<<<<<<<<<<<
 *         self.end_reason = strdup("end_reason")
 *         channel_number_val = NULL
 */
  __pyx_cur_scope->__pyx_v_self->start_time = strdup(((char const *)"start_time"));

  /* "pyslow5.pyx":163
 *         self.start_mux = strdup("start_mux")
 *         self.start_time = strdup("start_time")
 *         self.end_reason = strdup("end_reason")             # <<<<<<<<<<<<<<
 *         channel_number_val = NULL
 *         median_before_val = -1.0
 */
  __pyx_cur_scope->__pyx_v_self->end_reason = strdup(((char const *)"end_reason"));

  /* "pyslow5.pyx":164
 *         self.start_time = strdup("start_time")
 *         self.end_reason = strdup("end_reason")
 *         channel_number_val = NULL             # <<<<<<<<<<<<<<
 *         median_before_val = -1.0
 *         read_number_val = -1
 */
  __pyx_v_channel_number_val = NULL;

  /* "pyslow5.pyx":165
 *         self.end_reason = strdup("end_reason")
 *         channel_number_val = NULL
 *         median_before_val = -1.0             # <<<<<<<<<<<<<<
 *         read_number_val = -1
 *         start_mux_val = -1
 */
  __pyx_v_median_before_val = -1.0;

  /* "pyslow5.pyx":166
 *         channel_number_val = NULL
 *         median_before_val = -1.0
 *         read_number_val = -1             # <<<<<<<<<<<<<<
 *         start_mux_val = -1
 *         start_time_val = -1
 */
  __pyx_v_read_number_val = -1L;

  /* "pyslow5.pyx":167
 *         median_before_val = -1.0
 *         read_number_val = -1
 *         start_mux_val = -1             # <<<<<<<<<<<<<<
 *         start_time_val = -1
 *         channel_number_val_array = NULL
 */
  __pyx_v_start_mux_val = -1L;

  /* "pyslow5.pyx":168
 *         read_number_val = -1
 *         start_mux_val = -1
 *         start_time_val = -1             # <<<<<<<<<<<<<<
 *         channel_number_val_array = NULL
 *         median_before_val_array = NULL
 */
  __pyx_v_start_time_val = -1L;

  /* "pyslow5.pyx":169
 *         start_mux_val = -1
 *         start_time_val = -1
 *         channel_number_val_array = NULL             # <<<<<<<<<<<<<<
 *         median_before_val_array = NULL
 *         read_number_val_array = NULL
 */
  __pyx_v_channel_number_val_array = NULL;

  /* "pyslow5.pyx":170
 *         start_time_val = -1
 *         channel_number_val_array = NULL
 *         median_before_val_array = NULL             # <<<<<<<<<<<<<<
 *         read_number_val_array = NULL
 *         start_mux_val_array = NULL
 */
  __pyx_v_median_before_val_array = NULL;

  /* "pyslow5.pyx":171
 *         channel_number_val_array = NULL
 *         median_before_val_array = NULL
 *         read_number_val_array = NULL             # <<<<<<<<<<<<<<
 *         start_mux_val_array = NULL
 *         start_time_val_array = NULL
 */
  __pyx_v_read_number_val_array = NULL;

  /* "pyslow5.pyx":172
 *         median_before_val_array = NULL
 *         read_number_val_array = NULL
 *         start_mux_val_array = NULL             # <<<<<<<<<<<<<<
 *         start_time_val_array = NULL
 * 
 */
  __pyx_v_start_mux_val_array = NULL;

  /* "pyslow5.pyx":173
 *         read_number_val_array = NULL
 *         start_mux_val_array = NULL
 *         start_time_val_array = NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_start_time_val_array = NULL;

  /* "pyslow5.pyx":177
 * 
 *         # cdef something end_reason # some enum
 *         self.total_time_slow5_get_next = 0.0             # <<<<<<<<<<<<<<
 *         self.total_time_yield_reads = 0.0
 *         self.total_single_write_time = 0.0
 */
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next);
  __pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next = __pyx_float_0_0;

  /* "pyslow5.pyx":178
 *         # cdef something end_reason # some enum
 *         self.total_time_slow5_get_next = 0.0
 *         self.total_time_yield_reads = 0.0             # <<<<<<<<<<<<<<
 *         self.total_single_write_time = 0.0
 *         self.total_multi_write_signal_time = 0.0
 */
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads);
  __pyx_cur_scope->__pyx_v_self->total_time_yield_reads = __pyx_float_0_0;

  /* "pyslow5.pyx":179
 *         self.total_time_slow5_get_next = 0.0
 *         self.total_time_yield_reads = 0.0
 *         self.total_single_write_time = 0.0             # <<<<<<<<<<<<<<
 *         self.total_multi_write_signal_time = 0.0
 *         self.total_multi_write_time = 0.0
 */
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_single_write_time);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_single_write_time);
  __pyx_cur_scope->__pyx_v_self->total_single_write_time = __pyx_float_0_0;

  /* "pyslow5.pyx":180
 *         self.total_time_yield_reads = 0.0
 *         self.total_single_write_time = 0.0
 *         self.total_multi_write_signal_time = 0.0             # <<<<<<<<<<<<<<
 *         self.total_multi_write_time = 0.0
 * 
 */
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_multi_write_signal_time);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_multi_write_signal_time);
  __pyx_cur_scope->__pyx_v_self->total_multi_write_signal_time = __pyx_float_0_0;

  /* "pyslow5.pyx":181
 *         self.total_single_write_time = 0.0
 *         self.total_multi_write_signal_time = 0.0
 *         self.total_multi_write_time = 0.0             # <<<<<<<<<<<<<<
 * 
 *         # sets up logging level/verbosity
 */
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_multi_write_time);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_multi_write_time);
  __pyx_cur_scope->__pyx_v_self->total_multi_write_time = __pyx_float_0_0;

  /* "pyslow5.pyx":184
 * 
 *         # sets up logging level/verbosity
 *         self.V = DEBUG             # <<<<<<<<<<<<<<
 *         self.logger = logging.getLogger(__name__)
 *         if self.V == 1:
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_DEBUG); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 184, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_self->V = __pyx_t_1;

  /* "pyslow5.pyx":185
 *         # sets up logging level/verbosity
 *         self.V = DEBUG
 *         self.logger = logging.getLogger(__name__)             # <<<<<<<<<<<<<<
 *         if self.V == 1:
 *             lev = logging.DEBUG
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_logging); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_getLogger); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->logger);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->logger);
  __pyx_cur_scope->__pyx_v_self->logger = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":186
 *         self.V = DEBUG
 *         self.logger = logging.getLogger(__name__)
 *         if self.V == 1:             # <<<<<<<<<<<<<<
 *             lev = logging.DEBUG
 *         else:
 */
  __pyx_t_6 = ((__pyx_cur_scope->__pyx_v_self->V == 1) != 0);
  if (__pyx_t_6) {

    /* "pyslow5.pyx":187
 *         self.logger = logging.getLogger(__name__)
 *         if self.V == 1:
 *             lev = logging.DEBUG             # <<<<<<<<<<<<<<
 *         else:
 *             lev = logging.WARNING
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_DEBUG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_lev = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "pyslow5.pyx":186
 *         self.V = DEBUG
 *         self.logger = logging.getLogger(__name__)
 *         if self.V == 1:             # <<<<<<<<<<<<<<
 *             lev = logging.DEBUG
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyslow5.pyx":189
 *             lev = logging.DEBUG
 *         else:
 *             lev = logging.WARNING             # <<<<<<<<<<<<<<
 * 
 *         logging.basicConfig(format='%(asctime)s - [%(levelname)s]: %(message)s',
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_logging); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_WARNING); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_lev = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "pyslow5.pyx":191
 *             lev = logging.WARNING
 * 
 *         logging.basicConfig(format='%(asctime)s - [%(levelname)s]: %(message)s',             # <<<<<<<<<<<<<<
 *                             datefmt='%d-%b-%y %H:%M:%S', level=lev)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_basicConfig); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_format, __pyx_kp_u_asctime_s_levelname_s_message_s) < 0) __PYX_ERR(0, 191, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_datefmt, __pyx_kp_u_d_b_y_H_M_S) < 0) __PYX_ERR(0, 191, __pyx_L1_error)

  /* "pyslow5.pyx":192
 * 
 *         logging.basicConfig(format='%(asctime)s - [%(levelname)s]: %(message)s',
 *                             datefmt='%d-%b-%y %H:%M:%S', level=lev)             # <<<<<<<<<<<<<<
 * 
 *         # This should match slow5_defs.h error codes
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_level, __pyx_v_lev) < 0) __PYX_ERR(0, 191, __pyx_L1_error)

  /* "pyslow5.pyx":191
 *             lev = logging.WARNING
 * 
 *         logging.basicConfig(format='%(asctime)s - [%(levelname)s]: %(message)s',             # <<<<<<<<<<<<<<
 *                             datefmt='%d-%b-%y %H:%M:%S', level=lev)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":195
 * 
 *         # This should match slow5_defs.h error codes
 *         self.error_codes = {-1: ["SLOW5_ERR_EOF", "End Of File reached"],             # <<<<<<<<<<<<<<
 *                             -2: ["SLOW5_ERR_ARG", "bad argument (NULL)"],
 *                             -3: ["SLOW5_ERR_TRUNC", "file truncated"],
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(17); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_EOF);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_EOF);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_EOF);
  __Pyx_INCREF(__pyx_kp_u_End_Of_File_reached);
  __Pyx_GIVEREF(__pyx_kp_u_End_Of_File_reached);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_End_Of_File_reached);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_1, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":196
 *         # This should match slow5_defs.h error codes
 *         self.error_codes = {-1: ["SLOW5_ERR_EOF", "End Of File reached"],
 *                             -2: ["SLOW5_ERR_ARG", "bad argument (NULL)"],             # <<<<<<<<<<<<<<
 *                             -3: ["SLOW5_ERR_TRUNC", "file truncated"],
 *                             -4: ["SLOW5_ERR_RECPARSE", "record parsing error"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_ARG);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_ARG);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_ARG);
  __Pyx_INCREF(__pyx_kp_u_bad_argument_NULL);
  __Pyx_GIVEREF(__pyx_kp_u_bad_argument_NULL);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_bad_argument_NULL);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_2, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":197
 *         self.error_codes = {-1: ["SLOW5_ERR_EOF", "End Of File reached"],
 *                             -2: ["SLOW5_ERR_ARG", "bad argument (NULL)"],
 *                             -3: ["SLOW5_ERR_TRUNC", "file truncated"],             # <<<<<<<<<<<<<<
 *                             -4: ["SLOW5_ERR_RECPARSE", "record parsing error"],
 *                             -5: ["SLOW5_ERR_IO", "other file I/O error"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_TRUNC);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_TRUNC);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_TRUNC);
  __Pyx_INCREF(__pyx_kp_u_file_truncated);
  __Pyx_GIVEREF(__pyx_kp_u_file_truncated);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_file_truncated);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_3, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":198
 *                             -2: ["SLOW5_ERR_ARG", "bad argument (NULL)"],
 *                             -3: ["SLOW5_ERR_TRUNC", "file truncated"],
 *                             -4: ["SLOW5_ERR_RECPARSE", "record parsing error"],             # <<<<<<<<<<<<<<
 *                             -5: ["SLOW5_ERR_IO", "other file I/O error"],
 *                             -6: ["SLOW5_ERR_NOIDX", "index not loaded"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_RECPARSE);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_RECPARSE);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_RECPARSE);
  __Pyx_INCREF(__pyx_kp_u_record_parsing_error);
  __Pyx_GIVEREF(__pyx_kp_u_record_parsing_error);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_record_parsing_error);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_4, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":199
 *                             -3: ["SLOW5_ERR_TRUNC", "file truncated"],
 *                             -4: ["SLOW5_ERR_RECPARSE", "record parsing error"],
 *                             -5: ["SLOW5_ERR_IO", "other file I/O error"],             # <<<<<<<<<<<<<<
 *                             -6: ["SLOW5_ERR_NOIDX", "index not loaded"],
 *                             -7: ["SLOW5_ERR_NOTFOUND", "read id not found"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_IO);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_IO);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_IO);
  __Pyx_INCREF(__pyx_kp_u_other_file_I_O_error);
  __Pyx_GIVEREF(__pyx_kp_u_other_file_I_O_error);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_other_file_I_O_error);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_5, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":200
 *                             -4: ["SLOW5_ERR_RECPARSE", "record parsing error"],
 *                             -5: ["SLOW5_ERR_IO", "other file I/O error"],
 *                             -6: ["SLOW5_ERR_NOIDX", "index not loaded"],             # <<<<<<<<<<<<<<
 *                             -7: ["SLOW5_ERR_NOTFOUND", "read id not found"],
 *                             -8: ["SLOW5_ERR_OTH", "other error (big endian, internal error, etc.)"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_NOIDX);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_NOIDX);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_NOIDX);
  __Pyx_INCREF(__pyx_kp_u_index_not_loaded);
  __Pyx_GIVEREF(__pyx_kp_u_index_not_loaded);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_index_not_loaded);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_6, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":201
 *                             -5: ["SLOW5_ERR_IO", "other file I/O error"],
 *                             -6: ["SLOW5_ERR_NOIDX", "index not loaded"],
 *                             -7: ["SLOW5_ERR_NOTFOUND", "read id not found"],             # <<<<<<<<<<<<<<
 *                             -8: ["SLOW5_ERR_OTH", "other error (big endian, internal error, etc.)"],
 *                             -9: ["SLOW5_ERR_UNK", "file format unknown"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_NOTFOUND);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_NOTFOUND);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_NOTFOUND);
  __Pyx_INCREF(__pyx_kp_u_read_id_not_found);
  __Pyx_GIVEREF(__pyx_kp_u_read_id_not_found);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_read_id_not_found);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_7, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":202
 *                             -6: ["SLOW5_ERR_NOIDX", "index not loaded"],
 *                             -7: ["SLOW5_ERR_NOTFOUND", "read id not found"],
 *                             -8: ["SLOW5_ERR_OTH", "other error (big endian, internal error, etc.)"],             # <<<<<<<<<<<<<<
 *                             -9: ["SLOW5_ERR_UNK", "file format unknown"],
 *                             -10: ["SLOW5_ERR_MEM", "memory (re)allocation error"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_OTH);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_OTH);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_OTH);
  __Pyx_INCREF(__pyx_kp_u_other_error_big_endian_internal);
  __Pyx_GIVEREF(__pyx_kp_u_other_error_big_endian_internal);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_other_error_big_endian_internal);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_8, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":203
 *                             -7: ["SLOW5_ERR_NOTFOUND", "read id not found"],
 *                             -8: ["SLOW5_ERR_OTH", "other error (big endian, internal error, etc.)"],
 *                             -9: ["SLOW5_ERR_UNK", "file format unknown"],             # <<<<<<<<<<<<<<
 *                             -10: ["SLOW5_ERR_MEM", "memory (re)allocation error"],
 *                             -11: ["SLOW5_ERR_NOAUX", "no auxiliary map"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_UNK);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_UNK);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_UNK);
  __Pyx_INCREF(__pyx_kp_u_file_format_unknown);
  __Pyx_GIVEREF(__pyx_kp_u_file_format_unknown);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_file_format_unknown);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_9, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":204
 *                             -8: ["SLOW5_ERR_OTH", "other error (big endian, internal error, etc.)"],
 *                             -9: ["SLOW5_ERR_UNK", "file format unknown"],
 *                             -10: ["SLOW5_ERR_MEM", "memory (re)allocation error"],             # <<<<<<<<<<<<<<
 *                             -11: ["SLOW5_ERR_NOAUX", "no auxiliary map"],
 *                             -12: ["SLOW5_ERR_NOFLD", "field not found"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_MEM);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_MEM);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_MEM);
  __Pyx_INCREF(__pyx_kp_u_memory_re_allocation_error);
  __Pyx_GIVEREF(__pyx_kp_u_memory_re_allocation_error);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_memory_re_allocation_error);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_10, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":205
 *                             -9: ["SLOW5_ERR_UNK", "file format unknown"],
 *                             -10: ["SLOW5_ERR_MEM", "memory (re)allocation error"],
 *                             -11: ["SLOW5_ERR_NOAUX", "no auxiliary map"],             # <<<<<<<<<<<<<<
 *                             -12: ["SLOW5_ERR_NOFLD", "field not found"],
 *                             -13: ["SLOW5_ERR_PRESS", "(de)compression failure"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_NOAUX);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_NOAUX);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_NOAUX);
  __Pyx_INCREF(__pyx_kp_u_no_auxiliary_map);
  __Pyx_GIVEREF(__pyx_kp_u_no_auxiliary_map);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_no_auxiliary_map);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_11, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":206
 *                             -10: ["SLOW5_ERR_MEM", "memory (re)allocation error"],
 *                             -11: ["SLOW5_ERR_NOAUX", "no auxiliary map"],
 *                             -12: ["SLOW5_ERR_NOFLD", "field not found"],             # <<<<<<<<<<<<<<
 *                             -13: ["SLOW5_ERR_PRESS", "(de)compression failure"],
 *                             -14: ["SLOW5_ERR_MAGIC", "magic number invalid"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_NOFLD);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_NOFLD);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_NOFLD);
  __Pyx_INCREF(__pyx_kp_u_field_not_found);
  __Pyx_GIVEREF(__pyx_kp_u_field_not_found);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_field_not_found);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_12, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":207
 *                             -11: ["SLOW5_ERR_NOAUX", "no auxiliary map"],
 *                             -12: ["SLOW5_ERR_NOFLD", "field not found"],
 *                             -13: ["SLOW5_ERR_PRESS", "(de)compression failure"],             # <<<<<<<<<<<<<<
 *                             -14: ["SLOW5_ERR_MAGIC", "magic number invalid"],
 *                             -15: ["SLOW5_ERR_VERSION", "version incompatible"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_PRESS);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_PRESS);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_PRESS);
  __Pyx_INCREF(__pyx_kp_u_de_compression_failure);
  __Pyx_GIVEREF(__pyx_kp_u_de_compression_failure);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_de_compression_failure);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_13, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":208
 *                             -12: ["SLOW5_ERR_NOFLD", "field not found"],
 *                             -13: ["SLOW5_ERR_PRESS", "(de)compression failure"],
 *                             -14: ["SLOW5_ERR_MAGIC", "magic number invalid"],             # <<<<<<<<<<<<<<
 *                             -15: ["SLOW5_ERR_VERSION", "version incompatible"],
 *                             -16: ["SLOW5_ERR_HDRPARSE", "header parsing error"],
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_MAGIC);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_MAGIC);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_MAGIC);
  __Pyx_INCREF(__pyx_kp_u_magic_number_invalid);
  __Pyx_GIVEREF(__pyx_kp_u_magic_number_invalid);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_magic_number_invalid);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_14, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":209
 *                             -13: ["SLOW5_ERR_PRESS", "(de)compression failure"],
 *                             -14: ["SLOW5_ERR_MAGIC", "magic number invalid"],
 *                             -15: ["SLOW5_ERR_VERSION", "version incompatible"],             # <<<<<<<<<<<<<<
 *                             -16: ["SLOW5_ERR_HDRPARSE", "header parsing error"],
 *                             -17: ["SLOW5_ERR_TYPE", "error relating to slow5 type"]}
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_VERSION);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_VERSION);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_VERSION);
  __Pyx_INCREF(__pyx_kp_u_version_incompatible);
  __Pyx_GIVEREF(__pyx_kp_u_version_incompatible);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_version_incompatible);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_15, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":210
 *                             -14: ["SLOW5_ERR_MAGIC", "magic number invalid"],
 *                             -15: ["SLOW5_ERR_VERSION", "version incompatible"],
 *                             -16: ["SLOW5_ERR_HDRPARSE", "header parsing error"],             # <<<<<<<<<<<<<<
 *                             -17: ["SLOW5_ERR_TYPE", "error relating to slow5 type"]}
 *         # slow5_press_method from slow5_press.h
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_HDRPARSE);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_HDRPARSE);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_HDRPARSE);
  __Pyx_INCREF(__pyx_kp_u_header_parsing_error);
  __Pyx_GIVEREF(__pyx_kp_u_header_parsing_error);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_header_parsing_error);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_16, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":211
 *                             -15: ["SLOW5_ERR_VERSION", "version incompatible"],
 *                             -16: ["SLOW5_ERR_HDRPARSE", "header parsing error"],
 *                             -17: ["SLOW5_ERR_TYPE", "error relating to slow5 type"]}             # <<<<<<<<<<<<<<
 *         # slow5_press_method from slow5_press.h
 *         # enum slow5_press_method {
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SLOW5_ERR_TYPE);
  __Pyx_GIVEREF(__pyx_n_u_SLOW5_ERR_TYPE);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SLOW5_ERR_TYPE);
  __Pyx_INCREF(__pyx_kp_u_error_relating_to_slow5_type);
  __Pyx_GIVEREF(__pyx_kp_u_error_relating_to_slow5_type);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_error_relating_to_slow5_type);
  if (PyDict_SetItem(__pyx_t_3, __pyx_int_neg_17, __pyx_t_2) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":195
 * 
 *         # This should match slow5_defs.h error codes
 *         self.error_codes = {-1: ["SLOW5_ERR_EOF", "End Of File reached"],             # <<<<<<<<<<<<<<
 *                             -2: ["SLOW5_ERR_ARG", "bad argument (NULL)"],
 *                             -3: ["SLOW5_ERR_TRUNC", "file truncated"],
 */
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->error_codes);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->error_codes);
  __pyx_cur_scope->__pyx_v_self->error_codes = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":219
 *         #     SLOW5_COMPRESS_ZSTD,
 *         # };
 *         self.slow5_press_method = {"none": 0,             # <<<<<<<<<<<<<<
 *                                    "zlib": 1,
 *                                    "svb_zd": 2,
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_u_none, __pyx_int_0) < 0) __PYX_ERR(0, 219, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_u_zlib, __pyx_int_1) < 0) __PYX_ERR(0, 219, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_u_svb_zd, __pyx_int_2) < 0) __PYX_ERR(0, 219, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_u_zstd, __pyx_int_3) < 0) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->slow5_press_method);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->slow5_press_method);
  __pyx_cur_scope->__pyx_v_self->slow5_press_method = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":224
 *                                    "zstd": 3}
 * 
 *         p = str.encode(pathname)             # <<<<<<<<<<<<<<
 *         self.path = pathname
 *         self.p = strdup(p)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&PyUnicode_Type)), __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v_pathname) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_pathname);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_p = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":225
 * 
 *         p = str.encode(pathname)
 *         self.path = pathname             # <<<<<<<<<<<<<<
 *         self.p = strdup(p)
 *         m = str.encode(mode)
 */
  if (!(likely(PyUnicode_CheckExact(__pyx_v_pathname))||((__pyx_v_pathname) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_pathname)->tp_name), 0))) __PYX_ERR(0, 225, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_pathname;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->path);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->path);
  __pyx_cur_scope->__pyx_v_self->path = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":226
 *         p = str.encode(pathname)
 *         self.path = pathname
 *         self.p = strdup(p)             # <<<<<<<<<<<<<<
 *         m = str.encode(mode)
 *         self.mode = mode
 */
  __pyx_t_7 = __Pyx_PyObject_AsString(__pyx_v_p); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 226, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_self->p = strdup(__pyx_t_7);

  /* "pyslow5.pyx":227
 *         self.path = pathname
 *         self.p = strdup(p)
 *         m = str.encode(mode)             # <<<<<<<<<<<<<<
 *         self.mode = mode
 *         self.m = strdup(m)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&PyUnicode_Type)), __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v_mode) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_mode);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_m = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":228
 *         self.p = strdup(p)
 *         m = str.encode(mode)
 *         self.mode = mode             # <<<<<<<<<<<<<<
 *         self.m = strdup(m)
 *         self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 */
  if (!(likely(PyUnicode_CheckExact(__pyx_v_mode))||((__pyx_v_mode) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_mode)->tp_name), 0))) __PYX_ERR(0, 228, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_mode;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->mode);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->mode);
  __pyx_cur_scope->__pyx_v_self->mode = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":229
 *         m = str.encode(mode)
 *         self.mode = mode
 *         self.m = strdup(m)             # <<<<<<<<<<<<<<
 *         self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *         self.logger.debug("FILE: {}, mode: {}".format(self.p, self.m))
 */
  __pyx_t_7 = __Pyx_PyObject_AsString(__pyx_v_m); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 229, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_self->m = strdup(__pyx_t_7);

  /* "pyslow5.pyx":230
 *         self.mode = mode
 *         self.m = strdup(m)
 *         self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))             # <<<<<<<<<<<<<<
 *         self.logger.debug("FILE: {}, mode: {}".format(self.p, self.m))
 *         # Set state based on mode for file opening
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FILE_mode, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_8 = NULL;
  __pyx_t_1 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_1 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_self->path, __pyx_cur_scope->__pyx_v_self->mode};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_1, 2+__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_self->path, __pyx_cur_scope->__pyx_v_self->mode};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_1, 2+__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(2+__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_8) {
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    }
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->path);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->path);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_1, __pyx_cur_scope->__pyx_v_self->path);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->mode);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->mode);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_1, __pyx_cur_scope->__pyx_v_self->mode);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":231
 *         self.m = strdup(m)
 *         self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *         self.logger.debug("FILE: {}, mode: {}".format(self.p, self.m))             # <<<<<<<<<<<<<<
 *         # Set state based on mode for file opening
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FILE_mode, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->p); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->m); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_10 = NULL;
  __pyx_t_1 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_1 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_t_9, __pyx_t_8};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_1, 2+__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_t_9, __pyx_t_8};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_1, 2+__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  } else
  #endif
  {
    __pyx_t_11 = PyTuple_New(2+__pyx_t_1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_1, __pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_1, __pyx_t_8);
    __pyx_t_9 = 0;
    __pyx_t_8 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":234
 *         # Set state based on mode for file opening
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 *         if mode == "r":             # <<<<<<<<<<<<<<
 *             self.state = 0
 *         elif mode == "w":
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_n_u_r, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 234, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyslow5.pyx":235
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 *         if mode == "r":
 *             self.state = 0             # <<<<<<<<<<<<<<
 *         elif mode == "w":
 *             self.state = 1
 */
    __pyx_cur_scope->__pyx_v_self->state = 0;

    /* "pyslow5.pyx":234
 *         # Set state based on mode for file opening
 *         # state for read/write. -1=null, 0=read, 1=write, 2=append
 *         if mode == "r":             # <<<<<<<<<<<<<<
 *             self.state = 0
 *         elif mode == "w":
 */
    goto __pyx_L4;
  }

  /* "pyslow5.pyx":236
 *         if mode == "r":
 *             self.state = 0
 *         elif mode == "w":             # <<<<<<<<<<<<<<
 *             self.state = 1
 *             self.rec_press = rec_press
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_n_u_w, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 236, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyslow5.pyx":237
 *             self.state = 0
 *         elif mode == "w":
 *             self.state = 1             # <<<<<<<<<<<<<<
 *             self.rec_press = rec_press
 *             self.sig_press = sig_press
 */
    __pyx_cur_scope->__pyx_v_self->state = 1;

    /* "pyslow5.pyx":238
 *         elif mode == "w":
 *             self.state = 1
 *             self.rec_press = rec_press             # <<<<<<<<<<<<<<
 *             self.sig_press = sig_press
 *         elif mode == "a":
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_rec_press))||((__pyx_v_rec_press) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_rec_press)->tp_name), 0))) __PYX_ERR(0, 238, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_rec_press;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->rec_press);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->rec_press);
    __pyx_cur_scope->__pyx_v_self->rec_press = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyslow5.pyx":239
 *             self.state = 1
 *             self.rec_press = rec_press
 *             self.sig_press = sig_press             # <<<<<<<<<<<<<<
 *         elif mode == "a":
 *             self.state = 2
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_sig_press))||((__pyx_v_sig_press) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_sig_press)->tp_name), 0))) __PYX_ERR(0, 239, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_sig_press;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->sig_press);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->sig_press);
    __pyx_cur_scope->__pyx_v_self->sig_press = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyslow5.pyx":236
 *         if mode == "r":
 *             self.state = 0
 *         elif mode == "w":             # <<<<<<<<<<<<<<
 *             self.state = 1
 *             self.rec_press = rec_press
 */
    goto __pyx_L4;
  }

  /* "pyslow5.pyx":240
 *             self.rec_press = rec_press
 *             self.sig_press = sig_press
 *         elif mode == "a":             # <<<<<<<<<<<<<<
 *             self.state = 2
 *         else:
 */
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_n_u_a, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 240, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyslow5.pyx":241
 *             self.sig_press = sig_press
 *         elif mode == "a":
 *             self.state = 2             # <<<<<<<<<<<<<<
 *         else:
 *             self.state = -1
 */
    __pyx_cur_scope->__pyx_v_self->state = 2;

    /* "pyslow5.pyx":240
 *             self.rec_press = rec_press
 *             self.sig_press = sig_press
 *         elif mode == "a":             # <<<<<<<<<<<<<<
 *             self.state = 2
 *         else:
 */
    goto __pyx_L4;
  }

  /* "pyslow5.pyx":243
 *             self.state = 2
 *         else:
 *             self.state = -1             # <<<<<<<<<<<<<<
 *         # opens file and creates slow5 object for reading
 *         if self.state == 0:
 */
  /*else*/ {
    __pyx_cur_scope->__pyx_v_self->state = -1;
  }
  __pyx_L4:;

  /* "pyslow5.pyx":245
 *             self.state = -1
 *         # opens file and creates slow5 object for reading
 *         if self.state == 0:             # <<<<<<<<<<<<<<
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             self.logger.debug("Number of read_groups: {}".format(self.s5.header.num_read_groups))
 */
  switch (__pyx_cur_scope->__pyx_v_self->state) {
    case 0:

    /* "pyslow5.pyx":246
 *         # opens file and creates slow5 object for reading
 *         if self.state == 0:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)             # <<<<<<<<<<<<<<
 *             self.logger.debug("Number of read_groups: {}".format(self.s5.header.num_read_groups))
 *         elif self.state == 1:
 */
    __pyx_cur_scope->__pyx_v_self->s5 = slow5_open(__pyx_cur_scope->__pyx_v_self->p, __pyx_cur_scope->__pyx_v_self->m);

    /* "pyslow5.pyx":247
 *         if self.state == 0:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             self.logger.debug("Number of read_groups: {}".format(self.s5.header.num_read_groups))             # <<<<<<<<<<<<<<
 *         elif self.state == 1:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Number_of_read_groups, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_11 = __Pyx_PyInt_From_uint32_t(__pyx_cur_scope->__pyx_v_self->s5->header->num_read_groups); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":245
 *             self.state = -1
 *         # opens file and creates slow5 object for reading
 *         if self.state == 0:             # <<<<<<<<<<<<<<
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             self.logger.debug("Number of read_groups: {}".format(self.s5.header.num_read_groups))
 */
    break;
    case 1:

    /* "pyslow5.pyx":249
 *             self.logger.debug("Number of read_groups: {}".format(self.s5.header.num_read_groups))
 *         elif self.state == 1:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)             # <<<<<<<<<<<<<<
 *             if self.s5 is NULL:
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 */
    __pyx_cur_scope->__pyx_v_self->s5 = slow5_open(__pyx_cur_scope->__pyx_v_self->p, __pyx_cur_scope->__pyx_v_self->m);

    /* "pyslow5.pyx":250
 *         elif self.state == 1:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:             # <<<<<<<<<<<<<<
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 *             if "blow5" in self.path.split(".")[-1]:
 */
    __pyx_t_6 = ((__pyx_cur_scope->__pyx_v_self->s5 == NULL) != 0);
    if (__pyx_t_6) {

      /* "pyslow5.pyx":251
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))             # <<<<<<<<<<<<<<
 *             if "blow5" in self.path.split(".")[-1]:
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_File_could_not_be_opened_for_wri, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_11 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_11)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_11, __pyx_cur_scope->__pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_cur_scope->__pyx_v_self->path);
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":250
 *         elif self.state == 1:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:             # <<<<<<<<<<<<<<
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 *             if "blow5" in self.path.split(".")[-1]:
 */
    }

    /* "pyslow5.pyx":252
 *             if self.s5 is NULL:
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 *             if "blow5" in self.path.split(".")[-1]:             # <<<<<<<<<<<<<<
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 */
    if (unlikely(__pyx_cur_scope->__pyx_v_self->path == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "split");
      __PYX_ERR(0, 252, __pyx_L1_error)
    }
    __pyx_t_3 = PyUnicode_Split(__pyx_cur_scope->__pyx_v_self->path, __pyx_kp_u__2, -1L); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_t_3, -1L, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_blow5, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_12 = (__pyx_t_6 != 0);
    if (__pyx_t_12) {

      /* "pyslow5.pyx":253
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 *             if "blow5" in self.path.split(".")[-1]:
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():             # <<<<<<<<<<<<<<
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 *                     if ret != 0:
 */
      if (unlikely(__pyx_cur_scope->__pyx_v_self->slow5_press_method == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "keys");
        __PYX_ERR(0, 253, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyDict_Keys(__pyx_cur_scope->__pyx_v_self->slow5_press_method); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 253, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = (__Pyx_PySequence_ContainsTF(__pyx_cur_scope->__pyx_v_self->rec_press, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 253, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_13 = (__pyx_t_6 != 0);
      if (__pyx_t_13) {
      } else {
        __pyx_t_12 = __pyx_t_13;
        goto __pyx_L8_bool_binop_done;
      }
      if (unlikely(__pyx_cur_scope->__pyx_v_self->slow5_press_method == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "keys");
        __PYX_ERR(0, 253, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyDict_Keys(__pyx_cur_scope->__pyx_v_self->slow5_press_method); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 253, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_13 = (__Pyx_PySequence_ContainsTF(__pyx_cur_scope->__pyx_v_self->sig_press, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 253, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_6 = (__pyx_t_13 != 0);
      __pyx_t_12 = __pyx_t_6;
      __pyx_L8_bool_binop_done:;
      if (likely(__pyx_t_12)) {

        /* "pyslow5.pyx":254
 *             if "blow5" in self.path.split(".")[-1]:
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])             # <<<<<<<<<<<<<<
 *                     if ret != 0:
 *                         self.logger.error("slow5_set_press return not 0: {}".format(ret))
 */
        if (unlikely(__pyx_cur_scope->__pyx_v_self->slow5_press_method == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 254, __pyx_L1_error)
        }
        __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->slow5_press_method, __pyx_cur_scope->__pyx_v_self->rec_press); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 254, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 254, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(__pyx_cur_scope->__pyx_v_self->slow5_press_method == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 254, __pyx_L1_error)
        }
        __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->slow5_press_method, __pyx_cur_scope->__pyx_v_self->sig_press); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 254, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 254, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_v_ret = slow5_set_press(__pyx_cur_scope->__pyx_v_self->s5, __pyx_t_1, __pyx_t_14);

        /* "pyslow5.pyx":255
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 *                     if ret != 0:             # <<<<<<<<<<<<<<
 *                         self.logger.error("slow5_set_press return not 0: {}".format(ret))
 *                         raise RuntimeError("Unable to set compression")
 */
        __pyx_t_12 = ((__pyx_v_ret != 0) != 0);
        if (unlikely(__pyx_t_12)) {

          /* "pyslow5.pyx":256
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 *                     if ret != 0:
 *                         self.logger.error("slow5_set_press return not 0: {}".format(ret))             # <<<<<<<<<<<<<<
 *                         raise RuntimeError("Unable to set compression")
 *                 else:
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_set_press_return_not_0, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          __pyx_t_4 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_11);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_5 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_5)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":257
 *                     if ret != 0:
 *                         self.logger.error("slow5_set_press return not 0: {}".format(ret))
 *                         raise RuntimeError("Unable to set compression")             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))
 */
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 257, __pyx_L1_error)

          /* "pyslow5.pyx":255
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 *                     if ret != 0:             # <<<<<<<<<<<<<<
 *                         self.logger.error("slow5_set_press return not 0: {}".format(ret))
 *                         raise RuntimeError("Unable to set compression")
 */
        }

        /* "pyslow5.pyx":253
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 *             if "blow5" in self.path.split(".")[-1]:
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():             # <<<<<<<<<<<<<<
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 *                     if ret != 0:
 */
        goto __pyx_L7;
      }

      /* "pyslow5.pyx":259
 *                         raise RuntimeError("Unable to set compression")
 *                 else:
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))             # <<<<<<<<<<<<<<
 *                     self.logger.error("Please use only the following: {}".format(",".join(press for press in self.slow5_press_method.keys())))
 *                     raise KeyError
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 259, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Compression_type_rec_press_sig_p, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 259, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_cur_scope->__pyx_v_self->rec_press, __pyx_cur_scope->__pyx_v_self->sig_press};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_14, 2+__pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 259, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_cur_scope->__pyx_v_self->rec_press, __pyx_cur_scope->__pyx_v_self->sig_press};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_14, 2+__pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 259, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(2+__pyx_t_14); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 259, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->rec_press);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->rec_press);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_14, __pyx_cur_scope->__pyx_v_self->rec_press);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->sig_press);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->sig_press);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_14, __pyx_cur_scope->__pyx_v_self->sig_press);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 259, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":260
 *                 else:
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))
 *                     self.logger.error("Please use only the following: {}".format(",".join(press for press in self.slow5_press_method.keys())))             # <<<<<<<<<<<<<<
 *                     raise KeyError
 *             else:
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Please_use_only_the_following, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_8 = __pyx_pf_7pyslow5_4Open_9__cinit___genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_Generator_Next(__pyx_t_8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = PyUnicode_Join(__pyx_kp_u__4, __pyx_t_11); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_11, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8);
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 260, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":261
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))
 *                     self.logger.error("Please use only the following: {}".format(",".join(press for press in self.slow5_press_method.keys())))
 *                     raise KeyError             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.debug("Not writing blow5, skipping compression steps")
 */
        __Pyx_Raise(__pyx_builtin_KeyError, 0, 0, 0);
        __PYX_ERR(0, 261, __pyx_L1_error)
      }
      __pyx_L7:;

      /* "pyslow5.pyx":252
 *             if self.s5 is NULL:
 *                 self.logger.error("File '{}' could not be opened for writing.".format(self.path))
 *             if "blow5" in self.path.split(".")[-1]:             # <<<<<<<<<<<<<<
 *                 if self.rec_press in self.slow5_press_method.keys() and self.sig_press in self.slow5_press_method.keys():
 *                     ret = pyslow5.slow5_set_press(self.s5, self.slow5_press_method[self.rec_press], self.slow5_press_method[self.sig_press])
 */
      goto __pyx_L6;
    }

    /* "pyslow5.pyx":263
 *                     raise KeyError
 *             else:
 *                 self.logger.debug("Not writing blow5, skipping compression steps")             # <<<<<<<<<<<<<<
 *         elif self.state == 2:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_Not_writing_blow5_skipping_compr) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Not_writing_blow5_skipping_compr);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L6:;

    /* "pyslow5.pyx":248
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             self.logger.debug("Number of read_groups: {}".format(self.s5.header.num_read_groups))
 *         elif self.state == 1:             # <<<<<<<<<<<<<<
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:
 */
    break;
    case 2:

    /* "pyslow5.pyx":265
 *                 self.logger.debug("Not writing blow5, skipping compression steps")
 *         elif self.state == 2:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)             # <<<<<<<<<<<<<<
 *             if self.s5 is NULL:
 *                 self.logger.error("File '{}' could not be opened for writing - appending.".format(self.path))
 */
    __pyx_cur_scope->__pyx_v_self->s5 = slow5_open(__pyx_cur_scope->__pyx_v_self->p, __pyx_cur_scope->__pyx_v_self->m);

    /* "pyslow5.pyx":266
 *         elif self.state == 2:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:             # <<<<<<<<<<<<<<
 *                 self.logger.error("File '{}' could not be opened for writing - appending.".format(self.path))
 *         else:
 */
    __pyx_t_12 = ((__pyx_cur_scope->__pyx_v_self->s5 == NULL) != 0);
    if (__pyx_t_12) {

      /* "pyslow5.pyx":267
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:
 *                 self.logger.error("File '{}' could not be opened for writing - appending.".format(self.path))             # <<<<<<<<<<<<<<
 *         else:
 *             self.logger.error("File '{}' unknown open method: {}".format(self.path, self.mode))
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_File_could_not_be_opened_for_wri_2, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_cur_scope->__pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_cur_scope->__pyx_v_self->path);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":266
 *         elif self.state == 2:
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:             # <<<<<<<<<<<<<<
 *                 self.logger.error("File '{}' could not be opened for writing - appending.".format(self.path))
 *         else:
 */
    }

    /* "pyslow5.pyx":264
 *             else:
 *                 self.logger.debug("Not writing blow5, skipping compression steps")
 *         elif self.state == 2:             # <<<<<<<<<<<<<<
 *             self.s5 = pyslow5.slow5_open(self.p, self.m)
 *             if self.s5 is NULL:
 */
    break;
    default:

    /* "pyslow5.pyx":269
 *                 self.logger.error("File '{}' could not be opened for writing - appending.".format(self.path))
 *         else:
 *             self.logger.error("File '{}' unknown open method: {}".format(self.path, self.mode))             # <<<<<<<<<<<<<<
 *         # check object was actually created.
 *         if self.s5 is NULL:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_File_unknown_open_method, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = NULL;
    __pyx_t_14 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_14 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_self->path, __pyx_cur_scope->__pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_14, 2+__pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_self->path, __pyx_cur_scope->__pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_14, 2+__pyx_t_14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_11 = PyTuple_New(2+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->path);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->path);
      PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_cur_scope->__pyx_v_self->path);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->mode);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->mode);
      PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_cur_scope->__pyx_v_self->mode);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    break;
  }

  /* "pyslow5.pyx":271
 *             self.logger.error("File '{}' unknown open method: {}".format(self.path, self.mode))
 *         # check object was actually created.
 *         if self.s5 is NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 * 
 */
  __pyx_t_12 = ((__pyx_cur_scope->__pyx_v_self->s5 == NULL) != 0);
  if (unlikely(__pyx_t_12)) {

    /* "pyslow5.pyx":272
 *         # check object was actually created.
 *         if self.s5 is NULL:
 *             raise MemoryError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
    PyErr_NoMemory(); __PYX_ERR(0, 272, __pyx_L1_error)

    /* "pyslow5.pyx":271
 *             self.logger.error("File '{}' unknown open method: {}".format(self.path, self.mode))
 *         # check object was actually created.
 *         if self.s5 is NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 * 
 */
  }

  /* "pyslow5.pyx":105
 *     cdef pyslow5.float total_multi_write_time
 * 
 *     def __cinit__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):             # <<<<<<<<<<<<<<
 *         # Set to default NULL type
 *         self.s5 = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyslow5.Open.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_lev);
  __Pyx_XDECREF(__pyx_v_p);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_gb_7pyslow5_4Open_9__cinit___2generator6);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":275
 * 
 * 
 *     def __init__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):             # <<<<<<<<<<<<<<
 *         self.aux_names = []
 *         self.aux_types = []
 */

/* Python wrapper */
static int __pyx_pw_7pyslow5_4Open_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7pyslow5_4Open_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_pathname = 0;
  CYTHON_UNUSED PyObject *__pyx_v_mode = 0;
  CYTHON_UNUSED PyObject *__pyx_v_rec_press = 0;
  CYTHON_UNUSED PyObject *__pyx_v_sig_press = 0;
  CYTHON_UNUSED PyObject *__pyx_v_DEBUG = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pathname,&__pyx_n_s_mode,&__pyx_n_s_rec_press,&__pyx_n_s_sig_press,&__pyx_n_s_DEBUG,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[2] = ((PyObject *)__pyx_n_u_zlib);
    values[3] = ((PyObject *)__pyx_n_u_svb_zd);
    values[4] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pathname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 5, 1); __PYX_ERR(0, 275, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rec_press);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sig_press);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_DEBUG);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 275, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_pathname = values[0];
    __pyx_v_mode = values[1];
    __pyx_v_rec_press = values[2];
    __pyx_v_sig_press = values[3];
    __pyx_v_DEBUG = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 275, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_2__init__(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_pathname, __pyx_v_mode, __pyx_v_rec_press, __pyx_v_sig_press, __pyx_v_DEBUG);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7pyslow5_4Open_2__init__(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_pathname, CYTHON_UNUSED PyObject *__pyx_v_mode, CYTHON_UNUSED PyObject *__pyx_v_rec_press, CYTHON_UNUSED PyObject *__pyx_v_sig_press, CYTHON_UNUSED PyObject *__pyx_v_DEBUG) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_TraceCall("__init__", __pyx_f[0], 275, 0, __PYX_ERR(0, 275, __pyx_L1_error));

  /* "pyslow5.pyx":276
 * 
 *     def __init__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):
 *         self.aux_names = []             # <<<<<<<<<<<<<<
 *         self.aux_types = []
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->aux_names);
  __Pyx_DECREF(__pyx_v_self->aux_names);
  __pyx_v_self->aux_names = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":277
 *     def __init__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):
 *         self.aux_names = []
 *         self.aux_types = []             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->aux_types);
  __Pyx_DECREF(__pyx_v_self->aux_types);
  __pyx_v_self->aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":275
 * 
 * 
 *     def __init__(self, pathname, mode, rec_press="zlib", sig_press="svb_zd", DEBUG=0):             # <<<<<<<<<<<<<<
 *         self.aux_names = []
 *         self.aux_types = []
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyslow5.Open.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":280
 * 
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.p is not NULL:
 *             free(self.p)
 */

/* Python wrapper */
static void __pyx_pw_7pyslow5_4Open_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7pyslow5_4Open_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7pyslow5_4Open_4__dealloc__(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7pyslow5_4Open_4__dealloc__(struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);
  __Pyx_TraceCall("__dealloc__", __pyx_f[0], 280, 0, __PYX_ERR(0, 280, __pyx_L1_error));

  /* "pyslow5.pyx":281
 * 
 *     def __dealloc__(self):
 *         if self.p is not NULL:             # <<<<<<<<<<<<<<
 *             free(self.p)
 *         if self.m is not NULL:
 */
  __pyx_t_1 = ((__pyx_v_self->p != NULL) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":282
 *     def __dealloc__(self):
 *         if self.p is not NULL:
 *             free(self.p)             # <<<<<<<<<<<<<<
 *         if self.m is not NULL:
 *             free(self.m)
 */
    free(__pyx_v_self->p);

    /* "pyslow5.pyx":281
 * 
 *     def __dealloc__(self):
 *         if self.p is not NULL:             # <<<<<<<<<<<<<<
 *             free(self.p)
 *         if self.m is not NULL:
 */
  }

  /* "pyslow5.pyx":283
 *         if self.p is not NULL:
 *             free(self.p)
 *         if self.m is not NULL:             # <<<<<<<<<<<<<<
 *             free(self.m)
 *         if self.rec is not NULL:
 */
  __pyx_t_1 = ((__pyx_v_self->m != NULL) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":284
 *             free(self.p)
 *         if self.m is not NULL:
 *             free(self.m)             # <<<<<<<<<<<<<<
 *         if self.rec is not NULL:
 *             slow5_rec_free(self.rec)
 */
    free(__pyx_v_self->m);

    /* "pyslow5.pyx":283
 *         if self.p is not NULL:
 *             free(self.p)
 *         if self.m is not NULL:             # <<<<<<<<<<<<<<
 *             free(self.m)
 *         if self.rec is not NULL:
 */
  }

  /* "pyslow5.pyx":285
 *         if self.m is not NULL:
 *             free(self.m)
 *         if self.rec is not NULL:             # <<<<<<<<<<<<<<
 *             slow5_rec_free(self.rec)
 *         if self.read is not NULL:
 */
  __pyx_t_1 = ((__pyx_v_self->rec != NULL) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":286
 *             free(self.m)
 *         if self.rec is not NULL:
 *             slow5_rec_free(self.rec)             # <<<<<<<<<<<<<<
 *         if self.read is not NULL:
 *             slow5_rec_free(self.read)
 */
    slow5_rec_free(__pyx_v_self->rec);

    /* "pyslow5.pyx":285
 *         if self.m is not NULL:
 *             free(self.m)
 *         if self.rec is not NULL:             # <<<<<<<<<<<<<<
 *             slow5_rec_free(self.rec)
 *         if self.read is not NULL:
 */
  }

  /* "pyslow5.pyx":287
 *         if self.rec is not NULL:
 *             slow5_rec_free(self.rec)
 *         if self.read is not NULL:             # <<<<<<<<<<<<<<
 *             slow5_rec_free(self.read)
 *         if self.write is not NULL:
 */
  __pyx_t_1 = ((__pyx_v_self->read != NULL) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":288
 *             slow5_rec_free(self.rec)
 *         if self.read is not NULL:
 *             slow5_rec_free(self.read)             # <<<<<<<<<<<<<<
 *         if self.write is not NULL:
 *             slow5_rec_free(self.write)
 */
    slow5_rec_free(__pyx_v_self->read);

    /* "pyslow5.pyx":287
 *         if self.rec is not NULL:
 *             slow5_rec_free(self.rec)
 *         if self.read is not NULL:             # <<<<<<<<<<<<<<
 *             slow5_rec_free(self.read)
 *         if self.write is not NULL:
 */
  }

  /* "pyslow5.pyx":289
 *         if self.read is not NULL:
 *             slow5_rec_free(self.read)
 *         if self.write is not NULL:             # <<<<<<<<<<<<<<
 *             slow5_rec_free(self.write)
 *         if self.state in [1, 2]:
 */
  __pyx_t_1 = ((__pyx_v_self->write != NULL) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":290
 *             slow5_rec_free(self.read)
 *         if self.write is not NULL:
 *             slow5_rec_free(self.write)             # <<<<<<<<<<<<<<
 *         if self.state in [1, 2]:
 *             if not self.close_state:
 */
    slow5_rec_free(__pyx_v_self->write);

    /* "pyslow5.pyx":289
 *         if self.read is not NULL:
 *             slow5_rec_free(self.read)
 *         if self.write is not NULL:             # <<<<<<<<<<<<<<
 *             slow5_rec_free(self.write)
 *         if self.state in [1, 2]:
 */
  }

  /* "pyslow5.pyx":291
 *         if self.write is not NULL:
 *             slow5_rec_free(self.write)
 *         if self.state in [1, 2]:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
  switch (__pyx_v_self->state) {
    case 1:
    case 2:

    /* "pyslow5.pyx":292
 *             slow5_rec_free(self.write)
 *         if self.state in [1, 2]:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 */
    __pyx_t_1 = ((!(__pyx_v_self->close_state != 0)) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":293
 *         if self.state in [1, 2]:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
      __pyx_t_1 = ((__pyx_v_self->s5 != NULL) != 0);
      if (__pyx_t_1) {

        /* "pyslow5.pyx":294
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)             # <<<<<<<<<<<<<<
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))
 */
        (void)(slow5_close(__pyx_v_self->s5));

        /* "pyslow5.pyx":295
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 *                     self.close_state = True             # <<<<<<<<<<<<<<
 *                     self.logger.debug("{} closed".format(self.path))
 *             else:
 */
        __pyx_v_self->close_state = 1;

        /* "pyslow5.pyx":296
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.debug("{} already closed".format(self.path))
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":293
 *         if self.state in [1, 2]:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
      }

      /* "pyslow5.pyx":292
 *             slow5_rec_free(self.write)
 *         if self.state in [1, 2]:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 */
      goto __pyx_L8;
    }

    /* "pyslow5.pyx":298
 *                     self.logger.debug("{} closed".format(self.path))
 *             else:
 *                 self.logger.debug("{} already closed".format(self.path))             # <<<<<<<<<<<<<<
 *         if self.state == 0:
 *             if not self.close_state:
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_already_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L8:;

    /* "pyslow5.pyx":291
 *         if self.write is not NULL:
 *             slow5_rec_free(self.write)
 *         if self.state in [1, 2]:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
    break;
    default: break;
  }

  /* "pyslow5.pyx":299
 *             else:
 *                 self.logger.debug("{} already closed".format(self.path))
 *         if self.state == 0:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
  __pyx_t_1 = ((__pyx_v_self->state == 0) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":300
 *                 self.logger.debug("{} already closed".format(self.path))
 *         if self.state == 0:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_idx_unload(self.s5)
 */
    __pyx_t_1 = ((!(__pyx_v_self->close_state != 0)) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":301
 *         if self.state == 0:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_idx_unload(self.s5)
 *                     slow5_close(self.s5)
 */
      __pyx_t_1 = ((__pyx_v_self->s5 != NULL) != 0);
      if (__pyx_t_1) {

        /* "pyslow5.pyx":302
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 *                     slow5_idx_unload(self.s5)             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
        slow5_idx_unload(__pyx_v_self->s5);

        /* "pyslow5.pyx":303
 *                 if self.s5 is not NULL:
 *                     slow5_idx_unload(self.s5)
 *                     slow5_close(self.s5)             # <<<<<<<<<<<<<<
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))
 */
        (void)(slow5_close(__pyx_v_self->s5));

        /* "pyslow5.pyx":304
 *                     slow5_idx_unload(self.s5)
 *                     slow5_close(self.s5)
 *                     self.close_state = True             # <<<<<<<<<<<<<<
 *                     self.logger.debug("{} closed".format(self.path))
 * 
 */
        __pyx_v_self->close_state = 1;

        /* "pyslow5.pyx":305
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))             # <<<<<<<<<<<<<<
 * 
 *         free(self.channel_number)
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 305, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 305, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 305, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":301
 *         if self.state == 0:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_idx_unload(self.s5)
 *                     slow5_close(self.s5)
 */
      }

      /* "pyslow5.pyx":300
 *                 self.logger.debug("{} already closed".format(self.path))
 *         if self.state == 0:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_idx_unload(self.s5)
 */
    }

    /* "pyslow5.pyx":299
 *             else:
 *                 self.logger.debug("{} already closed".format(self.path))
 *         if self.state == 0:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
  }

  /* "pyslow5.pyx":307
 *                     self.logger.debug("{} closed".format(self.path))
 * 
 *         free(self.channel_number)             # <<<<<<<<<<<<<<
 *         free(self.median_before)
 *         free(self.read_number)
 */
  free(__pyx_v_self->channel_number);

  /* "pyslow5.pyx":308
 * 
 *         free(self.channel_number)
 *         free(self.median_before)             # <<<<<<<<<<<<<<
 *         free(self.read_number)
 *         free(self.start_mux)
 */
  free(__pyx_v_self->median_before);

  /* "pyslow5.pyx":309
 *         free(self.channel_number)
 *         free(self.median_before)
 *         free(self.read_number)             # <<<<<<<<<<<<<<
 *         free(self.start_mux)
 *         free(self.start_time)
 */
  free(__pyx_v_self->read_number);

  /* "pyslow5.pyx":310
 *         free(self.median_before)
 *         free(self.read_number)
 *         free(self.start_mux)             # <<<<<<<<<<<<<<
 *         free(self.start_time)
 *         free(self.end_reason)
 */
  free(__pyx_v_self->start_mux);

  /* "pyslow5.pyx":311
 *         free(self.read_number)
 *         free(self.start_mux)
 *         free(self.start_time)             # <<<<<<<<<<<<<<
 *         free(self.end_reason)
 * 
 */
  free(__pyx_v_self->start_time);

  /* "pyslow5.pyx":312
 *         free(self.start_mux)
 *         free(self.start_time)
 *         free(self.end_reason)             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("pathname: {}".format(self.path))
 */
  free(__pyx_v_self->end_reason);

  /* "pyslow5.pyx":314
 *         free(self.end_reason)
 * 
 *         self.logger.debug("pathname: {}".format(self.path))             # <<<<<<<<<<<<<<
 *         self.logger.debug("total_time_slow5_get_next: {} seconds".format(self.total_time_slow5_get_next))
 *         self.logger.debug("total_time_yield_reads: {} seconds".format(self.total_time_yield_reads))
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_pathname_2, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":315
 * 
 *         self.logger.debug("pathname: {}".format(self.path))
 *         self.logger.debug("total_time_slow5_get_next: {} seconds".format(self.total_time_slow5_get_next))             # <<<<<<<<<<<<<<
 *         self.logger.debug("total_time_yield_reads: {} seconds".format(self.total_time_yield_reads))
 *         self.logger.debug("total_single_write_time: {} seconds".format(self.total_single_write_time))
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_total_time_slow5_get_next_second, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->total_time_slow5_get_next) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->total_time_slow5_get_next);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":316
 *         self.logger.debug("pathname: {}".format(self.path))
 *         self.logger.debug("total_time_slow5_get_next: {} seconds".format(self.total_time_slow5_get_next))
 *         self.logger.debug("total_time_yield_reads: {} seconds".format(self.total_time_yield_reads))             # <<<<<<<<<<<<<<
 *         self.logger.debug("total_single_write_time: {} seconds".format(self.total_single_write_time))
 *         self.logger.debug("total_multi_write_signal_time: {} seconds".format(self.total_multi_write_signal_time))
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_total_time_yield_reads_seconds, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->total_time_yield_reads) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->total_time_yield_reads);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":317
 *         self.logger.debug("total_time_slow5_get_next: {} seconds".format(self.total_time_slow5_get_next))
 *         self.logger.debug("total_time_yield_reads: {} seconds".format(self.total_time_yield_reads))
 *         self.logger.debug("total_single_write_time: {} seconds".format(self.total_single_write_time))             # <<<<<<<<<<<<<<
 *         self.logger.debug("total_multi_write_signal_time: {} seconds".format(self.total_multi_write_signal_time))
 *         self.logger.debug("total_multi_write_time: {} seconds".format(self.total_multi_write_time))
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_total_single_write_time_seconds, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->total_single_write_time) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->total_single_write_time);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":318
 *         self.logger.debug("total_time_yield_reads: {} seconds".format(self.total_time_yield_reads))
 *         self.logger.debug("total_single_write_time: {} seconds".format(self.total_single_write_time))
 *         self.logger.debug("total_multi_write_signal_time: {} seconds".format(self.total_multi_write_signal_time))             # <<<<<<<<<<<<<<
 *         self.logger.debug("total_multi_write_time: {} seconds".format(self.total_multi_write_time))
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_total_multi_write_signal_time_se, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->total_multi_write_signal_time) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->total_multi_write_signal_time);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":319
 *         self.logger.debug("total_single_write_time: {} seconds".format(self.total_single_write_time))
 *         self.logger.debug("total_multi_write_signal_time: {} seconds".format(self.total_multi_write_signal_time))
 *         self.logger.debug("total_multi_write_time: {} seconds".format(self.total_multi_write_time))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_total_multi_write_time_seconds, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->total_multi_write_time) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->total_multi_write_time);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":280
 * 
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.p is not NULL:
 *             free(self.p)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pyslow5.Open.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
}

/* "pyslow5.pyx":322
 * 
 * 
 *     def _convert_to_pA(self, d):             # <<<<<<<<<<<<<<
 *         '''
 *         convert raw signal data to pA using digitisation, offset, and range
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_7_convert_to_pA(PyObject *__pyx_v_self, PyObject *__pyx_v_d); /*proto*/
static char __pyx_doc_7pyslow5_4Open_6_convert_to_pA[] = "\n        convert raw signal data to pA using digitisation, offset, and range\n        float raw_unit = range / digitisation;\n        for (int32_t j = 0; j < nsample; j++) {\n            rawptr[j] = (rawptr[j] + offset) * raw_unit;\n        }\n        TODO: Use memory view for the loop so it's faster\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_7_convert_to_pA(PyObject *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_convert_to_pA (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_6_convert_to_pA(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), ((PyObject *)__pyx_v_d));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_6_convert_to_pA(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_v_digitisation = NULL;
  PyObject *__pyx_v_range = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_raw_unit = NULL;
  PyObject *__pyx_v_new_raw = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_convert_to_pA", 0);
  __Pyx_TraceCall("_convert_to_pA", __pyx_f[0], 322, 0, __PYX_ERR(0, 322, __pyx_L1_error));

  /* "pyslow5.pyx":331
 *         TODO: Use memory view for the loop so it's faster
 *         '''
 *         digitisation = d['digitisation']             # <<<<<<<<<<<<<<
 *         range = d['range']
 *         offset = d['offset']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_d, __pyx_n_u_digitisation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_digitisation = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":332
 *         '''
 *         digitisation = d['digitisation']
 *         range = d['range']             # <<<<<<<<<<<<<<
 *         offset = d['offset']
 *         raw_unit = range / digitisation
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_d, __pyx_n_u_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_range = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":333
 *         digitisation = d['digitisation']
 *         range = d['range']
 *         offset = d['offset']             # <<<<<<<<<<<<<<
 *         raw_unit = range / digitisation
 *         # new_raw = []
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_d, __pyx_n_u_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_offset = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":334
 *         range = d['range']
 *         offset = d['offset']
 *         raw_unit = range / digitisation             # <<<<<<<<<<<<<<
 *         # new_raw = []
 *         # for i in d['signal']:
 */
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_range, __pyx_v_digitisation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_unit = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":340
 *         #     new_raw.append(float("{0:.2f}".format(round(j,2))))
 *         # new_raw = np.array(raw_unit * (d['signal'] + offset), dtype=np.float32)
 *         new_raw = (raw_unit * (d['signal'] + offset)).astype(np.float32)             # <<<<<<<<<<<<<<
 *         return new_raw
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_d, __pyx_n_u_signal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_v_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_raw_unit, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_astype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float32); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_new_raw = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":341
 *         # new_raw = np.array(raw_unit * (d['signal'] + offset), dtype=np.float32)
 *         new_raw = (raw_unit * (d['signal'] + offset)).astype(np.float32)
 *         return new_raw             # <<<<<<<<<<<<<<
 * 
 *     # ==========================================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_new_raw);
  __pyx_r = __pyx_v_new_raw;
  goto __pyx_L0;

  /* "pyslow5.pyx":322
 * 
 * 
 *     def _convert_to_pA(self, d):             # <<<<<<<<<<<<<<
 *         '''
 *         convert raw signal data to pA using digitisation, offset, and range
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyslow5.Open._convert_to_pA", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_digitisation);
  __Pyx_XDECREF(__pyx_v_range);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_raw_unit);
  __Pyx_XDECREF(__pyx_v_new_raw);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":347
 *     # ==========================================================================
 * 
 *     def _get_read(self, read_id, pA, aux):             # <<<<<<<<<<<<<<
 *         '''
 *         read_id = readID of individual read as a string, get's converted to b'str'
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_9_get_read(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_8_get_read[] = "\n        read_id = readID of individual read as a string, get's converted to b'str'\n        pA = Bool for converting signal to picoamps\n        aux = str 'name'/'all' or list of names of auxiliary fields added to return dictionary\n        returns dic = dictionary of main fields for read_id, with any aux fields added\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_9_get_read(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_id = 0;
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_read (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_id,&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_id)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read", 1, 3, 3, 1); __PYX_ERR(0, 347, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read", 1, 3, 3, 2); __PYX_ERR(0, 347, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_read") < 0)) __PYX_ERR(0, 347, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_read_id = values[0];
    __pyx_v_pA = values[1];
    __pyx_v_aux = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_read", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 347, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_8_get_read(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_id, __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_8_get_read(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_id, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  int __pyx_v_ret;
  PyObject *__pyx_v_dic = NULL;
  PyObject *__pyx_v_aux_dic = NULL;
  PyObject *__pyx_v_ID = NULL;
  int __pyx_v_found_single_aux;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_v_n_list = NULL;
  PyObject *__pyx_v_t_list = NULL;
  PyObject *__pyx_v_n_set = NULL;
  PyObject *__pyx_v_aux_set = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_signal = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  char const *__pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *(*__pyx_t_16)(PyObject *);
  int __pyx_t_17;
  char *__pyx_t_18;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_read", 0);
  __Pyx_TraceCall("_get_read", __pyx_f[0], 347, 0, __PYX_ERR(0, 347, __pyx_L1_error));

  /* "pyslow5.pyx":354
 *         returns dic = dictionary of main fields for read_id, with any aux fields added
 *         '''
 *         if not self.index_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 */
  __pyx_t_1 = ((!(__pyx_v_self->index_state != 0)) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":355
 *         '''
 *         if not self.index_state:
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))             # <<<<<<<<<<<<<<
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 *             self.logger.debug("Creating/loading index...")
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FILE_mode, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_self->path, __pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_self->path, __pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->path);
      __Pyx_GIVEREF(__pyx_v_self->path);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_self->path);
      __Pyx_INCREF(__pyx_v_self->mode);
      __Pyx_GIVEREF(__pyx_v_self->mode);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_self->mode);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":357
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 *             self.logger.debug("Creating/loading index...")             # <<<<<<<<<<<<<<
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_Creating_loading_index) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Creating_loading_index);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":358
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)             # <<<<<<<<<<<<<<
 *             if ret != 0:
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 */
    __pyx_v_ret = slow5_idx_load(__pyx_v_self->s5);

    /* "pyslow5.pyx":359
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:             # <<<<<<<<<<<<<<
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 */
    __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":360
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.index_state = True
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_idx_load_return_not_0, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(__pyx_v_self->error_codes == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 360, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_8, __pyx_t_9};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_8, __pyx_t_9};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 360, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_7, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_7, __pyx_t_9);
        __pyx_t_8 = 0;
        __pyx_t_9 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":359
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:             # <<<<<<<<<<<<<<
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 */
      goto __pyx_L4;
    }

    /* "pyslow5.pyx":362
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 *                 self.index_state = True             # <<<<<<<<<<<<<<
 *         dic = {}
 *         aux_dic = {}
 */
    /*else*/ {
      __pyx_v_self->index_state = 1;
    }
    __pyx_L4:;

    /* "pyslow5.pyx":354
 *         returns dic = dictionary of main fields for read_id, with any aux fields added
 *         '''
 *         if not self.index_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 */
  }

  /* "pyslow5.pyx":363
 *             else:
 *                 self.index_state = True
 *         dic = {}             # <<<<<<<<<<<<<<
 *         aux_dic = {}
 *         ID = str.encode(read_id)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_dic = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":364
 *                 self.index_state = True
 *         dic = {}
 *         aux_dic = {}             # <<<<<<<<<<<<<<
 *         ID = str.encode(read_id)
 *         # rec = NULL
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_aux_dic = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":365
 *         dic = {}
 *         aux_dic = {}
 *         ID = str.encode(read_id)             # <<<<<<<<<<<<<<
 *         # rec = NULL
 *         ret = slow5_get(ID, &self.rec, self.s5)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&PyUnicode_Type)), __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_read_id) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_read_id);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_ID = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":367
 *         ID = str.encode(read_id)
 *         # rec = NULL
 *         ret = slow5_get(ID, &self.rec, self.s5)             # <<<<<<<<<<<<<<
 *         if ret != 0:
 *             self.logger.debug("get_read return not 0: {}".format(ret))
 */
  __pyx_t_11 = __Pyx_PyObject_AsString(__pyx_v_ID); if (unlikely((!__pyx_t_11) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_v_ret = slow5_get(__pyx_t_11, (&__pyx_v_self->rec), __pyx_v_self->s5);

  /* "pyslow5.pyx":368
 *         # rec = NULL
 *         ret = slow5_get(ID, &self.rec, self.s5)
 *         if ret != 0:             # <<<<<<<<<<<<<<
 *             self.logger.debug("get_read return not 0: {}".format(ret))
 *             return None
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":369
 *         ret = slow5_get(ID, &self.rec, self.s5)
 *         if ret != 0:
 *             self.logger.debug("get_read return not 0: {}".format(ret))             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_return_not_0, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_9, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":370
 *         if ret != 0:
 *             self.logger.debug("get_read return not 0: {}".format(ret))
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         # check for aux fields
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pyslow5.pyx":368
 *         # rec = NULL
 *         ret = slow5_get(ID, &self.rec, self.s5)
 *         if ret != 0:             # <<<<<<<<<<<<<<
 *             self.logger.debug("get_read return not 0: {}".format(ret))
 *             return None
 */
  }

  /* "pyslow5.pyx":373
 * 
 *         # check for aux fields
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             if not self.aux_names or not self.aux_types:
 *                 self.aux_names = self.get_aux_names()
 */
  __pyx_t_1 = (__pyx_v_aux != Py_None);
  __pyx_t_12 = (__pyx_t_1 != 0);
  if (__pyx_t_12) {

    /* "pyslow5.pyx":374
 *         # check for aux fields
 *         if aux is not None:
 *             if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                 self.aux_names = self.get_aux_names()
 *                 self.aux_types = self.get_aux_types()
 */
    __pyx_t_1 = (__pyx_v_self->aux_names != Py_None)&&(PyList_GET_SIZE(__pyx_v_self->aux_names) != 0);
    __pyx_t_13 = ((!__pyx_t_1) != 0);
    if (!__pyx_t_13) {
    } else {
      __pyx_t_12 = __pyx_t_13;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_13 = (__pyx_v_self->aux_types != Py_None)&&(PyList_GET_SIZE(__pyx_v_self->aux_types) != 0);
    __pyx_t_1 = ((!__pyx_t_13) != 0);
    __pyx_t_12 = __pyx_t_1;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_12) {

      /* "pyslow5.pyx":375
 *         if aux is not None:
 *             if not self.aux_names or not self.aux_types:
 *                 self.aux_names = self.get_aux_names()             # <<<<<<<<<<<<<<
 *                 self.aux_types = self.get_aux_types()
 *             if type(aux) is str:
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_aux_names); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->aux_names);
      __Pyx_DECREF(__pyx_v_self->aux_names);
      __pyx_v_self->aux_names = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":376
 *             if not self.aux_names or not self.aux_types:
 *                 self.aux_names = self.get_aux_names()
 *                 self.aux_types = self.get_aux_types()             # <<<<<<<<<<<<<<
 *             if type(aux) is str:
 *                 # special type 'all'
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_aux_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->aux_types);
      __Pyx_DECREF(__pyx_v_self->aux_types);
      __pyx_v_self->aux_types = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":374
 *         # check for aux fields
 *         if aux is not None:
 *             if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                 self.aux_names = self.get_aux_names()
 *                 self.aux_types = self.get_aux_types()
 */
    }

    /* "pyslow5.pyx":377
 *                 self.aux_names = self.get_aux_names()
 *                 self.aux_types = self.get_aux_types()
 *             if type(aux) is str:             # <<<<<<<<<<<<<<
 *                 # special type 'all'
 *                 if aux == "all":
 */
    __pyx_t_12 = (((PyObject *)Py_TYPE(__pyx_v_aux)) == ((PyObject *)(&PyUnicode_Type)));
    __pyx_t_1 = (__pyx_t_12 != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":379
 *             if type(aux) is str:
 *                 # special type 'all'
 *                 if aux == "all":             # <<<<<<<<<<<<<<
 *                     aux_dic = self._get_read_aux(self.aux_names, self.aux_types)
 *                 else:
 */
      __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_aux, __pyx_n_u_all, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 379, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "pyslow5.pyx":380
 *                 # special type 'all'
 *                 if aux == "all":
 *                     aux_dic = self._get_read_aux(self.aux_names, self.aux_types)             # <<<<<<<<<<<<<<
 *                 else:
 *                     found_single_aux = False
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_read_aux); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_self->aux_names, __pyx_v_self->aux_types};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_self->aux_names, __pyx_v_self->aux_types};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_self->aux_names);
          __Pyx_GIVEREF(__pyx_v_self->aux_names);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_self->aux_names);
          __Pyx_INCREF(__pyx_v_self->aux_types);
          __Pyx_GIVEREF(__pyx_v_self->aux_types);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_v_self->aux_types);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF_SET(__pyx_v_aux_dic, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":379
 *             if type(aux) is str:
 *                 # special type 'all'
 *                 if aux == "all":             # <<<<<<<<<<<<<<
 *                     aux_dic = self._get_read_aux(self.aux_names, self.aux_types)
 *                 else:
 */
        goto __pyx_L11;
      }

      /* "pyslow5.pyx":382
 *                     aux_dic = self._get_read_aux(self.aux_names, self.aux_types)
 *                 else:
 *                     found_single_aux = False             # <<<<<<<<<<<<<<
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n == aux:
 */
      /*else*/ {
        __pyx_v_found_single_aux = 0;

        /* "pyslow5.pyx":383
 *                 else:
 *                     found_single_aux = False
 *                     for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                         if n == aux:
 *                             found_single_aux = True
 */
        __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_v_self->aux_names);
        __Pyx_GIVEREF(__pyx_v_self->aux_names);
        PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->aux_names);
        __Pyx_INCREF(__pyx_v_self->aux_types);
        __Pyx_GIVEREF(__pyx_v_self->aux_types);
        PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->aux_types);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
          __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_14 = 0;
          __pyx_t_15 = NULL;
        } else {
          __pyx_t_14 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_15 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 383, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        for (;;) {
          if (likely(!__pyx_t_15)) {
            if (likely(PyList_CheckExact(__pyx_t_2))) {
              if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_2)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
              #else
              __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            } else {
              if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
              #else
              __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            }
          } else {
            __pyx_t_3 = __pyx_t_15(__pyx_t_2);
            if (unlikely(!__pyx_t_3)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 383, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_3);
          }
          if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
            PyObject* sequence = __pyx_t_3;
            Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
            if (unlikely(size != 2)) {
              if (size > 2) __Pyx_RaiseTooManyValuesError(2);
              else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
              __PYX_ERR(0, 383, __pyx_L1_error)
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            if (likely(PyTuple_CheckExact(sequence))) {
              __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
              __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
            } else {
              __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
              __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
            }
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_4);
            #else
            __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 383, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 383, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            #endif
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          } else {
            Py_ssize_t index = -1;
            __pyx_t_10 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 383, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_16 = Py_TYPE(__pyx_t_10)->tp_iternext;
            index = 0; __pyx_t_5 = __pyx_t_16(__pyx_t_10); if (unlikely(!__pyx_t_5)) goto __pyx_L14_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_5);
            index = 1; __pyx_t_4 = __pyx_t_16(__pyx_t_10); if (unlikely(!__pyx_t_4)) goto __pyx_L14_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_4);
            if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_10), 2) < 0) __PYX_ERR(0, 383, __pyx_L1_error)
            __pyx_t_16 = NULL;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            goto __pyx_L15_unpacking_done;
            __pyx_L14_unpacking_failed:;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __pyx_t_16 = NULL;
            if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
            __PYX_ERR(0, 383, __pyx_L1_error)
            __pyx_L15_unpacking_done:;
          }
          __Pyx_XDECREF_SET(__pyx_v_n, __pyx_t_5);
          __pyx_t_5 = 0;
          __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":384
 *                     found_single_aux = False
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n == aux:             # <<<<<<<<<<<<<<
 *                             found_single_aux = True
 *                             aux_dic = self._get_read_aux([n], [t])
 */
          __pyx_t_3 = PyObject_RichCompare(__pyx_v_n, __pyx_v_aux, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 384, __pyx_L1_error)
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 384, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (__pyx_t_1) {

            /* "pyslow5.pyx":385
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n == aux:
 *                             found_single_aux = True             # <<<<<<<<<<<<<<
 *                             aux_dic = self._get_read_aux([n], [t])
 *                             break
 */
            __pyx_v_found_single_aux = 1;

            /* "pyslow5.pyx":386
 *                         if n == aux:
 *                             found_single_aux = True
 *                             aux_dic = self._get_read_aux([n], [t])             # <<<<<<<<<<<<<<
 *                             break
 *                     if not found_single_aux:
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_read_aux); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 386, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 386, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_INCREF(__pyx_v_n);
            __Pyx_GIVEREF(__pyx_v_n);
            PyList_SET_ITEM(__pyx_t_5, 0, __pyx_v_n);
            __pyx_t_10 = PyList_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 386, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_INCREF(__pyx_v_t);
            __Pyx_GIVEREF(__pyx_v_t);
            PyList_SET_ITEM(__pyx_t_10, 0, __pyx_v_t);
            __pyx_t_9 = NULL;
            __pyx_t_7 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_9)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
                __pyx_t_7 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_4)) {
              PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_5, __pyx_t_10};
              __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
              PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_5, __pyx_t_10};
              __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            } else
            #endif
            {
              __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 386, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              if (__pyx_t_9) {
                __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9); __pyx_t_9 = NULL;
              }
              __Pyx_GIVEREF(__pyx_t_5);
              PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_5);
              __Pyx_GIVEREF(__pyx_t_10);
              PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_10);
              __pyx_t_5 = 0;
              __pyx_t_10 = 0;
              __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF_SET(__pyx_v_aux_dic, __pyx_t_3);
            __pyx_t_3 = 0;

            /* "pyslow5.pyx":387
 *                             found_single_aux = True
 *                             aux_dic = self._get_read_aux([n], [t])
 *                             break             # <<<<<<<<<<<<<<
 *                     if not found_single_aux:
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))
 */
            goto __pyx_L13_break;

            /* "pyslow5.pyx":384
 *                     found_single_aux = False
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n == aux:             # <<<<<<<<<<<<<<
 *                             found_single_aux = True
 *                             aux_dic = self._get_read_aux([n], [t])
 */
          }

          /* "pyslow5.pyx":383
 *                 else:
 *                     found_single_aux = False
 *                     for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                         if n == aux:
 *                             found_single_aux = True
 */
        }
        __pyx_L13_break:;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":388
 *                             aux_dic = self._get_read_aux([n], [t])
 *                             break
 *                     if not found_single_aux:             # <<<<<<<<<<<<<<
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                         aux_dic.update({aux: None})
 */
        __pyx_t_1 = ((!(__pyx_v_found_single_aux != 0)) != 0);
        if (__pyx_t_1) {

          /* "pyslow5.pyx":389
 *                             break
 *                     if not found_single_aux:
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))             # <<<<<<<<<<<<<<
 *                         aux_dic.update({aux: None})
 *             elif type(aux) is list:
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 389, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_unknown_aux_name, __pyx_n_s_format); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 389, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_8, function);
            }
          }
          __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_10, __pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_aux);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 389, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_8, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":390
 *                     if not found_single_aux:
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                         aux_dic.update({aux: None})             # <<<<<<<<<<<<<<
 *             elif type(aux) is list:
 *                 n_list = []
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 390, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          if (PyDict_SetItem(__pyx_t_4, __pyx_v_aux, Py_None) < 0) __PYX_ERR(0, 390, __pyx_L1_error)
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_8, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":388
 *                             aux_dic = self._get_read_aux([n], [t])
 *                             break
 *                     if not found_single_aux:             # <<<<<<<<<<<<<<
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                         aux_dic.update({aux: None})
 */
        }
      }
      __pyx_L11:;

      /* "pyslow5.pyx":377
 *                 self.aux_names = self.get_aux_names()
 *                 self.aux_types = self.get_aux_types()
 *             if type(aux) is str:             # <<<<<<<<<<<<<<
 *                 # special type 'all'
 *                 if aux == "all":
 */
      goto __pyx_L10;
    }

    /* "pyslow5.pyx":391
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                         aux_dic.update({aux: None})
 *             elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                 n_list = []
 *                 t_list = []
 */
    __pyx_t_1 = (((PyObject *)Py_TYPE(__pyx_v_aux)) == ((PyObject *)(&PyList_Type)));
    __pyx_t_12 = (__pyx_t_1 != 0);
    if (__pyx_t_12) {

      /* "pyslow5.pyx":392
 *                         aux_dic.update({aux: None})
 *             elif type(aux) is list:
 *                 n_list = []             # <<<<<<<<<<<<<<
 *                 t_list = []
 *                 for n, t in zip(self.aux_names, self.aux_types):
 */
      __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_n_list = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":393
 *             elif type(aux) is list:
 *                 n_list = []
 *                 t_list = []             # <<<<<<<<<<<<<<
 *                 for n, t in zip(self.aux_names, self.aux_types):
 *                     if n in aux:
 */
      __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_t_list = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":394
 *                 n_list = []
 *                 t_list = []
 *                 for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                     if n in aux:
 *                         n_list.append(n)
 */
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 394, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_v_self->aux_names);
      __Pyx_GIVEREF(__pyx_v_self->aux_names);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->aux_names);
      __Pyx_INCREF(__pyx_v_self->aux_types);
      __Pyx_GIVEREF(__pyx_v_self->aux_types);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->aux_types);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
        __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_14 = 0;
        __pyx_t_15 = NULL;
      } else {
        __pyx_t_14 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_15 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 394, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      for (;;) {
        if (likely(!__pyx_t_15)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 394, __pyx_L1_error)
            #else
            __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          } else {
            if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 394, __pyx_L1_error)
            #else
            __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          }
        } else {
          __pyx_t_3 = __pyx_t_15(__pyx_t_2);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 394, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_3);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
          PyObject* sequence = __pyx_t_3;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 394, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_8);
          #else
          __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_10 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_16 = Py_TYPE(__pyx_t_10)->tp_iternext;
          index = 0; __pyx_t_4 = __pyx_t_16(__pyx_t_10); if (unlikely(!__pyx_t_4)) goto __pyx_L20_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_4);
          index = 1; __pyx_t_8 = __pyx_t_16(__pyx_t_10); if (unlikely(!__pyx_t_8)) goto __pyx_L20_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_8);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_10), 2) < 0) __PYX_ERR(0, 394, __pyx_L1_error)
          __pyx_t_16 = NULL;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          goto __pyx_L21_unpacking_done;
          __pyx_L20_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_16 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 394, __pyx_L1_error)
          __pyx_L21_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_n, __pyx_t_4);
        __pyx_t_4 = 0;
        __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_8);
        __pyx_t_8 = 0;

        /* "pyslow5.pyx":395
 *                 t_list = []
 *                 for n, t in zip(self.aux_names, self.aux_types):
 *                     if n in aux:             # <<<<<<<<<<<<<<
 *                         n_list.append(n)
 *                         t_list.append(t)
 */
        __pyx_t_12 = (__Pyx_PySequence_ContainsTF(__pyx_v_n, __pyx_v_aux, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 395, __pyx_L1_error)
        __pyx_t_1 = (__pyx_t_12 != 0);
        if (__pyx_t_1) {

          /* "pyslow5.pyx":396
 *                 for n, t in zip(self.aux_names, self.aux_types):
 *                     if n in aux:
 *                         n_list.append(n)             # <<<<<<<<<<<<<<
 *                         t_list.append(t)
 * 
 */
          __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_n_list, __pyx_v_n); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 396, __pyx_L1_error)

          /* "pyslow5.pyx":397
 *                     if n in aux:
 *                         n_list.append(n)
 *                         t_list.append(t)             # <<<<<<<<<<<<<<
 * 
 *                 aux_dic = self._get_read_aux(n_list, t_list)
 */
          __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_t_list, __pyx_v_t); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 397, __pyx_L1_error)

          /* "pyslow5.pyx":395
 *                 t_list = []
 *                 for n, t in zip(self.aux_names, self.aux_types):
 *                     if n in aux:             # <<<<<<<<<<<<<<
 *                         n_list.append(n)
 *                         t_list.append(t)
 */
        }

        /* "pyslow5.pyx":394
 *                 n_list = []
 *                 t_list = []
 *                 for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                     if n in aux:
 *                         n_list.append(n)
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":399
 *                         t_list.append(t)
 * 
 *                 aux_dic = self._get_read_aux(n_list, t_list)             # <<<<<<<<<<<<<<
 *                 # Check for items given that did not exist
 *                 n_set = set(n_list)
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_read_aux); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_v_n_list, __pyx_v_t_list};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 399, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_v_n_list, __pyx_v_t_list};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 399, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 399, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_INCREF(__pyx_v_n_list);
        __Pyx_GIVEREF(__pyx_v_n_list);
        PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_7, __pyx_v_n_list);
        __Pyx_INCREF(__pyx_v_t_list);
        __Pyx_GIVEREF(__pyx_v_t_list);
        PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_7, __pyx_v_t_list);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 399, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_aux_dic, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":401
 *                 aux_dic = self._get_read_aux(n_list, t_list)
 *                 # Check for items given that did not exist
 *                 n_set = set(n_list)             # <<<<<<<<<<<<<<
 *                 aux_set = set(aux)
 *                 if len(aux_set.difference(n_set)) > 0:
 */
      __pyx_t_2 = PySet_New(__pyx_v_n_list); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_n_set = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":402
 *                 # Check for items given that did not exist
 *                 n_set = set(n_list)
 *                 aux_set = set(aux)             # <<<<<<<<<<<<<<
 *                 if len(aux_set.difference(n_set)) > 0:
 *                     for i in aux_set.difference(n_set):
 */
      __pyx_t_2 = PySet_New(__pyx_v_aux); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_aux_set = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":403
 *                 n_set = set(n_list)
 *                 aux_set = set(aux)
 *                 if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                     for i in aux_set.difference(n_set):
 *                         self.logger.warning("get_read unknown aux name: {}".format(i))
 */
      __pyx_t_2 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_v_aux_set, __pyx_v_n_set); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_14 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_14 == ((Py_ssize_t)-1))) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_1 = ((__pyx_t_14 > 0) != 0);
      if (__pyx_t_1) {

        /* "pyslow5.pyx":404
 *                 aux_set = set(aux)
 *                 if len(aux_set.difference(n_set)) > 0:
 *                     for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                         self.logger.warning("get_read unknown aux name: {}".format(i))
 *                         aux_dic.update({i: None})
 */
        __pyx_t_2 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_v_aux_set, __pyx_v_n_set); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
          __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_14 = 0;
          __pyx_t_15 = NULL;
        } else {
          __pyx_t_14 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 404, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_15 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 404, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        for (;;) {
          if (likely(!__pyx_t_15)) {
            if (likely(PyList_CheckExact(__pyx_t_3))) {
              if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_3)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_14); __Pyx_INCREF(__pyx_t_2); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 404, __pyx_L1_error)
              #else
              __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              #endif
            } else {
              if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_14); __Pyx_INCREF(__pyx_t_2); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 404, __pyx_L1_error)
              #else
              __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              #endif
            }
          } else {
            __pyx_t_2 = __pyx_t_15(__pyx_t_3);
            if (unlikely(!__pyx_t_2)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 404, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_2);
          }
          __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
          __pyx_t_2 = 0;

          /* "pyslow5.pyx":405
 *                 if len(aux_set.difference(n_set)) > 0:
 *                     for i in aux_set.difference(n_set):
 *                         self.logger.warning("get_read unknown aux name: {}".format(i))             # <<<<<<<<<<<<<<
 *                         aux_dic.update({i: None})
 * 
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_unknown_aux_name, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_5 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_5)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
            }
          }
          __pyx_t_8 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_5, __pyx_v_i) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_i);
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
            }
          }
          __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":406
 *                     for i in aux_set.difference(n_set):
 *                         self.logger.warning("get_read unknown aux name: {}".format(i))
 *                         aux_dic.update({i: None})             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (PyDict_SetItem(__pyx_t_8, __pyx_v_i, Py_None) < 0) __PYX_ERR(0, 406, __pyx_L1_error)
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
            }
          }
          __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":404
 *                 aux_set = set(aux)
 *                 if len(aux_set.difference(n_set)) > 0:
 *                     for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                         self.logger.warning("get_read unknown aux name: {}".format(i))
 *                         aux_dic.update({i: None})
 */
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyslow5.pyx":403
 *                 n_set = set(n_list)
 *                 aux_set = set(aux)
 *                 if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                     for i in aux_set.difference(n_set):
 *                         self.logger.warning("get_read unknown aux name: {}".format(i))
 */
      }

      /* "pyslow5.pyx":391
 *                         self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                         aux_dic.update({aux: None})
 *             elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                 n_list = []
 *                 t_list = []
 */
      goto __pyx_L10;
    }

    /* "pyslow5.pyx":409
 * 
 *             else:
 *                 self.logger.debug("get_read aux type unknown, accepts str or list: {}".format(aux))             # <<<<<<<<<<<<<<
 * 
 *         # get read data
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_aux_type_unknown_accept, __pyx_n_s_format); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_10, __pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_aux);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_8, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_L10:;

    /* "pyslow5.pyx":373
 * 
 *         # check for aux fields
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             if not self.aux_names or not self.aux_types:
 *                 self.aux_names = self.get_aux_names()
 */
  }

  /* "pyslow5.pyx":412
 * 
 *         # get read data
 *         if type(self.rec.read_id) is bytes:             # <<<<<<<<<<<<<<
 *             dic['read_id'] = self.rec.read_id.decode()
 *         else:
 */
  __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_self->rec->read_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = (((PyObject *)Py_TYPE(__pyx_t_3)) == ((PyObject *)(&PyBytes_Type)));
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_12 = (__pyx_t_1 != 0);
  if (__pyx_t_12) {

    /* "pyslow5.pyx":413
 *         # get read data
 *         if type(self.rec.read_id) is bytes:
 *             dic['read_id'] = self.rec.read_id.decode()             # <<<<<<<<<<<<<<
 *         else:
 *             dic['read_id'] = self.rec.read_id
 */
    __pyx_t_18 = __pyx_v_self->rec->read_id;
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_t_18, 0, strlen(__pyx_t_18), NULL, NULL, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_read_id, __pyx_t_3) < 0)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":412
 * 
 *         # get read data
 *         if type(self.rec.read_id) is bytes:             # <<<<<<<<<<<<<<
 *             dic['read_id'] = self.rec.read_id.decode()
 *         else:
 */
    goto __pyx_L26;
  }

  /* "pyslow5.pyx":415
 *             dic['read_id'] = self.rec.read_id.decode()
 *         else:
 *             dic['read_id'] = self.rec.read_id             # <<<<<<<<<<<<<<
 *         dic['read_group'] = self.rec.read_group
 *         dic['digitisation'] = self.rec.digitisation
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_self->rec->read_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_read_id, __pyx_t_3) < 0)) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L26:;

  /* "pyslow5.pyx":416
 *         else:
 *             dic['read_id'] = self.rec.read_id
 *         dic['read_group'] = self.rec.read_group             # <<<<<<<<<<<<<<
 *         dic['digitisation'] = self.rec.digitisation
 *         dic['offset'] = self.rec.offset
 */
  __pyx_t_3 = __Pyx_PyInt_From_uint32_t(__pyx_v_self->rec->read_group); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_read_group, __pyx_t_3) < 0)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":417
 *             dic['read_id'] = self.rec.read_id
 *         dic['read_group'] = self.rec.read_group
 *         dic['digitisation'] = self.rec.digitisation             # <<<<<<<<<<<<<<
 *         dic['offset'] = self.rec.offset
 *         dic['range'] = self.rec.range
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->rec->digitisation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_digitisation, __pyx_t_3) < 0)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":418
 *         dic['read_group'] = self.rec.read_group
 *         dic['digitisation'] = self.rec.digitisation
 *         dic['offset'] = self.rec.offset             # <<<<<<<<<<<<<<
 *         dic['range'] = self.rec.range
 *         dic['sampling_rate'] = self.rec.sampling_rate
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->rec->offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_offset, __pyx_t_3) < 0)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":419
 *         dic['digitisation'] = self.rec.digitisation
 *         dic['offset'] = self.rec.offset
 *         dic['range'] = self.rec.range             # <<<<<<<<<<<<<<
 *         dic['sampling_rate'] = self.rec.sampling_rate
 *         dic['len_raw_signal'] = self.rec.len_raw_signal
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->rec->range); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_range, __pyx_t_3) < 0)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":420
 *         dic['offset'] = self.rec.offset
 *         dic['range'] = self.rec.range
 *         dic['sampling_rate'] = self.rec.sampling_rate             # <<<<<<<<<<<<<<
 *         dic['len_raw_signal'] = self.rec.len_raw_signal
 *         # This could be handled by numpy.fromiter() or similar
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->rec->sampling_rate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_sampling_rate, __pyx_t_3) < 0)) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":421
 *         dic['range'] = self.rec.range
 *         dic['sampling_rate'] = self.rec.sampling_rate
 *         dic['len_raw_signal'] = self.rec.len_raw_signal             # <<<<<<<<<<<<<<
 *         # This could be handled by numpy.fromiter() or similar
 *         # Probably MUCH faster
 */
  __pyx_t_3 = __Pyx_PyInt_From_uint64_t(__pyx_v_self->rec->len_raw_signal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_len_raw_signal, __pyx_t_3) < 0)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":428
 *         # dic['signal'] = [self.rec.raw_signal[i] for i in range(self.rec.len_raw_signal)]
 *         # cdef np.npy_intp shape_get[1]
 *         self.shape_get[0] = <np.npy_intp> self.rec.len_raw_signal             # <<<<<<<<<<<<<<
 *         signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,
 *                     np.NPY_INT16, <void *> self.rec.raw_signal))
 */
  (__pyx_v_self->shape_get[0]) = ((npy_intp)__pyx_v_self->rec->len_raw_signal);

  /* "pyslow5.pyx":429
 *         # cdef np.npy_intp shape_get[1]
 *         self.shape_get[0] = <np.npy_intp> self.rec.len_raw_signal
 *         signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,             # <<<<<<<<<<<<<<
 *                     np.NPY_INT16, <void *> self.rec.raw_signal))
 *         np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":430
 *         self.shape_get[0] = <np.npy_intp> self.rec.len_raw_signal
 *         signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,
 *                     np.NPY_INT16, <void *> self.rec.raw_signal))             # <<<<<<<<<<<<<<
 *         np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *         dic['signal'] = signal
 */
  __pyx_t_2 = PyArray_SimpleNewFromData(1, __pyx_v_self->shape_get, NPY_INT16, ((void *)__pyx_v_self->rec->raw_signal)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_signal = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":431
 *         signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,
 *                     np.NPY_INT16, <void *> self.rec.raw_signal))
 *         np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)             # <<<<<<<<<<<<<<
 *         dic['signal'] = signal
 * 
 */
  if (!(likely(((__pyx_v_signal) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_signal, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 431, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_signal, __pyx_n_s_flags); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_int(NPY_OWNDATA); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Or(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyArray_UpdateFlags(((PyArrayObject *)__pyx_v_signal), __pyx_t_7);

  /* "pyslow5.pyx":432
 *                     np.NPY_INT16, <void *> self.rec.raw_signal))
 *         np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *         dic['signal'] = signal             # <<<<<<<<<<<<<<
 * 
 *         # if pA=True, convert signal to pA
 */
  if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_signal, __pyx_v_signal) < 0)) __PYX_ERR(0, 432, __pyx_L1_error)

  /* "pyslow5.pyx":435
 * 
 *         # if pA=True, convert signal to pA
 *         if pA:             # <<<<<<<<<<<<<<
 *             dic['signal'] = self._convert_to_pA(dic)
 *         # if aux data, add to main dic
 */
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_v_pA); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 435, __pyx_L1_error)
  if (__pyx_t_12) {

    /* "pyslow5.pyx":436
 *         # if pA=True, convert signal to pA
 *         if pA:
 *             dic['signal'] = self._convert_to_pA(dic)             # <<<<<<<<<<<<<<
 *         # if aux data, add to main dic
 *         if aux_dic:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_convert_to_pA); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_dic) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_dic);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_n_u_signal, __pyx_t_2) < 0)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":435
 * 
 *         # if pA=True, convert signal to pA
 *         if pA:             # <<<<<<<<<<<<<<
 *             dic['signal'] = self._convert_to_pA(dic)
 *         # if aux data, add to main dic
 */
  }

  /* "pyslow5.pyx":438
 *             dic['signal'] = self._convert_to_pA(dic)
 *         # if aux data, add to main dic
 *         if aux_dic:             # <<<<<<<<<<<<<<
 *             dic.update(aux_dic)
 *         return dic
 */
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_v_aux_dic); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 438, __pyx_L1_error)
  if (__pyx_t_12) {

    /* "pyslow5.pyx":439
 *         # if aux data, add to main dic
 *         if aux_dic:
 *             dic.update(aux_dic)             # <<<<<<<<<<<<<<
 *         return dic
 * 
 */
    __pyx_t_2 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_v_dic, __pyx_v_aux_dic); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":438
 *             dic['signal'] = self._convert_to_pA(dic)
 *         # if aux data, add to main dic
 *         if aux_dic:             # <<<<<<<<<<<<<<
 *             dic.update(aux_dic)
 *         return dic
 */
  }

  /* "pyslow5.pyx":440
 *         if aux_dic:
 *             dic.update(aux_dic)
 *         return dic             # <<<<<<<<<<<<<<
 * 
 *     def _get_batches(self, read_list, size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_dic);
  __pyx_r = __pyx_v_dic;
  goto __pyx_L0;

  /* "pyslow5.pyx":347
 *     # ==========================================================================
 * 
 *     def _get_read(self, read_id, pA, aux):             # <<<<<<<<<<<<<<
 *         '''
 *         read_id = readID of individual read as a string, get's converted to b'str'
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pyslow5.Open._get_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dic);
  __Pyx_XDECREF(__pyx_v_aux_dic);
  __Pyx_XDECREF(__pyx_v_ID);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_n_list);
  __Pyx_XDECREF(__pyx_v_t_list);
  __Pyx_XDECREF(__pyx_v_n_set);
  __Pyx_XDECREF(__pyx_v_aux_set);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_signal);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_12generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":442
 *         return dic
 * 
 *     def _get_batches(self, read_list, size):             # <<<<<<<<<<<<<<
 *         """
 *         batchify readIDs
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_11_get_batches(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_10_get_batches[] = "\n        batchify readIDs\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_11_get_batches(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_list = 0;
  PyObject *__pyx_v_size = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_batches (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_list,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_list)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_batches", 1, 2, 2, 1); __PYX_ERR(0, 442, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_batches") < 0)) __PYX_ERR(0, 442, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_read_list = values[0];
    __pyx_v_size = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_batches", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 442, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_batches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_10_get_batches(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_list, __pyx_v_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_10_get_batches(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_size) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_batches", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *)__pyx_tp_new_7pyslow5___pyx_scope_struct_2__get_batches(__pyx_ptype_7pyslow5___pyx_scope_struct_2__get_batches, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 442, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_read_list = __pyx_v_read_list;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
  __pyx_cur_scope->__pyx_v_size = __pyx_v_size;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_size);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_size);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_12generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_get_batches, __pyx_n_s_Open__get_batches, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_batches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_12generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_batches", 0);
  __Pyx_TraceCall("_get_batches", __pyx_f[0], 442, 0, __PYX_ERR(0, 442, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 442, __pyx_L1_error)

  /* "pyslow5.pyx":446
 *         batchify readIDs
 *         """
 *         for i in range(0, len(read_list), size):             # <<<<<<<<<<<<<<
 *             yield read_list[i:i+size]
 * 
 */
  __pyx_t_1 = PyObject_Length(__pyx_cur_scope->__pyx_v_read_list); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_size);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_size);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_cur_scope->__pyx_v_size);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_1 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_1 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 446, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_1 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_1); __Pyx_INCREF(__pyx_t_2); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_1); __Pyx_INCREF(__pyx_t_2); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 446, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":447
 *         """
 *         for i in range(0, len(read_list), size):
 *             yield read_list[i:i+size]             # <<<<<<<<<<<<<<
 * 
 *     def _get_read_multi(self, read_list, threads, batchsize, pA, aux):
 */
    __pyx_t_2 = PyNumber_Add(__pyx_cur_scope->__pyx_v_i, __pyx_cur_scope->__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_cur_scope->__pyx_v_read_list, 0, 0, &__pyx_cur_scope->__pyx_v_i, &__pyx_t_2, NULL, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 447, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __Pyx_XGIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_4;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_TraceReturn(__pyx_r, 0);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = 0;
    __Pyx_XGOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 447, __pyx_L1_error)

    /* "pyslow5.pyx":446
 *         batchify readIDs
 *         """
 *         for i in range(0, len(read_list), size):             # <<<<<<<<<<<<<<
 *             yield read_list[i:i+size]
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyslow5.pyx":442
 *         return dic
 * 
 *     def _get_batches(self, read_list, size):             # <<<<<<<<<<<<<<
 *         """
 *         batchify readIDs
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("_get_batches", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_15generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":449
 *             yield read_list[i:i+size]
 * 
 *     def _get_read_multi(self, read_list, threads, batchsize, pA, aux):             # <<<<<<<<<<<<<<
 *         '''
 *         read_list = list of readIDs
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_14_get_read_multi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_13_get_read_multi[] = "\n        read_list = list of readIDs\n        pA = Bool for converting signal to picoamps\n        aux = str 'name'/'all' or list of names of auxiliary fields added to return dictionary\n        returns dic = dictionary of main fields for read_id, with any aux fields added\n        threads = how many threads to use to go fast\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_14_get_read_multi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_list = 0;
  PyObject *__pyx_v_threads = 0;
  PyObject *__pyx_v_batchsize = 0;
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_read_multi (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_list,&__pyx_n_s_threads,&__pyx_n_s_batchsize,&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_list)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_threads)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read_multi", 1, 5, 5, 1); __PYX_ERR(0, 449, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_batchsize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read_multi", 1, 5, 5, 2); __PYX_ERR(0, 449, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read_multi", 1, 5, 5, 3); __PYX_ERR(0, 449, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read_multi", 1, 5, 5, 4); __PYX_ERR(0, 449, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_read_multi") < 0)) __PYX_ERR(0, 449, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_read_list = values[0];
    __pyx_v_threads = values[1];
    __pyx_v_batchsize = values[2];
    __pyx_v_pA = values[3];
    __pyx_v_aux = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_read_multi", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 449, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_read_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_13_get_read_multi(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_list, __pyx_v_threads, __pyx_v_batchsize, __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_13_get_read_multi(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_read_multi", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *)__pyx_tp_new_7pyslow5___pyx_scope_struct_3__get_read_multi(__pyx_ptype_7pyslow5___pyx_scope_struct_3__get_read_multi, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 449, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_read_list = __pyx_v_read_list;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
  __pyx_cur_scope->__pyx_v_threads = __pyx_v_threads;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_threads);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_threads);
  __pyx_cur_scope->__pyx_v_batchsize = __pyx_v_batchsize;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_batchsize);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_batchsize);
  __pyx_cur_scope->__pyx_v_pA = __pyx_v_pA;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
  __pyx_cur_scope->__pyx_v_aux = __pyx_v_aux;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_15generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_get_read_multi, __pyx_n_s_Open__get_read_multi, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_read_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_15generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  Py_ssize_t __pyx_t_13;
  PyObject *(*__pyx_t_14)(PyObject *);
  size_t __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  char const *__pyx_t_18;
  Py_ssize_t __pyx_t_19;
  int __pyx_t_20;
  int __pyx_t_21;
  int __pyx_t_22;
  PyObject *(*__pyx_t_23)(PyObject *);
  PyObject *(*__pyx_t_24)(PyObject *);
  int __pyx_t_25;
  char *__pyx_t_26;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_read_multi", 0);
  __Pyx_TraceCall("_get_read_multi", __pyx_f[0], 449, 0, __PYX_ERR(0, 449, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L42_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 449, __pyx_L1_error)

  /* "pyslow5.pyx":457
 *         threads = how many threads to use to go fast
 *         '''
 *         if not self.index_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             self.logger.debug("Creating/loading index...")
 */
  __pyx_t_1 = ((!(__pyx_cur_scope->__pyx_v_self->index_state != 0)) != 0);
  if (__pyx_t_1) {

    /* "pyslow5.pyx":458
 *         '''
 *         if not self.index_state:
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))             # <<<<<<<<<<<<<<
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FILE_mode, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_self->path, __pyx_cur_scope->__pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_self->path, __pyx_cur_scope->__pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->path);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->path);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_cur_scope->__pyx_v_self->path);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->mode);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->mode);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_cur_scope->__pyx_v_self->mode);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":459
 *         if not self.index_state:
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             self.logger.debug("Creating/loading index...")             # <<<<<<<<<<<<<<
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_Creating_loading_index) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Creating_loading_index);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":460
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)             # <<<<<<<<<<<<<<
 *             if ret != 0:
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 */
    __pyx_cur_scope->__pyx_v_ret = slow5_idx_load(__pyx_cur_scope->__pyx_v_self->s5);

    /* "pyslow5.pyx":461
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:             # <<<<<<<<<<<<<<
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 */
    __pyx_t_1 = ((__pyx_cur_scope->__pyx_v_ret != 0) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":462
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.index_state = True
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_idx_load_return_not_0, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(__pyx_cur_scope->__pyx_v_self->error_codes == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 462, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->error_codes, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_8, __pyx_t_9};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 462, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_8, __pyx_t_9};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 462, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 462, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_7, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_7, __pyx_t_9);
        __pyx_t_8 = 0;
        __pyx_t_9 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 462, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":461
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:             # <<<<<<<<<<<<<<
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 */
      goto __pyx_L5;
    }

    /* "pyslow5.pyx":464
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 *                 self.index_state = True             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("Setting up batching...")
 */
    /*else*/ {
      __pyx_cur_scope->__pyx_v_self->index_state = 1;
    }
    __pyx_L5:;

    /* "pyslow5.pyx":457
 *         threads = how many threads to use to go fast
 *         '''
 *         if not self.index_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             self.logger.debug("Creating/loading index...")
 */
  }

  /* "pyslow5.pyx":466
 *                 self.index_state = True
 * 
 *         self.logger.debug("Setting up batching...")             # <<<<<<<<<<<<<<
 *         self.logger.debug("read_list: {}".format([i for i in read_list]))
 *         num_reads = len(read_list)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_Setting_up_batching) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Setting_up_batching);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":467
 * 
 *         self.logger.debug("Setting up batching...")
 *         self.logger.debug("read_list: {}".format([i for i in read_list]))             # <<<<<<<<<<<<<<
 *         num_reads = len(read_list)
 *         if num_reads > batchsize:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_read_list_2, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  { /* enter inner scope */
    __pyx_t_10 = PyList_New(0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_read_list)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_read_list)) {
      __pyx_t_9 = __pyx_cur_scope->__pyx_v_read_list; __Pyx_INCREF(__pyx_t_9); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_11 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_read_list); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 467, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_12 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 467, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_11); __Pyx_INCREF(__pyx_t_8); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 467, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_9, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 467, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        } else {
          if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_11); __Pyx_INCREF(__pyx_t_8); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 467, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_9, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 467, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        }
      } else {
        __pyx_t_8 = __pyx_t_12(__pyx_t_9);
        if (unlikely(!__pyx_t_8)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 467, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_8genexpr1__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_8genexpr1__pyx_v_i, __pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      __pyx_t_8 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_10, (PyObject*)__pyx_cur_scope->__pyx_8genexpr1__pyx_v_i))) __PYX_ERR(0, 467, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } /* exit inner scope */
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_9, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10);
  __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":468
 *         self.logger.debug("Setting up batching...")
 *         self.logger.debug("read_list: {}".format([i for i in read_list]))
 *         num_reads = len(read_list)             # <<<<<<<<<<<<<<
 *         if num_reads > batchsize:
 *             batches = self._get_batches(read_list, size=batchsize)
 */
  __pyx_t_11 = PyObject_Length(__pyx_cur_scope->__pyx_v_read_list); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 468, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_num_reads = __pyx_t_11;

  /* "pyslow5.pyx":469
 *         self.logger.debug("read_list: {}".format([i for i in read_list]))
 *         num_reads = len(read_list)
 *         if num_reads > batchsize:             # <<<<<<<<<<<<<<
 *             batches = self._get_batches(read_list, size=batchsize)
 *         else:
 */
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_cur_scope->__pyx_v_num_reads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_cur_scope->__pyx_v_batchsize, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_1) {

    /* "pyslow5.pyx":470
 *         num_reads = len(read_list)
 *         if num_reads > batchsize:
 *             batches = self._get_batches(read_list, size=batchsize)             # <<<<<<<<<<<<<<
 *         else:
 *             batches = self._get_batches(read_list, size=num_reads)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_batches); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_cur_scope->__pyx_v_read_list);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_size, __pyx_cur_scope->__pyx_v_batchsize) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_5);
    __pyx_cur_scope->__pyx_v_batches = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "pyslow5.pyx":469
 *         self.logger.debug("read_list: {}".format([i for i in read_list]))
 *         num_reads = len(read_list)
 *         if num_reads > batchsize:             # <<<<<<<<<<<<<<
 *             batches = self._get_batches(read_list, size=batchsize)
 *         else:
 */
    goto __pyx_L8;
  }

  /* "pyslow5.pyx":472
 *             batches = self._get_batches(read_list, size=batchsize)
 *         else:
 *             batches = self._get_batches(read_list, size=num_reads)             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("batch for loop start")
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_batches); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_cur_scope->__pyx_v_read_list);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_cur_scope->__pyx_v_num_reads); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_size, __pyx_t_3) < 0) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_v_batches = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L8:;

  /* "pyslow5.pyx":474
 *             batches = self._get_batches(read_list, size=num_reads)
 * 
 *         self.logger.debug("batch for loop start")             # <<<<<<<<<<<<<<
 *         for batch in chain(batches):
 *             self.logger.debug("Batch: {}".format(",".join([i for i in batch])))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_kp_u_batch_for_loop_start) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_batch_for_loop_start);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":475
 * 
 *         self.logger.debug("batch for loop start")
 *         for batch in chain(batches):             # <<<<<<<<<<<<<<
 *             self.logger.debug("Batch: {}".format(",".join([i for i in batch])))
 *             batch_len = len(batch)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_chain); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_cur_scope->__pyx_v_batches) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_cur_scope->__pyx_v_batches);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_11 = 0;
    __pyx_t_12 = NULL;
  } else {
    __pyx_t_11 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_12 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 475, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_12)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_11); __Pyx_INCREF(__pyx_t_3); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 475, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_11); __Pyx_INCREF(__pyx_t_3); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 475, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_12(__pyx_t_2);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 475, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_batch);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_batch, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyslow5.pyx":476
 *         self.logger.debug("batch for loop start")
 *         for batch in chain(batches):
 *             self.logger.debug("Batch: {}".format(",".join([i for i in batch])))             # <<<<<<<<<<<<<<
 *             batch_len = len(batch)
 *             self.logger.debug("Starting rid assignment")
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Batch, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    { /* enter inner scope */
      __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_batch)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_batch)) {
        __pyx_t_8 = __pyx_cur_scope->__pyx_v_batch; __Pyx_INCREF(__pyx_t_8); __pyx_t_13 = 0;
        __pyx_t_14 = NULL;
      } else {
        __pyx_t_13 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_batch); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 476, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_14 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 476, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_14)) {
          if (likely(PyList_CheckExact(__pyx_t_8))) {
            if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_8)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_13); __Pyx_INCREF(__pyx_t_6); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 476, __pyx_L1_error)
            #else
            __pyx_t_6 = PySequence_ITEM(__pyx_t_8, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            #endif
          } else {
            if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_13); __Pyx_INCREF(__pyx_t_6); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 476, __pyx_L1_error)
            #else
            __pyx_t_6 = PySequence_ITEM(__pyx_t_8, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            #endif
          }
        } else {
          __pyx_t_6 = __pyx_t_14(__pyx_t_8);
          if (unlikely(!__pyx_t_6)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 476, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_8genexpr2__pyx_v_i);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_8genexpr2__pyx_v_i, __pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        __pyx_t_6 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_9, (PyObject*)__pyx_cur_scope->__pyx_8genexpr2__pyx_v_i))) __PYX_ERR(0, 476, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    } /* exit inner scope */
    __pyx_t_8 = PyUnicode_Join(__pyx_kp_u__4, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_5 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":477
 *         for batch in chain(batches):
 *             self.logger.debug("Batch: {}".format(",".join([i for i in batch])))
 *             batch_len = len(batch)             # <<<<<<<<<<<<<<
 *             self.logger.debug("Starting rid assignment")
 *             self.rid = <char **> malloc(sizeof(char*)*batch_len)
 */
    __pyx_t_13 = PyObject_Length(__pyx_cur_scope->__pyx_v_batch); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 477, __pyx_L1_error)
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_13); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_batch_len);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_batch_len, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyslow5.pyx":478
 *             self.logger.debug("Batch: {}".format(",".join([i for i in batch])))
 *             batch_len = len(batch)
 *             self.logger.debug("Starting rid assignment")             # <<<<<<<<<<<<<<
 *             self.rid = <char **> malloc(sizeof(char*)*batch_len)
 *             for i in range(batch_len):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 478, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_kp_u_Starting_rid_assignment) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_u_Starting_rid_assignment);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 478, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":479
 *             batch_len = len(batch)
 *             self.logger.debug("Starting rid assignment")
 *             self.rid = <char **> malloc(sizeof(char*)*batch_len)             # <<<<<<<<<<<<<<
 *             for i in range(batch_len):
 *                 self.logger.debug("readID: {}, bin: {}".format(batch[i], batch[i].encode()))
 */
    __pyx_t_3 = __Pyx_PyInt_FromSize_t((sizeof(char *))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_cur_scope->__pyx_v_batch_len); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_15 = __Pyx_PyInt_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_15 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_cur_scope->__pyx_v_self->rid = ((char **)malloc(__pyx_t_15));

    /* "pyslow5.pyx":480
 *             self.logger.debug("Starting rid assignment")
 *             self.rid = <char **> malloc(sizeof(char*)*batch_len)
 *             for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                 self.logger.debug("readID: {}, bin: {}".format(batch[i], batch[i].encode()))
 *                 self.rid[i] = strdup(batch[i].encode())
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_cur_scope->__pyx_v_batch_len); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
    } else {
      __pyx_t_13 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 480, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 480, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_14)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_4); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 480, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 480, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_4); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 480, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 480, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_14(__pyx_t_3);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 480, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyslow5.pyx":481
 *             self.rid = <char **> malloc(sizeof(char*)*batch_len)
 *             for i in range(batch_len):
 *                 self.logger.debug("readID: {}, bin: {}".format(batch[i], batch[i].encode()))             # <<<<<<<<<<<<<<
 *                 self.rid[i] = strdup(batch[i].encode())
 *             self.logger.debug("rid assignment complete")
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_readID_bin, __pyx_n_s_format); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_batch, __pyx_cur_scope->__pyx_v_i); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_16 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_batch, __pyx_cur_scope->__pyx_v_i); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_n_s_encode); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_17))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_17);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_17, function);
        }
      }
      __pyx_t_6 = (__pyx_t_16) ? __Pyx_PyObject_CallOneArg(__pyx_t_17, __pyx_t_16) : __Pyx_PyObject_CallNoArg(__pyx_t_17);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_17 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_17)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_17);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[3] = {__pyx_t_17, __pyx_t_9, __pyx_t_6};
        __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 481, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[3] = {__pyx_t_17, __pyx_t_9, __pyx_t_6};
        __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 481, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_16 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 481, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        if (__pyx_t_17) {
          __Pyx_GIVEREF(__pyx_t_17); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_17); __pyx_t_17 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_7, __pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_7, __pyx_t_6);
        __pyx_t_9 = 0;
        __pyx_t_6 = 0;
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_16, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 481, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyslow5.pyx":482
 *             for i in range(batch_len):
 *                 self.logger.debug("readID: {}, bin: {}".format(batch[i], batch[i].encode()))
 *                 self.rid[i] = strdup(batch[i].encode())             # <<<<<<<<<<<<<<
 *             self.logger.debug("rid assignment complete")
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_batch, __pyx_cur_scope->__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 482, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_encode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 482, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 482, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_18 = __Pyx_PyObject_AsString(__pyx_t_4); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 482, __pyx_L1_error)
      __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_cur_scope->__pyx_v_i); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 482, __pyx_L1_error)
      (__pyx_cur_scope->__pyx_v_self->rid[__pyx_t_19]) = strdup(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyslow5.pyx":480
 *             self.logger.debug("Starting rid assignment")
 *             self.rid = <char **> malloc(sizeof(char*)*batch_len)
 *             for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                 self.logger.debug("readID: {}, bin: {}".format(batch[i], batch[i].encode()))
 *                 self.rid[i] = strdup(batch[i].encode())
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":483
 *                 self.logger.debug("readID: {}, bin: {}".format(batch[i], batch[i].encode()))
 *                 self.rid[i] = strdup(batch[i].encode())
 *             self.logger.debug("rid assignment complete")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_kp_u_rid_assignment_complete) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_u_rid_assignment_complete);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":486
 * 
 * 
 *             self.logger.debug("slow5_get_batch: num_reads: {}".format(batch_len))             # <<<<<<<<<<<<<<
 *             ret = slow5_get_batch(&self.trec, self.s5, self.rid, batch_len, threads);
 *             self.logger.debug("get_read_multi slow5_get_batch ret: {}".format(ret))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_batch_num_reads, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_10 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_cur_scope->__pyx_v_batch_len) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_cur_scope->__pyx_v_batch_len);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":487
 * 
 *             self.logger.debug("slow5_get_batch: num_reads: {}".format(batch_len))
 *             ret = slow5_get_batch(&self.trec, self.s5, self.rid, batch_len, threads);             # <<<<<<<<<<<<<<
 *             self.logger.debug("get_read_multi slow5_get_batch ret: {}".format(ret))
 *             if ret < 0:
 */
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_cur_scope->__pyx_v_batch_len); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L1_error)
    __pyx_t_20 = __Pyx_PyInt_As_int(__pyx_cur_scope->__pyx_v_threads); if (unlikely((__pyx_t_20 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L1_error)
    __pyx_cur_scope->__pyx_v_ret = slow5_get_batch((&__pyx_cur_scope->__pyx_v_self->trec), __pyx_cur_scope->__pyx_v_self->s5, __pyx_cur_scope->__pyx_v_self->rid, __pyx_t_7, __pyx_t_20);

    /* "pyslow5.pyx":488
 *             self.logger.debug("slow5_get_batch: num_reads: {}".format(batch_len))
 *             ret = slow5_get_batch(&self.trec, self.s5, self.rid, batch_len, threads);
 *             self.logger.debug("get_read_multi slow5_get_batch ret: {}".format(ret))             # <<<<<<<<<<<<<<
 *             if ret < 0:
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_multi_slow5_get_batch_r, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_16 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_16)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_10 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_16, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":489
 *             ret = slow5_get_batch(&self.trec, self.s5, self.rid, batch_len, threads);
 *             self.logger.debug("get_read_multi slow5_get_batch ret: {}".format(ret))
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 *                 break
 */
    __pyx_t_1 = ((__pyx_cur_scope->__pyx_v_ret < 0) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":490
 *             self.logger.debug("get_read_multi slow5_get_batch ret: {}".format(ret))
 *             if ret < 0:
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 *                 break
 *             if ret == 0:
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_error_code, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(__pyx_cur_scope->__pyx_v_self->error_codes == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 490, __pyx_L1_error)
      }
      __pyx_t_16 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->error_codes, __pyx_t_16); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = NULL;
      __pyx_t_20 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_20 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_t_8, __pyx_t_6};
        __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_t_8, __pyx_t_6};
        __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(2+__pyx_t_20); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_16) {
          __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_16); __pyx_t_16 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_20, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_20, __pyx_t_6);
        __pyx_t_8 = 0;
        __pyx_t_6 = 0;
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":491
 *             if ret < 0:
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 *                 break             # <<<<<<<<<<<<<<
 *             if ret == 0:
 *                 self.logger.debug("No more reads: {}".format(ret))
 */
      goto __pyx_L10_break;

      /* "pyslow5.pyx":489
 *             ret = slow5_get_batch(&self.trec, self.s5, self.rid, batch_len, threads);
 *             self.logger.debug("get_read_multi slow5_get_batch ret: {}".format(ret))
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 *                 break
 */
    }

    /* "pyslow5.pyx":492
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 *                 break
 *             if ret == 0:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("No more reads: {}".format(ret))
 *                 break
 */
    __pyx_t_1 = ((__pyx_cur_scope->__pyx_v_ret == 0) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":493
 *                 break
 *             if ret == 0:
 *                 self.logger.debug("No more reads: {}".format(ret))             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 493, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_No_more_reads, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 493, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 493, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_10 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 493, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 493, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":494
 *             if ret == 0:
 *                 self.logger.debug("No more reads: {}".format(ret))
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             for i in range(ret):
 */
      goto __pyx_L10_break;

      /* "pyslow5.pyx":492
 *                 self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 *                 break
 *             if ret == 0:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("No more reads: {}".format(ret))
 *                 break
 */
    }

    /* "pyslow5.pyx":496
 *                 break
 * 
 *             for i in range(ret):             # <<<<<<<<<<<<<<
 *                 python_parse_read_start = time.time()
 *                 self.rec = self.trec[i]
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
    } else {
      __pyx_t_13 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 496, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 496, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_14)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_4); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 496, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 496, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_4); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 496, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 496, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_14(__pyx_t_3);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 496, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyslow5.pyx":497
 * 
 *             for i in range(ret):
 *                 python_parse_read_start = time.time()             # <<<<<<<<<<<<<<
 *                 self.rec = self.trec[i]
 *                 dic = {}
 */
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_python_parse_read_start);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_python_parse_read_start, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyslow5.pyx":498
 *             for i in range(ret):
 *                 python_parse_read_start = time.time()
 *                 self.rec = self.trec[i]             # <<<<<<<<<<<<<<
 *                 dic = {}
 *                 aux_dic = {}
 */
      __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_cur_scope->__pyx_v_i); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 498, __pyx_L1_error)
      __pyx_cur_scope->__pyx_v_self->rec = (__pyx_cur_scope->__pyx_v_self->trec[__pyx_t_19]);

      /* "pyslow5.pyx":499
 *                 python_parse_read_start = time.time()
 *                 self.rec = self.trec[i]
 *                 dic = {}             # <<<<<<<<<<<<<<
 *                 aux_dic = {}
 * 
 */
      __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 499, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_dic);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_dic, ((PyObject*)__pyx_t_4));
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyslow5.pyx":500
 *                 self.rec = self.trec[i]
 *                 dic = {}
 *                 aux_dic = {}             # <<<<<<<<<<<<<<
 * 
 *                 # check for aux fields
 */
      __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 500, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyslow5.pyx":503
 * 
 *                 # check for aux fields
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()
 */
      __pyx_t_1 = (__pyx_cur_scope->__pyx_v_aux != Py_None);
      __pyx_t_21 = (__pyx_t_1 != 0);
      if (__pyx_t_21) {

        /* "pyslow5.pyx":504
 *                 # check for aux fields
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 */
        __pyx_t_1 = (__pyx_cur_scope->__pyx_v_self->aux_names != Py_None)&&(PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_self->aux_names) != 0);
        __pyx_t_22 = ((!__pyx_t_1) != 0);
        if (!__pyx_t_22) {
        } else {
          __pyx_t_21 = __pyx_t_22;
          goto __pyx_L21_bool_binop_done;
        }
        __pyx_t_22 = (__pyx_cur_scope->__pyx_v_self->aux_types != Py_None)&&(PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_self->aux_types) != 0);
        __pyx_t_1 = ((!__pyx_t_22) != 0);
        __pyx_t_21 = __pyx_t_1;
        __pyx_L21_bool_binop_done:;
        if (__pyx_t_21) {

          /* "pyslow5.pyx":505
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()             # <<<<<<<<<<<<<<
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:
 */
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_aux_names); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (!(likely(PyList_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(0, 505, __pyx_L1_error)
          __Pyx_GIVEREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __pyx_cur_scope->__pyx_v_self->aux_names = ((PyObject*)__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":506
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()             # <<<<<<<<<<<<<<
 *                     if type(aux) is str:
 *                         # special type 'all'
 */
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_aux_types); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 506, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 506, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (!(likely(PyList_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(0, 506, __pyx_L1_error)
          __Pyx_GIVEREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __pyx_cur_scope->__pyx_v_self->aux_types = ((PyObject*)__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":504
 *                 # check for aux fields
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 */
        }

        /* "pyslow5.pyx":507
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:             # <<<<<<<<<<<<<<
 *                         # special type 'all'
 *                         if aux == "all":
 */
        __pyx_t_21 = (((PyObject *)Py_TYPE(__pyx_cur_scope->__pyx_v_aux)) == ((PyObject *)(&PyUnicode_Type)));
        __pyx_t_1 = (__pyx_t_21 != 0);
        if (__pyx_t_1) {

          /* "pyslow5.pyx":509
 *                     if type(aux) is str:
 *                         # special type 'all'
 *                         if aux == "all":             # <<<<<<<<<<<<<<
 *                             aux_dic = self._get_read_aux(self.aux_names, self.aux_types)
 *                         else:
 */
          __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_aux, __pyx_n_u_all, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 509, __pyx_L1_error)
          if (__pyx_t_1) {

            /* "pyslow5.pyx":510
 *                         # special type 'all'
 *                         if aux == "all":
 *                             aux_dic = self._get_read_aux(self.aux_names, self.aux_types)             # <<<<<<<<<<<<<<
 *                         else:
 *                             found_single_aux = False
 */
            __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_read_aux); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 510, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_10 = NULL;
            __pyx_t_20 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
              __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
              if (likely(__pyx_t_10)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                __Pyx_INCREF(__pyx_t_10);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_5, function);
                __pyx_t_20 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_self->aux_names, __pyx_cur_scope->__pyx_v_self->aux_types};
              __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 510, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_4);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_self->aux_names, __pyx_cur_scope->__pyx_v_self->aux_types};
              __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 510, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_4);
            } else
            #endif
            {
              __pyx_t_9 = PyTuple_New(2+__pyx_t_20); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 510, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_9);
              if (__pyx_t_10) {
                __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_10); __pyx_t_10 = NULL;
              }
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
              __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
              PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_20, __pyx_cur_scope->__pyx_v_self->aux_names);
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
              __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
              PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_20, __pyx_cur_scope->__pyx_v_self->aux_types);
              __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 510, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            }
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
            __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_4);
            __pyx_t_4 = 0;

            /* "pyslow5.pyx":509
 *                     if type(aux) is str:
 *                         # special type 'all'
 *                         if aux == "all":             # <<<<<<<<<<<<<<
 *                             aux_dic = self._get_read_aux(self.aux_names, self.aux_types)
 *                         else:
 */
            goto __pyx_L24;
          }

          /* "pyslow5.pyx":512
 *                             aux_dic = self._get_read_aux(self.aux_names, self.aux_types)
 *                         else:
 *                             found_single_aux = False             # <<<<<<<<<<<<<<
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:
 */
          /*else*/ {
            __pyx_cur_scope->__pyx_v_found_single_aux = 0;

            /* "pyslow5.pyx":513
 *                         else:
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                                 if n == aux:
 *                                     found_single_aux = True
 */
            __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 513, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
            PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_cur_scope->__pyx_v_self->aux_names);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
            PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_cur_scope->__pyx_v_self->aux_types);
            __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 513, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
              __pyx_t_4 = __pyx_t_5; __Pyx_INCREF(__pyx_t_4); __pyx_t_19 = 0;
              __pyx_t_23 = NULL;
            } else {
              __pyx_t_19 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 513, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_23 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 513, __pyx_L1_error)
            }
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            for (;;) {
              if (likely(!__pyx_t_23)) {
                if (likely(PyList_CheckExact(__pyx_t_4))) {
                  if (__pyx_t_19 >= PyList_GET_SIZE(__pyx_t_4)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_5 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_19); __Pyx_INCREF(__pyx_t_5); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 513, __pyx_L1_error)
                  #else
                  __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 513, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  #endif
                } else {
                  if (__pyx_t_19 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_19); __Pyx_INCREF(__pyx_t_5); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 513, __pyx_L1_error)
                  #else
                  __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 513, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  #endif
                }
              } else {
                __pyx_t_5 = __pyx_t_23(__pyx_t_4);
                if (unlikely(!__pyx_t_5)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(0, 513, __pyx_L1_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_5);
              }
              if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
                PyObject* sequence = __pyx_t_5;
                Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
                if (unlikely(size != 2)) {
                  if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                  else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                  __PYX_ERR(0, 513, __pyx_L1_error)
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                if (likely(PyTuple_CheckExact(sequence))) {
                  __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
                  __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
                } else {
                  __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
                  __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
                }
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(__pyx_t_10);
                #else
                __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 513, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_9);
                __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 513, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                #endif
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              } else {
                Py_ssize_t index = -1;
                __pyx_t_6 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 513, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_6);
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                __pyx_t_24 = Py_TYPE(__pyx_t_6)->tp_iternext;
                index = 0; __pyx_t_9 = __pyx_t_24(__pyx_t_6); if (unlikely(!__pyx_t_9)) goto __pyx_L27_unpacking_failed;
                __Pyx_GOTREF(__pyx_t_9);
                index = 1; __pyx_t_10 = __pyx_t_24(__pyx_t_6); if (unlikely(!__pyx_t_10)) goto __pyx_L27_unpacking_failed;
                __Pyx_GOTREF(__pyx_t_10);
                if (__Pyx_IternextUnpackEndCheck(__pyx_t_24(__pyx_t_6), 2) < 0) __PYX_ERR(0, 513, __pyx_L1_error)
                __pyx_t_24 = NULL;
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                goto __pyx_L28_unpacking_done;
                __pyx_L27_unpacking_failed:;
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                __pyx_t_24 = NULL;
                if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
                __PYX_ERR(0, 513, __pyx_L1_error)
                __pyx_L28_unpacking_done:;
              }
              __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n);
              __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n, __pyx_t_9);
              __Pyx_GIVEREF(__pyx_t_9);
              __pyx_t_9 = 0;
              __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
              __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_10);
              __Pyx_GIVEREF(__pyx_t_10);
              __pyx_t_10 = 0;

              /* "pyslow5.pyx":514
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:             # <<<<<<<<<<<<<<
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_read_aux([n], [t])
 */
              __pyx_t_5 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_n, __pyx_cur_scope->__pyx_v_aux, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 514, __pyx_L1_error)
              __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 514, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              if (__pyx_t_1) {

                /* "pyslow5.pyx":515
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:
 *                                     found_single_aux = True             # <<<<<<<<<<<<<<
 *                                     aux_dic = self._get_read_aux([n], [t])
 *                                     break
 */
                __pyx_cur_scope->__pyx_v_found_single_aux = 1;

                /* "pyslow5.pyx":516
 *                                 if n == aux:
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_read_aux([n], [t])             # <<<<<<<<<<<<<<
 *                                     break
 *                             if not found_single_aux:
 */
                __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_read_aux); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 516, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                __pyx_t_9 = PyList_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 516, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_9);
                __Pyx_INCREF(__pyx_cur_scope->__pyx_v_n);
                __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_n);
                PyList_SET_ITEM(__pyx_t_9, 0, __pyx_cur_scope->__pyx_v_n);
                __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 516, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_6);
                __Pyx_INCREF(__pyx_cur_scope->__pyx_v_t);
                __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_t);
                PyList_SET_ITEM(__pyx_t_6, 0, __pyx_cur_scope->__pyx_v_t);
                __pyx_t_8 = NULL;
                __pyx_t_20 = 0;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
                  __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_10);
                  if (likely(__pyx_t_8)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                    __Pyx_INCREF(__pyx_t_8);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_10, function);
                    __pyx_t_20 = 1;
                  }
                }
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_10)) {
                  PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_9, __pyx_t_6};
                  __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 516, __pyx_L1_error)
                  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                  __Pyx_GOTREF(__pyx_t_5);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
                  PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_9, __pyx_t_6};
                  __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 516, __pyx_L1_error)
                  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                  __Pyx_GOTREF(__pyx_t_5);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                } else
                #endif
                {
                  __pyx_t_16 = PyTuple_New(2+__pyx_t_20); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 516, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_16);
                  if (__pyx_t_8) {
                    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_8); __pyx_t_8 = NULL;
                  }
                  __Pyx_GIVEREF(__pyx_t_9);
                  PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_20, __pyx_t_9);
                  __Pyx_GIVEREF(__pyx_t_6);
                  PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_20, __pyx_t_6);
                  __pyx_t_9 = 0;
                  __pyx_t_6 = 0;
                  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_16, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 516, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                }
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
                __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_5);
                __Pyx_GIVEREF(__pyx_t_5);
                __pyx_t_5 = 0;

                /* "pyslow5.pyx":517
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_read_aux([n], [t])
 *                                     break             # <<<<<<<<<<<<<<
 *                             if not found_single_aux:
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))
 */
                goto __pyx_L26_break;

                /* "pyslow5.pyx":514
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:             # <<<<<<<<<<<<<<
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_read_aux([n], [t])
 */
              }

              /* "pyslow5.pyx":513
 *                         else:
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                                 if n == aux:
 *                                     found_single_aux = True
 */
            }
            __pyx_L26_break:;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyslow5.pyx":518
 *                                     aux_dic = self._get_read_aux([n], [t])
 *                                     break
 *                             if not found_single_aux:             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 */
            __pyx_t_1 = ((!(__pyx_cur_scope->__pyx_v_found_single_aux != 0)) != 0);
            if (__pyx_t_1) {

              /* "pyslow5.pyx":519
 *                                     break
 *                             if not found_single_aux:
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))             # <<<<<<<<<<<<<<
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:
 */
              __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 519, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_unknown_aux_name, __pyx_n_s_format); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 519, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_16);
              __pyx_t_6 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_16))) {
                __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_16);
                if (likely(__pyx_t_6)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
                  __Pyx_INCREF(__pyx_t_6);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_16, function);
                }
              }
              __pyx_t_10 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_16, __pyx_t_6, __pyx_cur_scope->__pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_cur_scope->__pyx_v_aux);
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 519, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
              __pyx_t_16 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
                __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_5);
                if (likely(__pyx_t_16)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                  __Pyx_INCREF(__pyx_t_16);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_5, function);
                }
              }
              __pyx_t_4 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_16, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10);
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 519, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "pyslow5.pyx":520
 *                             if not found_single_aux:
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})             # <<<<<<<<<<<<<<
 *                     elif type(aux) is list:
 *                         n_list = []
 */
              __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 520, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 520, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (PyDict_SetItem(__pyx_t_10, __pyx_cur_scope->__pyx_v_aux, Py_None) < 0) __PYX_ERR(0, 520, __pyx_L1_error)
              __pyx_t_16 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
                __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_5);
                if (likely(__pyx_t_16)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                  __Pyx_INCREF(__pyx_t_16);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_5, function);
                }
              }
              __pyx_t_4 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_16, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_10);
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 520, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "pyslow5.pyx":518
 *                                     aux_dic = self._get_read_aux([n], [t])
 *                                     break
 *                             if not found_single_aux:             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 */
            }
          }
          __pyx_L24:;

          /* "pyslow5.pyx":507
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:             # <<<<<<<<<<<<<<
 *                         # special type 'all'
 *                         if aux == "all":
 */
          goto __pyx_L23;
        }

        /* "pyslow5.pyx":521
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                         n_list = []
 *                         t_list = []
 */
        __pyx_t_1 = (((PyObject *)Py_TYPE(__pyx_cur_scope->__pyx_v_aux)) == ((PyObject *)(&PyList_Type)));
        __pyx_t_21 = (__pyx_t_1 != 0);
        if (__pyx_t_21) {

          /* "pyslow5.pyx":522
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:
 *                         n_list = []             # <<<<<<<<<<<<<<
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):
 */
          __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 522, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n_list);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n_list, ((PyObject*)__pyx_t_4));
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":523
 *                     elif type(aux) is list:
 *                         n_list = []
 *                         t_list = []             # <<<<<<<<<<<<<<
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:
 */
          __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 523, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t_list);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t_list, ((PyObject*)__pyx_t_4));
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":524
 *                         n_list = []
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                             if n in aux:
 *                                 n_list.append(n)
 */
          __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 524, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_cur_scope->__pyx_v_self->aux_types);
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 524, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
            __pyx_t_4 = __pyx_t_5; __Pyx_INCREF(__pyx_t_4); __pyx_t_19 = 0;
            __pyx_t_23 = NULL;
          } else {
            __pyx_t_19 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 524, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_23 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 524, __pyx_L1_error)
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          for (;;) {
            if (likely(!__pyx_t_23)) {
              if (likely(PyList_CheckExact(__pyx_t_4))) {
                if (__pyx_t_19 >= PyList_GET_SIZE(__pyx_t_4)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_5 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_19); __Pyx_INCREF(__pyx_t_5); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 524, __pyx_L1_error)
                #else
                __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 524, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_5);
                #endif
              } else {
                if (__pyx_t_19 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_19); __Pyx_INCREF(__pyx_t_5); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 524, __pyx_L1_error)
                #else
                __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 524, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_5);
                #endif
              }
            } else {
              __pyx_t_5 = __pyx_t_23(__pyx_t_4);
              if (unlikely(!__pyx_t_5)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 524, __pyx_L1_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_5);
            }
            if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
              PyObject* sequence = __pyx_t_5;
              Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(0, 524, __pyx_L1_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              if (likely(PyTuple_CheckExact(sequence))) {
                __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
                __pyx_t_16 = PyTuple_GET_ITEM(sequence, 1); 
              } else {
                __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
                __pyx_t_16 = PyList_GET_ITEM(sequence, 1); 
              }
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_16);
              #else
              __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 524, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_16 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 524, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_16);
              #endif
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            } else {
              Py_ssize_t index = -1;
              __pyx_t_6 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 524, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __pyx_t_24 = Py_TYPE(__pyx_t_6)->tp_iternext;
              index = 0; __pyx_t_10 = __pyx_t_24(__pyx_t_6); if (unlikely(!__pyx_t_10)) goto __pyx_L33_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_10);
              index = 1; __pyx_t_16 = __pyx_t_24(__pyx_t_6); if (unlikely(!__pyx_t_16)) goto __pyx_L33_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_16);
              if (__Pyx_IternextUnpackEndCheck(__pyx_t_24(__pyx_t_6), 2) < 0) __PYX_ERR(0, 524, __pyx_L1_error)
              __pyx_t_24 = NULL;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              goto __pyx_L34_unpacking_done;
              __pyx_L33_unpacking_failed:;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              __pyx_t_24 = NULL;
              if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
              __PYX_ERR(0, 524, __pyx_L1_error)
              __pyx_L34_unpacking_done:;
            }
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n, __pyx_t_10);
            __Pyx_GIVEREF(__pyx_t_10);
            __pyx_t_10 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_16);
            __Pyx_GIVEREF(__pyx_t_16);
            __pyx_t_16 = 0;

            /* "pyslow5.pyx":525
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:             # <<<<<<<<<<<<<<
 *                                 n_list.append(n)
 *                                 t_list.append(t)
 */
            __pyx_t_21 = (__Pyx_PySequence_ContainsTF(__pyx_cur_scope->__pyx_v_n, __pyx_cur_scope->__pyx_v_aux, Py_EQ)); if (unlikely(__pyx_t_21 < 0)) __PYX_ERR(0, 525, __pyx_L1_error)
            __pyx_t_1 = (__pyx_t_21 != 0);
            if (__pyx_t_1) {

              /* "pyslow5.pyx":526
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:
 *                                 n_list.append(n)             # <<<<<<<<<<<<<<
 *                                 t_list.append(t)
 * 
 */
              __pyx_t_25 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_n); if (unlikely(__pyx_t_25 == ((int)-1))) __PYX_ERR(0, 526, __pyx_L1_error)

              /* "pyslow5.pyx":527
 *                             if n in aux:
 *                                 n_list.append(n)
 *                                 t_list.append(t)             # <<<<<<<<<<<<<<
 * 
 *                         aux_dic = self._get_read_aux(n_list, t_list)
 */
              __pyx_t_25 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_t_list, __pyx_cur_scope->__pyx_v_t); if (unlikely(__pyx_t_25 == ((int)-1))) __PYX_ERR(0, 527, __pyx_L1_error)

              /* "pyslow5.pyx":525
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:             # <<<<<<<<<<<<<<
 *                                 n_list.append(n)
 *                                 t_list.append(t)
 */
            }

            /* "pyslow5.pyx":524
 *                         n_list = []
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                             if n in aux:
 *                                 n_list.append(n)
 */
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "pyslow5.pyx":529
 *                                 t_list.append(t)
 * 
 *                         aux_dic = self._get_read_aux(n_list, t_list)             # <<<<<<<<<<<<<<
 *                         # Check for items given that did not exist
 *                         n_set = set(n_list)
 */
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_read_aux); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 529, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_16 = NULL;
          __pyx_t_20 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_16)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_16);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
              __pyx_t_20 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_t_list};
            __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 529, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_GOTREF(__pyx_t_4);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_t_list};
            __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 529, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_GOTREF(__pyx_t_4);
          } else
          #endif
          {
            __pyx_t_10 = PyTuple_New(2+__pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 529, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            if (__pyx_t_16) {
              __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_16); __pyx_t_16 = NULL;
            }
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_n_list);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_n_list);
            PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_20, __pyx_cur_scope->__pyx_v_n_list);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_t_list);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_t_list);
            PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_20, __pyx_cur_scope->__pyx_v_t_list);
            __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 529, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
          __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":531
 *                         aux_dic = self._get_read_aux(n_list, t_list)
 *                         # Check for items given that did not exist
 *                         n_set = set(n_list)             # <<<<<<<<<<<<<<
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:
 */
          __pyx_t_4 = PySet_New(__pyx_cur_scope->__pyx_v_n_list); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 531, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n_set);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n_set, ((PyObject*)__pyx_t_4));
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":532
 *                         # Check for items given that did not exist
 *                         n_set = set(n_list)
 *                         aux_set = set(aux)             # <<<<<<<<<<<<<<
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):
 */
          __pyx_t_4 = PySet_New(__pyx_cur_scope->__pyx_v_aux); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 532, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_aux_set);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_aux_set, ((PyObject*)__pyx_t_4));
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;

          /* "pyslow5.pyx":533
 *                         n_set = set(n_list)
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("get_read unknown aux name: {}".format(i))
 */
          __pyx_t_4 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_cur_scope->__pyx_v_aux_set, __pyx_cur_scope->__pyx_v_n_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 533, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_19 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_19 == ((Py_ssize_t)-1))) __PYX_ERR(0, 533, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_1 = ((__pyx_t_19 > 0) != 0);
          if (__pyx_t_1) {

            /* "pyslow5.pyx":534
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("get_read unknown aux name: {}".format(i))
 *                                 aux_dic.update({i: None})
 */
            __pyx_t_4 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_cur_scope->__pyx_v_aux_set, __pyx_cur_scope->__pyx_v_n_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 534, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
              __pyx_t_5 = __pyx_t_4; __Pyx_INCREF(__pyx_t_5); __pyx_t_19 = 0;
              __pyx_t_23 = NULL;
            } else {
              __pyx_t_19 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 534, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __pyx_t_23 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 534, __pyx_L1_error)
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            for (;;) {
              if (likely(!__pyx_t_23)) {
                if (likely(PyList_CheckExact(__pyx_t_5))) {
                  if (__pyx_t_19 >= PyList_GET_SIZE(__pyx_t_5)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_4 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_19); __Pyx_INCREF(__pyx_t_4); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 534, __pyx_L1_error)
                  #else
                  __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 534, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  #endif
                } else {
                  if (__pyx_t_19 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_19); __Pyx_INCREF(__pyx_t_4); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 534, __pyx_L1_error)
                  #else
                  __pyx_t_4 = PySequence_ITEM(__pyx_t_5, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 534, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  #endif
                }
              } else {
                __pyx_t_4 = __pyx_t_23(__pyx_t_5);
                if (unlikely(!__pyx_t_4)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(0, 534, __pyx_L1_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_4);
              }
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_i);
              __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_4);
              __Pyx_GIVEREF(__pyx_t_4);
              __pyx_t_4 = 0;

              /* "pyslow5.pyx":535
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("get_read unknown aux name: {}".format(i))             # <<<<<<<<<<<<<<
 *                                 aux_dic.update({i: None})
 * 
 */
              __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 535, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_unknown_aux_name, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 535, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_6);
              __pyx_t_9 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
                __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
                if (likely(__pyx_t_9)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                  __Pyx_INCREF(__pyx_t_9);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_6, function);
                }
              }
              __pyx_t_16 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_9, __pyx_cur_scope->__pyx_v_i) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_cur_scope->__pyx_v_i);
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 535, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_16);
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              __pyx_t_6 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
                __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_10);
                if (likely(__pyx_t_6)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                  __Pyx_INCREF(__pyx_t_6);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_10, function);
                }
              }
              __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_6, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_16);
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 535, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "pyslow5.pyx":536
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("get_read unknown aux name: {}".format(i))
 *                                 aux_dic.update({i: None})             # <<<<<<<<<<<<<<
 * 
 *                     else:
 */
              __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 536, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_16 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 536, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_16);
              if (PyDict_SetItem(__pyx_t_16, __pyx_cur_scope->__pyx_v_i, Py_None) < 0) __PYX_ERR(0, 536, __pyx_L1_error)
              __pyx_t_6 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
                __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_10);
                if (likely(__pyx_t_6)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                  __Pyx_INCREF(__pyx_t_6);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_10, function);
                }
              }
              __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_6, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_16);
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 536, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "pyslow5.pyx":534
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("get_read unknown aux name: {}".format(i))
 *                                 aux_dic.update({i: None})
 */
            }
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "pyslow5.pyx":533
 *                         n_set = set(n_list)
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("get_read unknown aux name: {}".format(i))
 */
          }

          /* "pyslow5.pyx":521
 *                                 self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                         n_list = []
 *                         t_list = []
 */
          goto __pyx_L23;
        }

        /* "pyslow5.pyx":539
 * 
 *                     else:
 *                         self.logger.debug("get_read aux type unknown, accepts str or list: {}".format(aux))             # <<<<<<<<<<<<<<
 * 
 *                 # get read data
 */
        /*else*/ {
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 539, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_aux_type_unknown_accept, __pyx_n_s_format); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 539, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_6 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_16))) {
            __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_16);
            if (likely(__pyx_t_6)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
              __Pyx_INCREF(__pyx_t_6);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_16, function);
            }
          }
          __pyx_t_10 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_16, __pyx_t_6, __pyx_cur_scope->__pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_cur_scope->__pyx_v_aux);
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          __pyx_t_16 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_16)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_16);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
            }
          }
          __pyx_t_5 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_16, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 539, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __pyx_L23:;

        /* "pyslow5.pyx":503
 * 
 *                 # check for aux fields
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()
 */
      }

      /* "pyslow5.pyx":542
 * 
 *                 # get read data
 *                 if type(self.rec.read_id) is bytes:             # <<<<<<<<<<<<<<
 *                     dic['read_id'] = self.rec.read_id.decode()
 *                 else:
 */
      __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->rec->read_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = (((PyObject *)Py_TYPE(__pyx_t_5)) == ((PyObject *)(&PyBytes_Type)));
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_21 = (__pyx_t_1 != 0);
      if (__pyx_t_21) {

        /* "pyslow5.pyx":543
 *                 # get read data
 *                 if type(self.rec.read_id) is bytes:
 *                     dic['read_id'] = self.rec.read_id.decode()             # <<<<<<<<<<<<<<
 *                 else:
 *                     dic['read_id'] = self.rec.read_id
 */
        __pyx_t_26 = __pyx_cur_scope->__pyx_v_self->rec->read_id;
        __pyx_t_5 = __Pyx_decode_c_string(__pyx_t_26, 0, strlen(__pyx_t_26), NULL, NULL, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 543, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_read_id, __pyx_t_5) < 0)) __PYX_ERR(0, 543, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "pyslow5.pyx":542
 * 
 *                 # get read data
 *                 if type(self.rec.read_id) is bytes:             # <<<<<<<<<<<<<<
 *                     dic['read_id'] = self.rec.read_id.decode()
 *                 else:
 */
        goto __pyx_L39;
      }

      /* "pyslow5.pyx":545
 *                     dic['read_id'] = self.rec.read_id.decode()
 *                 else:
 *                     dic['read_id'] = self.rec.read_id             # <<<<<<<<<<<<<<
 *                 dic['read_group'] = self.rec.read_group
 *                 dic['digitisation'] = self.rec.digitisation
 */
      /*else*/ {
        __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->rec->read_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_read_id, __pyx_t_5) < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __pyx_L39:;

      /* "pyslow5.pyx":546
 *                 else:
 *                     dic['read_id'] = self.rec.read_id
 *                 dic['read_group'] = self.rec.read_group             # <<<<<<<<<<<<<<
 *                 dic['digitisation'] = self.rec.digitisation
 *                 dic['offset'] = self.rec.offset
 */
      __pyx_t_5 = __Pyx_PyInt_From_uint32_t(__pyx_cur_scope->__pyx_v_self->rec->read_group); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_read_group, __pyx_t_5) < 0)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":547
 *                     dic['read_id'] = self.rec.read_id
 *                 dic['read_group'] = self.rec.read_group
 *                 dic['digitisation'] = self.rec.digitisation             # <<<<<<<<<<<<<<
 *                 dic['offset'] = self.rec.offset
 *                 dic['range'] = self.rec.range
 */
      __pyx_t_5 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->rec->digitisation); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_digitisation, __pyx_t_5) < 0)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":548
 *                 dic['read_group'] = self.rec.read_group
 *                 dic['digitisation'] = self.rec.digitisation
 *                 dic['offset'] = self.rec.offset             # <<<<<<<<<<<<<<
 *                 dic['range'] = self.rec.range
 *                 dic['sampling_rate'] = self.rec.sampling_rate
 */
      __pyx_t_5 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->rec->offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_offset, __pyx_t_5) < 0)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":549
 *                 dic['digitisation'] = self.rec.digitisation
 *                 dic['offset'] = self.rec.offset
 *                 dic['range'] = self.rec.range             # <<<<<<<<<<<<<<
 *                 dic['sampling_rate'] = self.rec.sampling_rate
 *                 dic['len_raw_signal'] = self.rec.len_raw_signal
 */
      __pyx_t_5 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->rec->range); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_range, __pyx_t_5) < 0)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":550
 *                 dic['offset'] = self.rec.offset
 *                 dic['range'] = self.rec.range
 *                 dic['sampling_rate'] = self.rec.sampling_rate             # <<<<<<<<<<<<<<
 *                 dic['len_raw_signal'] = self.rec.len_raw_signal
 *                 # This could be handled by numpy.fromiter() or similar
 */
      __pyx_t_5 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->rec->sampling_rate); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_sampling_rate, __pyx_t_5) < 0)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":551
 *                 dic['range'] = self.rec.range
 *                 dic['sampling_rate'] = self.rec.sampling_rate
 *                 dic['len_raw_signal'] = self.rec.len_raw_signal             # <<<<<<<<<<<<<<
 *                 # This could be handled by numpy.fromiter() or similar
 *                 # Probably MUCH faster
 */
      __pyx_t_5 = __Pyx_PyInt_From_uint64_t(__pyx_cur_scope->__pyx_v_self->rec->len_raw_signal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_len_raw_signal, __pyx_t_5) < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":558
 *                 # dic['signal'] = [self.rec.raw_signal[i] for i in range(self.rec.len_raw_signal)]
 *                 # cdef np.npy_intp shape_get[1]
 *                 self.shape_get[0] = <np.npy_intp> self.rec.len_raw_signal             # <<<<<<<<<<<<<<
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,
 *                             np.NPY_INT16, <void *> self.rec.raw_signal))
 */
      (__pyx_cur_scope->__pyx_v_self->shape_get[0]) = ((npy_intp)__pyx_cur_scope->__pyx_v_self->rec->len_raw_signal);

      /* "pyslow5.pyx":559
 *                 # cdef np.npy_intp shape_get[1]
 *                 self.shape_get[0] = <np.npy_intp> self.rec.len_raw_signal
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,             # <<<<<<<<<<<<<<
 *                             np.NPY_INT16, <void *> self.rec.raw_signal))
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_copy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyslow5.pyx":560
 *                 self.shape_get[0] = <np.npy_intp> self.rec.len_raw_signal
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,
 *                             np.NPY_INT16, <void *> self.rec.raw_signal))             # <<<<<<<<<<<<<<
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *                 dic['signal'] = signal
 */
      __pyx_t_4 = PyArray_SimpleNewFromData(1, __pyx_cur_scope->__pyx_v_self->shape_get, NPY_INT16, ((void *)__pyx_cur_scope->__pyx_v_self->rec->raw_signal)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_16 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      __pyx_t_5 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_16, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_signal);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_signal, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_5);
      __pyx_t_5 = 0;

      /* "pyslow5.pyx":561
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_get,
 *                             np.NPY_INT16, <void *> self.rec.raw_signal))
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)             # <<<<<<<<<<<<<<
 *                 dic['signal'] = signal
 * 
 */
      if (!(likely(((__pyx_cur_scope->__pyx_v_signal) == Py_None) || likely(__Pyx_TypeTest(__pyx_cur_scope->__pyx_v_signal, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 561, __pyx_L1_error)
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_signal, __pyx_n_s_flags); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 561, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_num); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 561, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyInt_From_int(NPY_OWNDATA); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 561, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyNumber_Or(__pyx_t_10, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 561, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_20 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_20 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 561, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      PyArray_UpdateFlags(((PyArrayObject *)__pyx_cur_scope->__pyx_v_signal), __pyx_t_20);

      /* "pyslow5.pyx":562
 *                             np.NPY_INT16, <void *> self.rec.raw_signal))
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *                 dic['signal'] = signal             # <<<<<<<<<<<<<<
 * 
 *                 # if pA=True, convert signal to pA
 */
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_signal, __pyx_cur_scope->__pyx_v_signal) < 0)) __PYX_ERR(0, 562, __pyx_L1_error)

      /* "pyslow5.pyx":565
 * 
 *                 # if pA=True, convert signal to pA
 *                 if pA:             # <<<<<<<<<<<<<<
 *                     dic['signal'] = self._convert_to_pA(dic)
 *                 # if aux data, add to main dic
 */
      __pyx_t_21 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_pA); if (unlikely(__pyx_t_21 < 0)) __PYX_ERR(0, 565, __pyx_L1_error)
      if (__pyx_t_21) {

        /* "pyslow5.pyx":566
 *                 # if pA=True, convert signal to pA
 *                 if pA:
 *                     dic['signal'] = self._convert_to_pA(dic)             # <<<<<<<<<<<<<<
 *                 # if aux data, add to main dic
 *                 if aux_dic:
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_convert_to_pA); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 566, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_10, __pyx_cur_scope->__pyx_v_dic) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_cur_scope->__pyx_v_dic);
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 566, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_dic, __pyx_n_u_signal, __pyx_t_4) < 0)) __PYX_ERR(0, 566, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pyslow5.pyx":565
 * 
 *                 # if pA=True, convert signal to pA
 *                 if pA:             # <<<<<<<<<<<<<<
 *                     dic['signal'] = self._convert_to_pA(dic)
 *                 # if aux data, add to main dic
 */
      }

      /* "pyslow5.pyx":568
 *                     dic['signal'] = self._convert_to_pA(dic)
 *                 # if aux data, add to main dic
 *                 if aux_dic:             # <<<<<<<<<<<<<<
 *                     dic.update(aux_dic)
 *                 yield dic
 */
      __pyx_t_21 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_aux_dic); if (unlikely(__pyx_t_21 < 0)) __PYX_ERR(0, 568, __pyx_L1_error)
      if (__pyx_t_21) {

        /* "pyslow5.pyx":569
 *                 # if aux data, add to main dic
 *                 if aux_dic:
 *                     dic.update(aux_dic)             # <<<<<<<<<<<<<<
 *                 yield dic
 * 
 */
        __pyx_t_4 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_cur_scope->__pyx_v_dic, __pyx_cur_scope->__pyx_v_aux_dic); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 569, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pyslow5.pyx":568
 *                     dic['signal'] = self._convert_to_pA(dic)
 *                 # if aux data, add to main dic
 *                 if aux_dic:             # <<<<<<<<<<<<<<
 *                     dic.update(aux_dic)
 *                 yield dic
 */
      }

      /* "pyslow5.pyx":570
 *                 if aux_dic:
 *                     dic.update(aux_dic)
 *                 yield dic             # <<<<<<<<<<<<<<
 * 
 *             slow5_free_batch(&self.trec, ret)
 */
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_dic);
      __pyx_r = __pyx_cur_scope->__pyx_v_dic;
      __Pyx_XGIVEREF(__pyx_t_2);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
      __Pyx_XGIVEREF(__pyx_t_3);
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_11;
      __pyx_cur_scope->__pyx_t_3 = __pyx_t_12;
      __pyx_cur_scope->__pyx_t_4 = __pyx_t_13;
      __pyx_cur_scope->__pyx_t_5 = __pyx_t_14;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_TraceReturn(__pyx_r, 0);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L42_resume_from_yield:;
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
      __pyx_cur_scope->__pyx_t_1 = 0;
      __Pyx_XGOTREF(__pyx_t_3);
      __pyx_t_11 = __pyx_cur_scope->__pyx_t_2;
      __pyx_t_12 = __pyx_cur_scope->__pyx_t_3;
      __pyx_t_13 = __pyx_cur_scope->__pyx_t_4;
      __pyx_t_14 = __pyx_cur_scope->__pyx_t_5;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 570, __pyx_L1_error)

      /* "pyslow5.pyx":496
 *                 break
 * 
 *             for i in range(ret):             # <<<<<<<<<<<<<<
 *                 python_parse_read_start = time.time()
 *                 self.rec = self.trec[i]
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":572
 *                 yield dic
 * 
 *             slow5_free_batch(&self.trec, ret)             # <<<<<<<<<<<<<<
 *             for i in range(batch_len):
 *                 free(self.rid[i])
 */
    slow5_free_batch((&__pyx_cur_scope->__pyx_v_self->trec), __pyx_cur_scope->__pyx_v_ret);

    /* "pyslow5.pyx":573
 * 
 *             slow5_free_batch(&self.trec, ret)
 *             for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                 free(self.rid[i])
 *             free(self.rid)
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_cur_scope->__pyx_v_batch_len); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
    } else {
      __pyx_t_13 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_14 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 573, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_14)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_13); __Pyx_INCREF(__pyx_t_3); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 573, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_13); __Pyx_INCREF(__pyx_t_3); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 573, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_14(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 573, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":574
 *             slow5_free_batch(&self.trec, ret)
 *             for i in range(batch_len):
 *                 free(self.rid[i])             # <<<<<<<<<<<<<<
 *             free(self.rid)
 *         self.rec = NULL
 */
      __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_cur_scope->__pyx_v_i); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 574, __pyx_L1_error)
      free((__pyx_cur_scope->__pyx_v_self->rid[__pyx_t_19]));

      /* "pyslow5.pyx":573
 * 
 *             slow5_free_batch(&self.trec, ret)
 *             for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                 free(self.rid[i])
 *             free(self.rid)
 */
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyslow5.pyx":575
 *             for i in range(batch_len):
 *                 free(self.rid[i])
 *             free(self.rid)             # <<<<<<<<<<<<<<
 *         self.rec = NULL
 *         self.logger.debug("seq_reads_multi timings:")
 */
    free(__pyx_cur_scope->__pyx_v_self->rid);

    /* "pyslow5.pyx":475
 * 
 *         self.logger.debug("batch for loop start")
 *         for batch in chain(batches):             # <<<<<<<<<<<<<<
 *             self.logger.debug("Batch: {}".format(",".join([i for i in batch])))
 *             batch_len = len(batch)
 */
  }
  __pyx_L10_break:;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":576
 *                 free(self.rid[i])
 *             free(self.rid)
 *         self.rec = NULL             # <<<<<<<<<<<<<<
 *         self.logger.debug("seq_reads_multi timings:")
 * 
 */
  __pyx_cur_scope->__pyx_v_self->rec = NULL;

  /* "pyslow5.pyx":577
 *             free(self.rid)
 *         self.rec = NULL
 *         self.logger.debug("seq_reads_multi timings:")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_kp_u_seq_reads_multi_timings) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_u_seq_reads_multi_timings);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyslow5.pyx":449
 *             yield read_list[i:i+size]
 * 
 *     def _get_read_multi(self, read_list, threads, batchsize, pA, aux):             # <<<<<<<<<<<<<<
 *         '''
 *         read_list = list of readIDs
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("_get_read_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":580
 * 
 * 
 *     def _get_read_aux(self, aux_names, aux_types):             # <<<<<<<<<<<<<<
 *         '''
 *         get aux field for read
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_17_get_read_aux(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_16_get_read_aux[] = "\n        get aux field for read\n        aux_names and aux_types need to be lists, even if a single element\n        both names and types need to be in the same order, to ensure correct type function is called\n        This ensures correct C -> python object conversion\n        Returns dictionary with dic[name] = value\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_17_get_read_aux(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_aux_names = 0;
  PyObject *__pyx_v_aux_types = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_read_aux (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_aux_names,&__pyx_n_s_aux_types,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux_names)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux_types)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_read_aux", 1, 2, 2, 1); __PYX_ERR(0, 580, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_read_aux") < 0)) __PYX_ERR(0, 580, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_aux_names = values[0];
    __pyx_v_aux_types = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_read_aux", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 580, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_read_aux", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_16_get_read_aux(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_aux_names, __pyx_v_aux_types);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_16_get_read_aux(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_aux_names, PyObject *__pyx_v_aux_types) {
  PyObject *__pyx_v_dic = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_atype = NULL;
  PyObject *__pyx_v_a_name = NULL;
  PyObject *__pyx_v_l = NULL;
  uint64_t __pyx_v_i;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  char const *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  char const *__pyx_t_16;
  char const *__pyx_t_17;
  char const *__pyx_t_18;
  char const *__pyx_t_19;
  char const *__pyx_t_20;
  char const *__pyx_t_21;
  char const *__pyx_t_22;
  char const *__pyx_t_23;
  char const *__pyx_t_24;
  char const *__pyx_t_25;
  char const *__pyx_t_26;
  char const *__pyx_t_27;
  uint64_t __pyx_t_28;
  uint64_t __pyx_t_29;
  uint64_t __pyx_t_30;
  int __pyx_t_31;
  char const *__pyx_t_32;
  char const *__pyx_t_33;
  char const *__pyx_t_34;
  char const *__pyx_t_35;
  char const *__pyx_t_36;
  char const *__pyx_t_37;
  char const *__pyx_t_38;
  char const *__pyx_t_39;
  char const *__pyx_t_40;
  char const *__pyx_t_41;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_read_aux", 0);
  __Pyx_TraceCall("_get_read_aux", __pyx_f[0], 580, 0, __PYX_ERR(0, 580, __pyx_L1_error));

  /* "pyslow5.pyx":588
 *         Returns dictionary with dic[name] = value
 *         '''
 *         dic = {}             # <<<<<<<<<<<<<<
 *         # treat all input as list to simplify
 *         for name, atype in zip(aux_names, aux_types):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dic = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":590
 *         dic = {}
 *         # treat all input as list to simplify
 *         for name, atype in zip(aux_names, aux_types):             # <<<<<<<<<<<<<<
 *             a_name = str.encode(name)
 *             if atype == 0:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_aux_names);
  __Pyx_GIVEREF(__pyx_v_aux_names);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_aux_names);
  __Pyx_INCREF(__pyx_v_aux_types);
  __Pyx_GIVEREF(__pyx_v_aux_types);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_aux_types);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 590, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 590, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 590, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 590, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 590, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 590, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 590, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 590, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 590, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 590, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 590, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 590, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_atype, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "pyslow5.pyx":591
 *         # treat all input as list to simplify
 *         for name, atype in zip(aux_names, aux_types):
 *             a_name = str.encode(name)             # <<<<<<<<<<<<<<
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&PyUnicode_Type)), __pyx_n_s_encode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_5, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_name);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_a_name, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":592
 *         for name, atype in zip(aux_names, aux_types):
 *             a_name = str.encode(name)
 *             if atype == 0:             # <<<<<<<<<<<<<<
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":593
 *             a_name = str.encode(name)
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e0
 */
      __pyx_t_10 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 593, __pyx_L1_error)
      __pyx_v_self->e0 = slow5_aux_get_int8(__pyx_v_self->rec, __pyx_t_10, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":594
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e0
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":595
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e0             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int8_t(__pyx_v_self->e0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 595, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 595, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":594
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e0
 *                 else:
 */
        goto __pyx_L8;
      }

      /* "pyslow5.pyx":597
 *                     dic[name] = self.e0
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 1:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 597, __pyx_L1_error)
        }
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 597, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 597, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 597, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_11 = 0;
          __pyx_t_13 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_15, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 597, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":598
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 598, __pyx_L1_error)
      }
      __pyx_L8:;

      /* "pyslow5.pyx":592
 *         for name, atype in zip(aux_names, aux_types):
 *             a_name = str.encode(name)
 *             if atype == 0:             # <<<<<<<<<<<<<<
 *                 self.e0 = slow5_aux_get_int8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":599
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 1:             # <<<<<<<<<<<<<<
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":600
 *                     dic[name] = None
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e1
 */
      __pyx_t_16 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_16) && PyErr_Occurred())) __PYX_ERR(0, 600, __pyx_L1_error)
      __pyx_v_self->e1 = slow5_aux_get_int16(__pyx_v_self->rec, __pyx_t_16, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":601
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e1
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":602
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e1             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int16_t(__pyx_v_self->e1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 602, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 602, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":601
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e1
 *                 else:
 */
        goto __pyx_L9;
      }

      /* "pyslow5.pyx":604
 *                     dic[name] = self.e1
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 2:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 604, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 604, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 604, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 604, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 604, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 604, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 604, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 604, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 604, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_15 = 0;
          __pyx_t_11 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 604, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 604, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":605
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 605, __pyx_L1_error)
      }
      __pyx_L9:;

      /* "pyslow5.pyx":599
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 1:             # <<<<<<<<<<<<<<
 *                 self.e1 = slow5_aux_get_int16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":606
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 2:             # <<<<<<<<<<<<<<
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":607
 *                     dic[name] = None
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e2
 */
      __pyx_t_17 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 607, __pyx_L1_error)
      __pyx_v_self->e2 = slow5_aux_get_int32(__pyx_v_self->rec, __pyx_t_17, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":608
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e2
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":609
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e2             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int32_t(__pyx_v_self->e2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 609, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 609, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":608
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e2
 *                 else:
 */
        goto __pyx_L10;
      }

      /* "pyslow5.pyx":611
 *                     dic[name] = self.e2
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 3:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 611, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 611, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 611, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 611, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 611, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 611, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 611, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 611, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 611, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_12 = 0;
          __pyx_t_15 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 611, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 611, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":612
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 612, __pyx_L1_error)
      }
      __pyx_L10:;

      /* "pyslow5.pyx":606
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 2:             # <<<<<<<<<<<<<<
 *                 self.e2 = slow5_aux_get_int32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":613
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 3:             # <<<<<<<<<<<<<<
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 613, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 613, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":614
 *                     dic[name] = None
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e3
 */
      __pyx_t_18 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 614, __pyx_L1_error)
      __pyx_v_self->e3 = slow5_aux_get_int64(__pyx_v_self->rec, __pyx_t_18, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":615
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e3
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":616
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e3             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int64_t(__pyx_v_self->e3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 616, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 616, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":615
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e3
 *                 else:
 */
        goto __pyx_L11;
      }

      /* "pyslow5.pyx":618
 *                     dic[name] = self.e3
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 4:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 618, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 618, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 618, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 618, __pyx_L1_error)
        }
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 618, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_12 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_15); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 618, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 618, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 618, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 618, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_15); __pyx_t_15 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_12);
          __pyx_t_13 = 0;
          __pyx_t_12 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 618, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 618, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":619
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 619, __pyx_L1_error)
      }
      __pyx_L11:;

      /* "pyslow5.pyx":613
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 3:             # <<<<<<<<<<<<<<
 *                 self.e3 = slow5_aux_get_int64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":620
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 4:             # <<<<<<<<<<<<<<
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_4, 4, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":621
 *                     dic[name] = None
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e4
 */
      __pyx_t_19 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 621, __pyx_L1_error)
      __pyx_v_self->e4 = slow5_aux_get_uint8(__pyx_v_self->rec, __pyx_t_19, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":622
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e4
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":623
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e4             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->e4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 623, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 623, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":622
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e4
 *                 else:
 */
        goto __pyx_L12;
      }

      /* "pyslow5.pyx":625
 *                     dic[name] = self.e4
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 5:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 625, __pyx_L1_error)
        }
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 625, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 625, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 625, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_11 = 0;
          __pyx_t_13 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_15, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 625, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":626
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 626, __pyx_L1_error)
      }
      __pyx_L12:;

      /* "pyslow5.pyx":620
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 4:             # <<<<<<<<<<<<<<
 *                 self.e4 = slow5_aux_get_uint8(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":627
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 5:             # <<<<<<<<<<<<<<
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_5, 5, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":628
 *                     dic[name] = None
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e5
 */
      __pyx_t_20 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_20) && PyErr_Occurred())) __PYX_ERR(0, 628, __pyx_L1_error)
      __pyx_v_self->e5 = slow5_aux_get_uint16(__pyx_v_self->rec, __pyx_t_20, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":629
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e5
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":630
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e5             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint16_t(__pyx_v_self->e5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 630, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 630, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":629
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e5
 *                 else:
 */
        goto __pyx_L13;
      }

      /* "pyslow5.pyx":632
 *                     dic[name] = self.e5
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 6:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 632, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 632, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 632, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 632, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 632, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 632, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 632, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 632, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 632, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_15 = 0;
          __pyx_t_11 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 632, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 632, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":633
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 633, __pyx_L1_error)
      }
      __pyx_L13:;

      /* "pyslow5.pyx":627
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 5:             # <<<<<<<<<<<<<<
 *                 self.e5 = slow5_aux_get_uint16(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":634
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 6:             # <<<<<<<<<<<<<<
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_6, 6, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 634, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":635
 *                     dic[name] = None
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e6
 */
      __pyx_t_21 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_21) && PyErr_Occurred())) __PYX_ERR(0, 635, __pyx_L1_error)
      __pyx_v_self->e6 = slow5_aux_get_uint32(__pyx_v_self->rec, __pyx_t_21, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":636
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e6
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":637
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e6             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint32_t(__pyx_v_self->e6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 637, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 637, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":636
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e6
 *                 else:
 */
        goto __pyx_L14;
      }

      /* "pyslow5.pyx":639
 *                     dic[name] = self.e6
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 7:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 639, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_12 = 0;
          __pyx_t_15 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":640
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 640, __pyx_L1_error)
      }
      __pyx_L14:;

      /* "pyslow5.pyx":634
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 6:             # <<<<<<<<<<<<<<
 *                 self.e6 = slow5_aux_get_uint32(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":641
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 7:             # <<<<<<<<<<<<<<
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_7, 7, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":642
 *                     dic[name] = None
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e7
 */
      __pyx_t_22 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_22) && PyErr_Occurred())) __PYX_ERR(0, 642, __pyx_L1_error)
      __pyx_v_self->e7 = slow5_aux_get_uint64(__pyx_v_self->rec, __pyx_t_22, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":643
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e7
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":644
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e7             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint64_t(__pyx_v_self->e7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 644, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 644, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":643
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e7
 *                 else:
 */
        goto __pyx_L15;
      }

      /* "pyslow5.pyx":646
 *                     dic[name] = self.e7
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 8:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 646, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 646, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 646, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 646, __pyx_L1_error)
        }
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 646, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_12 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_15); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 646, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 646, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 646, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 646, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_15); __pyx_t_15 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_12);
          __pyx_t_13 = 0;
          __pyx_t_12 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 646, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 646, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":647
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 647, __pyx_L1_error)
      }
      __pyx_L15:;

      /* "pyslow5.pyx":641
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 7:             # <<<<<<<<<<<<<<
 *                 self.e7 = slow5_aux_get_uint64(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":648
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 8:             # <<<<<<<<<<<<<<
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_8, 8, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 648, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 648, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":649
 *                     dic[name] = None
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e8
 */
      __pyx_t_23 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_23) && PyErr_Occurred())) __PYX_ERR(0, 649, __pyx_L1_error)
      __pyx_t_2 = PyFloat_FromDouble(slow5_aux_get_float(__pyx_v_self->rec, __pyx_t_23, (&__pyx_v_self->aux_get_err))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(PyFloat_CheckExact(__pyx_t_2))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->e8);
      __Pyx_DECREF(__pyx_v_self->e8);
      __pyx_v_self->e8 = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":650
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e8
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":651
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e8             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __pyx_v_self->e8;
        __Pyx_INCREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 651, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":650
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e8
 *                 else:
 */
        goto __pyx_L16;
      }

      /* "pyslow5.pyx":653
 *                     dic[name] = self.e8
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 9:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 653, __pyx_L1_error)
        }
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 653, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 653, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 653, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_11 = 0;
          __pyx_t_13 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_15, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 653, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":654
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 654, __pyx_L1_error)
      }
      __pyx_L16:;

      /* "pyslow5.pyx":648
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 8:             # <<<<<<<<<<<<<<
 *                 self.e8 = slow5_aux_get_float(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":655
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 9:             # <<<<<<<<<<<<<<
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_9, 9, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":656
 *                     dic[name] = None
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e9
 */
      __pyx_t_24 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_24) && PyErr_Occurred())) __PYX_ERR(0, 656, __pyx_L1_error)
      __pyx_v_self->e9 = slow5_aux_get_double(__pyx_v_self->rec, __pyx_t_24, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":657
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e9
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":658
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e9             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->e9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 658, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 658, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":657
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e9
 *                 else:
 */
        goto __pyx_L17;
      }

      /* "pyslow5.pyx":660
 *                     dic[name] = self.e9
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 10:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 660, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 660, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 660, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 660, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 660, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 660, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 660, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 660, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 660, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_15 = 0;
          __pyx_t_11 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 660, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 660, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":661
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 661, __pyx_L1_error)
      }
      __pyx_L17:;

      /* "pyslow5.pyx":655
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 9:             # <<<<<<<<<<<<<<
 *                 self.e9 = slow5_aux_get_double(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":662
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 10:             # <<<<<<<<<<<<<<
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_10, 10, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":663
 *                     dic[name] = None
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e10
 */
      __pyx_t_25 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_25) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L1_error)
      __pyx_v_self->e10 = slow5_aux_get_char(__pyx_v_self->rec, __pyx_t_25, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":664
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e10
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":665
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e10             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_char(__pyx_v_self->e10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 665, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 665, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":664
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e10
 *                 else:
 */
        goto __pyx_L18;
      }

      /* "pyslow5.pyx":667
 *                     dic[name] = self.e10
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 11:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 667, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 667, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 667, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 667, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_12 = 0;
          __pyx_t_15 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 667, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":668
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 668, __pyx_L1_error)
      }
      __pyx_L18:;

      /* "pyslow5.pyx":662
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 10:             # <<<<<<<<<<<<<<
 *                 self.e10 = slow5_aux_get_char(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":669
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 11:             # <<<<<<<<<<<<<<
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_11, 11, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":670
 *                     dic[name] = None
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e11
 */
      __pyx_t_26 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_26) && PyErr_Occurred())) __PYX_ERR(0, 670, __pyx_L1_error)
      __pyx_v_self->e11 = slow5_aux_get_enum(__pyx_v_self->rec, __pyx_t_26, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":671
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e11
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":672
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e11             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 */
        __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->e11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 672, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 672, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":671
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e11
 *                 else:
 */
        goto __pyx_L19;
      }

      /* "pyslow5.pyx":674
 *                     dic[name] = self.e11
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 12:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_5 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_v_atype) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_atype);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 674, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_5, __pyx_t_7, __pyx_t_15};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_5, __pyx_t_7, __pyx_t_15};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 674, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_5 = 0;
          __pyx_t_7 = 0;
          __pyx_t_15 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":675
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 675, __pyx_L1_error)
      }
      __pyx_L19:;

      /* "pyslow5.pyx":669
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 11:             # <<<<<<<<<<<<<<
 *                 self.e11 = slow5_aux_get_enum(self.rec, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":676
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                     dic[name] = None
 *             elif atype == 12:             # <<<<<<<<<<<<<<
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_12, 12, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":677
 *                     dic[name] = None
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_27 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_27) && PyErr_Occurred())) __PYX_ERR(0, 677, __pyx_L1_error)
      __pyx_v_self->e12 = slow5_aux_get_int8_array(__pyx_v_self->rec, __pyx_t_27, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":678
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":679
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e12[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 679, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":680
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e12[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":681
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e12[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int8_t((__pyx_v_self->e12[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 681, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":682
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e12[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 682, __pyx_L1_error)

        /* "pyslow5.pyx":678
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L20;
      }

      /* "pyslow5.pyx":684
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 13:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 684, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 684, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 684, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 684, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_7 = 0;
          __pyx_t_13 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 684, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":685
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 685, __pyx_L1_error)
      }
      __pyx_L20:;

      /* "pyslow5.pyx":676
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                     dic[name] = None
 *             elif atype == 12:             # <<<<<<<<<<<<<<
 *                 self.e12 = slow5_aux_get_int8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":686
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 13:             # <<<<<<<<<<<<<<
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_13, 13, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":687
 *                     dic[name] = None
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_32 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_32) && PyErr_Occurred())) __PYX_ERR(0, 687, __pyx_L1_error)
      __pyx_v_self->e13 = slow5_aux_get_int16_array(__pyx_v_self->rec, __pyx_t_32, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":688
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":689
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e13[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":690
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e13[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":691
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e13[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int16_t((__pyx_v_self->e13[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 691, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 691, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":692
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e13[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 692, __pyx_L1_error)

        /* "pyslow5.pyx":688
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":694
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 14:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 694, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_7);
          __pyx_t_11 = 0;
          __pyx_t_7 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 694, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":695
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 695, __pyx_L1_error)
      }
      __pyx_L23:;

      /* "pyslow5.pyx":686
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 13:             # <<<<<<<<<<<<<<
 *                 self.e13 = slow5_aux_get_int16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":696
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 14:             # <<<<<<<<<<<<<<
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_14, 14, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":697
 *                     dic[name] = None
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_33 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_33) && PyErr_Occurred())) __PYX_ERR(0, 697, __pyx_L1_error)
      __pyx_v_self->e14 = slow5_aux_get_int32_array(__pyx_v_self->rec, __pyx_t_33, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":698
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":699
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e14[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 699, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":700
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e14[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":701
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e14[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int32_t((__pyx_v_self->e14[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 701, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 701, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":702
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e14[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 702, __pyx_L1_error)

        /* "pyslow5.pyx":698
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L26;
      }

      /* "pyslow5.pyx":704
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 15:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 704, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 704, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 704, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 704, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 704, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 704, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 704, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 704, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 704, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_5 = 0;
          __pyx_t_11 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 704, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 704, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":705
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 705, __pyx_L1_error)
      }
      __pyx_L26:;

      /* "pyslow5.pyx":696
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 14:             # <<<<<<<<<<<<<<
 *                 self.e14 = slow5_aux_get_int32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":706
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 15:             # <<<<<<<<<<<<<<
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_15, 15, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":707
 *                     dic[name] = None
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_34 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_34) && PyErr_Occurred())) __PYX_ERR(0, 707, __pyx_L1_error)
      __pyx_v_self->e15 = slow5_aux_get_int64_array(__pyx_v_self->rec, __pyx_t_34, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":708
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":709
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e15[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 709, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":710
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e15[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":711
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e15[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int64_t((__pyx_v_self->e15[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 711, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 711, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":712
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e15[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 712, __pyx_L1_error)

        /* "pyslow5.pyx":708
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L29;
      }

      /* "pyslow5.pyx":714
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 16:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 714, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 714, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 714, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 714, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 714, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 714, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 714, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 714, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 714, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_14, __pyx_t_5);
          __pyx_t_13 = 0;
          __pyx_t_5 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_7, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 714, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 714, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":715
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 715, __pyx_L1_error)
      }
      __pyx_L29:;

      /* "pyslow5.pyx":706
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 15:             # <<<<<<<<<<<<<<
 *                 self.e15 = slow5_aux_get_int64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":716
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 16:             # <<<<<<<<<<<<<<
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_16, 16, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":717
 *                     dic[name] = None
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_35 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_35) && PyErr_Occurred())) __PYX_ERR(0, 717, __pyx_L1_error)
      __pyx_v_self->e16 = slow5_aux_get_uint8_array(__pyx_v_self->rec, __pyx_t_35, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":718
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":719
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e16[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 719, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":720
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e16[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":721
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e16[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint8_t((__pyx_v_self->e16[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 721, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 721, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":722
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e16[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 722, __pyx_L1_error)

        /* "pyslow5.pyx":718
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L32;
      }

      /* "pyslow5.pyx":724
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 17:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 724, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 724, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 724, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 724, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 724, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 724, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 724, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 724, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 724, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_7 = 0;
          __pyx_t_13 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 724, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 724, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":725
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 725, __pyx_L1_error)
      }
      __pyx_L32:;

      /* "pyslow5.pyx":716
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 16:             # <<<<<<<<<<<<<<
 *                 self.e16 = slow5_aux_get_uint8_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":726
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 17:             # <<<<<<<<<<<<<<
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_17, 17, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":727
 *                     dic[name] = None
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_36 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_36) && PyErr_Occurred())) __PYX_ERR(0, 727, __pyx_L1_error)
      __pyx_v_self->e17 = slow5_aux_get_uint16_array(__pyx_v_self->rec, __pyx_t_36, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":728
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":729
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e17[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 729, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":730
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e17[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":731
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e17[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint16_t((__pyx_v_self->e17[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 731, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 731, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":732
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e17[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 732, __pyx_L1_error)

        /* "pyslow5.pyx":728
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L35;
      }

      /* "pyslow5.pyx":734
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 18:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 734, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_7);
          __pyx_t_11 = 0;
          __pyx_t_7 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":735
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 735, __pyx_L1_error)
      }
      __pyx_L35:;

      /* "pyslow5.pyx":726
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 17:             # <<<<<<<<<<<<<<
 *                 self.e17 = slow5_aux_get_uint16_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":736
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 18:             # <<<<<<<<<<<<<<
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_18, 18, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":737
 *                     dic[name] = None
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_37 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_37) && PyErr_Occurred())) __PYX_ERR(0, 737, __pyx_L1_error)
      __pyx_v_self->e18 = slow5_aux_get_uint32_array(__pyx_v_self->rec, __pyx_t_37, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":738
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":739
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e18[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 739, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":740
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e18[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":741
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e18[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint32_t((__pyx_v_self->e18[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 741, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 741, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":742
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e18[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 742, __pyx_L1_error)

        /* "pyslow5.pyx":738
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L38;
      }

      /* "pyslow5.pyx":744
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 19:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 744, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 744, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 744, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 744, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 744, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 744, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 744, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 744, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 744, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_5 = 0;
          __pyx_t_11 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 744, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":745
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 745, __pyx_L1_error)
      }
      __pyx_L38:;

      /* "pyslow5.pyx":736
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 18:             # <<<<<<<<<<<<<<
 *                 self.e18 = slow5_aux_get_uint32_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":746
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 19:             # <<<<<<<<<<<<<<
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_19, 19, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":747
 *                     dic[name] = None
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_38 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_38) && PyErr_Occurred())) __PYX_ERR(0, 747, __pyx_L1_error)
      __pyx_v_self->e19 = slow5_aux_get_uint64_array(__pyx_v_self->rec, __pyx_t_38, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":748
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":749
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e19[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 749, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":750
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e19[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":751
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e19[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint64_t((__pyx_v_self->e19[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 751, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 751, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":752
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e19[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 752, __pyx_L1_error)

        /* "pyslow5.pyx":748
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L41;
      }

      /* "pyslow5.pyx":754
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 20:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 754, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 754, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 754, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 754, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_14, __pyx_t_5);
          __pyx_t_13 = 0;
          __pyx_t_5 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_7, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 754, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":755
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 755, __pyx_L1_error)
      }
      __pyx_L41:;

      /* "pyslow5.pyx":746
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 19:             # <<<<<<<<<<<<<<
 *                 self.e19 = slow5_aux_get_uint64_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":756
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 20:             # <<<<<<<<<<<<<<
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_20, 20, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 756, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 756, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":757
 *                     dic[name] = None
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_39 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_39) && PyErr_Occurred())) __PYX_ERR(0, 757, __pyx_L1_error)
      __pyx_v_self->e20 = slow5_aux_get_float_array(__pyx_v_self->rec, __pyx_t_39, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":758
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":759
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e20[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 759, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":760
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e20[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":761
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e20[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->e20[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 761, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 761, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":762
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e20[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 762, __pyx_L1_error)

        /* "pyslow5.pyx":758
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L44;
      }

      /* "pyslow5.pyx":764
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 21:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 764, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 764, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 764, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 764, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_7 = 0;
          __pyx_t_13 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 764, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":765
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 765, __pyx_L1_error)
      }
      __pyx_L44:;

      /* "pyslow5.pyx":756
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 20:             # <<<<<<<<<<<<<<
 *                 self.e20 = slow5_aux_get_float_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":766
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 21:             # <<<<<<<<<<<<<<
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_21, 21, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 766, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":767
 *                     dic[name] = None
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_40 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_40) && PyErr_Occurred())) __PYX_ERR(0, 767, __pyx_L1_error)
      __pyx_v_self->e21 = slow5_aux_get_double_array(__pyx_v_self->rec, __pyx_t_40, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":768
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":769
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e21[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 769, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":770
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e21[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":771
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e21[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->e21[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 771, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 771, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":772
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e21[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 772, __pyx_L1_error)

        /* "pyslow5.pyx":768
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L47;
      }

      /* "pyslow5.pyx":774
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 22:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 774, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 774, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 774, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 774, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_7);
          __pyx_t_11 = 0;
          __pyx_t_7 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 774, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":775
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 775, __pyx_L1_error)
      }
      __pyx_L47:;

      /* "pyslow5.pyx":766
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 21:             # <<<<<<<<<<<<<<
 *                 self.e21 = slow5_aux_get_double_array(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":776
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_22, 22, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":777
 *                     dic[name] = None
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_41 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_41) && PyErr_Occurred())) __PYX_ERR(0, 777, __pyx_L1_error)
      __pyx_v_self->e22 = slow5_aux_get_string(__pyx_v_self->rec, __pyx_t_41, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":778
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     s = ""
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":779
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     s = ""
 *                     for i in range(self.aux_get_len):
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 779, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":780
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     s = ""             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(chr(self.e22[i]))
 */
        __Pyx_INCREF(__pyx_kp_u_);
        __Pyx_XDECREF_SET(__pyx_v_s, __pyx_kp_u_);

        /* "pyslow5.pyx":781
 *                     l = []
 *                     s = ""
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(chr(self.e22[i]))
 *                         s = "".join(l)
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":782
 *                     s = ""
 *                     for i in range(self.aux_get_len):
 *                         l.append(chr(self.e22[i]))             # <<<<<<<<<<<<<<
 *                         s = "".join(l)
 *                     dic[name] = s
 */
          __pyx_t_2 = __Pyx_PyInt_From_char((__pyx_v_self->e22[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 782, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 782, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_6); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 782, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "pyslow5.pyx":783
 *                     for i in range(self.aux_get_len):
 *                         l.append(chr(self.e22[i]))
 *                         s = "".join(l)             # <<<<<<<<<<<<<<
 *                     dic[name] = s
 *                 else:
 */
          __pyx_t_6 = PyUnicode_Join(__pyx_kp_u_, __pyx_v_l); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 783, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_s, ((PyObject*)__pyx_t_6));
          __pyx_t_6 = 0;
        }

        /* "pyslow5.pyx":784
 *                         l.append(chr(self.e22[i]))
 *                         s = "".join(l)
 *                     dic[name] = s             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_s) < 0)) __PYX_ERR(0, 784, __pyx_L1_error)

        /* "pyslow5.pyx":778
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     s = ""
 */
        goto __pyx_L50;
      }

      /* "pyslow5.pyx":786
 *                     dic[name] = s
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 22:
 */
      /*else*/ {
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 786, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 786, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 786, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 786, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_5 = 0;
          __pyx_t_11 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 786, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        __pyx_t_6 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyslow5.pyx":787
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 22:
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 787, __pyx_L1_error)
      }
      __pyx_L50:;

      /* "pyslow5.pyx":776
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.e22 = slow5_aux_get_string(self.rec, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":788
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                 dic[name] = None
 */
    __pyx_t_6 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_22, 22, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 788, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 788, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":789
 *                     dic[name] = None
 *             elif atype == 22:
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])             # <<<<<<<<<<<<<<
 *                 dic[name] = None
 *             else:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_NOT_IMPLEMENTED_YET_get_aux_type, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_15, function);
        }
      }
      __pyx_t_12 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_15, __pyx_t_13, __pyx_v_atype) : __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_v_atype);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      if (unlikely(__pyx_v_self->error_codes == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 789, __pyx_L1_error)
      }
      __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      __pyx_t_14 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_14 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_12, __pyx_t_15, __pyx_t_11};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 789, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_12, __pyx_t_15, __pyx_t_11};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 789, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 789, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (__pyx_t_13) {
          __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_t_12);
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_11);
        __pyx_t_12 = 0;
        __pyx_t_15 = 0;
        __pyx_t_11 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 789, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyslow5.pyx":790
 *             elif atype == 22:
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                 dic[name] = None             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.debug("get_read_aux atype not known, skipping: {}".format(atype))
 */
      if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 790, __pyx_L1_error)

      /* "pyslow5.pyx":788
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                 dic[name] = None
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":792
 *                 dic[name] = None
 *             else:
 *                 self.logger.debug("get_read_aux atype not known, skipping: {}".format(atype))             # <<<<<<<<<<<<<<
 * 
 *         return dic
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_aux_atype_not_known_ski, __pyx_n_s_format); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_15 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_15)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_15);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
        }
      }
      __pyx_t_5 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_15, __pyx_v_atype) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_atype);
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_11)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_6 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_11, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __pyx_L7:;

    /* "pyslow5.pyx":590
 *         dic = {}
 *         # treat all input as list to simplify
 *         for name, atype in zip(aux_names, aux_types):             # <<<<<<<<<<<<<<
 *             a_name = str.encode(name)
 *             if atype == 0:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":794
 *                 self.logger.debug("get_read_aux atype not known, skipping: {}".format(atype))
 * 
 *         return dic             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_dic);
  __pyx_r = __pyx_v_dic;
  goto __pyx_L0;

  /* "pyslow5.pyx":580
 * 
 * 
 *     def _get_read_aux(self, aux_names, aux_types):             # <<<<<<<<<<<<<<
 *         '''
 *         get aux field for read
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("pyslow5.Open._get_read_aux", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dic);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_atype);
  __Pyx_XDECREF(__pyx_v_a_name);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":797
 * 
 * 
 *     def _get_seq_read_aux(self, aux_names, aux_types):             # <<<<<<<<<<<<<<
 *         '''
 *         get aux field for read - when doing sequential read, because of separate self.read (vs rec) call
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_19_get_seq_read_aux(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_18_get_seq_read_aux[] = "\n        get aux field for read - when doing sequential read, because of separate self.read (vs rec) call\n        aux_names and aux_types need to be lists, even if a single element\n        both names and types need to be in the same order, to ensure correct type function is called\n        This ensures correct C -> python object conversion\n        Returns dictionary with dic[name] = value\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_19_get_seq_read_aux(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_aux_names = 0;
  PyObject *__pyx_v_aux_types = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_seq_read_aux (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_aux_names,&__pyx_n_s_aux_types,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux_names)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux_types)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_seq_read_aux", 1, 2, 2, 1); __PYX_ERR(0, 797, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_seq_read_aux") < 0)) __PYX_ERR(0, 797, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_aux_names = values[0];
    __pyx_v_aux_types = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_seq_read_aux", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 797, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._get_seq_read_aux", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_18_get_seq_read_aux(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_aux_names, __pyx_v_aux_types);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_18_get_seq_read_aux(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_aux_names, PyObject *__pyx_v_aux_types) {
  PyObject *__pyx_v_dic = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_atype = NULL;
  PyObject *__pyx_v_a_name = NULL;
  PyObject *__pyx_v_l = NULL;
  uint64_t __pyx_v_i;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  char const *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  char const *__pyx_t_16;
  char const *__pyx_t_17;
  char const *__pyx_t_18;
  char const *__pyx_t_19;
  char const *__pyx_t_20;
  char const *__pyx_t_21;
  char const *__pyx_t_22;
  char const *__pyx_t_23;
  char const *__pyx_t_24;
  char const *__pyx_t_25;
  char const *__pyx_t_26;
  char const *__pyx_t_27;
  uint64_t __pyx_t_28;
  uint64_t __pyx_t_29;
  uint64_t __pyx_t_30;
  int __pyx_t_31;
  char const *__pyx_t_32;
  char const *__pyx_t_33;
  char const *__pyx_t_34;
  char const *__pyx_t_35;
  char const *__pyx_t_36;
  char const *__pyx_t_37;
  char const *__pyx_t_38;
  char const *__pyx_t_39;
  char const *__pyx_t_40;
  char const *__pyx_t_41;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_seq_read_aux", 0);
  __Pyx_TraceCall("_get_seq_read_aux", __pyx_f[0], 797, 0, __PYX_ERR(0, 797, __pyx_L1_error));

  /* "pyslow5.pyx":805
 *         Returns dictionary with dic[name] = value
 *         '''
 *         dic = {}             # <<<<<<<<<<<<<<
 *         # treat all input as a list to simplify
 *         for name, atype in zip(aux_names, aux_types):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 805, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dic = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":807
 *         dic = {}
 *         # treat all input as a list to simplify
 *         for name, atype in zip(aux_names, aux_types):             # <<<<<<<<<<<<<<
 *             a_name = str.encode(name)
 *             if atype == 0:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_aux_names);
  __Pyx_GIVEREF(__pyx_v_aux_names);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_aux_names);
  __Pyx_INCREF(__pyx_v_aux_types);
  __Pyx_GIVEREF(__pyx_v_aux_types);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_aux_types);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 807, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 807, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 807, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 807, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 807, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 807, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 807, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 807, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 807, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 807, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 807, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 807, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_atype, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "pyslow5.pyx":808
 *         # treat all input as a list to simplify
 *         for name, atype in zip(aux_names, aux_types):
 *             a_name = str.encode(name)             # <<<<<<<<<<<<<<
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&PyUnicode_Type)), __pyx_n_s_encode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_5, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_name);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_a_name, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":809
 *         for name, atype in zip(aux_names, aux_types):
 *             a_name = str.encode(name)
 *             if atype == 0:             # <<<<<<<<<<<<<<
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":810
 *             a_name = str.encode(name)
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e0
 */
      __pyx_t_10 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 810, __pyx_L1_error)
      __pyx_v_self->e0 = slow5_aux_get_int8(__pyx_v_self->read, __pyx_t_10, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":811
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e0
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":812
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e0             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int8_t(__pyx_v_self->e0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 812, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 812, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":811
 *             if atype == 0:
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e0
 *                 else:
 */
        goto __pyx_L8;
      }

      /* "pyslow5.pyx":814
 *                     dic[name] = self.e0
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 1:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 814, __pyx_L1_error)
        }
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 814, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 814, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 814, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_11 = 0;
          __pyx_t_13 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_15, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 814, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":815
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 815, __pyx_L1_error)
      }
      __pyx_L8:;

      /* "pyslow5.pyx":809
 *         for name, atype in zip(aux_names, aux_types):
 *             a_name = str.encode(name)
 *             if atype == 0:             # <<<<<<<<<<<<<<
 *                 self.e0 = slow5_aux_get_int8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":816
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 1:             # <<<<<<<<<<<<<<
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 816, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 816, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":817
 *                     dic[name] = None
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e1
 */
      __pyx_t_16 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_16) && PyErr_Occurred())) __PYX_ERR(0, 817, __pyx_L1_error)
      __pyx_v_self->e1 = slow5_aux_get_int16(__pyx_v_self->read, __pyx_t_16, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":818
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e1
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":819
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e1             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int16_t(__pyx_v_self->e1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 819, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":818
 *             elif atype == 1:
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e1
 *                 else:
 */
        goto __pyx_L9;
      }

      /* "pyslow5.pyx":821
 *                     dic[name] = self.e1
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 2:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 821, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 821, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 821, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 821, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 821, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 821, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 821, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 821, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 821, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_15 = 0;
          __pyx_t_11 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 821, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 821, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":822
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 822, __pyx_L1_error)
      }
      __pyx_L9:;

      /* "pyslow5.pyx":816
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 1:             # <<<<<<<<<<<<<<
 *                 self.e1 = slow5_aux_get_int16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":823
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 2:             # <<<<<<<<<<<<<<
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 823, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 823, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":824
 *                     dic[name] = None
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e2
 */
      __pyx_t_17 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 824, __pyx_L1_error)
      __pyx_v_self->e2 = slow5_aux_get_int32(__pyx_v_self->read, __pyx_t_17, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":825
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e2
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":826
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e2             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int32_t(__pyx_v_self->e2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 826, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 826, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":825
 *             elif atype == 2:
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e2
 *                 else:
 */
        goto __pyx_L10;
      }

      /* "pyslow5.pyx":828
 *                     dic[name] = self.e2
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 3:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 828, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 828, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 828, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 828, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 828, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 828, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 828, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 828, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 828, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_12 = 0;
          __pyx_t_15 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 828, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 828, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":829
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 829, __pyx_L1_error)
      }
      __pyx_L10:;

      /* "pyslow5.pyx":823
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 2:             # <<<<<<<<<<<<<<
 *                 self.e2 = slow5_aux_get_int32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":830
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 3:             # <<<<<<<<<<<<<<
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":831
 *                     dic[name] = None
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e3
 */
      __pyx_t_18 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 831, __pyx_L1_error)
      __pyx_v_self->e3 = slow5_aux_get_int64(__pyx_v_self->read, __pyx_t_18, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":832
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e3
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":833
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e3             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_int64_t(__pyx_v_self->e3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 833, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 833, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":832
 *             elif atype == 3:
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e3
 *                 else:
 */
        goto __pyx_L11;
      }

      /* "pyslow5.pyx":835
 *                     dic[name] = self.e3
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 4:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 835, __pyx_L1_error)
        }
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_12 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_15); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 835, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_15); __pyx_t_15 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_12);
          __pyx_t_13 = 0;
          __pyx_t_12 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":836
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 836, __pyx_L1_error)
      }
      __pyx_L11:;

      /* "pyslow5.pyx":830
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 3:             # <<<<<<<<<<<<<<
 *                 self.e3 = slow5_aux_get_int64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":837
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 4:             # <<<<<<<<<<<<<<
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_4, 4, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 837, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":838
 *                     dic[name] = None
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e4
 */
      __pyx_t_19 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 838, __pyx_L1_error)
      __pyx_v_self->e4 = slow5_aux_get_uint8(__pyx_v_self->read, __pyx_t_19, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":839
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e4
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":840
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e4             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->e4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 840, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 840, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":839
 *             elif atype == 4:
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e4
 *                 else:
 */
        goto __pyx_L12;
      }

      /* "pyslow5.pyx":842
 *                     dic[name] = self.e4
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 5:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 842, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 842, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 842, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 842, __pyx_L1_error)
        }
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 842, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 842, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 842, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_11 = 0;
          __pyx_t_13 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_15, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 842, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":843
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 843, __pyx_L1_error)
      }
      __pyx_L12:;

      /* "pyslow5.pyx":837
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 4:             # <<<<<<<<<<<<<<
 *                 self.e4 = slow5_aux_get_uint8(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":844
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 5:             # <<<<<<<<<<<<<<
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_5, 5, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":845
 *                     dic[name] = None
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e5
 */
      __pyx_t_20 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_20) && PyErr_Occurred())) __PYX_ERR(0, 845, __pyx_L1_error)
      __pyx_v_self->e5 = slow5_aux_get_uint16(__pyx_v_self->read, __pyx_t_20, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":846
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e5
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":847
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e5             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint16_t(__pyx_v_self->e5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 847, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 847, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":846
 *             elif atype == 5:
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e5
 *                 else:
 */
        goto __pyx_L13;
      }

      /* "pyslow5.pyx":849
 *                     dic[name] = self.e5
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 6:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 849, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 849, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 849, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 849, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_15 = 0;
          __pyx_t_11 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 849, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":850
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 850, __pyx_L1_error)
      }
      __pyx_L13:;

      /* "pyslow5.pyx":844
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 5:             # <<<<<<<<<<<<<<
 *                 self.e5 = slow5_aux_get_uint16(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":851
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 6:             # <<<<<<<<<<<<<<
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_6, 6, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 851, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 851, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":852
 *                     dic[name] = None
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e6
 */
      __pyx_t_21 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_21) && PyErr_Occurred())) __PYX_ERR(0, 852, __pyx_L1_error)
      __pyx_v_self->e6 = slow5_aux_get_uint32(__pyx_v_self->read, __pyx_t_21, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":853
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e6
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":854
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e6             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint32_t(__pyx_v_self->e6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 854, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 854, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":853
 *             elif atype == 6:
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e6
 *                 else:
 */
        goto __pyx_L14;
      }

      /* "pyslow5.pyx":856
 *                     dic[name] = self.e6
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 7:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 856, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 856, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 856, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 856, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_12 = 0;
          __pyx_t_15 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 856, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":857
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 857, __pyx_L1_error)
      }
      __pyx_L14:;

      /* "pyslow5.pyx":851
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 6:             # <<<<<<<<<<<<<<
 *                 self.e6 = slow5_aux_get_uint32(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":858
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 7:             # <<<<<<<<<<<<<<
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_7, 7, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":859
 *                     dic[name] = None
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e7
 */
      __pyx_t_22 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_22) && PyErr_Occurred())) __PYX_ERR(0, 859, __pyx_L1_error)
      __pyx_v_self->e7 = slow5_aux_get_uint64(__pyx_v_self->read, __pyx_t_22, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":860
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e7
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":861
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e7             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_uint64_t(__pyx_v_self->e7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 861, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 861, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":860
 *             elif atype == 7:
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e7
 *                 else:
 */
        goto __pyx_L15;
      }

      /* "pyslow5.pyx":863
 *                     dic[name] = self.e7
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 8:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 863, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 863, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 863, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 863, __pyx_L1_error)
        }
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 863, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_12 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_15); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 863, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 863, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_15, __pyx_v_atype, __pyx_t_13, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 863, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 863, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_15); __pyx_t_15 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_12);
          __pyx_t_13 = 0;
          __pyx_t_12 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 863, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 863, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":864
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 864, __pyx_L1_error)
      }
      __pyx_L15:;

      /* "pyslow5.pyx":858
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 7:             # <<<<<<<<<<<<<<
 *                 self.e7 = slow5_aux_get_uint64(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":865
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 8:             # <<<<<<<<<<<<<<
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_8, 8, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":866
 *                     dic[name] = None
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e8
 */
      __pyx_t_23 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_23) && PyErr_Occurred())) __PYX_ERR(0, 866, __pyx_L1_error)
      __pyx_t_2 = PyFloat_FromDouble(slow5_aux_get_float(__pyx_v_self->read, __pyx_t_23, (&__pyx_v_self->aux_get_err))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 866, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(PyFloat_CheckExact(__pyx_t_2))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 866, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->e8);
      __Pyx_DECREF(__pyx_v_self->e8);
      __pyx_v_self->e8 = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":867
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e8
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":868
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e8             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __pyx_v_self->e8;
        __Pyx_INCREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 868, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":867
 *             elif atype == 8:
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e8
 *                 else:
 */
        goto __pyx_L16;
      }

      /* "pyslow5.pyx":870
 *                     dic[name] = self.e8
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 9:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 870, __pyx_L1_error)
        }
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 870, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_12, __pyx_v_atype, __pyx_t_11, __pyx_t_13};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 870, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 870, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_11 = 0;
          __pyx_t_13 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_15, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 870, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":871
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 871, __pyx_L1_error)
      }
      __pyx_L16:;

      /* "pyslow5.pyx":865
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 8:             # <<<<<<<<<<<<<<
 *                 self.e8 = slow5_aux_get_float(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":872
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 9:             # <<<<<<<<<<<<<<
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_9, 9, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":873
 *                     dic[name] = None
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e9
 */
      __pyx_t_24 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_24) && PyErr_Occurred())) __PYX_ERR(0, 873, __pyx_L1_error)
      __pyx_v_self->e9 = slow5_aux_get_double(__pyx_v_self->read, __pyx_t_24, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":874
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e9
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":875
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e9             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->e9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 875, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":874
 *             elif atype == 9:
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e9
 *                 else:
 */
        goto __pyx_L17;
      }

      /* "pyslow5.pyx":877
 *                     dic[name] = self.e9
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 10:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 877, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 877, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_15, __pyx_t_11};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 877, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 877, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_15 = 0;
          __pyx_t_11 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 877, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":878
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 878, __pyx_L1_error)
      }
      __pyx_L17:;

      /* "pyslow5.pyx":872
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 9:             # <<<<<<<<<<<<<<
 *                 self.e9 = slow5_aux_get_double(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":879
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 10:             # <<<<<<<<<<<<<<
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_10, 10, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 879, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 879, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":880
 *                     dic[name] = None
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e10
 */
      __pyx_t_25 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_25) && PyErr_Occurred())) __PYX_ERR(0, 880, __pyx_L1_error)
      __pyx_v_self->e10 = slow5_aux_get_char(__pyx_v_self->read, __pyx_t_25, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":881
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e10
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":882
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e10             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        __pyx_t_2 = __Pyx_PyInt_From_char(__pyx_v_self->e10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":881
 *             elif atype == 10:
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e10
 *                 else:
 */
        goto __pyx_L18;
      }

      /* "pyslow5.pyx":884
 *                     dic[name] = self.e10
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 11:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 884, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 884, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_12, __pyx_t_15};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 884, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 884, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_12 = 0;
          __pyx_t_15 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 884, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":885
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 885, __pyx_L1_error)
      }
      __pyx_L18:;

      /* "pyslow5.pyx":879
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 10:             # <<<<<<<<<<<<<<
 *                 self.e10 = slow5_aux_get_char(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":886
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 11:             # <<<<<<<<<<<<<<
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_11, 11, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 886, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 886, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":887
 *                     dic[name] = None
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e11
 */
      __pyx_t_26 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_26) && PyErr_Occurred())) __PYX_ERR(0, 887, __pyx_L1_error)
      __pyx_v_self->e11 = slow5_aux_get_enum(__pyx_v_self->read, __pyx_t_26, (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":888
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e11
 *                 else:
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":889
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     dic[name] = self.e11             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 */
        __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->e11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 889, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_t_2) < 0)) __PYX_ERR(0, 889, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":888
 *             elif atype == 11:
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     dic[name] = self.e11
 *                 else:
 */
        goto __pyx_L19;
      }

      /* "pyslow5.pyx":891
 *                     dic[name] = self.e11
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 12:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_5 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_v_atype) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_atype);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 891, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_5, __pyx_t_7, __pyx_t_15};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_5, __pyx_t_7, __pyx_t_15};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_14, __pyx_t_15);
          __pyx_t_5 = 0;
          __pyx_t_7 = 0;
          __pyx_t_15 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":892
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 892, __pyx_L1_error)
      }
      __pyx_L19:;

      /* "pyslow5.pyx":886
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 11:             # <<<<<<<<<<<<<<
 *                 self.e11 = slow5_aux_get_enum(self.read, a_name, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":893
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                     dic[name] = None
 *             elif atype == 12:             # <<<<<<<<<<<<<<
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_12, 12, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 893, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":894
 *                     dic[name] = None
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_27 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_27) && PyErr_Occurred())) __PYX_ERR(0, 894, __pyx_L1_error)
      __pyx_v_self->e12 = slow5_aux_get_int8_array(__pyx_v_self->read, __pyx_t_27, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":895
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":896
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e12[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 896, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":897
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e12[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":898
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e12[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int8_t((__pyx_v_self->e12[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 898, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 898, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":899
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e12[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 899, __pyx_L1_error)

        /* "pyslow5.pyx":895
 *             elif atype == 12:
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L20;
      }

      /* "pyslow5.pyx":901
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 13:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 901, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 901, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 901, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 901, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_7 = 0;
          __pyx_t_13 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 901, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":902
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 902, __pyx_L1_error)
      }
      __pyx_L20:;

      /* "pyslow5.pyx":893
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                     dic[name] = None
 *             elif atype == 12:             # <<<<<<<<<<<<<<
 *                 self.e12 = slow5_aux_get_int8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":903
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 13:             # <<<<<<<<<<<<<<
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_13, 13, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 903, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 903, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":904
 *                     dic[name] = None
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_32 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_32) && PyErr_Occurred())) __PYX_ERR(0, 904, __pyx_L1_error)
      __pyx_v_self->e13 = slow5_aux_get_int16_array(__pyx_v_self->read, __pyx_t_32, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":905
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":906
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e13[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 906, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":907
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e13[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":908
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e13[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int16_t((__pyx_v_self->e13[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 908, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 908, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":909
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e13[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 909, __pyx_L1_error)

        /* "pyslow5.pyx":905
 *             elif atype == 13:
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":911
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 14:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 911, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 911, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 911, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 911, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 911, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 911, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 911, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 911, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 911, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_7);
          __pyx_t_11 = 0;
          __pyx_t_7 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 911, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 911, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":912
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 912, __pyx_L1_error)
      }
      __pyx_L23:;

      /* "pyslow5.pyx":903
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 13:             # <<<<<<<<<<<<<<
 *                 self.e13 = slow5_aux_get_int16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":913
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 14:             # <<<<<<<<<<<<<<
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_14, 14, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 913, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":914
 *                     dic[name] = None
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_33 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_33) && PyErr_Occurred())) __PYX_ERR(0, 914, __pyx_L1_error)
      __pyx_v_self->e14 = slow5_aux_get_int32_array(__pyx_v_self->read, __pyx_t_33, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":915
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":916
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e14[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 916, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":917
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e14[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":918
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e14[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int32_t((__pyx_v_self->e14[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 918, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 918, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":919
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e14[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 919, __pyx_L1_error)

        /* "pyslow5.pyx":915
 *             elif atype == 14:
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L26;
      }

      /* "pyslow5.pyx":921
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 15:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 921, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 921, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 921, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 921, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 921, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 921, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 921, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 921, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 921, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_5 = 0;
          __pyx_t_11 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 921, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 921, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":922
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 922, __pyx_L1_error)
      }
      __pyx_L26:;

      /* "pyslow5.pyx":913
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 14:             # <<<<<<<<<<<<<<
 *                 self.e14 = slow5_aux_get_int32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":923
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 15:             # <<<<<<<<<<<<<<
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_15, 15, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 923, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 923, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":924
 *                     dic[name] = None
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_34 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_34) && PyErr_Occurred())) __PYX_ERR(0, 924, __pyx_L1_error)
      __pyx_v_self->e15 = slow5_aux_get_int64_array(__pyx_v_self->read, __pyx_t_34, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":925
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":926
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e15[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 926, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":927
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e15[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":928
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e15[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_int64_t((__pyx_v_self->e15[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 928, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 928, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":929
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e15[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 929, __pyx_L1_error)

        /* "pyslow5.pyx":925
 *             elif atype == 15:
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L29;
      }

      /* "pyslow5.pyx":931
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 16:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 931, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 931, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 931, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 931, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 931, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 931, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 931, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 931, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 931, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_14, __pyx_t_5);
          __pyx_t_13 = 0;
          __pyx_t_5 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_7, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 931, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 931, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":932
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 932, __pyx_L1_error)
      }
      __pyx_L29:;

      /* "pyslow5.pyx":923
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 15:             # <<<<<<<<<<<<<<
 *                 self.e15 = slow5_aux_get_int64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":933
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 16:             # <<<<<<<<<<<<<<
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_16, 16, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 933, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 933, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":934
 *                     dic[name] = None
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_35 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_35) && PyErr_Occurred())) __PYX_ERR(0, 934, __pyx_L1_error)
      __pyx_v_self->e16 = slow5_aux_get_uint8_array(__pyx_v_self->read, __pyx_t_35, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":935
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":936
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e16[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":937
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e16[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":938
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e16[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint8_t((__pyx_v_self->e16[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 938, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 938, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":939
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e16[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 939, __pyx_L1_error)

        /* "pyslow5.pyx":935
 *             elif atype == 16:
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L32;
      }

      /* "pyslow5.pyx":941
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 17:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 941, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 941, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 941, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 941, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_7 = 0;
          __pyx_t_13 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 941, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 941, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":942
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 942, __pyx_L1_error)
      }
      __pyx_L32:;

      /* "pyslow5.pyx":933
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 16:             # <<<<<<<<<<<<<<
 *                 self.e16 = slow5_aux_get_uint8_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":943
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 17:             # <<<<<<<<<<<<<<
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_17, 17, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 943, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 943, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":944
 *                     dic[name] = None
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_36 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_36) && PyErr_Occurred())) __PYX_ERR(0, 944, __pyx_L1_error)
      __pyx_v_self->e17 = slow5_aux_get_uint16_array(__pyx_v_self->read, __pyx_t_36, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":945
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":946
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e17[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 946, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":947
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e17[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":948
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e17[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint16_t((__pyx_v_self->e17[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 948, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":949
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e17[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 949, __pyx_L1_error)

        /* "pyslow5.pyx":945
 *             elif atype == 17:
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L35;
      }

      /* "pyslow5.pyx":951
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 18:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 951, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 951, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 951, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 951, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_7);
          __pyx_t_11 = 0;
          __pyx_t_7 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 951, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":952
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 952, __pyx_L1_error)
      }
      __pyx_L35:;

      /* "pyslow5.pyx":943
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 17:             # <<<<<<<<<<<<<<
 *                 self.e17 = slow5_aux_get_uint16_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":953
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 18:             # <<<<<<<<<<<<<<
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_18, 18, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 953, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 953, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":954
 *                     dic[name] = None
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_37 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_37) && PyErr_Occurred())) __PYX_ERR(0, 954, __pyx_L1_error)
      __pyx_v_self->e18 = slow5_aux_get_uint32_array(__pyx_v_self->read, __pyx_t_37, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":955
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":956
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e18[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 956, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":957
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e18[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":958
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e18[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint32_t((__pyx_v_self->e18[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 958, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 958, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":959
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e18[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 959, __pyx_L1_error)

        /* "pyslow5.pyx":955
 *             elif atype == 18:
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L38;
      }

      /* "pyslow5.pyx":961
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 19:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 961, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 961, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 961, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 961, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_5 = 0;
          __pyx_t_11 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 961, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":962
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 962, __pyx_L1_error)
      }
      __pyx_L38:;

      /* "pyslow5.pyx":953
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 18:             # <<<<<<<<<<<<<<
 *                 self.e18 = slow5_aux_get_uint32_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":963
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 19:             # <<<<<<<<<<<<<<
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_19, 19, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 963, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 963, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":964
 *                     dic[name] = None
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_38 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_38) && PyErr_Occurred())) __PYX_ERR(0, 964, __pyx_L1_error)
      __pyx_v_self->e19 = slow5_aux_get_uint64_array(__pyx_v_self->read, __pyx_t_38, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":965
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":966
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e19[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 966, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":967
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e19[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":968
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e19[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = __Pyx_PyInt_From_uint64_t((__pyx_v_self->e19[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 968, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 968, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":969
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e19[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 969, __pyx_L1_error)

        /* "pyslow5.pyx":965
 *             elif atype == 19:
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L41;
      }

      /* "pyslow5.pyx":971
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 20:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 971, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 971, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_v_atype, __pyx_t_13, __pyx_t_5};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 971, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 971, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_14, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_14, __pyx_t_5);
          __pyx_t_13 = 0;
          __pyx_t_5 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_7, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 971, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 971, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":972
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 972, __pyx_L1_error)
      }
      __pyx_L41:;

      /* "pyslow5.pyx":963
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 19:             # <<<<<<<<<<<<<<
 *                 self.e19 = slow5_aux_get_uint64_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":973
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 20:             # <<<<<<<<<<<<<<
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_20, 20, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":974
 *                     dic[name] = None
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_39 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_39) && PyErr_Occurred())) __PYX_ERR(0, 974, __pyx_L1_error)
      __pyx_v_self->e20 = slow5_aux_get_float_array(__pyx_v_self->read, __pyx_t_39, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":975
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":976
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e20[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 976, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":977
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e20[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":978
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e20[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->e20[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 978, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 978, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":979
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e20[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 979, __pyx_L1_error)

        /* "pyslow5.pyx":975
 *             elif atype == 20:
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L44;
      }

      /* "pyslow5.pyx":981
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 21:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 981, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 981, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 981, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 981, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 981, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 981, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 981, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_atype, __pyx_t_7, __pyx_t_13};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 981, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 981, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_14, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_14, __pyx_t_13);
          __pyx_t_7 = 0;
          __pyx_t_13 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 981, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":982
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 982, __pyx_L1_error)
      }
      __pyx_L44:;

      /* "pyslow5.pyx":973
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 20:             # <<<<<<<<<<<<<<
 *                 self.e20 = slow5_aux_get_float_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":983
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 21:             # <<<<<<<<<<<<<<
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_21, 21, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 983, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 983, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":984
 *                     dic[name] = None
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_40 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_40) && PyErr_Occurred())) __PYX_ERR(0, 984, __pyx_L1_error)
      __pyx_v_self->e21 = slow5_aux_get_double_array(__pyx_v_self->read, __pyx_t_40, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":985
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":986
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e21[i])
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 986, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":987
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(self.e21[i])
 *                     dic[name] = l
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":988
 *                     l = []
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e21[i])             # <<<<<<<<<<<<<<
 *                     dic[name] = l
 *                 else:
 */
          __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->e21[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 988, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_2); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 988, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "pyslow5.pyx":989
 *                     for i in range(self.aux_get_len):
 *                         l.append(self.e21[i])
 *                     dic[name] = l             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_l) < 0)) __PYX_ERR(0, 989, __pyx_L1_error)

        /* "pyslow5.pyx":985
 *             elif atype == 21:
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     for i in range(self.aux_get_len):
 */
        goto __pyx_L47;
      }

      /* "pyslow5.pyx":991
 *                     dic[name] = l
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 22:
 */
      /*else*/ {
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 991, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 991, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 991, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 991, __pyx_L1_error)
        }
        __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 991, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 991, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 991, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_v_atype, __pyx_t_11, __pyx_t_7};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 991, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 991, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_7);
          __pyx_t_11 = 0;
          __pyx_t_7 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 991, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_2 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 991, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":992
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 992, __pyx_L1_error)
      }
      __pyx_L47:;

      /* "pyslow5.pyx":983
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 21:             # <<<<<<<<<<<<<<
 *                 self.e21 = slow5_aux_get_double_array(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":993
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_22, 22, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 993, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":994
 *                     dic[name] = None
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)             # <<<<<<<<<<<<<<
 *                 if self.aux_get_err == 0:
 *                     l = []
 */
      __pyx_t_41 = __Pyx_PyObject_AsString(__pyx_v_a_name); if (unlikely((!__pyx_t_41) && PyErr_Occurred())) __PYX_ERR(0, 994, __pyx_L1_error)
      __pyx_v_self->e22 = slow5_aux_get_string(__pyx_v_self->read, __pyx_t_41, (&__pyx_v_self->aux_get_len), (&__pyx_v_self->aux_get_err));

      /* "pyslow5.pyx":995
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     s = ""
 */
      __pyx_t_9 = ((__pyx_v_self->aux_get_err == 0) != 0);
      if (__pyx_t_9) {

        /* "pyslow5.pyx":996
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 *                     l = []             # <<<<<<<<<<<<<<
 *                     s = ""
 *                     for i in range(self.aux_get_len):
 */
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 996, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":997
 *                 if self.aux_get_err == 0:
 *                     l = []
 *                     s = ""             # <<<<<<<<<<<<<<
 *                     for i in range(self.aux_get_len):
 *                         l.append(chr(self.e22[i]))
 */
        __Pyx_INCREF(__pyx_kp_u_);
        __Pyx_XDECREF_SET(__pyx_v_s, __pyx_kp_u_);

        /* "pyslow5.pyx":998
 *                     l = []
 *                     s = ""
 *                     for i in range(self.aux_get_len):             # <<<<<<<<<<<<<<
 *                         l.append(chr(self.e22[i]))
 *                         s = "".join(l)
 */
        __pyx_t_28 = __pyx_v_self->aux_get_len;
        __pyx_t_29 = __pyx_t_28;
        for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {
          __pyx_v_i = __pyx_t_30;

          /* "pyslow5.pyx":999
 *                     s = ""
 *                     for i in range(self.aux_get_len):
 *                         l.append(chr(self.e22[i]))             # <<<<<<<<<<<<<<
 *                         s = "".join(l)
 *                     dic[name] = s
 */
          __pyx_t_2 = __Pyx_PyInt_From_char((__pyx_v_self->e22[__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 999, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 999, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_6); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 999, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "pyslow5.pyx":1000
 *                     for i in range(self.aux_get_len):
 *                         l.append(chr(self.e22[i]))
 *                         s = "".join(l)             # <<<<<<<<<<<<<<
 *                     dic[name] = s
 *                 else:
 */
          __pyx_t_6 = PyUnicode_Join(__pyx_kp_u_, __pyx_v_l); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1000, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_s, ((PyObject*)__pyx_t_6));
          __pyx_t_6 = 0;
        }

        /* "pyslow5.pyx":1001
 *                         l.append(chr(self.e22[i]))
 *                         s = "".join(l)
 *                     dic[name] = s             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, __pyx_v_s) < 0)) __PYX_ERR(0, 1001, __pyx_L1_error)

        /* "pyslow5.pyx":995
 *             elif atype == 22:
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:             # <<<<<<<<<<<<<<
 *                     l = []
 *                     s = ""
 */
        goto __pyx_L50;
      }

      /* "pyslow5.pyx":1003
 *                     dic[name] = s
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))             # <<<<<<<<<<<<<<
 *                     dic[name] = None
 *             elif atype == 22:
 */
      /*else*/ {
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1003, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1003, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1003, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (unlikely(__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1003, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1003, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1003, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_14 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_14 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1003, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_atype, __pyx_t_5, __pyx_t_11};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1003, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1003, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_atype);
          __Pyx_GIVEREF(__pyx_v_atype);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_14, __pyx_v_atype);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_14, __pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_14, __pyx_t_11);
          __pyx_t_5 = 0;
          __pyx_t_11 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1003, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        __pyx_t_6 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_15, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_12);
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1003, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyslow5.pyx":1004
 *                 else:
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None             # <<<<<<<<<<<<<<
 *             elif atype == 22:
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 */
        if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 1004, __pyx_L1_error)
      }
      __pyx_L50:;

      /* "pyslow5.pyx":993
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.e22 = slow5_aux_get_string(self.read, a_name, &self.aux_get_len, &self.aux_get_err)
 *                 if self.aux_get_err == 0:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":1005
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                 dic[name] = None
 */
    __pyx_t_6 = __Pyx_PyInt_EqObjC(__pyx_v_atype, __pyx_int_22, 22, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "pyslow5.pyx":1006
 *                     dic[name] = None
 *             elif atype == 22:
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])             # <<<<<<<<<<<<<<
 *                 dic[name] = None
 *             else:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_NOT_IMPLEMENTED_YET_get_aux_type, __pyx_n_s_format); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_15);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_15, function);
        }
      }
      __pyx_t_12 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_15, __pyx_t_13, __pyx_v_atype) : __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_v_atype);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      if (unlikely(__pyx_v_self->error_codes == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1006, __pyx_L1_error)
      }
      __pyx_t_13 = __Pyx_PyInt_From_int(__pyx_v_self->aux_get_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      __pyx_t_14 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_14 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_12, __pyx_t_15, __pyx_t_11};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1006, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_12, __pyx_t_15, __pyx_t_11};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_14, 3+__pyx_t_14); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1006, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(3+__pyx_t_14); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1006, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (__pyx_t_13) {
          __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_13); __pyx_t_13 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_14, __pyx_t_12);
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_14, __pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_14, __pyx_t_11);
        __pyx_t_12 = 0;
        __pyx_t_15 = 0;
        __pyx_t_11 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1006, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyslow5.pyx":1007
 *             elif atype == 22:
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                 dic[name] = None             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.debug("get_read_aux atype not known, skipping: {}".format(atype))
 */
      if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_name, Py_None) < 0)) __PYX_ERR(0, 1007, __pyx_L1_error)

      /* "pyslow5.pyx":1005
 *                     self.logger.debug("get_aux_types {} self.aux_get_err is {}: {}".format(atype, self.aux_get_err, self.error_codes[self.aux_get_err]))
 *                     dic[name] = None
 *             elif atype == 22:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("NOT IMPLEMENTED YET: get_aux_types {} self.aux_get_err is {}: {}".format(atype), self.aux_get_err, self.error_codes[self.aux_get_err])
 *                 dic[name] = None
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":1009
 *                 dic[name] = None
 *             else:
 *                 self.logger.debug("get_read_aux atype not known, skipping: {}".format(atype))             # <<<<<<<<<<<<<<
 * 
 *         return dic
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1009, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_aux_atype_not_known_ski, __pyx_n_s_format); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1009, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_15 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_15)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_15);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
        }
      }
      __pyx_t_5 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_15, __pyx_v_atype) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_atype);
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1009, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_11)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_6 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_11, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1009, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __pyx_L7:;

    /* "pyslow5.pyx":807
 *         dic = {}
 *         # treat all input as a list to simplify
 *         for name, atype in zip(aux_names, aux_types):             # <<<<<<<<<<<<<<
 *             a_name = str.encode(name)
 *             if atype == 0:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1011
 *                 self.logger.debug("get_read_aux atype not known, skipping: {}".format(atype))
 * 
 *         return dic             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_dic);
  __pyx_r = __pyx_v_dic;
  goto __pyx_L0;

  /* "pyslow5.pyx":797
 * 
 * 
 *     def _get_seq_read_aux(self, aux_names, aux_types):             # <<<<<<<<<<<<<<
 *         '''
 *         get aux field for read - when doing sequential read, because of separate self.read (vs rec) call
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("pyslow5.Open._get_seq_read_aux", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dic);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_atype);
  __Pyx_XDECREF(__pyx_v_a_name);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1014
 * 
 * 
 *     def get_read_ids(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get all read_ids from index
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_21get_read_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7pyslow5_4Open_20get_read_ids[] = "\n        get all read_ids from index\n        if no index, build it then get read_ids\n        No idea why this is needed but whatever\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_21get_read_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_read_ids (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_20get_read_ids(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_20get_read_ids(struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_v_rids = NULL;
  int __pyx_v_ret;
  uint64_t __pyx_v_i;
  uint64_t __pyx_v_rids_len;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  uint64_t __pyx_t_11;
  uint64_t __pyx_t_12;
  uint64_t __pyx_t_13;
  char *__pyx_t_14;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_read_ids", 0);
  __Pyx_TraceCall("get_read_ids", __pyx_f[0], 1014, 0, __PYX_ERR(0, 1014, __pyx_L1_error));

  /* "pyslow5.pyx":1020
 *         No idea why this is needed but whatever
 *         '''
 *         rids = []             # <<<<<<<<<<<<<<
 *         if not self.index_state:
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1020, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rids = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1021
 *         '''
 *         rids = []
 *         if not self.index_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 */
  __pyx_t_2 = ((!(__pyx_v_self->index_state != 0)) != 0);
  if (__pyx_t_2) {

    /* "pyslow5.pyx":1022
 *         rids = []
 *         if not self.index_state:
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))             # <<<<<<<<<<<<<<
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 *             self.logger.debug("Creating/loading index...")
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FILE_mode, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_self->path, __pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1022, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_self->path, __pyx_v_self->mode};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1022, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1022, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->path);
      __Pyx_GIVEREF(__pyx_v_self->path);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_self->path);
      __Pyx_INCREF(__pyx_v_self->mode);
      __Pyx_GIVEREF(__pyx_v_self->mode);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_self->mode);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1022, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1024
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 *             self.logger.debug("Creating/loading index...")             # <<<<<<<<<<<<<<
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1024, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_Creating_loading_index) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Creating_loading_index);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1024, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1025
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)             # <<<<<<<<<<<<<<
 *             if ret != 0:
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 */
    __pyx_v_ret = slow5_idx_load(__pyx_v_self->s5);

    /* "pyslow5.pyx":1026
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:             # <<<<<<<<<<<<<<
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 */
    __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_2) {

      /* "pyslow5.pyx":1027
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.index_state = True
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_idx_load_return_not_0, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(__pyx_v_self->error_codes == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1027, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = __Pyx_PyDict_GetItem(__pyx_v_self->error_codes, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_8, __pyx_t_9};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1027, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_8, __pyx_t_9};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1027, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1027, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_7, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_7, __pyx_t_9);
        __pyx_t_8 = 0;
        __pyx_t_9 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1027, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1026
 *             self.logger.debug("Creating/loading index...")
 *             ret = slow5_idx_load(self.s5)
 *             if ret != 0:             # <<<<<<<<<<<<<<
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 */
      goto __pyx_L4;
    }

    /* "pyslow5.pyx":1029
 *                 self.logger.warning("slow5_idx_load return not 0: {}: {}".format(ret, self.error_codes[ret]))
 *             else:
 *                 self.index_state = True             # <<<<<<<<<<<<<<
 *         self.rids = slow5_get_rids(self.s5, &self.rids_len)
 * 
 */
    /*else*/ {
      __pyx_v_self->index_state = 1;
    }
    __pyx_L4:;

    /* "pyslow5.pyx":1021
 *         '''
 *         rids = []
 *         if not self.index_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("FILE: {}, mode: {}".format(self.path, self.mode))
 *             # self.logger.debug("FILE: {}, mode: {}".format(self.path, self.m))
 */
  }

  /* "pyslow5.pyx":1030
 *             else:
 *                 self.index_state = True
 *         self.rids = slow5_get_rids(self.s5, &self.rids_len)             # <<<<<<<<<<<<<<
 * 
 *         if self.rids_len == 0:
 */
  __pyx_v_self->rids = slow5_get_rids(__pyx_v_self->s5, (&__pyx_v_self->rids_len));

  /* "pyslow5.pyx":1032
 *         self.rids = slow5_get_rids(self.s5, &self.rids_len)
 * 
 *         if self.rids_len == 0:             # <<<<<<<<<<<<<<
 *             self.logger.error("get_read_ids: length of read_ids is 0, something when wrong")
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->rids_len == 0) != 0);
  if (__pyx_t_2) {

    /* "pyslow5.pyx":1033
 * 
 *         if self.rids_len == 0:
 *             self.logger.error("get_read_ids: length of read_ids is 0, something when wrong")             # <<<<<<<<<<<<<<
 * 
 *         if self.rids is NULL:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1033, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_get_read_ids_length_of_read_ids) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_get_read_ids_length_of_read_ids);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1033, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1032
 *         self.rids = slow5_get_rids(self.s5, &self.rids_len)
 * 
 *         if self.rids_len == 0:             # <<<<<<<<<<<<<<
 *             self.logger.error("get_read_ids: length of read_ids is 0, something when wrong")
 * 
 */
  }

  /* "pyslow5.pyx":1035
 *             self.logger.error("get_read_ids: length of read_ids is 0, something when wrong")
 * 
 *         if self.rids is NULL:             # <<<<<<<<<<<<<<
 *             self.logger.error("get_read_ids: returned rids is NULL, something when wrong")
 *         else:
 */
  __pyx_t_2 = ((__pyx_v_self->rids == NULL) != 0);
  if (__pyx_t_2) {

    /* "pyslow5.pyx":1036
 * 
 *         if self.rids is NULL:
 *             self.logger.error("get_read_ids: returned rids is NULL, something when wrong")             # <<<<<<<<<<<<<<
 *         else:
 *             for i in range(self.rids_len):
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1036, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_kp_u_get_read_ids_returned_rids_is_NU) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_get_read_ids_returned_rids_is_NU);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1036, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1035
 *             self.logger.error("get_read_ids: length of read_ids is 0, something when wrong")
 * 
 *         if self.rids is NULL:             # <<<<<<<<<<<<<<
 *             self.logger.error("get_read_ids: returned rids is NULL, something when wrong")
 *         else:
 */
    goto __pyx_L6;
  }

  /* "pyslow5.pyx":1038
 *             self.logger.error("get_read_ids: returned rids is NULL, something when wrong")
 *         else:
 *             for i in range(self.rids_len):             # <<<<<<<<<<<<<<
 *                 rids.append(self.rids[i].decode())
 *         rids_len = self.rids_len
 */
  /*else*/ {
    __pyx_t_11 = __pyx_v_self->rids_len;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "pyslow5.pyx":1039
 *         else:
 *             for i in range(self.rids_len):
 *                 rids.append(self.rids[i].decode())             # <<<<<<<<<<<<<<
 *         rids_len = self.rids_len
 *         return rids, rids_len
 */
      __pyx_t_14 = (__pyx_v_self->rids[__pyx_v_i]);
      __pyx_t_1 = __Pyx_decode_c_string(__pyx_t_14, 0, strlen(__pyx_t_14), NULL, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1039, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_15 = __Pyx_PyList_Append(__pyx_v_rids, __pyx_t_1); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 1039, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
  }
  __pyx_L6:;

  /* "pyslow5.pyx":1040
 *             for i in range(self.rids_len):
 *                 rids.append(self.rids[i].decode())
 *         rids_len = self.rids_len             # <<<<<<<<<<<<<<
 *         return rids, rids_len
 * 
 */
  __pyx_t_11 = __pyx_v_self->rids_len;
  __pyx_v_rids_len = __pyx_t_11;

  /* "pyslow5.pyx":1041
 *                 rids.append(self.rids[i].decode())
 *         rids_len = self.rids_len
 *         return rids, rids_len             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint64_t(__pyx_v_rids_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_rids);
  __Pyx_GIVEREF(__pyx_v_rids);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_rids);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyslow5.pyx":1014
 * 
 * 
 *     def get_read_ids(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get all read_ids from index
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pyslow5.Open.get_read_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rids);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1044
 * 
 * 
 *     def get_read(self, read_id, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         return self._get_read(read_id, pA, aux)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_23get_read(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7pyslow5_4Open_23get_read(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_id = 0;
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_read (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_id,&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_False);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_id)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_read") < 0)) __PYX_ERR(0, 1044, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_read_id = values[0];
    __pyx_v_pA = values[1];
    __pyx_v_aux = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_read", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1044, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_22get_read(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_id, __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_22get_read(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_id, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_read", 0);
  __Pyx_TraceCall("get_read", __pyx_f[0], 1044, 0, __PYX_ERR(0, 1044, __pyx_L1_error));

  /* "pyslow5.pyx":1045
 * 
 *     def get_read(self, read_id, pA=False, aux=None):
 *         return self._get_read(read_id, pA, aux)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_read); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1045, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_read_id, __pyx_v_pA, __pyx_v_aux};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1045, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_read_id, __pyx_v_pA, __pyx_v_aux};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1045, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1045, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_read_id);
    __Pyx_GIVEREF(__pyx_v_read_id);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_read_id);
    __Pyx_INCREF(__pyx_v_pA);
    __Pyx_GIVEREF(__pyx_v_pA);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_pA);
    __Pyx_INCREF(__pyx_v_aux);
    __Pyx_GIVEREF(__pyx_v_aux);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_aux);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1045, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyslow5.pyx":1044
 * 
 * 
 *     def get_read(self, read_id, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         return self._get_read(read_id, pA, aux)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyslow5.Open.get_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_26generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":1048
 * 
 * 
 *     def seq_reads(self, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for sequential reading of slow5 file
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_25seq_reads(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_24seq_reads[] = "\n        returns generator for sequential reading of slow5 file\n        for pA and aux, see _get_read\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_25seq_reads(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seq_reads (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_False);
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "seq_reads") < 0)) __PYX_ERR(0, 1048, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_pA = values[0];
    __pyx_v_aux = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("seq_reads", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1048, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.seq_reads", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_24seq_reads(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_24seq_reads(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seq_reads", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *)__pyx_tp_new_7pyslow5___pyx_scope_struct_4_seq_reads(__pyx_ptype_7pyslow5___pyx_scope_struct_4_seq_reads, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1048, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_pA = __pyx_v_pA;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
  __pyx_cur_scope->__pyx_v_aux = __pyx_v_aux;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_26generator2, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_seq_reads, __pyx_n_s_Open_seq_reads, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 1048, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open.seq_reads", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_26generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  PyObject *(*__pyx_t_14)(PyObject *);
  PyObject *(*__pyx_t_15)(PyObject *);
  int __pyx_t_16;
  char *__pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seq_reads", 0);
  __Pyx_TraceCall("seq_reads", __pyx_f[0], 1048, 0, __PYX_ERR(0, 1048, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L31_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1048, __pyx_L1_error)

  /* "pyslow5.pyx":1053
 *         for pA and aux, see _get_read
 *         '''
 *         aux_dic = {}             # <<<<<<<<<<<<<<
 *         row = {}
 *         ret = 0
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_aux_dic = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1054
 *         '''
 *         aux_dic = {}
 *         row = {}             # <<<<<<<<<<<<<<
 *         ret = 0
 *         # While loops check ret of previous read for errors as fail safe
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_row = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1055
 *         aux_dic = {}
 *         row = {}
 *         ret = 0             # <<<<<<<<<<<<<<
 *         # While loops check ret of previous read for errors as fail safe
 *         while ret >= 0:
 */
  __pyx_cur_scope->__pyx_v_ret = 0;

  /* "pyslow5.pyx":1057
 *         ret = 0
 *         # While loops check ret of previous read for errors as fail safe
 *         while ret >= 0:             # <<<<<<<<<<<<<<
 *             start_slow5_get_next = time.time()
 *             ret = slow5_get_next(&self.read, self.s5)
 */
  while (1) {
    __pyx_t_2 = ((__pyx_cur_scope->__pyx_v_ret >= 0) != 0);
    if (!__pyx_t_2) break;

    /* "pyslow5.pyx":1058
 *         # While loops check ret of previous read for errors as fail safe
 *         while ret >= 0:
 *             start_slow5_get_next = time.time()             # <<<<<<<<<<<<<<
 *             ret = slow5_get_next(&self.read, self.s5)
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1058, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1058, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1058, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_start_slow5_get_next);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_start_slow5_get_next, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1059
 *         while ret >= 0:
 *             start_slow5_get_next = time.time()
 *             ret = slow5_get_next(&self.read, self.s5)             # <<<<<<<<<<<<<<
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)
 * 
 */
    __pyx_cur_scope->__pyx_v_ret = slow5_get_next((&__pyx_cur_scope->__pyx_v_self->read), __pyx_cur_scope->__pyx_v_self->s5);

    /* "pyslow5.pyx":1060
 *             start_slow5_get_next = time.time()
 *             ret = slow5_get_next(&self.read, self.s5)
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)             # <<<<<<<<<<<<<<
 * 
 *             self.logger.debug("slow5_get_next return: {}".format(ret))
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_cur_scope->__pyx_v_start_slow5_get_next); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Add(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyFloat_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next);
    __pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1062
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)
 * 
 *             self.logger.debug("slow5_get_next return: {}".format(ret))             # <<<<<<<<<<<<<<
 *             # check for EOF or other errors
 *             if ret < 0:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_return, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1064
 *             self.logger.debug("slow5_get_next return: {}".format(ret))
 *             # check for EOF or other errors
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 if ret == -1:
 *                     self.logger.debug("slow5_get_next reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 */
    __pyx_t_2 = ((__pyx_cur_scope->__pyx_v_ret < 0) != 0);
    if (__pyx_t_2) {

      /* "pyslow5.pyx":1065
 *             # check for EOF or other errors
 *             if ret < 0:
 *                 if ret == -1:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("slow5_get_next reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 *                 else:
 */
      __pyx_t_2 = ((__pyx_cur_scope->__pyx_v_ret == -1L) != 0);
      if (__pyx_t_2) {

        /* "pyslow5.pyx":1066
 *             if ret < 0:
 *                 if ret == -1:
 *                     self.logger.debug("slow5_get_next reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1066, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_reached_end_of_fi, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1066, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1066, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (unlikely(__pyx_cur_scope->__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1066, __pyx_L1_error)
        }
        __pyx_t_7 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1066, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->error_codes, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1066, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_9 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_9 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_t_8};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1066, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_t_8};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1066, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1066, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_8);
          __pyx_t_6 = 0;
          __pyx_t_8 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1066, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1066, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1065
 *             # check for EOF or other errors
 *             if ret < 0:
 *                 if ret == -1:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("slow5_get_next reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 *                 else:
 */
        goto __pyx_L7;
      }

      /* "pyslow5.pyx":1068
 *                     self.logger.debug("slow5_get_next reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 *                 else:
 *                     self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 * 
 *                 break
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1068, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_error_code, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1068, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_10 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1068, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (unlikely(__pyx_cur_scope->__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1068, __pyx_L1_error)
        }
        __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1068, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->error_codes, __pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1068, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = NULL;
        __pyx_t_9 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_9 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_10, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1068, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_10, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1068, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1068, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_8) {
            __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8); __pyx_t_8 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_9, __pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_9, __pyx_t_6);
          __pyx_t_10 = 0;
          __pyx_t_6 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1068, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __pyx_L7:;

      /* "pyslow5.pyx":1070
 *                     self.logger.error("slow5_get_next error code: {}: {}".format(ret, self.error_codes[ret]))
 * 
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             aux_dic = {}
 */
      goto __pyx_L5_break;

      /* "pyslow5.pyx":1064
 *             self.logger.debug("slow5_get_next return: {}".format(ret))
 *             # check for EOF or other errors
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 if ret == -1:
 *                     self.logger.debug("slow5_get_next reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 */
    }

    /* "pyslow5.pyx":1072
 *                 break
 * 
 *             aux_dic = {}             # <<<<<<<<<<<<<<
 *             row = {}
 *             # get aux fields
 */
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1072, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1073
 * 
 *             aux_dic = {}
 *             row = {}             # <<<<<<<<<<<<<<
 *             # get aux fields
 *             if aux is not None:
 */
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1073, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_row);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_row, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1075
 *             row = {}
 *             # get aux fields
 *             if aux is not None:             # <<<<<<<<<<<<<<
 *                 if not self.aux_names or not self.aux_types:
 *                     self.aux_names = self.get_aux_names()
 */
    __pyx_t_2 = (__pyx_cur_scope->__pyx_v_aux != Py_None);
    __pyx_t_11 = (__pyx_t_2 != 0);
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1076
 *             # get aux fields
 *             if aux is not None:
 *                 if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                     self.aux_names = self.get_aux_names()
 *                     self.aux_types = self.get_aux_types()
 */
      __pyx_t_2 = (__pyx_cur_scope->__pyx_v_self->aux_names != Py_None)&&(PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_self->aux_names) != 0);
      __pyx_t_12 = ((!__pyx_t_2) != 0);
      if (!__pyx_t_12) {
      } else {
        __pyx_t_11 = __pyx_t_12;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_12 = (__pyx_cur_scope->__pyx_v_self->aux_types != Py_None)&&(PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_self->aux_types) != 0);
      __pyx_t_2 = ((!__pyx_t_12) != 0);
      __pyx_t_11 = __pyx_t_2;
      __pyx_L10_bool_binop_done:;
      if (__pyx_t_11) {

        /* "pyslow5.pyx":1077
 *             if aux is not None:
 *                 if not self.aux_names or not self.aux_types:
 *                     self.aux_names = self.get_aux_names()             # <<<<<<<<<<<<<<
 *                     self.aux_types = self.get_aux_types()
 *                 if type(aux) is str:
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_aux_names); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1077, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1077, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 1077, __pyx_L1_error)
        __Pyx_GIVEREF(__pyx_t_1);
        __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->aux_names);
        __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->aux_names);
        __pyx_cur_scope->__pyx_v_self->aux_names = ((PyObject*)__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1078
 *                 if not self.aux_names or not self.aux_types:
 *                     self.aux_names = self.get_aux_names()
 *                     self.aux_types = self.get_aux_types()             # <<<<<<<<<<<<<<
 *                 if type(aux) is str:
 *                     if aux == "all":
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_aux_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1078, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 1078, __pyx_L1_error)
        __Pyx_GIVEREF(__pyx_t_1);
        __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->aux_types);
        __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->aux_types);
        __pyx_cur_scope->__pyx_v_self->aux_types = ((PyObject*)__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1076
 *             # get aux fields
 *             if aux is not None:
 *                 if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                     self.aux_names = self.get_aux_names()
 *                     self.aux_types = self.get_aux_types()
 */
      }

      /* "pyslow5.pyx":1079
 *                     self.aux_names = self.get_aux_names()
 *                     self.aux_types = self.get_aux_types()
 *                 if type(aux) is str:             # <<<<<<<<<<<<<<
 *                     if aux == "all":
 *                         aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 */
      __pyx_t_11 = (((PyObject *)Py_TYPE(__pyx_cur_scope->__pyx_v_aux)) == ((PyObject *)(&PyUnicode_Type)));
      __pyx_t_2 = (__pyx_t_11 != 0);
      if (__pyx_t_2) {

        /* "pyslow5.pyx":1080
 *                     self.aux_types = self.get_aux_types()
 *                 if type(aux) is str:
 *                     if aux == "all":             # <<<<<<<<<<<<<<
 *                         aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 *                     else:
 */
        __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_aux, __pyx_n_u_all, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1080, __pyx_L1_error)
        if (__pyx_t_2) {

          /* "pyslow5.pyx":1081
 *                 if type(aux) is str:
 *                     if aux == "all":
 *                         aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)             # <<<<<<<<<<<<<<
 *                     else:
 *                         found_single_aux = False
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_seq_read_aux); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1081, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = NULL;
          __pyx_t_9 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_4)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
              __pyx_t_9 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_3)) {
            PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_cur_scope->__pyx_v_self->aux_names, __pyx_cur_scope->__pyx_v_self->aux_types};
            __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1081, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_GOTREF(__pyx_t_1);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
            PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_cur_scope->__pyx_v_self->aux_names, __pyx_cur_scope->__pyx_v_self->aux_types};
            __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1081, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_GOTREF(__pyx_t_1);
          } else
          #endif
          {
            __pyx_t_5 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1081, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            if (__pyx_t_4) {
              __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
            }
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
            PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_9, __pyx_cur_scope->__pyx_v_self->aux_names);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
            PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_9, __pyx_cur_scope->__pyx_v_self->aux_types);
            __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1081, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
          __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __pyx_t_1 = 0;

          /* "pyslow5.pyx":1080
 *                     self.aux_types = self.get_aux_types()
 *                 if type(aux) is str:
 *                     if aux == "all":             # <<<<<<<<<<<<<<
 *                         aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 *                     else:
 */
          goto __pyx_L13;
        }

        /* "pyslow5.pyx":1083
 *                         aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 *                     else:
 *                         found_single_aux = False             # <<<<<<<<<<<<<<
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n == aux:
 */
        /*else*/ {
          __pyx_cur_scope->__pyx_v_found_single_aux = 0;

          /* "pyslow5.pyx":1084
 *                     else:
 *                         found_single_aux = False
 *                         for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                             if n == aux:
 *                                 found_single_aux = True
 */
          __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1084, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_cur_scope->__pyx_v_self->aux_types);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1084, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
            __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_13 = 0;
            __pyx_t_14 = NULL;
          } else {
            __pyx_t_13 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1084, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_14 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1084, __pyx_L1_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          for (;;) {
            if (likely(!__pyx_t_14)) {
              if (likely(PyList_CheckExact(__pyx_t_1))) {
                if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_1)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_13); __Pyx_INCREF(__pyx_t_3); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1084, __pyx_L1_error)
                #else
                __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1084, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_3);
                #endif
              } else {
                if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_13); __Pyx_INCREF(__pyx_t_3); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1084, __pyx_L1_error)
                #else
                __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1084, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_3);
                #endif
              }
            } else {
              __pyx_t_3 = __pyx_t_14(__pyx_t_1);
              if (unlikely(!__pyx_t_3)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 1084, __pyx_L1_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_3);
            }
            if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
              PyObject* sequence = __pyx_t_3;
              Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(0, 1084, __pyx_L1_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              if (likely(PyTuple_CheckExact(sequence))) {
                __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
                __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
              } else {
                __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
                __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
              }
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_4);
              #else
              __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1084, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1084, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            } else {
              Py_ssize_t index = -1;
              __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1084, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __pyx_t_15 = Py_TYPE(__pyx_t_7)->tp_iternext;
              index = 0; __pyx_t_5 = __pyx_t_15(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L16_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_5);
              index = 1; __pyx_t_4 = __pyx_t_15(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L16_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_4);
              if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_7), 2) < 0) __PYX_ERR(0, 1084, __pyx_L1_error)
              __pyx_t_15 = NULL;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              goto __pyx_L17_unpacking_done;
              __pyx_L16_unpacking_failed:;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_15 = NULL;
              if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
              __PYX_ERR(0, 1084, __pyx_L1_error)
              __pyx_L17_unpacking_done:;
            }
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n, __pyx_t_5);
            __Pyx_GIVEREF(__pyx_t_5);
            __pyx_t_5 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_4);
            __pyx_t_4 = 0;

            /* "pyslow5.pyx":1085
 *                         found_single_aux = False
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n == aux:             # <<<<<<<<<<<<<<
 *                                 found_single_aux = True
 *                                 aux_dic = self._get_seq_read_aux([n], [t])
 */
            __pyx_t_3 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_n, __pyx_cur_scope->__pyx_v_aux, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1085, __pyx_L1_error)
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1085, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (__pyx_t_2) {

              /* "pyslow5.pyx":1086
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n == aux:
 *                                 found_single_aux = True             # <<<<<<<<<<<<<<
 *                                 aux_dic = self._get_seq_read_aux([n], [t])
 *                                 break
 */
              __pyx_cur_scope->__pyx_v_found_single_aux = 1;

              /* "pyslow5.pyx":1087
 *                             if n == aux:
 *                                 found_single_aux = True
 *                                 aux_dic = self._get_seq_read_aux([n], [t])             # <<<<<<<<<<<<<<
 *                                 break
 *                         if not found_single_aux:
 */
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_seq_read_aux); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1087, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1087, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_n);
              __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_n);
              PyList_SET_ITEM(__pyx_t_5, 0, __pyx_cur_scope->__pyx_v_n);
              __pyx_t_7 = PyList_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1087, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_t);
              __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_t);
              PyList_SET_ITEM(__pyx_t_7, 0, __pyx_cur_scope->__pyx_v_t);
              __pyx_t_6 = NULL;
              __pyx_t_9 = 0;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_6)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_6);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                  __pyx_t_9 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_5, __pyx_t_7};
                __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1087, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_5, __pyx_t_7};
                __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1087, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              } else
              #endif
              {
                __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1087, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                if (__pyx_t_6) {
                  __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
                }
                __Pyx_GIVEREF(__pyx_t_5);
                PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_5);
                __Pyx_GIVEREF(__pyx_t_7);
                PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_7);
                __pyx_t_5 = 0;
                __pyx_t_7 = 0;
                __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1087, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              }
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
              __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_3);
              __Pyx_GIVEREF(__pyx_t_3);
              __pyx_t_3 = 0;

              /* "pyslow5.pyx":1088
 *                                 found_single_aux = True
 *                                 aux_dic = self._get_seq_read_aux([n], [t])
 *                                 break             # <<<<<<<<<<<<<<
 *                         if not found_single_aux:
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))
 */
              goto __pyx_L15_break;

              /* "pyslow5.pyx":1085
 *                         found_single_aux = False
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n == aux:             # <<<<<<<<<<<<<<
 *                                 found_single_aux = True
 *                                 aux_dic = self._get_seq_read_aux([n], [t])
 */
            }

            /* "pyslow5.pyx":1084
 *                     else:
 *                         found_single_aux = False
 *                         for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                             if n == aux:
 *                                 found_single_aux = True
 */
          }
          __pyx_L15_break:;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pyslow5.pyx":1089
 *                                 aux_dic = self._get_seq_read_aux([n], [t])
 *                                 break
 *                         if not found_single_aux:             # <<<<<<<<<<<<<<
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                             aux_dic.update({aux: None})
 */
          __pyx_t_2 = ((!(__pyx_cur_scope->__pyx_v_found_single_aux != 0)) != 0);
          if (__pyx_t_2) {

            /* "pyslow5.pyx":1090
 *                                 break
 *                         if not found_single_aux:
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))             # <<<<<<<<<<<<<<
 *                             aux_dic.update({aux: None})
 *                 elif type(aux) is list:
 */
            __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1090, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_unknown_aux_name, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1090, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_7 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
              if (likely(__pyx_t_7)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_7);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_10, function);
              }
            }
            __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_7, __pyx_cur_scope->__pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_cur_scope->__pyx_v_aux);
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1090, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __pyx_t_10 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
              if (likely(__pyx_t_10)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
                __Pyx_INCREF(__pyx_t_10);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_3, function);
              }
            }
            __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1090, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyslow5.pyx":1091
 *                         if not found_single_aux:
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                             aux_dic.update({aux: None})             # <<<<<<<<<<<<<<
 *                 elif type(aux) is list:
 *                     n_list = []
 */
            __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1091, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1091, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            if (PyDict_SetItem(__pyx_t_4, __pyx_cur_scope->__pyx_v_aux, Py_None) < 0) __PYX_ERR(0, 1091, __pyx_L1_error)
            __pyx_t_10 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
              if (likely(__pyx_t_10)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
                __Pyx_INCREF(__pyx_t_10);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_3, function);
              }
            }
            __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1091, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyslow5.pyx":1089
 *                                 aux_dic = self._get_seq_read_aux([n], [t])
 *                                 break
 *                         if not found_single_aux:             # <<<<<<<<<<<<<<
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                             aux_dic.update({aux: None})
 */
          }
        }
        __pyx_L13:;

        /* "pyslow5.pyx":1079
 *                     self.aux_names = self.get_aux_names()
 *                     self.aux_types = self.get_aux_types()
 *                 if type(aux) is str:             # <<<<<<<<<<<<<<
 *                     if aux == "all":
 *                         aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 */
        goto __pyx_L12;
      }

      /* "pyslow5.pyx":1092
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                             aux_dic.update({aux: None})
 *                 elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                     n_list = []
 *                     t_list = []
 */
      __pyx_t_2 = (((PyObject *)Py_TYPE(__pyx_cur_scope->__pyx_v_aux)) == ((PyObject *)(&PyList_Type)));
      __pyx_t_11 = (__pyx_t_2 != 0);
      if (__pyx_t_11) {

        /* "pyslow5.pyx":1093
 *                             aux_dic.update({aux: None})
 *                 elif type(aux) is list:
 *                     n_list = []             # <<<<<<<<<<<<<<
 *                     t_list = []
 *                     for n, t in zip(self.aux_names, self.aux_types):
 */
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1093, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n_list);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n_list, ((PyObject*)__pyx_t_1));
        __Pyx_GIVEREF(__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1094
 *                 elif type(aux) is list:
 *                     n_list = []
 *                     t_list = []             # <<<<<<<<<<<<<<
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n in aux:
 */
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1094, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t_list);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t_list, ((PyObject*)__pyx_t_1));
        __Pyx_GIVEREF(__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1095
 *                     n_list = []
 *                     t_list = []
 *                     for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                         if n in aux:
 *                             n_list.append(n)
 */
        __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1095, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
        __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_cur_scope->__pyx_v_self->aux_names);
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
        __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_cur_scope->__pyx_v_self->aux_types);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1095, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
          __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_13 = 0;
          __pyx_t_14 = NULL;
        } else {
          __pyx_t_13 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1095, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_14 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1095, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        for (;;) {
          if (likely(!__pyx_t_14)) {
            if (likely(PyList_CheckExact(__pyx_t_1))) {
              if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_13); __Pyx_INCREF(__pyx_t_3); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1095, __pyx_L1_error)
              #else
              __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1095, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            } else {
              if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_13); __Pyx_INCREF(__pyx_t_3); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1095, __pyx_L1_error)
              #else
              __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1095, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            }
          } else {
            __pyx_t_3 = __pyx_t_14(__pyx_t_1);
            if (unlikely(!__pyx_t_3)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 1095, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_3);
          }
          if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
            PyObject* sequence = __pyx_t_3;
            Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
            if (unlikely(size != 2)) {
              if (size > 2) __Pyx_RaiseTooManyValuesError(2);
              else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
              __PYX_ERR(0, 1095, __pyx_L1_error)
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            if (likely(PyTuple_CheckExact(sequence))) {
              __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
              __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
            } else {
              __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
              __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
            }
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_10);
            #else
            __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1095, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1095, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            #endif
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          } else {
            Py_ssize_t index = -1;
            __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1095, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_15 = Py_TYPE(__pyx_t_7)->tp_iternext;
            index = 0; __pyx_t_4 = __pyx_t_15(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L22_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_4);
            index = 1; __pyx_t_10 = __pyx_t_15(__pyx_t_7); if (unlikely(!__pyx_t_10)) goto __pyx_L22_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_10);
            if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_7), 2) < 0) __PYX_ERR(0, 1095, __pyx_L1_error)
            __pyx_t_15 = NULL;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            goto __pyx_L23_unpacking_done;
            __pyx_L22_unpacking_failed:;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_15 = NULL;
            if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
            __PYX_ERR(0, 1095, __pyx_L1_error)
            __pyx_L23_unpacking_done:;
          }
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_10);
          __pyx_t_10 = 0;

          /* "pyslow5.pyx":1096
 *                     t_list = []
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n in aux:             # <<<<<<<<<<<<<<
 *                             n_list.append(n)
 *                             t_list.append(t)
 */
          __pyx_t_11 = (__Pyx_PySequence_ContainsTF(__pyx_cur_scope->__pyx_v_n, __pyx_cur_scope->__pyx_v_aux, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1096, __pyx_L1_error)
          __pyx_t_2 = (__pyx_t_11 != 0);
          if (__pyx_t_2) {

            /* "pyslow5.pyx":1097
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n in aux:
 *                             n_list.append(n)             # <<<<<<<<<<<<<<
 *                             t_list.append(t)
 * 
 */
            __pyx_t_16 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_n); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 1097, __pyx_L1_error)

            /* "pyslow5.pyx":1098
 *                         if n in aux:
 *                             n_list.append(n)
 *                             t_list.append(t)             # <<<<<<<<<<<<<<
 * 
 *                     aux_dic = self._get_seq_read_aux(n_list, t_list)
 */
            __pyx_t_16 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_t_list, __pyx_cur_scope->__pyx_v_t); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 1098, __pyx_L1_error)

            /* "pyslow5.pyx":1096
 *                     t_list = []
 *                     for n, t in zip(self.aux_names, self.aux_types):
 *                         if n in aux:             # <<<<<<<<<<<<<<
 *                             n_list.append(n)
 *                             t_list.append(t)
 */
          }

          /* "pyslow5.pyx":1095
 *                     n_list = []
 *                     t_list = []
 *                     for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                         if n in aux:
 *                             n_list.append(n)
 */
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1100
 *                             t_list.append(t)
 * 
 *                     aux_dic = self._get_seq_read_aux(n_list, t_list)             # <<<<<<<<<<<<<<
 *                     # check for items in given list that do not exist
 *                     n_set = set(n_list)
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_seq_read_aux); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1100, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = NULL;
        __pyx_t_9 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_9 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_t_list};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1100, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_t_list};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1100, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else
        #endif
        {
          __pyx_t_4 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1100, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          if (__pyx_t_10) {
            __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_10); __pyx_t_10 = NULL;
          }
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_n_list);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_n_list);
          PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_9, __pyx_cur_scope->__pyx_v_n_list);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_t_list);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_t_list);
          PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_9, __pyx_cur_scope->__pyx_v_t_list);
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1100, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
        __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1102
 *                     aux_dic = self._get_seq_read_aux(n_list, t_list)
 *                     # check for items in given list that do not exist
 *                     n_set = set(n_list)             # <<<<<<<<<<<<<<
 *                     aux_set = set(aux)
 *                     if len(aux_set.difference(n_set)) > 0:
 */
        __pyx_t_1 = PySet_New(__pyx_cur_scope->__pyx_v_n_list); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1102, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n_set);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n_set, ((PyObject*)__pyx_t_1));
        __Pyx_GIVEREF(__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1103
 *                     # check for items in given list that do not exist
 *                     n_set = set(n_list)
 *                     aux_set = set(aux)             # <<<<<<<<<<<<<<
 *                     if len(aux_set.difference(n_set)) > 0:
 *                         for i in aux_set.difference(n_set):
 */
        __pyx_t_1 = PySet_New(__pyx_cur_scope->__pyx_v_aux); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1103, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_aux_set);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_aux_set, ((PyObject*)__pyx_t_1));
        __Pyx_GIVEREF(__pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1104
 *                     n_set = set(n_list)
 *                     aux_set = set(aux)
 *                     if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                         for i in aux_set.difference(n_set):
 *                             self.logger.warning("get_read unknown aux name: {}".format(i))
 */
        __pyx_t_1 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_cur_scope->__pyx_v_aux_set, __pyx_cur_scope->__pyx_v_n_set); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1104, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_13 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1104, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_2 = ((__pyx_t_13 > 0) != 0);
        if (__pyx_t_2) {

          /* "pyslow5.pyx":1105
 *                     aux_set = set(aux)
 *                     if len(aux_set.difference(n_set)) > 0:
 *                         for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                             self.logger.warning("get_read unknown aux name: {}".format(i))
 *                             aux_dic.update({i: None})
 */
          __pyx_t_1 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_cur_scope->__pyx_v_aux_set, __pyx_cur_scope->__pyx_v_n_set); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1105, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
            __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_13 = 0;
            __pyx_t_14 = NULL;
          } else {
            __pyx_t_13 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1105, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_14 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1105, __pyx_L1_error)
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          for (;;) {
            if (likely(!__pyx_t_14)) {
              if (likely(PyList_CheckExact(__pyx_t_3))) {
                if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_3)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_1); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1105, __pyx_L1_error)
                #else
                __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1105, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_1);
                #endif
              } else {
                if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_1); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1105, __pyx_L1_error)
                #else
                __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1105, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_1);
                #endif
              }
            } else {
              __pyx_t_1 = __pyx_t_14(__pyx_t_3);
              if (unlikely(!__pyx_t_1)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 1105, __pyx_L1_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_1);
            }
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_1);
            __pyx_t_1 = 0;

            /* "pyslow5.pyx":1106
 *                     if len(aux_set.difference(n_set)) > 0:
 *                         for i in aux_set.difference(n_set):
 *                             self.logger.warning("get_read unknown aux name: {}".format(i))             # <<<<<<<<<<<<<<
 *                             aux_dic.update({i: None})
 * 
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1106, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_unknown_aux_name, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1106, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_5 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
              if (likely(__pyx_t_5)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_7, function);
              }
            }
            __pyx_t_10 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_5, __pyx_cur_scope->__pyx_v_i) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_cur_scope->__pyx_v_i);
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1106, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_7 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_7)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_7);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_7, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyslow5.pyx":1107
 *                         for i in aux_set.difference(n_set):
 *                             self.logger.warning("get_read unknown aux name: {}".format(i))
 *                             aux_dic.update({i: None})             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1107, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1107, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            if (PyDict_SetItem(__pyx_t_10, __pyx_cur_scope->__pyx_v_i, Py_None) < 0) __PYX_ERR(0, 1107, __pyx_L1_error)
            __pyx_t_7 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_7)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_7);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_7, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1107, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyslow5.pyx":1105
 *                     aux_set = set(aux)
 *                     if len(aux_set.difference(n_set)) > 0:
 *                         for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                             self.logger.warning("get_read unknown aux name: {}".format(i))
 *                             aux_dic.update({i: None})
 */
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "pyslow5.pyx":1104
 *                     n_set = set(n_list)
 *                     aux_set = set(aux)
 *                     if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                         for i in aux_set.difference(n_set):
 *                             self.logger.warning("get_read unknown aux name: {}".format(i))
 */
        }

        /* "pyslow5.pyx":1092
 *                             self.logger.warning("get_read unknown aux name: {}".format(aux))
 *                             aux_dic.update({aux: None})
 *                 elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                     n_list = []
 *                     t_list = []
 */
        goto __pyx_L12;
      }

      /* "pyslow5.pyx":1110
 * 
 *                 else:
 *                     self.logger.debug("get_read aux type unknown, accepts str or list: {}".format(aux))             # <<<<<<<<<<<<<<
 * 
 *             # Get read data
 */
      /*else*/ {
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1110, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_read_aux_type_unknown_accept, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1110, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_7, __pyx_cur_scope->__pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_cur_scope->__pyx_v_aux);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1110, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1110, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __pyx_L12:;

      /* "pyslow5.pyx":1075
 *             row = {}
 *             # get aux fields
 *             if aux is not None:             # <<<<<<<<<<<<<<
 *                 if not self.aux_names or not self.aux_types:
 *                     self.aux_names = self.get_aux_names()
 */
    }

    /* "pyslow5.pyx":1113
 * 
 *             # Get read data
 *             if type(self.read.read_id) is bytes:             # <<<<<<<<<<<<<<
 *                 row['read_id'] = self.read.read_id.decode()
 *             else:
 */
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->read->read_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (((PyObject *)Py_TYPE(__pyx_t_3)) == ((PyObject *)(&PyBytes_Type)));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_11 = (__pyx_t_2 != 0);
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1114
 *             # Get read data
 *             if type(self.read.read_id) is bytes:
 *                 row['read_id'] = self.read.read_id.decode()             # <<<<<<<<<<<<<<
 *             else:
 *                 row['read_id'] = self.read.read_id
 */
      __pyx_t_17 = __pyx_cur_scope->__pyx_v_self->read->read_id;
      __pyx_t_3 = __Pyx_decode_c_string(__pyx_t_17, 0, strlen(__pyx_t_17), NULL, NULL, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_read_id, __pyx_t_3) < 0)) __PYX_ERR(0, 1114, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1113
 * 
 *             # Get read data
 *             if type(self.read.read_id) is bytes:             # <<<<<<<<<<<<<<
 *                 row['read_id'] = self.read.read_id.decode()
 *             else:
 */
      goto __pyx_L28;
    }

    /* "pyslow5.pyx":1116
 *                 row['read_id'] = self.read.read_id.decode()
 *             else:
 *                 row['read_id'] = self.read.read_id             # <<<<<<<<<<<<<<
 *             row['read_group'] = self.read.read_group
 *             row['digitisation'] = self.read.digitisation
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->read->read_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_read_id, __pyx_t_3) < 0)) __PYX_ERR(0, 1116, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_L28:;

    /* "pyslow5.pyx":1117
 *             else:
 *                 row['read_id'] = self.read.read_id
 *             row['read_group'] = self.read.read_group             # <<<<<<<<<<<<<<
 *             row['digitisation'] = self.read.digitisation
 *             row['offset'] = self.read.offset
 */
    __pyx_t_3 = __Pyx_PyInt_From_uint32_t(__pyx_cur_scope->__pyx_v_self->read->read_group); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_read_group, __pyx_t_3) < 0)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1118
 *                 row['read_id'] = self.read.read_id
 *             row['read_group'] = self.read.read_group
 *             row['digitisation'] = self.read.digitisation             # <<<<<<<<<<<<<<
 *             row['offset'] = self.read.offset
 *             row['range'] = self.read.range
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->digitisation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_digitisation, __pyx_t_3) < 0)) __PYX_ERR(0, 1118, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1119
 *             row['read_group'] = self.read.read_group
 *             row['digitisation'] = self.read.digitisation
 *             row['offset'] = self.read.offset             # <<<<<<<<<<<<<<
 *             row['range'] = self.read.range
 *             row['sampling_rate'] = self.read.sampling_rate
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_offset, __pyx_t_3) < 0)) __PYX_ERR(0, 1119, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1120
 *             row['digitisation'] = self.read.digitisation
 *             row['offset'] = self.read.offset
 *             row['range'] = self.read.range             # <<<<<<<<<<<<<<
 *             row['sampling_rate'] = self.read.sampling_rate
 *             row['len_raw_signal'] = self.read.len_raw_signal
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->range); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1120, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_range, __pyx_t_3) < 0)) __PYX_ERR(0, 1120, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1121
 *             row['offset'] = self.read.offset
 *             row['range'] = self.read.range
 *             row['sampling_rate'] = self.read.sampling_rate             # <<<<<<<<<<<<<<
 *             row['len_raw_signal'] = self.read.len_raw_signal
 *             self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->sampling_rate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_sampling_rate, __pyx_t_3) < 0)) __PYX_ERR(0, 1121, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1122
 *             row['range'] = self.read.range
 *             row['sampling_rate'] = self.read.sampling_rate
 *             row['len_raw_signal'] = self.read.len_raw_signal             # <<<<<<<<<<<<<<
 *             self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 *             signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 */
    __pyx_t_3 = __Pyx_PyInt_From_uint64_t(__pyx_cur_scope->__pyx_v_self->read->len_raw_signal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_len_raw_signal, __pyx_t_3) < 0)) __PYX_ERR(0, 1122, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1123
 *             row['sampling_rate'] = self.read.sampling_rate
 *             row['len_raw_signal'] = self.read.len_raw_signal
 *             self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal             # <<<<<<<<<<<<<<
 *             signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 *                         np.NPY_INT16, <void *> self.read.raw_signal))
 */
    (__pyx_cur_scope->__pyx_v_self->shape_seq[0]) = ((npy_intp)__pyx_cur_scope->__pyx_v_self->read->len_raw_signal);

    /* "pyslow5.pyx":1124
 *             row['len_raw_signal'] = self.read.len_raw_signal
 *             self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 *             signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,             # <<<<<<<<<<<<<<
 *                         np.NPY_INT16, <void *> self.read.raw_signal))
 *             np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1125
 *             self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 *             signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 *                         np.NPY_INT16, <void *> self.read.raw_signal))             # <<<<<<<<<<<<<<
 *             np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *             row['signal'] = signal
 */
    __pyx_t_1 = PyArray_SimpleNewFromData(1, __pyx_cur_scope->__pyx_v_self->shape_seq, NPY_INT16, ((void *)__pyx_cur_scope->__pyx_v_self->read->raw_signal)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_signal);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_signal, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyslow5.pyx":1126
 *             signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 *                         np.NPY_INT16, <void *> self.read.raw_signal))
 *             np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)             # <<<<<<<<<<<<<<
 *             row['signal'] = signal
 * 
 */
    if (!(likely(((__pyx_cur_scope->__pyx_v_signal) == Py_None) || likely(__Pyx_TypeTest(__pyx_cur_scope->__pyx_v_signal, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1126, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_signal, __pyx_n_s_flags); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_From_int(NPY_OWNDATA); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyNumber_Or(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    PyArray_UpdateFlags(((PyArrayObject *)__pyx_cur_scope->__pyx_v_signal), __pyx_t_9);

    /* "pyslow5.pyx":1127
 *                         np.NPY_INT16, <void *> self.read.raw_signal))
 *             np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *             row['signal'] = signal             # <<<<<<<<<<<<<<
 * 
 *             # if pA=True, convert signal to pA
 */
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_signal, __pyx_cur_scope->__pyx_v_signal) < 0)) __PYX_ERR(0, 1127, __pyx_L1_error)

    /* "pyslow5.pyx":1130
 * 
 *             # if pA=True, convert signal to pA
 *             if pA:             # <<<<<<<<<<<<<<
 *                 row['signal'] = self._convert_to_pA(row)
 *             # if aux data update main dic
 */
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_pA); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1130, __pyx_L1_error)
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1131
 *             # if pA=True, convert signal to pA
 *             if pA:
 *                 row['signal'] = self._convert_to_pA(row)             # <<<<<<<<<<<<<<
 *             # if aux data update main dic
 *             if aux_dic:
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_convert_to_pA); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_cur_scope->__pyx_v_row) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_cur_scope->__pyx_v_row);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_signal, __pyx_t_1) < 0)) __PYX_ERR(0, 1131, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1130
 * 
 *             # if pA=True, convert signal to pA
 *             if pA:             # <<<<<<<<<<<<<<
 *                 row['signal'] = self._convert_to_pA(row)
 *             # if aux data update main dic
 */
    }

    /* "pyslow5.pyx":1133
 *                 row['signal'] = self._convert_to_pA(row)
 *             # if aux data update main dic
 *             if aux_dic:             # <<<<<<<<<<<<<<
 *                 row.update(aux_dic)
 *             self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - start_slow5_get_next)
 */
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_aux_dic); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1133, __pyx_L1_error)
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1134
 *             # if aux data update main dic
 *             if aux_dic:
 *                 row.update(aux_dic)             # <<<<<<<<<<<<<<
 *             self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - start_slow5_get_next)
 *             yield row
 */
      __pyx_t_1 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_cur_scope->__pyx_v_row, __pyx_cur_scope->__pyx_v_aux_dic); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1133
 *                 row['signal'] = self._convert_to_pA(row)
 *             # if aux data update main dic
 *             if aux_dic:             # <<<<<<<<<<<<<<
 *                 row.update(aux_dic)
 *             self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - start_slow5_get_next)
 */
    }

    /* "pyslow5.pyx":1135
 *             if aux_dic:
 *                 row.update(aux_dic)
 *             self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - start_slow5_get_next)             # <<<<<<<<<<<<<<
 *             yield row
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_Subtract(__pyx_t_1, __pyx_cur_scope->__pyx_v_start_slow5_get_next); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Add(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!(likely(PyFloat_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads);
    __pyx_cur_scope->__pyx_v_self->total_time_yield_reads = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1136
 *                 row.update(aux_dic)
 *             self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - start_slow5_get_next)
 *             yield row             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_row);
    __pyx_r = __pyx_cur_scope->__pyx_v_row;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_TraceReturn(__pyx_r, 0);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L31_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1136, __pyx_L1_error)
  }
  __pyx_L5_break:;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyslow5.pyx":1048
 * 
 * 
 *     def seq_reads(self, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for sequential reading of slow5 file
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("seq_reads", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_29generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":1139
 * 
 * 
 *     def seq_reads_multi(self, threads=4, batchsize=4096, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for sequential reading of slow5 file
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_28seq_reads_multi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_27seq_reads_multi[] = "\n        returns generator for sequential reading of slow5 file\n        for pA and aux, see _get_read\n        threads: number of threads to use\n        batchsize: Number of reads to process with thread pool in parallel\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_28seq_reads_multi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_threads = 0;
  PyObject *__pyx_v_batchsize = 0;
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seq_reads_multi (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_threads,&__pyx_n_s_batchsize,&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[4] = {0,0,0,0};
    values[0] = ((PyObject *)__pyx_int_4);
    values[1] = ((PyObject *)__pyx_int_4096);
    values[2] = ((PyObject *)Py_False);
    values[3] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_threads);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_batchsize);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "seq_reads_multi") < 0)) __PYX_ERR(0, 1139, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_threads = values[0];
    __pyx_v_batchsize = values[1];
    __pyx_v_pA = values[2];
    __pyx_v_aux = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("seq_reads_multi", 0, 0, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1139, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.seq_reads_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_27seq_reads_multi(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_threads, __pyx_v_batchsize, __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_27seq_reads_multi(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seq_reads_multi", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *)__pyx_tp_new_7pyslow5___pyx_scope_struct_5_seq_reads_multi(__pyx_ptype_7pyslow5___pyx_scope_struct_5_seq_reads_multi, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1139, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_threads = __pyx_v_threads;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_threads);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_threads);
  __pyx_cur_scope->__pyx_v_batchsize = __pyx_v_batchsize;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_batchsize);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_batchsize);
  __pyx_cur_scope->__pyx_v_pA = __pyx_v_pA;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
  __pyx_cur_scope->__pyx_v_aux = __pyx_v_aux;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_29generator3, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_seq_reads_multi, __pyx_n_s_Open_seq_reads_multi, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 1139, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open.seq_reads_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_29generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  PyObject *(*__pyx_t_17)(PyObject *);
  PyObject *(*__pyx_t_18)(PyObject *);
  int __pyx_t_19;
  char *__pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seq_reads_multi", 0);
  __Pyx_TraceCall("seq_reads_multi", __pyx_f[0], 1139, 0, __PYX_ERR(0, 1139, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L33_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1139, __pyx_L1_error)

  /* "pyslow5.pyx":1146
 *         batchsize: Number of reads to process with thread pool in parallel
 *         '''
 *         aux_dic = {}             # <<<<<<<<<<<<<<
 *         row = {}
 *         ret = 1
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_aux_dic = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1147
 *         '''
 *         aux_dic = {}
 *         row = {}             # <<<<<<<<<<<<<<
 *         ret = 1
 *         timedic = {"aux_total_time": 0,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_row = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1148
 *         aux_dic = {}
 *         row = {}
 *         ret = 1             # <<<<<<<<<<<<<<
 *         timedic = {"aux_total_time": 0,
 *                    "primary_total_time": 0,
 */
  __pyx_cur_scope->__pyx_v_ret = 1;

  /* "pyslow5.pyx":1149
 *         row = {}
 *         ret = 1
 *         timedic = {"aux_total_time": 0,             # <<<<<<<<<<<<<<
 *                    "primary_total_time": 0,
 *                    "pA_total_time": 0,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aux_total_time, __pyx_int_0) < 0) __PYX_ERR(0, 1149, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_primary_total_time, __pyx_int_0) < 0) __PYX_ERR(0, 1149, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_pA_total_time, __pyx_int_0) < 0) __PYX_ERR(0, 1149, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_signal_total_time, __pyx_int_0) < 0) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_timedic = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1155
 * 
 *         # While loops check ret of previous read for errors as fail safe
 *         while ret > 0:             # <<<<<<<<<<<<<<
 *             start_slow5_get_next = time.time()
 *             ret = slow5_get_next_batch(&self.trec, self.s5, batchsize, threads)
 */
  while (1) {
    __pyx_t_2 = ((__pyx_cur_scope->__pyx_v_ret > 0) != 0);
    if (!__pyx_t_2) break;

    /* "pyslow5.pyx":1156
 *         # While loops check ret of previous read for errors as fail safe
 *         while ret > 0:
 *             start_slow5_get_next = time.time()             # <<<<<<<<<<<<<<
 *             ret = slow5_get_next_batch(&self.trec, self.s5, batchsize, threads)
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_start_slow5_get_next);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_start_slow5_get_next, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1157
 *         while ret > 0:
 *             start_slow5_get_next = time.time()
 *             ret = slow5_get_next_batch(&self.trec, self.s5, batchsize, threads)             # <<<<<<<<<<<<<<
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)
 *             self.logger.debug("slow5_get_next_multi return: {}".format(ret))
 */
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_cur_scope->__pyx_v_batchsize); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1157, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_cur_scope->__pyx_v_threads); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1157, __pyx_L1_error)
    __pyx_cur_scope->__pyx_v_ret = slow5_get_next_batch((&__pyx_cur_scope->__pyx_v_self->trec), __pyx_cur_scope->__pyx_v_self->s5, __pyx_t_5, __pyx_t_6);

    /* "pyslow5.pyx":1158
 *             start_slow5_get_next = time.time()
 *             ret = slow5_get_next_batch(&self.trec, self.s5, batchsize, threads)
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)             # <<<<<<<<<<<<<<
 *             self.logger.debug("slow5_get_next_multi return: {}".format(ret))
 *             # check for EOF or other errors
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_cur_scope->__pyx_v_start_slow5_get_next); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Add(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyFloat_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next);
    __pyx_cur_scope->__pyx_v_self->total_time_slow5_get_next = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1159
 *             ret = slow5_get_next_batch(&self.trec, self.s5, batchsize, threads)
 *             self.total_time_slow5_get_next = self.total_time_slow5_get_next + (time.time() - start_slow5_get_next)
 *             self.logger.debug("slow5_get_next_multi return: {}".format(ret))             # <<<<<<<<<<<<<<
 *             # check for EOF or other errors
 *             if ret < 0:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_return, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_4 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1161
 *             self.logger.debug("slow5_get_next_multi return: {}".format(ret))
 *             # check for EOF or other errors
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 if ret == -1:
 *                     self.logger.debug("slow5_get_next_multi reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 */
    __pyx_t_2 = ((__pyx_cur_scope->__pyx_v_ret < 0) != 0);
    if (__pyx_t_2) {

      /* "pyslow5.pyx":1162
 *             # check for EOF or other errors
 *             if ret < 0:
 *                 if ret == -1:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("slow5_get_next_multi reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 *                 else:
 */
      __pyx_t_2 = ((__pyx_cur_scope->__pyx_v_ret == -1L) != 0);
      if (__pyx_t_2) {

        /* "pyslow5.pyx":1163
 *             if ret < 0:
 *                 if ret == -1:
 *                     self.logger.debug("slow5_get_next_multi reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.error("slow5_get_next_multi error code: {}: {}".format(ret, self.error_codes[ret]))
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_reached_end, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (unlikely(__pyx_cur_scope->__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1163, __pyx_L1_error)
        }
        __pyx_t_9 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->error_codes, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_8, __pyx_t_10};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1163, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_8, __pyx_t_10};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1163, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1163, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (__pyx_t_9) {
            __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_6, __pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_6, __pyx_t_10);
          __pyx_t_8 = 0;
          __pyx_t_10 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1163, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1162
 *             # check for EOF or other errors
 *             if ret < 0:
 *                 if ret == -1:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("slow5_get_next_multi reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 *                 else:
 */
        goto __pyx_L7;
      }

      /* "pyslow5.pyx":1165
 *                     self.logger.debug("slow5_get_next_multi reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 *                 else:
 *                     self.logger.error("slow5_get_next_multi error code: {}: {}".format(ret, self.error_codes[ret]))             # <<<<<<<<<<<<<<
 * 
 *                 break
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_error_code, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_cur_scope->__pyx_v_self->error_codes == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1165, __pyx_L1_error)
        }
        __pyx_t_10 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_self->error_codes, __pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_t_11, __pyx_t_8};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_t_11, __pyx_t_8};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_10) {
            __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_10); __pyx_t_10 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_6, __pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_6, __pyx_t_8);
          __pyx_t_11 = 0;
          __pyx_t_8 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __pyx_L7:;

      /* "pyslow5.pyx":1167
 *                     self.logger.error("slow5_get_next_multi error code: {}: {}".format(ret, self.error_codes[ret]))
 * 
 *                 break             # <<<<<<<<<<<<<<
 *             for i in range(ret):
 *                 python_parse_read_start = time.time()
 */
      goto __pyx_L5_break;

      /* "pyslow5.pyx":1161
 *             self.logger.debug("slow5_get_next_multi return: {}".format(ret))
 *             # check for EOF or other errors
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 if ret == -1:
 *                     self.logger.debug("slow5_get_next_multi reached end of file (EOF)(-1): {}: {}".format(ret, self.error_codes[ret]))
 */
    }

    /* "pyslow5.pyx":1168
 * 
 *                 break
 *             for i in range(ret):             # <<<<<<<<<<<<<<
 *                 python_parse_read_start = time.time()
 *                 self.read = self.trec[i]
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_12 = 0;
      __pyx_t_13 = NULL;
    } else {
      __pyx_t_12 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_13 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1168, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_13)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_12); __Pyx_INCREF(__pyx_t_3); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 1168, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_12); __Pyx_INCREF(__pyx_t_3); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 1168, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_13(__pyx_t_1);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1168, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1169
 *                 break
 *             for i in range(ret):
 *                 python_parse_read_start = time.time()             # <<<<<<<<<<<<<<
 *                 self.read = self.trec[i]
 *                 aux_dic = {}
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1169, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1169, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1169, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_python_parse_read_start);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_python_parse_read_start, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1170
 *             for i in range(ret):
 *                 python_parse_read_start = time.time()
 *                 self.read = self.trec[i]             # <<<<<<<<<<<<<<
 *                 aux_dic = {}
 *                 row = {}
 */
      __pyx_t_14 = __Pyx_PyIndex_AsSsize_t(__pyx_cur_scope->__pyx_v_i); if (unlikely((__pyx_t_14 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1170, __pyx_L1_error)
      __pyx_cur_scope->__pyx_v_self->read = (__pyx_cur_scope->__pyx_v_self->trec[__pyx_t_14]);

      /* "pyslow5.pyx":1171
 *                 python_parse_read_start = time.time()
 *                 self.read = self.trec[i]
 *                 aux_dic = {}             # <<<<<<<<<<<<<<
 *                 row = {}
 *                 # get aux fields
 */
      __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1171, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1172
 *                 self.read = self.trec[i]
 *                 aux_dic = {}
 *                 row = {}             # <<<<<<<<<<<<<<
 *                 # get aux fields
 *                 aux_time_start = time.time()
 */
      __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1172, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_row);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_row, ((PyObject*)__pyx_t_3));
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1174
 *                 row = {}
 *                 # get aux fields
 *                 aux_time_start = time.time()             # <<<<<<<<<<<<<<
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1174, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1174, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1174, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_aux_time_start);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_aux_time_start, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1175
 *                 # get aux fields
 *                 aux_time_start = time.time()
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()
 */
      __pyx_t_2 = (__pyx_cur_scope->__pyx_v_aux != Py_None);
      __pyx_t_15 = (__pyx_t_2 != 0);
      if (__pyx_t_15) {

        /* "pyslow5.pyx":1176
 *                 aux_time_start = time.time()
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 */
        __pyx_t_2 = (__pyx_cur_scope->__pyx_v_self->aux_names != Py_None)&&(PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_self->aux_names) != 0);
        __pyx_t_16 = ((!__pyx_t_2) != 0);
        if (!__pyx_t_16) {
        } else {
          __pyx_t_15 = __pyx_t_16;
          goto __pyx_L12_bool_binop_done;
        }
        __pyx_t_16 = (__pyx_cur_scope->__pyx_v_self->aux_types != Py_None)&&(PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_self->aux_types) != 0);
        __pyx_t_2 = ((!__pyx_t_16) != 0);
        __pyx_t_15 = __pyx_t_2;
        __pyx_L12_bool_binop_done:;
        if (__pyx_t_15) {

          /* "pyslow5.pyx":1177
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()             # <<<<<<<<<<<<<<
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_aux_names); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1177, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_7 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_7)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_7);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
            }
          }
          __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1177, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 1177, __pyx_L1_error)
          __Pyx_GIVEREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __pyx_cur_scope->__pyx_v_self->aux_names = ((PyObject*)__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1178
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()             # <<<<<<<<<<<<<<
 *                     if type(aux) is str:
 *                         if aux == "all":
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_aux_types); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1178, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_7 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_7)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_7);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
            }
          }
          __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1178, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 1178, __pyx_L1_error)
          __Pyx_GIVEREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __pyx_cur_scope->__pyx_v_self->aux_types = ((PyObject*)__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1176
 *                 aux_time_start = time.time()
 *                 if aux is not None:
 *                     if not self.aux_names or not self.aux_types:             # <<<<<<<<<<<<<<
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 */
        }

        /* "pyslow5.pyx":1179
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:             # <<<<<<<<<<<<<<
 *                         if aux == "all":
 *                             aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 */
        __pyx_t_15 = (((PyObject *)Py_TYPE(__pyx_cur_scope->__pyx_v_aux)) == ((PyObject *)(&PyUnicode_Type)));
        __pyx_t_2 = (__pyx_t_15 != 0);
        if (__pyx_t_2) {

          /* "pyslow5.pyx":1180
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:
 *                         if aux == "all":             # <<<<<<<<<<<<<<
 *                             aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 *                         else:
 */
          __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_aux, __pyx_n_u_all, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1180, __pyx_L1_error)
          if (__pyx_t_2) {

            /* "pyslow5.pyx":1181
 *                     if type(aux) is str:
 *                         if aux == "all":
 *                             aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)             # <<<<<<<<<<<<<<
 *                         else:
 *                             found_single_aux = False
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_seq_read_aux); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1181, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_7 = NULL;
            __pyx_t_6 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_7)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_7);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
                __pyx_t_6 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_4)) {
              PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_cur_scope->__pyx_v_self->aux_names, __pyx_cur_scope->__pyx_v_self->aux_types};
              __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_GOTREF(__pyx_t_3);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
              PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_cur_scope->__pyx_v_self->aux_names, __pyx_cur_scope->__pyx_v_self->aux_types};
              __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_GOTREF(__pyx_t_3);
            } else
            #endif
            {
              __pyx_t_9 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1181, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_9);
              if (__pyx_t_7) {
                __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
              }
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
              __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
              PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_6, __pyx_cur_scope->__pyx_v_self->aux_names);
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
              __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
              PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_6, __pyx_cur_scope->__pyx_v_self->aux_types);
              __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
            __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_3);
            __pyx_t_3 = 0;

            /* "pyslow5.pyx":1180
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:
 *                         if aux == "all":             # <<<<<<<<<<<<<<
 *                             aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 *                         else:
 */
            goto __pyx_L15;
          }

          /* "pyslow5.pyx":1183
 *                             aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 *                         else:
 *                             found_single_aux = False             # <<<<<<<<<<<<<<
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:
 */
          /*else*/ {
            __pyx_cur_scope->__pyx_v_found_single_aux = 0;

            /* "pyslow5.pyx":1184
 *                         else:
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                                 if n == aux:
 *                                     found_single_aux = True
 */
            __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1184, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
            PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_cur_scope->__pyx_v_self->aux_names);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
            PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_cur_scope->__pyx_v_self->aux_types);
            __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1184, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
              __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_14 = 0;
              __pyx_t_17 = NULL;
            } else {
              __pyx_t_14 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1184, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_17 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1184, __pyx_L1_error)
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            for (;;) {
              if (likely(!__pyx_t_17)) {
                if (likely(PyList_CheckExact(__pyx_t_3))) {
                  if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_3)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_14); __Pyx_INCREF(__pyx_t_4); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1184, __pyx_L1_error)
                  #else
                  __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1184, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  #endif
                } else {
                  if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_14); __Pyx_INCREF(__pyx_t_4); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1184, __pyx_L1_error)
                  #else
                  __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1184, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  #endif
                }
              } else {
                __pyx_t_4 = __pyx_t_17(__pyx_t_3);
                if (unlikely(!__pyx_t_4)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(0, 1184, __pyx_L1_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_4);
              }
              if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
                PyObject* sequence = __pyx_t_4;
                Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
                if (unlikely(size != 2)) {
                  if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                  else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                  __PYX_ERR(0, 1184, __pyx_L1_error)
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                if (likely(PyTuple_CheckExact(sequence))) {
                  __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
                  __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
                } else {
                  __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
                  __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
                }
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(__pyx_t_7);
                #else
                __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1184, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_9);
                __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1184, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_7);
                #endif
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              } else {
                Py_ssize_t index = -1;
                __pyx_t_8 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1184, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __pyx_t_18 = Py_TYPE(__pyx_t_8)->tp_iternext;
                index = 0; __pyx_t_9 = __pyx_t_18(__pyx_t_8); if (unlikely(!__pyx_t_9)) goto __pyx_L18_unpacking_failed;
                __Pyx_GOTREF(__pyx_t_9);
                index = 1; __pyx_t_7 = __pyx_t_18(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L18_unpacking_failed;
                __Pyx_GOTREF(__pyx_t_7);
                if (__Pyx_IternextUnpackEndCheck(__pyx_t_18(__pyx_t_8), 2) < 0) __PYX_ERR(0, 1184, __pyx_L1_error)
                __pyx_t_18 = NULL;
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                goto __pyx_L19_unpacking_done;
                __pyx_L18_unpacking_failed:;
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                __pyx_t_18 = NULL;
                if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
                __PYX_ERR(0, 1184, __pyx_L1_error)
                __pyx_L19_unpacking_done:;
              }
              __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n);
              __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n, __pyx_t_9);
              __Pyx_GIVEREF(__pyx_t_9);
              __pyx_t_9 = 0;
              __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
              __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_7);
              __Pyx_GIVEREF(__pyx_t_7);
              __pyx_t_7 = 0;

              /* "pyslow5.pyx":1185
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:             # <<<<<<<<<<<<<<
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_seq_read_aux([n], [t])
 */
              __pyx_t_4 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_n, __pyx_cur_scope->__pyx_v_aux, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1185, __pyx_L1_error)
              __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1185, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (__pyx_t_2) {

                /* "pyslow5.pyx":1186
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:
 *                                     found_single_aux = True             # <<<<<<<<<<<<<<
 *                                     aux_dic = self._get_seq_read_aux([n], [t])
 *                                     break
 */
                __pyx_cur_scope->__pyx_v_found_single_aux = 1;

                /* "pyslow5.pyx":1187
 *                                 if n == aux:
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_seq_read_aux([n], [t])             # <<<<<<<<<<<<<<
 *                                     break
 *                             if not found_single_aux:
 */
                __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_seq_read_aux); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1187, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_7);
                __pyx_t_9 = PyList_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1187, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_9);
                __Pyx_INCREF(__pyx_cur_scope->__pyx_v_n);
                __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_n);
                PyList_SET_ITEM(__pyx_t_9, 0, __pyx_cur_scope->__pyx_v_n);
                __pyx_t_8 = PyList_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1187, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_INCREF(__pyx_cur_scope->__pyx_v_t);
                __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_t);
                PyList_SET_ITEM(__pyx_t_8, 0, __pyx_cur_scope->__pyx_v_t);
                __pyx_t_11 = NULL;
                __pyx_t_6 = 0;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                  __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
                  if (likely(__pyx_t_11)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                    __Pyx_INCREF(__pyx_t_11);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_7, function);
                    __pyx_t_6 = 1;
                  }
                }
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_7)) {
                  PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_t_9, __pyx_t_8};
                  __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
                  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                  __Pyx_GOTREF(__pyx_t_4);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
                  PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_t_9, __pyx_t_8};
                  __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
                  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                  __Pyx_GOTREF(__pyx_t_4);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                } else
                #endif
                {
                  __pyx_t_10 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1187, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_10);
                  if (__pyx_t_11) {
                    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_11); __pyx_t_11 = NULL;
                  }
                  __Pyx_GIVEREF(__pyx_t_9);
                  PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_6, __pyx_t_9);
                  __Pyx_GIVEREF(__pyx_t_8);
                  PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_6, __pyx_t_8);
                  __pyx_t_9 = 0;
                  __pyx_t_8 = 0;
                  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                }
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
                __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_4);
                __Pyx_GIVEREF(__pyx_t_4);
                __pyx_t_4 = 0;

                /* "pyslow5.pyx":1188
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_seq_read_aux([n], [t])
 *                                     break             # <<<<<<<<<<<<<<
 *                             if not found_single_aux:
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))
 */
                goto __pyx_L17_break;

                /* "pyslow5.pyx":1185
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):
 *                                 if n == aux:             # <<<<<<<<<<<<<<
 *                                     found_single_aux = True
 *                                     aux_dic = self._get_seq_read_aux([n], [t])
 */
              }

              /* "pyslow5.pyx":1184
 *                         else:
 *                             found_single_aux = False
 *                             for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                                 if n == aux:
 *                                     found_single_aux = True
 */
            }
            __pyx_L17_break:;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "pyslow5.pyx":1189
 *                                     aux_dic = self._get_seq_read_aux([n], [t])
 *                                     break
 *                             if not found_single_aux:             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 */
            __pyx_t_2 = ((!(__pyx_cur_scope->__pyx_v_found_single_aux != 0)) != 0);
            if (__pyx_t_2) {

              /* "pyslow5.pyx":1190
 *                                     break
 *                             if not found_single_aux:
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))             # <<<<<<<<<<<<<<
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:
 */
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1190, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_unknown_aux, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1190, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_8 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
                __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_10);
                if (likely(__pyx_t_8)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                  __Pyx_INCREF(__pyx_t_8);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_10, function);
                }
              }
              __pyx_t_7 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_8, __pyx_cur_scope->__pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_cur_scope->__pyx_v_aux);
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1190, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __pyx_t_10 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_10)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_10);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                }
              }
              __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1190, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "pyslow5.pyx":1191
 *                             if not found_single_aux:
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})             # <<<<<<<<<<<<<<
 *                     elif type(aux) is list:
 *                         n_list = []
 */
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1191, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1191, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              if (PyDict_SetItem(__pyx_t_7, __pyx_cur_scope->__pyx_v_aux, Py_None) < 0) __PYX_ERR(0, 1191, __pyx_L1_error)
              __pyx_t_10 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_10)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_10);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                }
              }
              __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1191, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "pyslow5.pyx":1189
 *                                     aux_dic = self._get_seq_read_aux([n], [t])
 *                                     break
 *                             if not found_single_aux:             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 */
            }
          }
          __pyx_L15:;

          /* "pyslow5.pyx":1179
 *                         self.aux_names = self.get_aux_names()
 *                         self.aux_types = self.get_aux_types()
 *                     if type(aux) is str:             # <<<<<<<<<<<<<<
 *                         if aux == "all":
 *                             aux_dic = self._get_seq_read_aux(self.aux_names, self.aux_types)
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1192
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                         n_list = []
 *                         t_list = []
 */
        __pyx_t_2 = (((PyObject *)Py_TYPE(__pyx_cur_scope->__pyx_v_aux)) == ((PyObject *)(&PyList_Type)));
        __pyx_t_15 = (__pyx_t_2 != 0);
        if (__pyx_t_15) {

          /* "pyslow5.pyx":1193
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:
 *                         n_list = []             # <<<<<<<<<<<<<<
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):
 */
          __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1193, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n_list);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n_list, ((PyObject*)__pyx_t_3));
          __Pyx_GIVEREF(__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1194
 *                     elif type(aux) is list:
 *                         n_list = []
 *                         t_list = []             # <<<<<<<<<<<<<<
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:
 */
          __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1194, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t_list);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t_list, ((PyObject*)__pyx_t_3));
          __Pyx_GIVEREF(__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1195
 *                         n_list = []
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                             if n in aux:
 *                                 n_list.append(n)
 */
          __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1195, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_names);
          PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_cur_scope->__pyx_v_self->aux_names);
          __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->aux_types);
          PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_cur_scope->__pyx_v_self->aux_types);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1195, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
            __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_14 = 0;
            __pyx_t_17 = NULL;
          } else {
            __pyx_t_14 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1195, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_17 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1195, __pyx_L1_error)
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          for (;;) {
            if (likely(!__pyx_t_17)) {
              if (likely(PyList_CheckExact(__pyx_t_3))) {
                if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_3)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_14); __Pyx_INCREF(__pyx_t_4); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1195, __pyx_L1_error)
                #else
                __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1195, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_4);
                #endif
              } else {
                if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_14); __Pyx_INCREF(__pyx_t_4); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1195, __pyx_L1_error)
                #else
                __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1195, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_4);
                #endif
              }
            } else {
              __pyx_t_4 = __pyx_t_17(__pyx_t_3);
              if (unlikely(!__pyx_t_4)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 1195, __pyx_L1_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_4);
            }
            if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
              PyObject* sequence = __pyx_t_4;
              Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(0, 1195, __pyx_L1_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              if (likely(PyTuple_CheckExact(sequence))) {
                __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
                __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
              } else {
                __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
                __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
              }
              __Pyx_INCREF(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_10);
              #else
              __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1195, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1195, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              #endif
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            } else {
              Py_ssize_t index = -1;
              __pyx_t_8 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1195, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_18 = Py_TYPE(__pyx_t_8)->tp_iternext;
              index = 0; __pyx_t_7 = __pyx_t_18(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L24_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_7);
              index = 1; __pyx_t_10 = __pyx_t_18(__pyx_t_8); if (unlikely(!__pyx_t_10)) goto __pyx_L24_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_10);
              if (__Pyx_IternextUnpackEndCheck(__pyx_t_18(__pyx_t_8), 2) < 0) __PYX_ERR(0, 1195, __pyx_L1_error)
              __pyx_t_18 = NULL;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              goto __pyx_L25_unpacking_done;
              __pyx_L24_unpacking_failed:;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_18 = NULL;
              if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
              __PYX_ERR(0, 1195, __pyx_L1_error)
              __pyx_L25_unpacking_done:;
            }
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n, __pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_7);
            __pyx_t_7 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_10);
            __Pyx_GIVEREF(__pyx_t_10);
            __pyx_t_10 = 0;

            /* "pyslow5.pyx":1196
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:             # <<<<<<<<<<<<<<
 *                                 n_list.append(n)
 *                                 t_list.append(t)
 */
            __pyx_t_15 = (__Pyx_PySequence_ContainsTF(__pyx_cur_scope->__pyx_v_n, __pyx_cur_scope->__pyx_v_aux, Py_EQ)); if (unlikely(__pyx_t_15 < 0)) __PYX_ERR(0, 1196, __pyx_L1_error)
            __pyx_t_2 = (__pyx_t_15 != 0);
            if (__pyx_t_2) {

              /* "pyslow5.pyx":1197
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:
 *                                 n_list.append(n)             # <<<<<<<<<<<<<<
 *                                 t_list.append(t)
 * 
 */
              __pyx_t_19 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_n); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(0, 1197, __pyx_L1_error)

              /* "pyslow5.pyx":1198
 *                             if n in aux:
 *                                 n_list.append(n)
 *                                 t_list.append(t)             # <<<<<<<<<<<<<<
 * 
 *                         aux_dic = self._get_seq_read_aux(n_list, t_list)
 */
              __pyx_t_19 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_t_list, __pyx_cur_scope->__pyx_v_t); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(0, 1198, __pyx_L1_error)

              /* "pyslow5.pyx":1196
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):
 *                             if n in aux:             # <<<<<<<<<<<<<<
 *                                 n_list.append(n)
 *                                 t_list.append(t)
 */
            }

            /* "pyslow5.pyx":1195
 *                         n_list = []
 *                         t_list = []
 *                         for n, t in zip(self.aux_names, self.aux_types):             # <<<<<<<<<<<<<<
 *                             if n in aux:
 *                                 n_list.append(n)
 */
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "pyslow5.pyx":1200
 *                                 t_list.append(t)
 * 
 *                         aux_dic = self._get_seq_read_aux(n_list, t_list)             # <<<<<<<<<<<<<<
 *                         # check for items in given list that do not exist
 *                         n_set = set(n_list)
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_seq_read_aux); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1200, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_10 = NULL;
          __pyx_t_6 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
              __pyx_t_6 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_4)) {
            PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_t_list};
            __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1200, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_3);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
            PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_n_list, __pyx_cur_scope->__pyx_v_t_list};
            __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1200, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_3);
          } else
          #endif
          {
            __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1200, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            if (__pyx_t_10) {
              __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_10); __pyx_t_10 = NULL;
            }
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_n_list);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_n_list);
            PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_cur_scope->__pyx_v_n_list);
            __Pyx_INCREF(__pyx_cur_scope->__pyx_v_t_list);
            __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_t_list);
            PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_cur_scope->__pyx_v_t_list);
            __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1200, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_aux_dic);
          __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1202
 *                         aux_dic = self._get_seq_read_aux(n_list, t_list)
 *                         # check for items in given list that do not exist
 *                         n_set = set(n_list)             # <<<<<<<<<<<<<<
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:
 */
          __pyx_t_3 = PySet_New(__pyx_cur_scope->__pyx_v_n_list); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1202, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_n_set);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_n_set, ((PyObject*)__pyx_t_3));
          __Pyx_GIVEREF(__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1203
 *                         # check for items in given list that do not exist
 *                         n_set = set(n_list)
 *                         aux_set = set(aux)             # <<<<<<<<<<<<<<
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):
 */
          __pyx_t_3 = PySet_New(__pyx_cur_scope->__pyx_v_aux); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1203, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_aux_set);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_aux_set, ((PyObject*)__pyx_t_3));
          __Pyx_GIVEREF(__pyx_t_3);
          __pyx_t_3 = 0;

          /* "pyslow5.pyx":1204
 *                         n_set = set(n_list)
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(i))
 */
          __pyx_t_3 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_cur_scope->__pyx_v_aux_set, __pyx_cur_scope->__pyx_v_n_set); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1204, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_14 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_14 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1204, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_2 = ((__pyx_t_14 > 0) != 0);
          if (__pyx_t_2) {

            /* "pyslow5.pyx":1205
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(i))
 *                                 aux_dic.update({i: None})
 */
            __pyx_t_3 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PySet_Type_difference, __pyx_cur_scope->__pyx_v_aux_set, __pyx_cur_scope->__pyx_v_n_set); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1205, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
              __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4); __pyx_t_14 = 0;
              __pyx_t_17 = NULL;
            } else {
              __pyx_t_14 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1205, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_17 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1205, __pyx_L1_error)
            }
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            for (;;) {
              if (likely(!__pyx_t_17)) {
                if (likely(PyList_CheckExact(__pyx_t_4))) {
                  if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_4)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1205, __pyx_L1_error)
                  #else
                  __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1205, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  #endif
                } else {
                  if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1205, __pyx_L1_error)
                  #else
                  __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1205, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  #endif
                }
              } else {
                __pyx_t_3 = __pyx_t_17(__pyx_t_4);
                if (unlikely(!__pyx_t_3)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(0, 1205, __pyx_L1_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_3);
              }
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_i);
              __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_3);
              __Pyx_GIVEREF(__pyx_t_3);
              __pyx_t_3 = 0;

              /* "pyslow5.pyx":1206
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(i))             # <<<<<<<<<<<<<<
 *                                 aux_dic.update({i: None})
 * 
 */
              __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1206, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_unknown_aux, __pyx_n_s_format); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1206, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              __pyx_t_9 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
                __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
                if (likely(__pyx_t_9)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
                  __Pyx_INCREF(__pyx_t_9);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_8, function);
                }
              }
              __pyx_t_10 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_cur_scope->__pyx_v_i) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_cur_scope->__pyx_v_i);
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1206, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
                if (likely(__pyx_t_8)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                  __Pyx_INCREF(__pyx_t_8);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_7, function);
                }
              }
              __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_10);
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1206, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "pyslow5.pyx":1207
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(i))
 *                                 aux_dic.update({i: None})             # <<<<<<<<<<<<<<
 * 
 *                     else:
 */
              __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_aux_dic, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1207, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1207, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (PyDict_SetItem(__pyx_t_10, __pyx_cur_scope->__pyx_v_i, Py_None) < 0) __PYX_ERR(0, 1207, __pyx_L1_error)
              __pyx_t_8 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
                if (likely(__pyx_t_8)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                  __Pyx_INCREF(__pyx_t_8);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_7, function);
                }
              }
              __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_10);
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1207, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "pyslow5.pyx":1205
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:
 *                             for i in aux_set.difference(n_set):             # <<<<<<<<<<<<<<
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(i))
 *                                 aux_dic.update({i: None})
 */
            }
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyslow5.pyx":1204
 *                         n_set = set(n_list)
 *                         aux_set = set(aux)
 *                         if len(aux_set.difference(n_set)) > 0:             # <<<<<<<<<<<<<<
 *                             for i in aux_set.difference(n_set):
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(i))
 */
          }

          /* "pyslow5.pyx":1192
 *                                 self.logger.warning("slow5_get_next_multi unknown aux name: {}".format(aux))
 *                                 aux_dic.update({aux: None})
 *                     elif type(aux) is list:             # <<<<<<<<<<<<<<
 *                         n_list = []
 *                         t_list = []
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1210
 * 
 *                     else:
 *                         self.logger.debug("slow5_get_next_multi aux type unknown, accepts str or list: {}".format(aux))             # <<<<<<<<<<<<<<
 *                 timedic["aux_total_time"] = timedic["aux_total_time"] + (time.time() - aux_time_start)
 *                 # Get read data
 */
        /*else*/ {
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1210, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_aux_type_un, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1210, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
            }
          }
          __pyx_t_7 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_8, __pyx_cur_scope->__pyx_v_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_cur_scope->__pyx_v_aux);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1210, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1210, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
        __pyx_L14:;

        /* "pyslow5.pyx":1175
 *                 # get aux fields
 *                 aux_time_start = time.time()
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     if not self.aux_names or not self.aux_types:
 *                         self.aux_names = self.get_aux_names()
 */
      }

      /* "pyslow5.pyx":1211
 *                     else:
 *                         self.logger.debug("slow5_get_next_multi aux type unknown, accepts str or list: {}".format(aux))
 *                 timedic["aux_total_time"] = timedic["aux_total_time"] + (time.time() - aux_time_start)             # <<<<<<<<<<<<<<
 *                 # Get read data
 *                 primary_start_time = time.time()
 */
      __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_aux_total_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = PyNumber_Subtract(__pyx_t_3, __pyx_cur_scope->__pyx_v_aux_time_start); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyNumber_Add(__pyx_t_4, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_aux_total_time, __pyx_t_3) < 0)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1213
 *                 timedic["aux_total_time"] = timedic["aux_total_time"] + (time.time() - aux_time_start)
 *                 # Get read data
 *                 primary_start_time = time.time()             # <<<<<<<<<<<<<<
 *                 if type(self.read.read_id) is bytes:
 *                     row['read_id'] = self.read.read_id.decode()
 */
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_primary_start_time);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_primary_start_time, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1214
 *                 # Get read data
 *                 primary_start_time = time.time()
 *                 if type(self.read.read_id) is bytes:             # <<<<<<<<<<<<<<
 *                     row['read_id'] = self.read.read_id.decode()
 *                 else:
 */
      __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->read->read_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = (((PyObject *)Py_TYPE(__pyx_t_3)) == ((PyObject *)(&PyBytes_Type)));
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_15 = (__pyx_t_2 != 0);
      if (__pyx_t_15) {

        /* "pyslow5.pyx":1215
 *                 primary_start_time = time.time()
 *                 if type(self.read.read_id) is bytes:
 *                     row['read_id'] = self.read.read_id.decode()             # <<<<<<<<<<<<<<
 *                 else:
 *                     row['read_id'] = self.read.read_id
 */
        __pyx_t_20 = __pyx_cur_scope->__pyx_v_self->read->read_id;
        __pyx_t_3 = __Pyx_decode_c_string(__pyx_t_20, 0, strlen(__pyx_t_20), NULL, NULL, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1215, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_read_id, __pyx_t_3) < 0)) __PYX_ERR(0, 1215, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyslow5.pyx":1214
 *                 # Get read data
 *                 primary_start_time = time.time()
 *                 if type(self.read.read_id) is bytes:             # <<<<<<<<<<<<<<
 *                     row['read_id'] = self.read.read_id.decode()
 *                 else:
 */
        goto __pyx_L30;
      }

      /* "pyslow5.pyx":1217
 *                     row['read_id'] = self.read.read_id.decode()
 *                 else:
 *                     row['read_id'] = self.read.read_id             # <<<<<<<<<<<<<<
 *                 # row['read_id'] = self.read.read_id.decode()
 *                 row['read_group'] = self.read.read_group
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_cur_scope->__pyx_v_self->read->read_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1217, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_read_id, __pyx_t_3) < 0)) __PYX_ERR(0, 1217, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __pyx_L30:;

      /* "pyslow5.pyx":1219
 *                     row['read_id'] = self.read.read_id
 *                 # row['read_id'] = self.read.read_id.decode()
 *                 row['read_group'] = self.read.read_group             # <<<<<<<<<<<<<<
 *                 row['digitisation'] = self.read.digitisation
 *                 row['offset'] = self.read.offset
 */
      __pyx_t_3 = __Pyx_PyInt_From_uint32_t(__pyx_cur_scope->__pyx_v_self->read->read_group); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_read_group, __pyx_t_3) < 0)) __PYX_ERR(0, 1219, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1220
 *                 # row['read_id'] = self.read.read_id.decode()
 *                 row['read_group'] = self.read.read_group
 *                 row['digitisation'] = self.read.digitisation             # <<<<<<<<<<<<<<
 *                 row['offset'] = self.read.offset
 *                 row['range'] = self.read.range
 */
      __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->digitisation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1220, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_digitisation, __pyx_t_3) < 0)) __PYX_ERR(0, 1220, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1221
 *                 row['read_group'] = self.read.read_group
 *                 row['digitisation'] = self.read.digitisation
 *                 row['offset'] = self.read.offset             # <<<<<<<<<<<<<<
 *                 row['range'] = self.read.range
 *                 row['sampling_rate'] = self.read.sampling_rate
 */
      __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1221, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_offset, __pyx_t_3) < 0)) __PYX_ERR(0, 1221, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1222
 *                 row['digitisation'] = self.read.digitisation
 *                 row['offset'] = self.read.offset
 *                 row['range'] = self.read.range             # <<<<<<<<<<<<<<
 *                 row['sampling_rate'] = self.read.sampling_rate
 *                 row['len_raw_signal'] = self.read.len_raw_signal
 */
      __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->range); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_range, __pyx_t_3) < 0)) __PYX_ERR(0, 1222, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1223
 *                 row['offset'] = self.read.offset
 *                 row['range'] = self.read.range
 *                 row['sampling_rate'] = self.read.sampling_rate             # <<<<<<<<<<<<<<
 *                 row['len_raw_signal'] = self.read.len_raw_signal
 *                 signal_start_time = time.time()
 */
      __pyx_t_3 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->read->sampling_rate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1223, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_sampling_rate, __pyx_t_3) < 0)) __PYX_ERR(0, 1223, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1224
 *                 row['range'] = self.read.range
 *                 row['sampling_rate'] = self.read.sampling_rate
 *                 row['len_raw_signal'] = self.read.len_raw_signal             # <<<<<<<<<<<<<<
 *                 signal_start_time = time.time()
 *                 self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 */
      __pyx_t_3 = __Pyx_PyInt_From_uint64_t(__pyx_cur_scope->__pyx_v_self->read->len_raw_signal); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_len_raw_signal, __pyx_t_3) < 0)) __PYX_ERR(0, 1224, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1225
 *                 row['sampling_rate'] = self.read.sampling_rate
 *                 row['len_raw_signal'] = self.read.len_raw_signal
 *                 signal_start_time = time.time()             # <<<<<<<<<<<<<<
 *                 self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1225, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1225, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1225, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_signal_start_time);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_signal_start_time, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1226
 *                 row['len_raw_signal'] = self.read.len_raw_signal
 *                 signal_start_time = time.time()
 *                 self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal             # <<<<<<<<<<<<<<
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 *                             np.NPY_INT16, <void *> self.read.raw_signal))
 */
      (__pyx_cur_scope->__pyx_v_self->shape_seq[0]) = ((npy_intp)__pyx_cur_scope->__pyx_v_self->read->len_raw_signal);

      /* "pyslow5.pyx":1227
 *                 signal_start_time = time.time()
 *                 self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,             # <<<<<<<<<<<<<<
 *                             np.NPY_INT16, <void *> self.read.raw_signal))
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 */
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_copy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "pyslow5.pyx":1228
 *                 self.shape_seq[0] = <np.npy_intp> self.read.len_raw_signal
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 *                             np.NPY_INT16, <void *> self.read.raw_signal))             # <<<<<<<<<<<<<<
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *                 row['signal'] = signal
 */
      __pyx_t_10 = PyArray_SimpleNewFromData(1, __pyx_cur_scope->__pyx_v_self->shape_seq, NPY_INT16, ((void *)__pyx_cur_scope->__pyx_v_self->read->raw_signal)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_7, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_signal);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_signal, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1229
 *                 signal = copy.deepcopy(np.PyArray_SimpleNewFromData(1, self.shape_seq,
 *                             np.NPY_INT16, <void *> self.read.raw_signal))
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)             # <<<<<<<<<<<<<<
 *                 row['signal'] = signal
 *                 timedic["signal_total_time"] = timedic["signal_total_time"] + (time.time() - signal_start_time)
 */
      if (!(likely(((__pyx_cur_scope->__pyx_v_signal) == Py_None) || likely(__Pyx_TypeTest(__pyx_cur_scope->__pyx_v_signal, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1229, __pyx_L1_error)
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_signal, __pyx_n_s_flags); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyInt_From_int(NPY_OWNDATA); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_10 = PyNumber_Or(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_10); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1229, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      PyArray_UpdateFlags(((PyArrayObject *)__pyx_cur_scope->__pyx_v_signal), __pyx_t_6);

      /* "pyslow5.pyx":1230
 *                             np.NPY_INT16, <void *> self.read.raw_signal))
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *                 row['signal'] = signal             # <<<<<<<<<<<<<<
 *                 timedic["signal_total_time"] = timedic["signal_total_time"] + (time.time() - signal_start_time)
 *                 timedic["primary_total_time"] = timedic["primary_total_time"] + (time.time() - primary_start_time)
 */
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_signal, __pyx_cur_scope->__pyx_v_signal) < 0)) __PYX_ERR(0, 1230, __pyx_L1_error)

      /* "pyslow5.pyx":1231
 *                 np.PyArray_UpdateFlags(signal, signal.flags.num | np.NPY_OWNDATA)
 *                 row['signal'] = signal
 *                 timedic["signal_total_time"] = timedic["signal_total_time"] + (time.time() - signal_start_time)             # <<<<<<<<<<<<<<
 *                 timedic["primary_total_time"] = timedic["primary_total_time"] + (time.time() - primary_start_time)
 *                 # if pA=True, convert signal to pA
 */
      __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_signal_total_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyNumber_Subtract(__pyx_t_3, __pyx_cur_scope->__pyx_v_signal_start_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyNumber_Add(__pyx_t_10, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_signal_total_time, __pyx_t_3) < 0)) __PYX_ERR(0, 1231, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1232
 *                 row['signal'] = signal
 *                 timedic["signal_total_time"] = timedic["signal_total_time"] + (time.time() - signal_start_time)
 *                 timedic["primary_total_time"] = timedic["primary_total_time"] + (time.time() - primary_start_time)             # <<<<<<<<<<<<<<
 *                 # if pA=True, convert signal to pA
 *                 if pA:
 */
      __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_primary_total_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_7 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyNumber_Subtract(__pyx_t_7, __pyx_cur_scope->__pyx_v_primary_start_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyNumber_Add(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_primary_total_time, __pyx_t_7) < 0)) __PYX_ERR(0, 1232, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyslow5.pyx":1234
 *                 timedic["primary_total_time"] = timedic["primary_total_time"] + (time.time() - primary_start_time)
 *                 # if pA=True, convert signal to pA
 *                 if pA:             # <<<<<<<<<<<<<<
 *                     pA_start_time = time.time()
 *                     row['signal'] = self._convert_to_pA(row)
 */
      __pyx_t_15 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_pA); if (unlikely(__pyx_t_15 < 0)) __PYX_ERR(0, 1234, __pyx_L1_error)
      if (__pyx_t_15) {

        /* "pyslow5.pyx":1235
 *                 # if pA=True, convert signal to pA
 *                 if pA:
 *                     pA_start_time = time.time()             # <<<<<<<<<<<<<<
 *                     row['signal'] = self._convert_to_pA(row)
 *                     timedic["pA_total_time"] = timedic["pA_total_time"] + (time.time() - pA_start_time)
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_7 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_pA_start_time);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_pA_start_time, __pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_7);
        __pyx_t_7 = 0;

        /* "pyslow5.pyx":1236
 *                 if pA:
 *                     pA_start_time = time.time()
 *                     row['signal'] = self._convert_to_pA(row)             # <<<<<<<<<<<<<<
 *                     timedic["pA_total_time"] = timedic["pA_total_time"] + (time.time() - pA_start_time)
 *                 # if aux data update main dic
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_convert_to_pA); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1236, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_7 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_cur_scope->__pyx_v_row) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_cur_scope->__pyx_v_row);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1236, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_row, __pyx_n_u_signal, __pyx_t_7) < 0)) __PYX_ERR(0, 1236, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyslow5.pyx":1237
 *                     pA_start_time = time.time()
 *                     row['signal'] = self._convert_to_pA(row)
 *                     timedic["pA_total_time"] = timedic["pA_total_time"] + (time.time() - pA_start_time)             # <<<<<<<<<<<<<<
 *                 # if aux data update main dic
 *                 if aux_dic:
 */
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_pA_total_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = PyNumber_Subtract(__pyx_t_3, __pyx_cur_scope->__pyx_v_pA_start_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyNumber_Add(__pyx_t_7, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_n_u_pA_total_time, __pyx_t_3) < 0)) __PYX_ERR(0, 1237, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyslow5.pyx":1234
 *                 timedic["primary_total_time"] = timedic["primary_total_time"] + (time.time() - primary_start_time)
 *                 # if pA=True, convert signal to pA
 *                 if pA:             # <<<<<<<<<<<<<<
 *                     pA_start_time = time.time()
 *                     row['signal'] = self._convert_to_pA(row)
 */
      }

      /* "pyslow5.pyx":1239
 *                     timedic["pA_total_time"] = timedic["pA_total_time"] + (time.time() - pA_start_time)
 *                 # if aux data update main dic
 *                 if aux_dic:             # <<<<<<<<<<<<<<
 *                     row.update(aux_dic)
 *                 self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - python_parse_read_start)
 */
      __pyx_t_15 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_aux_dic); if (unlikely(__pyx_t_15 < 0)) __PYX_ERR(0, 1239, __pyx_L1_error)
      if (__pyx_t_15) {

        /* "pyslow5.pyx":1240
 *                 # if aux data update main dic
 *                 if aux_dic:
 *                     row.update(aux_dic)             # <<<<<<<<<<<<<<
 *                 self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - python_parse_read_start)
 *                 yield row
 */
        __pyx_t_3 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_cur_scope->__pyx_v_row, __pyx_cur_scope->__pyx_v_aux_dic); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1240, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyslow5.pyx":1239
 *                     timedic["pA_total_time"] = timedic["pA_total_time"] + (time.time() - pA_start_time)
 *                 # if aux data update main dic
 *                 if aux_dic:             # <<<<<<<<<<<<<<
 *                     row.update(aux_dic)
 *                 self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - python_parse_read_start)
 */
      }

      /* "pyslow5.pyx":1241
 *                 if aux_dic:
 *                     row.update(aux_dic)
 *                 self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - python_parse_read_start)             # <<<<<<<<<<<<<<
 *                 yield row
 *             slow5_free_batch(&self.trec, ret)
 */
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyNumber_Subtract(__pyx_t_3, __pyx_cur_scope->__pyx_v_python_parse_read_start); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyNumber_Add(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (!(likely(PyFloat_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 1241, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads);
      __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->total_time_yield_reads);
      __pyx_cur_scope->__pyx_v_self->total_time_yield_reads = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyslow5.pyx":1242
 *                     row.update(aux_dic)
 *                 self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - python_parse_read_start)
 *                 yield row             # <<<<<<<<<<<<<<
 *             slow5_free_batch(&self.trec, ret)
 *             if ret < batchsize:
 */
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_row);
      __pyx_r = __pyx_cur_scope->__pyx_v_row;
      __Pyx_XGIVEREF(__pyx_t_1);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_12;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_13;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_TraceReturn(__pyx_r, 0);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L33_resume_from_yield:;
      __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_12 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_13 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1242, __pyx_L1_error)

      /* "pyslow5.pyx":1168
 * 
 *                 break
 *             for i in range(ret):             # <<<<<<<<<<<<<<
 *                 python_parse_read_start = time.time()
 *                 self.read = self.trec[i]
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1243
 *                 self.total_time_yield_reads = self.total_time_yield_reads + (time.time() - python_parse_read_start)
 *                 yield row
 *             slow5_free_batch(&self.trec, ret)             # <<<<<<<<<<<<<<
 *             if ret < batchsize:
 *                 self.logger.debug("slow5_get_next_multi has no more batches - batchsize:{} ret:{}".format(batchsize, ret))
 */
    slow5_free_batch((&__pyx_cur_scope->__pyx_v_self->trec), __pyx_cur_scope->__pyx_v_ret);

    /* "pyslow5.pyx":1244
 *                 yield row
 *             slow5_free_batch(&self.trec, ret)
 *             if ret < batchsize:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("slow5_get_next_multi has no more batches - batchsize:{} ret:{}".format(batchsize, ret))
 *                 break
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_cur_scope->__pyx_v_batchsize, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1244, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_15 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_15 < 0)) __PYX_ERR(0, 1244, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_15) {

      /* "pyslow5.pyx":1245
 *             slow5_free_batch(&self.trec, ret)
 *             if ret < batchsize:
 *                 self.logger.debug("slow5_get_next_multi has no more batches - batchsize:{} ret:{}".format(batchsize, ret))             # <<<<<<<<<<<<<<
 *                 break
 *         self.read = NULL
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1245, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_next_multi_has_no_more, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1245, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_cur_scope->__pyx_v_ret); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1245, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = NULL;
      __pyx_t_6 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
          __pyx_t_6 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_batchsize, __pyx_t_4};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1245, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_batchsize, __pyx_t_4};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1245, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1245, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_batchsize);
        __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_batchsize);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_6, __pyx_cur_scope->__pyx_v_batchsize);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_6, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1245, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1245, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1246
 *             if ret < batchsize:
 *                 self.logger.debug("slow5_get_next_multi has no more batches - batchsize:{} ret:{}".format(batchsize, ret))
 *                 break             # <<<<<<<<<<<<<<
 *         self.read = NULL
 *         self.logger.debug("seq_reads_multi timings:")
 */
      goto __pyx_L5_break;

      /* "pyslow5.pyx":1244
 *                 yield row
 *             slow5_free_batch(&self.trec, ret)
 *             if ret < batchsize:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("slow5_get_next_multi has no more batches - batchsize:{} ret:{}".format(batchsize, ret))
 *                 break
 */
    }
  }
  __pyx_L5_break:;

  /* "pyslow5.pyx":1247
 *                 self.logger.debug("slow5_get_next_multi has no more batches - batchsize:{} ret:{}".format(batchsize, ret))
 *                 break
 *         self.read = NULL             # <<<<<<<<<<<<<<
 *         self.logger.debug("seq_reads_multi timings:")
 *         for i in timedic:
 */
  __pyx_cur_scope->__pyx_v_self->read = NULL;

  /* "pyslow5.pyx":1248
 *                 break
 *         self.read = NULL
 *         self.logger.debug("seq_reads_multi timings:")             # <<<<<<<<<<<<<<
 *         for i in timedic:
 *             self.logger.debug("{}: {}".format(i, timedic[i]))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_7, __pyx_kp_u_seq_reads_multi_timings) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_seq_reads_multi_timings);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":1249
 *         self.read = NULL
 *         self.logger.debug("seq_reads_multi timings:")
 *         for i in timedic:             # <<<<<<<<<<<<<<
 *             self.logger.debug("{}: {}".format(i, timedic[i]))
 * 
 */
  __pyx_t_12 = 0;
  __pyx_t_1 = __Pyx_dict_iterator(__pyx_cur_scope->__pyx_v_timedic, 1, ((PyObject *)NULL), (&__pyx_t_14), (&__pyx_t_6)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __pyx_t_3 = __pyx_t_1;
  __pyx_t_1 = 0;
  while (1) {
    __pyx_t_5 = __Pyx_dict_iter_next(__pyx_t_3, __pyx_t_14, &__pyx_t_12, &__pyx_t_1, NULL, NULL, __pyx_t_6);
    if (unlikely(__pyx_t_5 == 0)) break;
    if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 1249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1250
 *         self.logger.debug("seq_reads_multi timings:")
 *         for i in timedic:
 *             self.logger.debug("{}: {}".format(i, timedic[i]))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u__5, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_cur_scope->__pyx_v_timedic, __pyx_cur_scope->__pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_9)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_i, __pyx_t_4};
      __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1250, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_i, __pyx_t_4};
      __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1250, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_11 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1250, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_i);
      PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_5, __pyx_cur_scope->__pyx_v_i);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_5, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1250, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyslow5.pyx":1139
 * 
 * 
 *     def seq_reads_multi(self, threads=4, batchsize=4096, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for sequential reading of slow5 file
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("seq_reads_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_32generator4(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":1253
 * 
 * 
 *     def get_read_list(self, read_list, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for random access of slow5 file
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_31get_read_list(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_30get_read_list[] = "\n        returns generator for random access of slow5 file\n        read_list = list of readIDs, if readID not in file, None type returned (need EOF to work)\n        for pA and aux see _get_read\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_31get_read_list(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_list = 0;
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_read_list (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_list,&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_False);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_list)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_read_list") < 0)) __PYX_ERR(0, 1253, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_read_list = values[0];
    __pyx_v_pA = values[1];
    __pyx_v_aux = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_read_list", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1253, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_read_list", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_30get_read_list(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_list, __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_30get_read_list(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_read_list", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *)__pyx_tp_new_7pyslow5___pyx_scope_struct_6_get_read_list(__pyx_ptype_7pyslow5___pyx_scope_struct_6_get_read_list, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1253, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_read_list = __pyx_v_read_list;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
  __pyx_cur_scope->__pyx_v_pA = __pyx_v_pA;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
  __pyx_cur_scope->__pyx_v_aux = __pyx_v_aux;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_32generator4, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_get_read_list, __pyx_n_s_Open_get_read_list, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 1253, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_read_list", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_32generator4(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_read_list", 0);
  __Pyx_TraceCall("get_read_list", __pyx_f[0], 1253, 0, __PYX_ERR(0, 1253, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1253, __pyx_L1_error)

  /* "pyslow5.pyx":1259
 *         for pA and aux see _get_read
 *         '''
 *         for r in read_list:             # <<<<<<<<<<<<<<
 *             yield self._get_read(r, pA, aux)
 * 
 */
  if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_read_list)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_read_list)) {
    __pyx_t_1 = __pyx_cur_scope->__pyx_v_read_list; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_read_list); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1259, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1259, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1259, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1259, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1259, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1259, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_r);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_r, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyslow5.pyx":1260
 *         '''
 *         for r in read_list:
 *             yield self._get_read(r, pA, aux)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_read); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_r, __pyx_cur_scope->__pyx_v_pA, __pyx_cur_scope->__pyx_v_aux};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1260, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_r, __pyx_cur_scope->__pyx_v_pA, __pyx_cur_scope->__pyx_v_aux};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1260, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1260, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_r);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_r);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_cur_scope->__pyx_v_r);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_cur_scope->__pyx_v_pA);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
      PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_cur_scope->__pyx_v_aux);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1260, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    __Pyx_XGIVEREF(__pyx_t_1);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_TraceReturn(__pyx_r, 0);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1260, __pyx_L1_error)

    /* "pyslow5.pyx":1259
 *         for pA and aux see _get_read
 *         '''
 *         for r in read_list:             # <<<<<<<<<<<<<<
 *             yield self._get_read(r, pA, aux)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyslow5.pyx":1253
 * 
 * 
 *     def get_read_list(self, read_list, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for random access of slow5 file
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("get_read_list", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyslow5_4Open_35generator5(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyslow5.pyx":1263
 * 
 * 
 *     def get_read_list_multi(self, read_list, threads=4, batchsize=100, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for random access of slow5 file
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_34get_read_list_multi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_33get_read_list_multi[] = "\n        returns generator for random access of slow5 file\n        read_list = list of readIDs, if readID not in file, None type returned (need EOF to work)\n        for pA and aux see _get_read\n        threads = number of threads to use to do batched random access\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_34get_read_list_multi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_list = 0;
  PyObject *__pyx_v_threads = 0;
  PyObject *__pyx_v_batchsize = 0;
  PyObject *__pyx_v_pA = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_read_list_multi (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_list,&__pyx_n_s_threads,&__pyx_n_s_batchsize,&__pyx_n_s_pA,&__pyx_n_s_aux,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[1] = ((PyObject *)__pyx_int_4);
    values[2] = ((PyObject *)__pyx_int_100);
    values[3] = ((PyObject *)Py_False);
    values[4] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_list)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_threads);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_batchsize);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pA);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_read_list_multi") < 0)) __PYX_ERR(0, 1263, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_read_list = values[0];
    __pyx_v_threads = values[1];
    __pyx_v_batchsize = values[2];
    __pyx_v_pA = values[3];
    __pyx_v_aux = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_read_list_multi", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1263, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_read_list_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_33get_read_list_multi(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_list, __pyx_v_threads, __pyx_v_batchsize, __pyx_v_pA, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_33get_read_list_multi(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_list, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_pA, PyObject *__pyx_v_aux) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_read_list_multi", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *)__pyx_tp_new_7pyslow5___pyx_scope_struct_7_get_read_list_multi(__pyx_ptype_7pyslow5___pyx_scope_struct_7_get_read_list_multi, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1263, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_read_list = __pyx_v_read_list;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
  __pyx_cur_scope->__pyx_v_threads = __pyx_v_threads;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_threads);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_threads);
  __pyx_cur_scope->__pyx_v_batchsize = __pyx_v_batchsize;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_batchsize);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_batchsize);
  __pyx_cur_scope->__pyx_v_pA = __pyx_v_pA;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
  __pyx_cur_scope->__pyx_v_aux = __pyx_v_aux;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyslow5_4Open_35generator5, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_get_read_list_multi, __pyx_n_s_Open_get_read_list_multi, __pyx_n_s_pyslow5); if (unlikely(!gen)) __PYX_ERR(0, 1263, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_read_list_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyslow5_4Open_35generator5(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *__pyx_cur_scope = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_read_list_multi", 0);
  __Pyx_TraceCall("get_read_list_multi", __pyx_f[0], 1263, 0, __PYX_ERR(0, 1263, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1263, __pyx_L1_error)

  /* "pyslow5.pyx":1270
 *         threads = number of threads to use to do batched random access
 *         '''
 *         for r in self._get_read_multi(read_list, threads, batchsize, pA, aux):             # <<<<<<<<<<<<<<
 *             yield r
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_get_read_multi); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[6] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_read_list, __pyx_cur_scope->__pyx_v_threads, __pyx_cur_scope->__pyx_v_batchsize, __pyx_cur_scope->__pyx_v_pA, __pyx_cur_scope->__pyx_v_aux};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 5+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1270, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[6] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_read_list, __pyx_cur_scope->__pyx_v_threads, __pyx_cur_scope->__pyx_v_batchsize, __pyx_cur_scope->__pyx_v_pA, __pyx_cur_scope->__pyx_v_aux};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 5+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1270, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(5+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_read_list);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_read_list);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_cur_scope->__pyx_v_read_list);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_threads);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_threads);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_cur_scope->__pyx_v_threads);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_batchsize);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_batchsize);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_cur_scope->__pyx_v_batchsize);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_pA);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_pA);
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_cur_scope->__pyx_v_pA);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_aux);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_aux);
    PyTuple_SET_ITEM(__pyx_t_5, 4+__pyx_t_4, __pyx_cur_scope->__pyx_v_aux);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1270, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1270, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1270, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1270, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_7(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1270, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_r);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_r, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1271
 *         '''
 *         for r in self._get_read_multi(read_list, threads, batchsize, pA, aux):
 *             yield r             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_r);
    __pyx_r = __pyx_cur_scope->__pyx_v_r;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_TraceReturn(__pyx_r, 0);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1271, __pyx_L1_error)

    /* "pyslow5.pyx":1270
 *         threads = number of threads to use to do batched random access
 *         '''
 *         for r in self._get_read_multi(read_list, threads, batchsize, pA, aux):             # <<<<<<<<<<<<<<
 *             yield r
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyslow5.pyx":1263
 * 
 * 
 *     def get_read_list_multi(self, read_list, threads=4, batchsize=100, pA=False, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         returns generator for random access of slow5 file
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("get_read_list_multi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1274
 * 
 * 
 *     def get_header_names(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get all header names and return list
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_37get_header_names(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7pyslow5_4Open_36get_header_names[] = "\n        get all header names and return list\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_37get_header_names(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_header_names (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_36get_header_names(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_36get_header_names(struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_v_headers = NULL;
  char const **__pyx_v_ret;
  uint64_t __pyx_8genexpr3__pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  uint64_t __pyx_t_8;
  uint64_t __pyx_t_9;
  uint64_t __pyx_t_10;
  char const *__pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_header_names", 0);
  __Pyx_TraceCall("get_header_names", __pyx_f[0], 1274, 0, __PYX_ERR(0, 1274, __pyx_L1_error));

  /* "pyslow5.pyx":1278
 *         get all header names and return list
 *         '''
 *         headers = []             # <<<<<<<<<<<<<<
 *         ret = slow5_get_hdr_keys(self.s5.header, &self.head_len)
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_headers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1279
 *         '''
 *         headers = []
 *         ret = slow5_get_hdr_keys(self.s5.header, &self.head_len)             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("slow5_get_hdr_keys head_len: {}".format(self.head_len))
 */
  __pyx_v_ret = slow5_get_hdr_keys(__pyx_v_self->s5->header, (&__pyx_v_self->head_len));

  /* "pyslow5.pyx":1281
 *         ret = slow5_get_hdr_keys(self.s5.header, &self.head_len)
 * 
 *         self.logger.debug("slow5_get_hdr_keys head_len: {}".format(self.head_len))             # <<<<<<<<<<<<<<
 *         if ret == NULL:
 *             self.logger.debug("slow5_get_hdr_keys ret is NULL")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_slow5_get_hdr_keys_head_len, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_uint64_t(__pyx_v_self->head_len); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1282
 * 
 *         self.logger.debug("slow5_get_hdr_keys head_len: {}".format(self.head_len))
 *         if ret == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.debug("slow5_get_hdr_keys ret is NULL")
 *             return headers
 */
  __pyx_t_7 = ((__pyx_v_ret == NULL) != 0);
  if (__pyx_t_7) {

    /* "pyslow5.pyx":1283
 *         self.logger.debug("slow5_get_hdr_keys head_len: {}".format(self.head_len))
 *         if ret == NULL:
 *             self.logger.debug("slow5_get_hdr_keys ret is NULL")             # <<<<<<<<<<<<<<
 *             return headers
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_slow5_get_hdr_keys_ret_is_NULL) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_slow5_get_hdr_keys_ret_is_NULL);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1284
 *         if ret == NULL:
 *             self.logger.debug("slow5_get_hdr_keys ret is NULL")
 *             return headers             # <<<<<<<<<<<<<<
 * 
 *         headers = [ret[i].decode() for i in range(self.head_len)]
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_headers);
    __pyx_r = __pyx_v_headers;
    goto __pyx_L0;

    /* "pyslow5.pyx":1282
 * 
 *         self.logger.debug("slow5_get_hdr_keys head_len: {}".format(self.head_len))
 *         if ret == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.debug("slow5_get_hdr_keys ret is NULL")
 *             return headers
 */
  }

  /* "pyslow5.pyx":1286
 *             return headers
 * 
 *         headers = [ret[i].decode() for i in range(self.head_len)]             # <<<<<<<<<<<<<<
 *         free(ret)
 *         return headers
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __pyx_v_self->head_len;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_8genexpr3__pyx_v_i = __pyx_t_10;
      __pyx_t_11 = (__pyx_v_ret[__pyx_8genexpr3__pyx_v_i]);
      __pyx_t_2 = __Pyx_decode_c_string(__pyx_t_11, 0, strlen(__pyx_t_11), NULL, NULL, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 1286, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
  } /* exit inner scope */
  __Pyx_DECREF_SET(__pyx_v_headers, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1287
 * 
 *         headers = [ret[i].decode() for i in range(self.head_len)]
 *         free(ret)             # <<<<<<<<<<<<<<
 *         return headers
 * 
 */
  free(__pyx_v_ret);

  /* "pyslow5.pyx":1288
 *         headers = [ret[i].decode() for i in range(self.head_len)]
 *         free(ret)
 *         return headers             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_headers);
  __pyx_r = __pyx_v_headers;
  goto __pyx_L0;

  /* "pyslow5.pyx":1274
 * 
 * 
 *     def get_header_names(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get all header names and return list
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyslow5.Open.get_header_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_headers);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1291
 * 
 * 
 *     def get_header_value(self, attr, read_group=0):             # <<<<<<<<<<<<<<
 *         '''
 *         get header attribute value
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_39get_header_value(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_38get_header_value[] = "\n        get header attribute value\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_39get_header_value(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_attr = 0;
  PyObject *__pyx_v_read_group = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_header_value (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_attr,&__pyx_n_s_read_group,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_attr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_group);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_header_value") < 0)) __PYX_ERR(0, 1291, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_attr = values[0];
    __pyx_v_read_group = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_header_value", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1291, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_header_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_38get_header_value(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_attr, __pyx_v_read_group);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_38get_header_value(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_attr, PyObject *__pyx_v_read_group) {
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_v_ret = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char const *__pyx_t_4;
  uint32_t __pyx_t_5;
  char *__pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_header_value", 0);
  __Pyx_TraceCall("get_header_value", __pyx_f[0], 1291, 0, __PYX_ERR(0, 1291, __pyx_L1_error));

  /* "pyslow5.pyx":1295
 *         get header attribute value
 *         '''
 *         a = str.encode(attr)             # <<<<<<<<<<<<<<
 *         ret = ''
 *         ret = slow5_hdr_get(a, read_group, self.s5.header).decode()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&PyUnicode_Type)), __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_attr) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_attr);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1296
 *         '''
 *         a = str.encode(attr)
 *         ret = ''             # <<<<<<<<<<<<<<
 *         ret = slow5_hdr_get(a, read_group, self.s5.header).decode()
 *         if not ret:
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __pyx_v_ret = __pyx_kp_u_;

  /* "pyslow5.pyx":1297
 *         a = str.encode(attr)
 *         ret = ''
 *         ret = slow5_hdr_get(a, read_group, self.s5.header).decode()             # <<<<<<<<<<<<<<
 *         if not ret:
 *             self.logger.warning("get_header_value header value not found: {} - rg: {}".format(attr, read_group))
 */
  __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_a); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1297, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_uint32_t(__pyx_v_read_group); if (unlikely((__pyx_t_5 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1297, __pyx_L1_error)
  __pyx_t_6 = slow5_hdr_get(__pyx_t_4, __pyx_t_5, __pyx_v_self->s5->header);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_t_6, 0, strlen(__pyx_t_6), NULL, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_ret, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1298
 *         ret = ''
 *         ret = slow5_hdr_get(a, read_group, self.s5.header).decode()
 *         if not ret:             # <<<<<<<<<<<<<<
 *             self.logger.warning("get_header_value header value not found: {} - rg: {}".format(attr, read_group))
 *             return None
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_ret); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 1298, __pyx_L1_error)
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "pyslow5.pyx":1299
 *         ret = slow5_hdr_get(a, read_group, self.s5.header).decode()
 *         if not ret:
 *             self.logger.warning("get_header_value header value not found: {} - rg: {}".format(attr, read_group))             # <<<<<<<<<<<<<<
 *             return None
 *         return ret
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_header_value_header_value_no, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = NULL;
    __pyx_t_11 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_11 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_9)) {
      PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_attr, __pyx_v_read_group};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
      PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_attr, __pyx_v_read_group};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_10) {
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
      }
      __Pyx_INCREF(__pyx_v_attr);
      __Pyx_GIVEREF(__pyx_v_attr);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_v_attr);
      __Pyx_INCREF(__pyx_v_read_group);
      __Pyx_GIVEREF(__pyx_v_read_group);
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_v_read_group);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_12, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1300
 *         if not ret:
 *             self.logger.warning("get_header_value header value not found: {} - rg: {}".format(attr, read_group))
 *             return None             # <<<<<<<<<<<<<<
 *         return ret
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pyslow5.pyx":1298
 *         ret = ''
 *         ret = slow5_hdr_get(a, read_group, self.s5.header).decode()
 *         if not ret:             # <<<<<<<<<<<<<<
 *             self.logger.warning("get_header_value header value not found: {} - rg: {}".format(attr, read_group))
 *             return None
 */
  }

  /* "pyslow5.pyx":1301
 *             self.logger.warning("get_header_value header value not found: {} - rg: {}".format(attr, read_group))
 *             return None
 *         return ret             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ret);
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "pyslow5.pyx":1291
 * 
 * 
 *     def get_header_value(self, attr, read_group=0):             # <<<<<<<<<<<<<<
 *         '''
 *         get header attribute value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("pyslow5.Open.get_header_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1304
 * 
 * 
 *     def get_all_headers(self, read_group=0):             # <<<<<<<<<<<<<<
 *         '''
 *         get all headers present and return dictionary
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_41get_all_headers(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_40get_all_headers[] = "\n        get all headers present and return dictionary\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_41get_all_headers(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_group = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_all_headers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_group,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_group);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_all_headers") < 0)) __PYX_ERR(0, 1304, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_read_group = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_all_headers", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1304, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_all_headers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_40get_all_headers(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_read_group);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_40get_all_headers(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_read_group) {
  PyObject *__pyx_v_dic = NULL;
  PyObject *__pyx_v_headers = NULL;
  PyObject *__pyx_v_header = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_all_headers", 0);
  __Pyx_TraceCall("get_all_headers", __pyx_f[0], 1304, 0, __PYX_ERR(0, 1304, __pyx_L1_error));

  /* "pyslow5.pyx":1308
 *         get all headers present and return dictionary
 *         '''
 *         dic = {}             # <<<<<<<<<<<<<<
 *         headers = self.get_header_names()
 *         for header in headers:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dic = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1309
 *         '''
 *         dic = {}
 *         headers = self.get_header_names()             # <<<<<<<<<<<<<<
 *         for header in headers:
 *             val = self.get_header_value(header, read_group=read_group)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_header_names); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_headers = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1310
 *         dic = {}
 *         headers = self.get_header_names()
 *         for header in headers:             # <<<<<<<<<<<<<<
 *             val = self.get_header_value(header, read_group=read_group)
 *             dic[header] = val
 */
  if (likely(PyList_CheckExact(__pyx_v_headers)) || PyTuple_CheckExact(__pyx_v_headers)) {
    __pyx_t_1 = __pyx_v_headers; __Pyx_INCREF(__pyx_t_1); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_headers); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1310, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 1310, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1310, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 1310, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1310, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1310, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_header, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1311
 *         headers = self.get_header_names()
 *         for header in headers:
 *             val = self.get_header_value(header, read_group=read_group)             # <<<<<<<<<<<<<<
 *             dic[header] = val
 *         return dic
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_header_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_header);
    __Pyx_GIVEREF(__pyx_v_header);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_header);
    __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_read_group, __pyx_v_read_group) < 0) __PYX_ERR(0, 1311, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyslow5.pyx":1312
 *         for header in headers:
 *             val = self.get_header_value(header, read_group=read_group)
 *             dic[header] = val             # <<<<<<<<<<<<<<
 *         return dic
 * 
 */
    if (unlikely(PyDict_SetItem(__pyx_v_dic, __pyx_v_header, __pyx_v_val) < 0)) __PYX_ERR(0, 1312, __pyx_L1_error)

    /* "pyslow5.pyx":1310
 *         dic = {}
 *         headers = self.get_header_names()
 *         for header in headers:             # <<<<<<<<<<<<<<
 *             val = self.get_header_value(header, read_group=read_group)
 *             dic[header] = val
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1313
 *             val = self.get_header_value(header, read_group=read_group)
 *             dic[header] = val
 *         return dic             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_dic);
  __pyx_r = __pyx_v_dic;
  goto __pyx_L0;

  /* "pyslow5.pyx":1304
 * 
 * 
 *     def get_all_headers(self, read_group=0):             # <<<<<<<<<<<<<<
 *         '''
 *         get all headers present and return dictionary
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyslow5.Open.get_all_headers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dic);
  __Pyx_XDECREF(__pyx_v_headers);
  __Pyx_XDECREF(__pyx_v_header);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1316
 * 
 * 
 *     def get_aux_names(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get all aux field names, returns list
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_43get_aux_names(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7pyslow5_4Open_42get_aux_names[] = "\n        get all aux field names, returns list\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_43get_aux_names(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_aux_names (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_42get_aux_names(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_42get_aux_names(struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_v_aux_names = NULL;
  char **__pyx_v_ret;
  uint64_t __pyx_8genexpr4__pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  uint64_t __pyx_t_8;
  uint64_t __pyx_t_9;
  uint64_t __pyx_t_10;
  char *__pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_aux_names", 0);
  __Pyx_TraceCall("get_aux_names", __pyx_f[0], 1316, 0, __PYX_ERR(0, 1316, __pyx_L1_error));

  /* "pyslow5.pyx":1320
 *         get all aux field names, returns list
 *         '''
 *         self.logger.debug("get_aux_names starting")             # <<<<<<<<<<<<<<
 *         aux_names = []
 *         ret = slow5_get_aux_names(self.s5.header, &self.aux_len)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_get_aux_names_starting) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_get_aux_names_starting);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1321
 *         '''
 *         self.logger.debug("get_aux_names starting")
 *         aux_names = []             # <<<<<<<<<<<<<<
 *         ret = slow5_get_aux_names(self.s5.header, &self.aux_len)
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_aux_names = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1322
 *         self.logger.debug("get_aux_names starting")
 *         aux_names = []
 *         ret = slow5_get_aux_names(self.s5.header, &self.aux_len)             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("get_aux_names aux_len: {}".format(self.aux_len))
 */
  __pyx_v_ret = slow5_get_aux_names(__pyx_v_self->s5->header, (&__pyx_v_self->aux_len));

  /* "pyslow5.pyx":1324
 *         ret = slow5_get_aux_names(self.s5.header, &self.aux_len)
 * 
 *         self.logger.debug("get_aux_names aux_len: {}".format(self.aux_len))             # <<<<<<<<<<<<<<
 *         if ret == NULL:
 *             self.logger.warning("get_aux_names ret is NULL")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_names_aux_len, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_uint64_t(__pyx_v_self->aux_len); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1325
 * 
 *         self.logger.debug("get_aux_names aux_len: {}".format(self.aux_len))
 *         if ret == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.warning("get_aux_names ret is NULL")
 *             return aux_names
 */
  __pyx_t_7 = ((__pyx_v_ret == NULL) != 0);
  if (__pyx_t_7) {

    /* "pyslow5.pyx":1326
 *         self.logger.debug("get_aux_names aux_len: {}".format(self.aux_len))
 *         if ret == NULL:
 *             self.logger.warning("get_aux_names ret is NULL")             # <<<<<<<<<<<<<<
 *             return aux_names
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_get_aux_names_ret_is_NULL) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_get_aux_names_ret_is_NULL);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1327
 *         if ret == NULL:
 *             self.logger.warning("get_aux_names ret is NULL")
 *             return aux_names             # <<<<<<<<<<<<<<
 * 
 *         aux_names = [ret[i].decode() for i in range(self.aux_len)]
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_aux_names);
    __pyx_r = __pyx_v_aux_names;
    goto __pyx_L0;

    /* "pyslow5.pyx":1325
 * 
 *         self.logger.debug("get_aux_names aux_len: {}".format(self.aux_len))
 *         if ret == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.warning("get_aux_names ret is NULL")
 *             return aux_names
 */
  }

  /* "pyslow5.pyx":1329
 *             return aux_names
 * 
 *         aux_names = [ret[i].decode() for i in range(self.aux_len)]             # <<<<<<<<<<<<<<
 *         return aux_names
 * 
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __pyx_v_self->aux_len;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_8genexpr4__pyx_v_i = __pyx_t_10;
      __pyx_t_11 = (__pyx_v_ret[__pyx_8genexpr4__pyx_v_i]);
      __pyx_t_2 = __Pyx_decode_c_string(__pyx_t_11, 0, strlen(__pyx_t_11), NULL, NULL, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 1329, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
  } /* exit inner scope */
  __Pyx_DECREF_SET(__pyx_v_aux_names, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1330
 * 
 *         aux_names = [ret[i].decode() for i in range(self.aux_len)]
 *         return aux_names             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_aux_names);
  __pyx_r = __pyx_v_aux_names;
  goto __pyx_L0;

  /* "pyslow5.pyx":1316
 * 
 * 
 *     def get_aux_names(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get all aux field names, returns list
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyslow5.Open.get_aux_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_aux_names);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1333
 * 
 * 
 *     def get_aux_types(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get aux attribute type
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_45get_aux_types(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7pyslow5_4Open_44get_aux_types[] = "\n        get aux attribute type\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_45get_aux_types(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_aux_types (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_44get_aux_types(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_44get_aux_types(struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_v_aux_types = NULL;
  uint64_t __pyx_8genexpr5__pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  uint64_t __pyx_t_8;
  uint64_t __pyx_t_9;
  uint64_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_aux_types", 0);
  __Pyx_TraceCall("get_aux_types", __pyx_f[0], 1333, 0, __PYX_ERR(0, 1333, __pyx_L1_error));

  /* "pyslow5.pyx":1337
 *         get aux attribute type
 *         '''
 *         self.logger.debug("get_aux_types starting")             # <<<<<<<<<<<<<<
 *         aux_types = []
 *         self.s5_aux_type = slow5_get_aux_types(self.s5.header, &self.aux_len)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_get_aux_types_starting) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_get_aux_types_starting);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1338
 *         '''
 *         self.logger.debug("get_aux_types starting")
 *         aux_types = []             # <<<<<<<<<<<<<<
 *         self.s5_aux_type = slow5_get_aux_types(self.s5.header, &self.aux_len)
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1338, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1339
 *         self.logger.debug("get_aux_types starting")
 *         aux_types = []
 *         self.s5_aux_type = slow5_get_aux_types(self.s5.header, &self.aux_len)             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("get_aux_types aux_len: {}".format(self.aux_len))
 */
  __pyx_v_self->s5_aux_type = slow5_get_aux_types(__pyx_v_self->s5->header, (&__pyx_v_self->aux_len));

  /* "pyslow5.pyx":1341
 *         self.s5_aux_type = slow5_get_aux_types(self.s5.header, &self.aux_len)
 * 
 *         self.logger.debug("get_aux_types aux_len: {}".format(self.aux_len))             # <<<<<<<<<<<<<<
 *         if self.s5_aux_type == NULL:
 *             self.logger.warning("get_aux_types self.s5_aux_type is NULL")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_get_aux_types_aux_len, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_uint64_t(__pyx_v_self->aux_len); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1342
 * 
 *         self.logger.debug("get_aux_types aux_len: {}".format(self.aux_len))
 *         if self.s5_aux_type == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.warning("get_aux_types self.s5_aux_type is NULL")
 *             return aux_types
 */
  __pyx_t_7 = ((__pyx_v_self->s5_aux_type == NULL) != 0);
  if (__pyx_t_7) {

    /* "pyslow5.pyx":1343
 *         self.logger.debug("get_aux_types aux_len: {}".format(self.aux_len))
 *         if self.s5_aux_type == NULL:
 *             self.logger.warning("get_aux_types self.s5_aux_type is NULL")             # <<<<<<<<<<<<<<
 *             return aux_types
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_get_aux_types_self_s5_aux_type_i) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_get_aux_types_self_s5_aux_type_i);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1344
 *         if self.s5_aux_type == NULL:
 *             self.logger.warning("get_aux_types self.s5_aux_type is NULL")
 *             return aux_types             # <<<<<<<<<<<<<<
 * 
 *         aux_types = [self.s5_aux_type[i] for i in range(self.aux_len)]
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_aux_types);
    __pyx_r = __pyx_v_aux_types;
    goto __pyx_L0;

    /* "pyslow5.pyx":1342
 * 
 *         self.logger.debug("get_aux_types aux_len: {}".format(self.aux_len))
 *         if self.s5_aux_type == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.warning("get_aux_types self.s5_aux_type is NULL")
 *             return aux_types
 */
  }

  /* "pyslow5.pyx":1346
 *             return aux_types
 * 
 *         aux_types = [self.s5_aux_type[i] for i in range(self.aux_len)]             # <<<<<<<<<<<<<<
 *         return aux_types
 * 
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1346, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __pyx_v_self->aux_len;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_8genexpr5__pyx_v_i = __pyx_t_10;
      __pyx_t_2 = __Pyx_PyInt_From_enum__slow5_aux_type((__pyx_v_self->s5_aux_type[__pyx_8genexpr5__pyx_v_i])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1346, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 1346, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
  } /* exit inner scope */
  __Pyx_DECREF_SET(__pyx_v_aux_types, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1347
 * 
 *         aux_types = [self.s5_aux_type[i] for i in range(self.aux_len)]
 *         return aux_types             # <<<<<<<<<<<<<<
 * 
 *     # ==========================================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_aux_types);
  __pyx_r = __pyx_v_aux_types;
  goto __pyx_L0;

  /* "pyslow5.pyx":1333
 * 
 * 
 *     def get_aux_types(self):             # <<<<<<<<<<<<<<
 *         '''
 *         get aux attribute type
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyslow5.Open.get_aux_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_aux_types);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1353
 *     # ==========================================================================
 * 
 *     def get_empty_header(self):             # <<<<<<<<<<<<<<
 *         '''
 *         returns example empty header dic for user to populate
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_47get_empty_header(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7pyslow5_4Open_46get_empty_header[] = "\n        returns example empty header dic for user to populate\n        Any values not populated will be skipped\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_47get_empty_header(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_empty_header (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_46get_empty_header(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_46get_empty_header(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_v_header = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_empty_header", 0);
  __Pyx_TraceCall("get_empty_header", __pyx_f[0], 1353, 0, __PYX_ERR(0, 1353, __pyx_L1_error));

  /* "pyslow5.pyx":1358
 *         Any values not populated will be skipped
 *         '''
 *         header = {"asic_id": None,             # <<<<<<<<<<<<<<
 *                   "asic_id_eeprom": None,
 *                   "asic_temp": None,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(38); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_asic_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1359
 *         '''
 *         header = {"asic_id": None,
 *                   "asic_id_eeprom": None,             # <<<<<<<<<<<<<<
 *                   "asic_temp": None,
 *                   "asic_version": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_asic_id_eeprom, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1360
 *         header = {"asic_id": None,
 *                   "asic_id_eeprom": None,
 *                   "asic_temp": None,             # <<<<<<<<<<<<<<
 *                   "asic_version": None,
 *                   "auto_update": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_asic_temp, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1361
 *                   "asic_id_eeprom": None,
 *                   "asic_temp": None,
 *                   "asic_version": None,             # <<<<<<<<<<<<<<
 *                   "auto_update": None,
 *                   "auto_update_source": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_asic_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1362
 *                   "asic_temp": None,
 *                   "asic_version": None,
 *                   "auto_update": None,             # <<<<<<<<<<<<<<
 *                   "auto_update_source": None,
 *                   "barcoding_enabled": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_auto_update, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1363
 *                   "asic_version": None,
 *                   "auto_update": None,
 *                   "auto_update_source": None,             # <<<<<<<<<<<<<<
 *                   "barcoding_enabled": None,
 *                   "bream_is_standard": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_auto_update_source, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1364
 *                   "auto_update": None,
 *                   "auto_update_source": None,
 *                   "barcoding_enabled": None,             # <<<<<<<<<<<<<<
 *                   "bream_is_standard": None,
 *                   "configuration_version": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_barcoding_enabled, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1365
 *                   "auto_update_source": None,
 *                   "barcoding_enabled": None,
 *                   "bream_is_standard": None,             # <<<<<<<<<<<<<<
 *                   "configuration_version": None,
 *                   "device_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_bream_is_standard, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1366
 *                   "barcoding_enabled": None,
 *                   "bream_is_standard": None,
 *                   "configuration_version": None,             # <<<<<<<<<<<<<<
 *                   "device_id": None,
 *                   "device_type": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_configuration_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1367
 *                   "bream_is_standard": None,
 *                   "configuration_version": None,
 *                   "device_id": None,             # <<<<<<<<<<<<<<
 *                   "device_type": None,
 *                   "distribution_status": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_device_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1368
 *                   "configuration_version": None,
 *                   "device_id": None,
 *                   "device_type": None,             # <<<<<<<<<<<<<<
 *                   "distribution_status": None,
 *                   "distribution_version": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_device_type, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1369
 *                   "device_id": None,
 *                   "device_type": None,
 *                   "distribution_status": None,             # <<<<<<<<<<<<<<
 *                   "distribution_version": None,
 *                   "exp_script_name": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_distribution_status, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1370
 *                   "device_type": None,
 *                   "distribution_status": None,
 *                   "distribution_version": None,             # <<<<<<<<<<<<<<
 *                   "exp_script_name": None,
 *                   "exp_script_purpose": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_distribution_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1371
 *                   "distribution_status": None,
 *                   "distribution_version": None,
 *                   "exp_script_name": None,             # <<<<<<<<<<<<<<
 *                   "exp_script_purpose": None,
 *                   "exp_start_time": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_exp_script_name, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1372
 *                   "distribution_version": None,
 *                   "exp_script_name": None,
 *                   "exp_script_purpose": None,             # <<<<<<<<<<<<<<
 *                   "exp_start_time": None,
 *                   "experiment_duration_set": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_exp_script_purpose, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1373
 *                   "exp_script_name": None,
 *                   "exp_script_purpose": None,
 *                   "exp_start_time": None,             # <<<<<<<<<<<<<<
 *                   "experiment_duration_set": None,
 *                   "flow_cell_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_exp_start_time, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1374
 *                   "exp_script_purpose": None,
 *                   "exp_start_time": None,
 *                   "experiment_duration_set": None,             # <<<<<<<<<<<<<<
 *                   "flow_cell_id": None,
 *                   "flow_cell_product_code": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_experiment_duration_set, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1375
 *                   "exp_start_time": None,
 *                   "experiment_duration_set": None,
 *                   "flow_cell_id": None,             # <<<<<<<<<<<<<<
 *                   "flow_cell_product_code": None,
 *                   "guppy_version": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_flow_cell_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1376
 *                   "experiment_duration_set": None,
 *                   "flow_cell_id": None,
 *                   "flow_cell_product_code": None,             # <<<<<<<<<<<<<<
 *                   "guppy_version": None,
 *                   "heatsink_temp": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_flow_cell_product_code, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1377
 *                   "flow_cell_id": None,
 *                   "flow_cell_product_code": None,
 *                   "guppy_version": None,             # <<<<<<<<<<<<<<
 *                   "heatsink_temp": None,
 *                   "hostname": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_guppy_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1378
 *                   "flow_cell_product_code": None,
 *                   "guppy_version": None,
 *                   "heatsink_temp": None,             # <<<<<<<<<<<<<<
 *                   "hostname": None,
 *                   "installation_type": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_heatsink_temp, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1379
 *                   "guppy_version": None,
 *                   "heatsink_temp": None,
 *                   "hostname": None,             # <<<<<<<<<<<<<<
 *                   "installation_type": None,
 *                   "local_basecalling": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hostname, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1380
 *                   "heatsink_temp": None,
 *                   "hostname": None,
 *                   "installation_type": None,             # <<<<<<<<<<<<<<
 *                   "local_basecalling": None,
 *                   "operating_system": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_installation_type, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1381
 *                   "hostname": None,
 *                   "installation_type": None,
 *                   "local_basecalling": None,             # <<<<<<<<<<<<<<
 *                   "operating_system": None,
 *                   "package": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_local_basecalling, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1382
 *                   "installation_type": None,
 *                   "local_basecalling": None,
 *                   "operating_system": None,             # <<<<<<<<<<<<<<
 *                   "package": None,
 *                   "protocol_group_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_operating_system, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1383
 *                   "local_basecalling": None,
 *                   "operating_system": None,
 *                   "package": None,             # <<<<<<<<<<<<<<
 *                   "protocol_group_id": None,
 *                   "protocol_run_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_package, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1384
 *                   "operating_system": None,
 *                   "package": None,
 *                   "protocol_group_id": None,             # <<<<<<<<<<<<<<
 *                   "protocol_run_id": None,
 *                   "protocol_start_time": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_protocol_group_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1385
 *                   "package": None,
 *                   "protocol_group_id": None,
 *                   "protocol_run_id": None,             # <<<<<<<<<<<<<<
 *                   "protocol_start_time": None,
 *                   "protocols_version": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_protocol_run_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1386
 *                   "protocol_group_id": None,
 *                   "protocol_run_id": None,
 *                   "protocol_start_time": None,             # <<<<<<<<<<<<<<
 *                   "protocols_version": None,
 *                   "run_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_protocol_start_time, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1387
 *                   "protocol_run_id": None,
 *                   "protocol_start_time": None,
 *                   "protocols_version": None,             # <<<<<<<<<<<<<<
 *                   "run_id": None,
 *                   "sample_frequency": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_protocols_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1388
 *                   "protocol_start_time": None,
 *                   "protocols_version": None,
 *                   "run_id": None,             # <<<<<<<<<<<<<<
 *                   "sample_frequency": None,
 *                   "sample_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_run_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1389
 *                   "protocols_version": None,
 *                   "run_id": None,
 *                   "sample_frequency": None,             # <<<<<<<<<<<<<<
 *                   "sample_id": None,
 *                   "sequencing_kit": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sample_frequency, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1390
 *                   "run_id": None,
 *                   "sample_frequency": None,
 *                   "sample_id": None,             # <<<<<<<<<<<<<<
 *                   "sequencing_kit": None,
 *                   "usb_config": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sample_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1391
 *                   "sample_frequency": None,
 *                   "sample_id": None,
 *                   "sequencing_kit": None,             # <<<<<<<<<<<<<<
 *                   "usb_config": None,
 *                   "version": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sequencing_kit, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1392
 *                   "sample_id": None,
 *                   "sequencing_kit": None,
 *                   "usb_config": None,             # <<<<<<<<<<<<<<
 *                   "version": None,
 *                   "hublett_board_id": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_usb_config, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1393
 *                   "sequencing_kit": None,
 *                   "usb_config": None,
 *                   "version": None,             # <<<<<<<<<<<<<<
 *                   "hublett_board_id": None,
 *                   "satellite_firmware_version": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1394
 *                   "usb_config": None,
 *                   "version": None,
 *                   "hublett_board_id": None,             # <<<<<<<<<<<<<<
 *                   "satellite_firmware_version": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hublett_board_id, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)

  /* "pyslow5.pyx":1395
 *                   "version": None,
 *                   "hublett_board_id": None,
 *                   "satellite_firmware_version": None}             # <<<<<<<<<<<<<<
 * 
 *         return header
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_satellite_firmware_version, Py_None) < 0) __PYX_ERR(0, 1358, __pyx_L1_error)
  __pyx_v_header = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1397
 *                   "satellite_firmware_version": None}
 * 
 *         return header             # <<<<<<<<<<<<<<
 * 
 *     def get_empty_record(self, aux=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_header);
  __pyx_r = __pyx_v_header;
  goto __pyx_L0;

  /* "pyslow5.pyx":1353
 *     # ==========================================================================
 * 
 *     def get_empty_header(self):             # <<<<<<<<<<<<<<
 *         '''
 *         returns example empty header dic for user to populate
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyslow5.Open.get_empty_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_header);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1399
 *         return header
 * 
 *     def get_empty_record(self, aux=False):             # <<<<<<<<<<<<<<
 *         '''
 *         Return a dictionary of empty record and types
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_49get_empty_record(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_48get_empty_record[] = "\n        Return a dictionary of empty record and types\n        if axu=True, a second dictionary will be returned with optional aux record\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_49get_empty_record(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_empty_record (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_aux,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_empty_record") < 0)) __PYX_ERR(0, 1399, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_aux = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_empty_record", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1399, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.get_empty_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_48get_empty_record(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_48get_empty_record(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_aux) {
  PyObject *__pyx_v_record = NULL;
  PyObject *__pyx_v_aux_rec = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_empty_record", 0);
  __Pyx_TraceCall("get_empty_record", __pyx_f[0], 1399, 0, __PYX_ERR(0, 1399, __pyx_L1_error));

  /* "pyslow5.pyx":1404
 *         if axu=True, a second dictionary will be returned with optional aux record
 *         '''
 *         record = {"read_id": None,             # <<<<<<<<<<<<<<
 *                   "read_group": 0,
 *                   "digitisation": None,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_id, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_group, __pyx_int_0) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)

  /* "pyslow5.pyx":1406
 *         record = {"read_id": None,
 *                   "read_group": 0,
 *                   "digitisation": None,             # <<<<<<<<<<<<<<
 *                   "offset": None,
 *                   "range": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_digitisation, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)

  /* "pyslow5.pyx":1407
 *                   "read_group": 0,
 *                   "digitisation": None,
 *                   "offset": None,             # <<<<<<<<<<<<<<
 *                   "range": None,
 *                   "sampling_rate": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_offset, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)

  /* "pyslow5.pyx":1408
 *                   "digitisation": None,
 *                   "offset": None,
 *                   "range": None,             # <<<<<<<<<<<<<<
 *                   "sampling_rate": None,
 *                   "len_raw_signal": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_range, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)

  /* "pyslow5.pyx":1409
 *                   "offset": None,
 *                   "range": None,
 *                   "sampling_rate": None,             # <<<<<<<<<<<<<<
 *                   "len_raw_signal": None,
 *                   "signal": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sampling_rate, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)

  /* "pyslow5.pyx":1410
 *                   "range": None,
 *                   "sampling_rate": None,
 *                   "len_raw_signal": None,             # <<<<<<<<<<<<<<
 *                   "signal": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_len_raw_signal, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)

  /* "pyslow5.pyx":1411
 *                   "sampling_rate": None,
 *                   "len_raw_signal": None,
 *                   "signal": None}             # <<<<<<<<<<<<<<
 * 
 *         aux_rec = {"channel_number": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_signal, Py_None) < 0) __PYX_ERR(0, 1404, __pyx_L1_error)
  __pyx_v_record = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1413
 *                   "signal": None}
 * 
 *         aux_rec = {"channel_number": None,             # <<<<<<<<<<<<<<
 *                "median_before": None,
 *                "read_number": None,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, Py_None) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)

  /* "pyslow5.pyx":1414
 * 
 *         aux_rec = {"channel_number": None,
 *                "median_before": None,             # <<<<<<<<<<<<<<
 *                "read_number": None,
 *                "start_mux": None,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, Py_None) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)

  /* "pyslow5.pyx":1415
 *         aux_rec = {"channel_number": None,
 *                "median_before": None,
 *                "read_number": None,             # <<<<<<<<<<<<<<
 *                "start_mux": None,
 *                "start_time": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, Py_None) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)

  /* "pyslow5.pyx":1416
 *                "median_before": None,
 *                "read_number": None,
 *                "start_mux": None,             # <<<<<<<<<<<<<<
 *                "start_time": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, Py_None) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)

  /* "pyslow5.pyx":1417
 *                "read_number": None,
 *                "start_mux": None,
 *                "start_time": None}             # <<<<<<<<<<<<<<
 * 
 *         if aux:
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, Py_None) < 0) __PYX_ERR(0, 1413, __pyx_L1_error)
  __pyx_v_aux_rec = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1419
 *                "start_time": None}
 * 
 *         if aux:             # <<<<<<<<<<<<<<
 *             return record, aux_rec
 *         return record
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_aux); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1419, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "pyslow5.pyx":1420
 * 
 *         if aux:
 *             return record, aux_rec             # <<<<<<<<<<<<<<
 *         return record
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_record);
    __Pyx_GIVEREF(__pyx_v_record);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_record);
    __Pyx_INCREF(__pyx_v_aux_rec);
    __Pyx_GIVEREF(__pyx_v_aux_rec);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_aux_rec);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "pyslow5.pyx":1419
 *                "start_time": None}
 * 
 *         if aux:             # <<<<<<<<<<<<<<
 *             return record, aux_rec
 *         return record
 */
  }

  /* "pyslow5.pyx":1421
 *         if aux:
 *             return record, aux_rec
 *         return record             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_record);
  __pyx_r = __pyx_v_record;
  goto __pyx_L0;

  /* "pyslow5.pyx":1399
 *         return header
 * 
 *     def get_empty_record(self, aux=False):             # <<<<<<<<<<<<<<
 *         '''
 *         Return a dictionary of empty record and types
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyslow5.Open.get_empty_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_record);
  __Pyx_XDECREF(__pyx_v_aux_rec);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1424
 * 
 * 
 *     def _header_type_validation(self, header):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate header types before pushing to C side
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_51_header_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_v_header); /*proto*/
static char __pyx_doc_7pyslow5_4Open_50_header_type_validation[] = "\n        internal function to validate header types before pushing to C side\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_51_header_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_v_header) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_header_type_validation (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_50_header_type_validation(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), ((PyObject *)__pyx_v_header));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_50_header_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_header) {
  PyObject *__pyx_v_h = NULL;
  PyTypeObject *__pyx_v_t = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_header_type_validation", 0);
  __Pyx_TraceCall("_header_type_validation", __pyx_f[0], 1424, 0, __PYX_ERR(0, 1424, __pyx_L1_error));

  /* "pyslow5.pyx":1428
 *         internal function to validate header types before pushing to C side
 *         '''
 *         for h in header:             # <<<<<<<<<<<<<<
 *             if header[h] is not None:
 *                 t = type(header[h])
 */
  if (likely(PyList_CheckExact(__pyx_v_header)) || PyTuple_CheckExact(__pyx_v_header)) {
    __pyx_t_1 = __pyx_v_header; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_header); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1428, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1428, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1428, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1428, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1428, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1428, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1428, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_h, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyslow5.pyx":1429
 *         '''
 *         for h in header:
 *             if header[h] is not None:             # <<<<<<<<<<<<<<
 *                 t = type(header[h])
 *                 if t is not type("string"):
 */
    __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_header, __pyx_v_h); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = (__pyx_t_4 != Py_None);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {

      /* "pyslow5.pyx":1430
 *         for h in header:
 *             if header[h] is not None:
 *                 t = type(header[h])             # <<<<<<<<<<<<<<
 *                 if t is not type("string"):
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))
 */
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_header, __pyx_v_h); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_t_4)));
      __Pyx_XDECREF_SET(__pyx_v_t, ((PyTypeObject*)((PyObject *)Py_TYPE(__pyx_t_4))));
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyslow5.pyx":1431
 *             if header[h] is not None:
 *                 t = type(header[h])
 *                 if t is not type("string"):             # <<<<<<<<<<<<<<
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))
 *                     #try and convert
 */
      __pyx_t_6 = (__pyx_v_t != ((PyTypeObject*)((PyObject *)Py_TYPE(__pyx_n_u_string))));
      __pyx_t_5 = (__pyx_t_6 != 0);
      if (__pyx_t_5) {

        /* "pyslow5.pyx":1432
 *                 t = type(header[h])
 *                 if t is not type("string"):
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))             # <<<<<<<<<<<<<<
 *                     #try and convert
 *                     try:
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1432, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_header_type_validation_is_not_a, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1432, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_v_header, __pyx_v_h); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1432, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = NULL;
        __pyx_t_12 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_12 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_v_h, __pyx_t_10};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1432, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_v_h, __pyx_t_10};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1432, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1432, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_h);
          __Pyx_GIVEREF(__pyx_v_h);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_v_h);
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_t_10);
          __pyx_t_10 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1432, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_4 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1432, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pyslow5.pyx":1434
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))
 *                     #try and convert
 *                     try:             # <<<<<<<<<<<<<<
 *                         s = str(header[h])
 *                         header[h] = s
 */
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          /*try:*/ {

            /* "pyslow5.pyx":1435
 *                     #try and convert
 *                     try:
 *                         s = str(header[h])             # <<<<<<<<<<<<<<
 *                         header[h] = s
 *                         self.logger.warning("_header_type_validation {}: {} conversion successful".format(h, s))
 */
            __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_header, __pyx_v_h); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1435, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1435, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_7);
            __pyx_t_7 = 0;

            /* "pyslow5.pyx":1436
 *                     try:
 *                         s = str(header[h])
 *                         header[h] = s             # <<<<<<<<<<<<<<
 *                         self.logger.warning("_header_type_validation {}: {} conversion successful".format(h, s))
 *                     except:
 */
            if (unlikely(PyObject_SetItem(__pyx_v_header, __pyx_v_h, __pyx_v_s) < 0)) __PYX_ERR(0, 1436, __pyx_L7_error)

            /* "pyslow5.pyx":1437
 *                         s = str(header[h])
 *                         header[h] = s
 *                         self.logger.warning("_header_type_validation {}: {} conversion successful".format(h, s))             # <<<<<<<<<<<<<<
 *                     except:
 *                         self.logger.error("_header_type_validation {}: {} could not convert value to string".format(h, header[h]))
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1437, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_header_type_validation_conversi, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1437, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_13 = NULL;
            __pyx_t_12 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
              __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_9);
              if (likely(__pyx_t_13)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
                __Pyx_INCREF(__pyx_t_13);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_9, function);
                __pyx_t_12 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_h, __pyx_v_s};
              __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1437, __pyx_L7_error)
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_h, __pyx_v_s};
              __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1437, __pyx_L7_error)
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            {
              __pyx_t_10 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1437, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (__pyx_t_13) {
                __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_13); __pyx_t_13 = NULL;
              }
              __Pyx_INCREF(__pyx_v_h);
              __Pyx_GIVEREF(__pyx_v_h);
              PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_12, __pyx_v_h);
              __Pyx_INCREF(__pyx_v_s);
              __Pyx_GIVEREF(__pyx_v_s);
              PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_12, __pyx_v_s);
              __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1437, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __pyx_t_9 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_9)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            __pyx_t_7 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1437, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

            /* "pyslow5.pyx":1434
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))
 *                     #try and convert
 *                     try:             # <<<<<<<<<<<<<<
 *                         s = str(header[h])
 *                         header[h] = s
 */
          }
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L14_try_end;
          __pyx_L7_error:;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

          /* "pyslow5.pyx":1438
 *                         header[h] = s
 *                         self.logger.warning("_header_type_validation {}: {} conversion successful".format(h, s))
 *                     except:             # <<<<<<<<<<<<<<
 *                         self.logger.error("_header_type_validation {}: {} could not convert value to string".format(h, header[h]))
 *                         raise
 */
          /*except:*/ {
            __Pyx_AddTraceback("pyslow5.Open._header_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_4, &__pyx_t_8) < 0) __PYX_ERR(0, 1438, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_GOTREF(__pyx_t_8);

            /* "pyslow5.pyx":1439
 *                         self.logger.warning("_header_type_validation {}: {} conversion successful".format(h, s))
 *                     except:
 *                         self.logger.error("_header_type_validation {}: {} could not convert value to string".format(h, header[h]))             # <<<<<<<<<<<<<<
 *                         raise
 *         return header
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_header_type_validation_could_no, __pyx_n_s_format); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_17 = __Pyx_PyObject_GetItem(__pyx_v_header, __pyx_v_h); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_18 = NULL;
            __pyx_t_12 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
              __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_11);
              if (likely(__pyx_t_18)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
                __Pyx_INCREF(__pyx_t_18);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_11, function);
                __pyx_t_12 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_11)) {
              PyObject *__pyx_temp[3] = {__pyx_t_18, __pyx_v_h, __pyx_t_17};
              __pyx_t_13 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
              __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
              __Pyx_GOTREF(__pyx_t_13);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
              PyObject *__pyx_temp[3] = {__pyx_t_18, __pyx_v_h, __pyx_t_17};
              __pyx_t_13 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
              __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
              __Pyx_GOTREF(__pyx_t_13);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            } else
            #endif
            {
              __pyx_t_19 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
              __Pyx_GOTREF(__pyx_t_19);
              if (__pyx_t_18) {
                __Pyx_GIVEREF(__pyx_t_18); PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_t_18); __pyx_t_18 = NULL;
              }
              __Pyx_INCREF(__pyx_v_h);
              __Pyx_GIVEREF(__pyx_v_h);
              PyTuple_SET_ITEM(__pyx_t_19, 0+__pyx_t_12, __pyx_v_h);
              __Pyx_GIVEREF(__pyx_t_17);
              PyTuple_SET_ITEM(__pyx_t_19, 1+__pyx_t_12, __pyx_t_17);
              __pyx_t_17 = 0;
              __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_19, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
              __Pyx_GOTREF(__pyx_t_13);
              __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
            }
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __pyx_t_11 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
              if (likely(__pyx_t_11)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_11);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_10, function);
              }
            }
            __pyx_t_9 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_13) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_13);
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1439, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

            /* "pyslow5.pyx":1440
 *                     except:
 *                         self.logger.error("_header_type_validation {}: {} could not convert value to string".format(h, header[h]))
 *                         raise             # <<<<<<<<<<<<<<
 *         return header
 * 
 */
            __Pyx_GIVEREF(__pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_8);
            __Pyx_ErrRestoreWithState(__pyx_t_7, __pyx_t_4, __pyx_t_8);
            __pyx_t_7 = 0; __pyx_t_4 = 0; __pyx_t_8 = 0; 
            __PYX_ERR(0, 1440, __pyx_L9_except_error)
          }
          __pyx_L9_except_error:;

          /* "pyslow5.pyx":1434
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))
 *                     #try and convert
 *                     try:             # <<<<<<<<<<<<<<
 *                         s = str(header[h])
 *                         header[h] = s
 */
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L1_error;
          __pyx_L14_try_end:;
        }

        /* "pyslow5.pyx":1431
 *             if header[h] is not None:
 *                 t = type(header[h])
 *                 if t is not type("string"):             # <<<<<<<<<<<<<<
 *                     self.logger.warning("_header_type_validation {}: {} is not a string type, attempting to convert".format(h, header[h]))
 *                     #try and convert
 */
      }

      /* "pyslow5.pyx":1429
 *         '''
 *         for h in header:
 *             if header[h] is not None:             # <<<<<<<<<<<<<<
 *                 t = type(header[h])
 *                 if t is not type("string"):
 */
    }

    /* "pyslow5.pyx":1428
 *         internal function to validate header types before pushing to C side
 *         '''
 *         for h in header:             # <<<<<<<<<<<<<<
 *             if header[h] is not None:
 *                 t = type(header[h])
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1441
 *                         self.logger.error("_header_type_validation {}: {} could not convert value to string".format(h, header[h]))
 *                         raise
 *         return header             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_header);
  __pyx_r = __pyx_v_header;
  goto __pyx_L0;

  /* "pyslow5.pyx":1424
 * 
 * 
 *     def _header_type_validation(self, header):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate header types before pushing to C side
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_AddTraceback("pyslow5.Open._header_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_h);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1444
 * 
 * 
 *     def _aux_header_type_validation(self, user_aux_types):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate and convert aux types before pushing to C side
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_53_aux_header_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_v_user_aux_types); /*proto*/
static char __pyx_doc_7pyslow5_4Open_52_aux_header_type_validation[] = "\n        internal function to validate and convert aux types before pushing to C side\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_53_aux_header_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_v_user_aux_types) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_aux_header_type_validation (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_52_aux_header_type_validation(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), ((PyObject *)__pyx_v_user_aux_types));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_52_aux_header_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_user_aux_types) {
  PyObject *__pyx_v_slow5_aux_types = NULL;
  PyObject *__pyx_v_py_aux_types = NULL;
  PyObject *__pyx_v_C_aux_types = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_aux_header_type_validation", 0);
  __Pyx_TraceCall("_aux_header_type_validation", __pyx_f[0], 1444, 0, __PYX_ERR(0, 1444, __pyx_L1_error));

  /* "pyslow5.pyx":1448
 *         internal function to validate and convert aux types before pushing to C side
 *         '''
 *         slow5_aux_types = {}             # <<<<<<<<<<<<<<
 * 
 *         py_aux_types = {"channel_number": type("string"),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_slow5_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1450
 *         slow5_aux_types = {}
 * 
 *         py_aux_types = {"channel_number": type("string"),             # <<<<<<<<<<<<<<
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1450, __pyx_L1_error)

  /* "pyslow5.pyx":1451
 * 
 *         py_aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),             # <<<<<<<<<<<<<<
 *                      "read_number": type(10),
 *                      "start_mux": type(1),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1450, __pyx_L1_error)

  /* "pyslow5.pyx":1452
 *         py_aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),             # <<<<<<<<<<<<<<
 *                      "start_mux": type(1),
 *                      "start_time": type(100),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1450, __pyx_L1_error)

  /* "pyslow5.pyx":1453
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),
 *                      "start_mux": type(1),             # <<<<<<<<<<<<<<
 *                      "start_time": type(100),
 *                      "end_reason": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1450, __pyx_L1_error)

  /* "pyslow5.pyx":1454
 *                      "read_number": type(10),
 *                      "start_mux": type(1),
 *                      "start_time": type(100),             # <<<<<<<<<<<<<<
 *                      "end_reason": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, ((PyObject *)Py_TYPE(__pyx_int_100))) < 0) __PYX_ERR(0, 1450, __pyx_L1_error)

  /* "pyslow5.pyx":1455
 *                      "start_mux": type(1),
 *                      "start_time": type(100),
 *                      "end_reason": None}             # <<<<<<<<<<<<<<
 * 
 *         C_aux_types = {"channel_number": SLOW5_STRING,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_end_reason, Py_None) < 0) __PYX_ERR(0, 1450, __pyx_L1_error)
  __pyx_v_py_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1457
 *                      "end_reason": None}
 * 
 *         C_aux_types = {"channel_number": SLOW5_STRING,             # <<<<<<<<<<<<<<
 *                      "median_before":SLOW5_DOUBLE,
 *                      "read_number": SLOW5_INT32_T,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_enum__slow5_aux_type(SLOW5_STRING); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, __pyx_t_2) < 0) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":1458
 * 
 *         C_aux_types = {"channel_number": SLOW5_STRING,
 *                      "median_before":SLOW5_DOUBLE,             # <<<<<<<<<<<<<<
 *                      "read_number": SLOW5_INT32_T,
 *                      "start_mux": SLOW5_UINT8_T,
 */
  __pyx_t_2 = __Pyx_PyInt_From_enum__slow5_aux_type(SLOW5_DOUBLE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, __pyx_t_2) < 0) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":1459
 *         C_aux_types = {"channel_number": SLOW5_STRING,
 *                      "median_before":SLOW5_DOUBLE,
 *                      "read_number": SLOW5_INT32_T,             # <<<<<<<<<<<<<<
 *                      "start_mux": SLOW5_UINT8_T,
 *                      "start_time": SLOW5_UINT64_T,
 */
  __pyx_t_2 = __Pyx_PyInt_From_enum__slow5_aux_type(SLOW5_INT32_T); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, __pyx_t_2) < 0) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":1460
 *                      "median_before":SLOW5_DOUBLE,
 *                      "read_number": SLOW5_INT32_T,
 *                      "start_mux": SLOW5_UINT8_T,             # <<<<<<<<<<<<<<
 *                      "start_time": SLOW5_UINT64_T,
 *                      "end_reason": None}
 */
  __pyx_t_2 = __Pyx_PyInt_From_enum__slow5_aux_type(SLOW5_UINT8_T); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, __pyx_t_2) < 0) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":1461
 *                      "read_number": SLOW5_INT32_T,
 *                      "start_mux": SLOW5_UINT8_T,
 *                      "start_time": SLOW5_UINT64_T,             # <<<<<<<<<<<<<<
 *                      "end_reason": None}
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_enum__slow5_aux_type(SLOW5_UINT64_T); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, __pyx_t_2) < 0) __PYX_ERR(0, 1457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":1462
 *                      "start_mux": SLOW5_UINT8_T,
 *                      "start_time": SLOW5_UINT64_T,
 *                      "end_reason": None}             # <<<<<<<<<<<<<<
 * 
 *         for a in user_aux_types:
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_end_reason, Py_None) < 0) __PYX_ERR(0, 1457, __pyx_L1_error)
  __pyx_v_C_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1464
 *                      "end_reason": None}
 * 
 *         for a in user_aux_types:             # <<<<<<<<<<<<<<
 *             if user_aux_types[a] is None:
 *                 continue
 */
  if (likely(PyList_CheckExact(__pyx_v_user_aux_types)) || PyTuple_CheckExact(__pyx_v_user_aux_types)) {
    __pyx_t_1 = __pyx_v_user_aux_types; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_user_aux_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1464, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 1464, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1464, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 1464, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1464, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1464, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1465
 * 
 *         for a in user_aux_types:
 *             if user_aux_types[a] is None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if a not in py_aux_types:
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_user_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = (__pyx_t_2 == Py_None);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {

      /* "pyslow5.pyx":1466
 *         for a in user_aux_types:
 *             if user_aux_types[a] is None:
 *                 continue             # <<<<<<<<<<<<<<
 *             if a not in py_aux_types:
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))
 */
      goto __pyx_L3_continue;

      /* "pyslow5.pyx":1465
 * 
 *         for a in user_aux_types:
 *             if user_aux_types[a] is None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if a not in py_aux_types:
 */
    }

    /* "pyslow5.pyx":1467
 *             if user_aux_types[a] is None:
 *                 continue
 *             if a not in py_aux_types:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None
 */
    __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_v_a, __pyx_v_py_aux_types, Py_NE)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 1467, __pyx_L1_error)
    __pyx_t_5 = (__pyx_t_6 != 0);
    if (__pyx_t_5) {

      /* "pyslow5.pyx":1468
 *                 continue
 *             if a not in py_aux_types:
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))             # <<<<<<<<<<<<<<
 *                 return None
 *             if user_aux_types[a] == py_aux_types[a]:
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_aux_type_validation_user_aux_fi, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_v_user_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = NULL;
      __pyx_t_12 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_11)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
          __pyx_t_12 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_v_a, __pyx_t_10};
        __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1468, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_v_a, __pyx_t_10};
        __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1468, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      {
        __pyx_t_13 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1468, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (__pyx_t_11) {
          __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_t_10);
        __pyx_t_10 = 0;
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1468, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_2 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":1469
 *             if a not in py_aux_types:
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None             # <<<<<<<<<<<<<<
 *             if user_aux_types[a] == py_aux_types[a]:
 *                 slow5_aux_types[a] = C_aux_types[a]
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "pyslow5.pyx":1467
 *             if user_aux_types[a] is None:
 *                 continue
 *             if a not in py_aux_types:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None
 */
    }

    /* "pyslow5.pyx":1470
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None
 *             if user_aux_types[a] == py_aux_types[a]:             # <<<<<<<<<<<<<<
 *                 slow5_aux_types[a] = C_aux_types[a]
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_user_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_py_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_2, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1470, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 1470, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_5) {

      /* "pyslow5.pyx":1471
 *                 return None
 *             if user_aux_types[a] == py_aux_types[a]:
 *                 slow5_aux_types[a] = C_aux_types[a]             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.error("_aux_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, user_aux_types[a]))
 */
      __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_v_C_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1471, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(PyDict_SetItem(__pyx_v_slow5_aux_types, __pyx_v_a, __pyx_t_8) < 0)) __PYX_ERR(0, 1471, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyslow5.pyx":1470
 *                 self.logger.error("_aux_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None
 *             if user_aux_types[a] == py_aux_types[a]:             # <<<<<<<<<<<<<<
 *                 slow5_aux_types[a] = C_aux_types[a]
 *             else:
 */
      goto __pyx_L7;
    }

    /* "pyslow5.pyx":1473
 *                 slow5_aux_types[a] = C_aux_types[a]
 *             else:
 *                 self.logger.error("_aux_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, user_aux_types[a]))             # <<<<<<<<<<<<<<
 *                 return None
 *         return slow5_aux_types
 */
    /*else*/ {
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1473, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_aux_type_validation_user_aux_fi_2, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1473, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_user_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1473, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_10 = NULL;
      __pyx_t_12 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
          __pyx_t_12 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_a, __pyx_t_13};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1473, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_a, __pyx_t_13};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1473, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1473, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (__pyx_t_10) {
          __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_12, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_13);
        PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_12, __pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1473, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_8 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1473, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyslow5.pyx":1474
 *             else:
 *                 self.logger.error("_aux_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None             # <<<<<<<<<<<<<<
 *         return slow5_aux_types
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __pyx_L7:;

    /* "pyslow5.pyx":1464
 *                      "end_reason": None}
 * 
 *         for a in user_aux_types:             # <<<<<<<<<<<<<<
 *             if user_aux_types[a] is None:
 *                 continue
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1475
 *                 self.logger.error("_aux_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, user_aux_types[a]))
 *                 return None
 *         return slow5_aux_types             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_slow5_aux_types);
  __pyx_r = __pyx_v_slow5_aux_types;
  goto __pyx_L0;

  /* "pyslow5.pyx":1444
 * 
 * 
 *     def _aux_header_type_validation(self, user_aux_types):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate and convert aux types before pushing to C side
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyslow5.Open._aux_header_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_slow5_aux_types);
  __Pyx_XDECREF(__pyx_v_py_aux_types);
  __Pyx_XDECREF(__pyx_v_C_aux_types);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1478
 * 
 * 
 *     def _record_type_validation(self, user_record, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate and convert aux types before pushing to C side
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_55_record_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_54_record_type_validation[] = "\n        internal function to validate and convert aux types before pushing to C side\n        returns None, None on error\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_55_record_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_user_record = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_record_type_validation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_user_record,&__pyx_n_s_aux,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_user_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_record_type_validation") < 0)) __PYX_ERR(0, 1478, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_user_record = values[0];
    __pyx_v_aux = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_record_type_validation", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1478, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._record_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_54_record_type_validation(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_user_record, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_54_record_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_user_record, PyObject *__pyx_v_aux) {
  PyObject *__pyx_v_py_record_types = NULL;
  PyObject *__pyx_v_py_aux_types = NULL;
  PyObject *__pyx_v_new_aux = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  char const *__pyx_t_14;
  double __pyx_t_15;
  int32_t __pyx_t_16;
  uint8_t __pyx_t_17;
  uint64_t __pyx_t_18;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_record_type_validation", 0);
  __Pyx_TraceCall("_record_type_validation", __pyx_f[0], 1478, 0, __PYX_ERR(0, 1478, __pyx_L1_error));

  /* "pyslow5.pyx":1484
 *         '''
 * 
 *         py_record_types = {"read_id": type("string"),             # <<<<<<<<<<<<<<
 *                            "read_group": type(1),
 *                            "digitisation": type(1.0),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_id, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1485
 * 
 *         py_record_types = {"read_id": type("string"),
 *                            "read_group": type(1),             # <<<<<<<<<<<<<<
 *                            "digitisation": type(1.0),
 *                            "offset": type(1.0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_group, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1486
 *         py_record_types = {"read_id": type("string"),
 *                            "read_group": type(1),
 *                            "digitisation": type(1.0),             # <<<<<<<<<<<<<<
 *                            "offset": type(1.0),
 *                            "range": type(1.0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_digitisation, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1487
 *                            "read_group": type(1),
 *                            "digitisation": type(1.0),
 *                            "offset": type(1.0),             # <<<<<<<<<<<<<<
 *                            "range": type(1.0),
 *                            "sampling_rate": type(1.0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_offset, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1488
 *                            "digitisation": type(1.0),
 *                            "offset": type(1.0),
 *                            "range": type(1.0),             # <<<<<<<<<<<<<<
 *                            "sampling_rate": type(1.0),
 *                            "len_raw_signal": type(10),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_range, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1489
 *                            "offset": type(1.0),
 *                            "range": type(1.0),
 *                            "sampling_rate": type(1.0),             # <<<<<<<<<<<<<<
 *                            "len_raw_signal": type(10),
 *                            "signal": type(np.array([1, 2, 3], np.int16))}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sampling_rate, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1490
 *                            "range": type(1.0),
 *                            "sampling_rate": type(1.0),
 *                            "len_raw_signal": type(10),             # <<<<<<<<<<<<<<
 *                            "signal": type(np.array([1, 2, 3], np.int16))}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_len_raw_signal, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)

  /* "pyslow5.pyx":1491
 *                            "sampling_rate": type(1.0),
 *                            "len_raw_signal": type(10),
 *                            "signal": type(np.array([1, 2, 3], np.int16))}             # <<<<<<<<<<<<<<
 * 
 *         py_aux_types = {"channel_number": type("string"),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyList_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_int_1);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_int_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_3, 2, __pyx_int_3);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int16); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1491, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1491, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1491, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1491, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_signal, ((PyObject *)Py_TYPE(__pyx_t_2))) < 0) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_py_record_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1493
 *                            "signal": type(np.array([1, 2, 3], np.int16))}
 * 
 *         py_aux_types = {"channel_number": type("string"),             # <<<<<<<<<<<<<<
 *                         "median_before": type(1.0),
 *                         "read_number": type(10),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)

  /* "pyslow5.pyx":1494
 * 
 *         py_aux_types = {"channel_number": type("string"),
 *                         "median_before": type(1.0),             # <<<<<<<<<<<<<<
 *                         "read_number": type(10),
 *                         "start_mux": type(1),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)

  /* "pyslow5.pyx":1495
 *         py_aux_types = {"channel_number": type("string"),
 *                         "median_before": type(1.0),
 *                         "read_number": type(10),             # <<<<<<<<<<<<<<
 *                         "start_mux": type(1),
 *                         "start_time": type(100),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)

  /* "pyslow5.pyx":1496
 *                         "median_before": type(1.0),
 *                         "read_number": type(10),
 *                         "start_mux": type(1),             # <<<<<<<<<<<<<<
 *                         "start_time": type(100),
 *                         "end_reason": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)

  /* "pyslow5.pyx":1497
 *                         "read_number": type(10),
 *                         "start_mux": type(1),
 *                         "start_time": type(100),             # <<<<<<<<<<<<<<
 *                         "end_reason": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, ((PyObject *)Py_TYPE(__pyx_int_100))) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)

  /* "pyslow5.pyx":1498
 *                         "start_mux": type(1),
 *                         "start_time": type(100),
 *                         "end_reason": None}             # <<<<<<<<<<<<<<
 * 
 *         new_aux = {}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_end_reason, Py_None) < 0) __PYX_ERR(0, 1493, __pyx_L1_error)
  __pyx_v_py_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1500
 *                         "end_reason": None}
 * 
 *         new_aux = {}             # <<<<<<<<<<<<<<
 * 
 *         for a in user_record:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_aux = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1502
 *         new_aux = {}
 * 
 *         for a in user_record:             # <<<<<<<<<<<<<<
 *             if user_record[a] is None:
 *                 continue
 */
  if (likely(PyList_CheckExact(__pyx_v_user_record)) || PyTuple_CheckExact(__pyx_v_user_record)) {
    __pyx_t_1 = __pyx_v_user_record; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_user_record); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1502, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1502, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1502, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1502, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1502, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_10(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1502, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1503
 * 
 *         for a in user_record:
 *             if user_record[a] is None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if a not in py_record_types:
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = (__pyx_t_2 == Py_None);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_12 = (__pyx_t_11 != 0);
    if (__pyx_t_12) {

      /* "pyslow5.pyx":1504
 *         for a in user_record:
 *             if user_record[a] is None:
 *                 continue             # <<<<<<<<<<<<<<
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 */
      goto __pyx_L3_continue;

      /* "pyslow5.pyx":1503
 * 
 *         for a in user_record:
 *             if user_record[a] is None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if a not in py_record_types:
 */
    }

    /* "pyslow5.pyx":1505
 *             if user_record[a] is None:
 *                 continue
 *             if a not in py_record_types:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    __pyx_t_12 = (__Pyx_PyDict_ContainsTF(__pyx_v_a, __pyx_v_py_record_types, Py_NE)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1505, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_12 != 0);
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1506
 *                 continue
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))             # <<<<<<<<<<<<<<
 *                 return None, None
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1506, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_pri, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1506, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1506, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
        __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1506, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
        __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1506, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_13 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1506, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_7, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_7, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1506, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1506, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":1507
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None             # <<<<<<<<<<<<<<
 * 
 *             if type(user_record[a]) != py_record_types[a]:
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_tuple__6);
      __pyx_r = __pyx_tuple__6;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "pyslow5.pyx":1505
 *             if user_record[a] is None:
 *                 continue
 *             if a not in py_record_types:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    }

    /* "pyslow5.pyx":1509
 *                 return None, None
 * 
 *             if type(user_record[a]) != py_record_types[a]:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_py_record_types, __pyx_v_a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_t_2)), __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1509, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1509, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1510
 * 
 *             if type(user_record[a]) != py_record_types[a]:
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))             # <<<<<<<<<<<<<<
 *                 return None, None
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_pri_2, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_3 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_a, __pyx_t_13};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1510, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_a, __pyx_t_13};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1510, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1510, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (__pyx_t_3) {
          __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_13);
        PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1510, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_8 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyslow5.pyx":1511
 *             if type(user_record[a]) != py_record_types[a]:
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None             # <<<<<<<<<<<<<<
 * 
 *         # check aux if given
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_tuple__6);
      __pyx_r = __pyx_tuple__6;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "pyslow5.pyx":1509
 *                 return None, None
 * 
 *             if type(user_record[a]) != py_record_types[a]:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    }

    /* "pyslow5.pyx":1502
 *         new_aux = {}
 * 
 *         for a in user_record:             # <<<<<<<<<<<<<<
 *             if user_record[a] is None:
 *                 continue
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1514
 * 
 *         # check aux if given
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 */
  __pyx_t_11 = (__pyx_v_aux != Py_None);
  __pyx_t_12 = (__pyx_t_11 != 0);
  if (__pyx_t_12) {

    /* "pyslow5.pyx":1515
 *         # check aux if given
 *         if aux is not None:
 *             self.logger.debug("_record_type_validation: doing aux stuff...")             # <<<<<<<<<<<<<<
 *             for a in aux:
 *                 if a not in py_aux_types:
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1515, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_4, __pyx_kp_u_record_type_validation_doing_au) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_kp_u_record_type_validation_doing_au);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1515, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1516
 *         if aux is not None:
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:             # <<<<<<<<<<<<<<
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 */
    if (likely(PyList_CheckExact(__pyx_v_aux)) || PyTuple_CheckExact(__pyx_v_aux)) {
      __pyx_t_1 = __pyx_v_aux; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_aux); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1516, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1516, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_8); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1516, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1516, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_8); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1516, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1516, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        }
      } else {
        __pyx_t_8 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_8)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1516, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyslow5.pyx":1517
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 *                 if a not in py_aux_types:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      __pyx_t_12 = (__Pyx_PyDict_ContainsTF(__pyx_v_a, __pyx_v_py_aux_types, Py_NE)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1517, __pyx_L1_error)
      __pyx_t_11 = (__pyx_t_12 != 0);
      if (__pyx_t_11) {

        /* "pyslow5.pyx":1518
 *             for a in aux:
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))             # <<<<<<<<<<<<<<
 *                     return None, None
 *                 if aux[a] is None:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1518, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_aux, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1518, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1518, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_a, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1518, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_a, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1518, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_3 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1518, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_a);
          __Pyx_GIVEREF(__pyx_v_a);
          PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_7, __pyx_v_a);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_7, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1518, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_8 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1518, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyslow5.pyx":1519
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None             # <<<<<<<<<<<<<<
 *                 if aux[a] is None:
 *                     continue
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_tuple__6);
        __pyx_r = __pyx_tuple__6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;

        /* "pyslow5.pyx":1517
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 *                 if a not in py_aux_types:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      }

      /* "pyslow5.pyx":1520
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 *                 if aux[a] is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:
 */
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1520, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_11 = (__pyx_t_8 == Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_12 = (__pyx_t_11 != 0);
      if (__pyx_t_12) {

        /* "pyslow5.pyx":1521
 *                     return None, None
 *                 if aux[a] is None:
 *                     continue             # <<<<<<<<<<<<<<
 *                 if type(aux[a]) != py_aux_types[a]:
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 */
        goto __pyx_L9_continue;

        /* "pyslow5.pyx":1520
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 *                 if aux[a] is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:
 */
      }

      /* "pyslow5.pyx":1522
 *                 if aux[a] is None:
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1522, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_py_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1522, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_t_8)), __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1522, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1522, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_12) {

        /* "pyslow5.pyx":1523
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))             # <<<<<<<<<<<<<<
 *                     return None, None
 *                 else:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1523, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_aux_2, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1523, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1523, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1523, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1523, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1523, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_a);
          __Pyx_GIVEREF(__pyx_v_a);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_7, __pyx_v_a);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_7, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1523, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1523, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1524
 *                 if type(aux[a]) != py_aux_types[a]:
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_tuple__6);
        __pyx_r = __pyx_tuple__6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;

        /* "pyslow5.pyx":1522
 *                 if aux[a] is None:
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      }

      /* "pyslow5.pyx":1526
 *                     return None, None
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")             # <<<<<<<<<<<<<<
 *                     if a == "channel_number":
 *                         self.channel_number_val=strdup(aux[a].encode())
 */
      /*else*/ {
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1526, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, __pyx_kp_u_record_type_validation_aux_pass) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_u_record_type_validation_aux_pass);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1526, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1527
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 *                     if a == "channel_number":             # <<<<<<<<<<<<<<
 *                         self.channel_number_val=strdup(aux[a].encode())
 *                         new_aux[a] = aux[a]
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_channel_number, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1527, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1528
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 *                     if a == "channel_number":
 *                         self.channel_number_val=strdup(aux[a].encode())             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":
 */
          __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1528, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_encode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1528, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_4 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
            if (likely(__pyx_t_4)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_8, function);
            }
          }
          __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_8);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1528, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_14 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 1528, __pyx_L1_error)
          __pyx_v_self->channel_number_val = strdup(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1529
 *                     if a == "channel_number":
 *                         self.channel_number_val=strdup(aux[a].encode())
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "median_before":
 *                         self.median_before_val = <double>aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1529, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1529, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1527
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 *                     if a == "channel_number":             # <<<<<<<<<<<<<<
 *                         self.channel_number_val=strdup(aux[a].encode())
 *                         new_aux[a] = aux[a]
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1530
 *                         self.channel_number_val=strdup(aux[a].encode())
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":             # <<<<<<<<<<<<<<
 *                         self.median_before_val = <double>aux[a]
 *                         new_aux[a] = aux[a]
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_median_before, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1530, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1531
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":
 *                         self.median_before_val = <double>aux[a]             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1531, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1531, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_self->median_before_val = ((double)__pyx_t_15);

          /* "pyslow5.pyx":1532
 *                     elif a == "median_before":
 *                         self.median_before_val = <double>aux[a]
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "read_number":
 *                         self.read_number_val = <int32_t>aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1532, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1532, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1530
 *                         self.channel_number_val=strdup(aux[a].encode())
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":             # <<<<<<<<<<<<<<
 *                         self.median_before_val = <double>aux[a]
 *                         new_aux[a] = aux[a]
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1533
 *                         self.median_before_val = <double>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":             # <<<<<<<<<<<<<<
 *                         self.read_number_val = <int32_t>aux[a]
 *                         new_aux[a] = aux[a]
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_read_number, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1533, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1534
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":
 *                         self.read_number_val = <int32_t>aux[a]             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1534, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_16 = __Pyx_PyInt_As_int32_t(__pyx_t_2); if (unlikely((__pyx_t_16 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1534, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_self->read_number_val = ((int32_t)__pyx_t_16);

          /* "pyslow5.pyx":1535
 *                     elif a == "read_number":
 *                         self.read_number_val = <int32_t>aux[a]
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "start_mux":
 *                         self.start_mux_val = <uint8_t>aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1535, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1535, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1533
 *                         self.median_before_val = <double>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":             # <<<<<<<<<<<<<<
 *                         self.read_number_val = <int32_t>aux[a]
 *                         new_aux[a] = aux[a]
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1536
 *                         self.read_number_val = <int32_t>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                         self.start_mux_val = <uint8_t>aux[a]
 *                         new_aux[a] = aux[a]
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_mux, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1536, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1537
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":
 *                         self.start_mux_val = <uint8_t>aux[a]             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1537, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_17 = __Pyx_PyInt_As_uint8_t(__pyx_t_2); if (unlikely((__pyx_t_17 == ((uint8_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1537, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_self->start_mux_val = ((uint8_t)__pyx_t_17);

          /* "pyslow5.pyx":1538
 *                     elif a == "start_mux":
 *                         self.start_mux_val = <uint8_t>aux[a]
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "start_time":
 *                         self.start_time_val = <uint64_t>aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1538, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1538, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1536
 *                         self.read_number_val = <int32_t>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                         self.start_mux_val = <uint8_t>aux[a]
 *                         new_aux[a] = aux[a]
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1539
 *                         self.start_mux_val = <uint8_t>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":             # <<<<<<<<<<<<<<
 *                         self.start_time_val = <uint64_t>aux[a]
 *                         new_aux[a] = aux[a]
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_time, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1539, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1540
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":
 *                         self.start_time_val = <uint64_t>aux[a]             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1540, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_18 = __Pyx_PyInt_As_uint64_t(__pyx_t_2); if (unlikely((__pyx_t_18 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1540, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_self->start_time_val = ((uint64_t)__pyx_t_18);

          /* "pyslow5.pyx":1541
 *                     elif a == "start_time":
 *                         self.start_time_val = <uint64_t>aux[a]
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "end_reason":
 *                         continue
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1541, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1541, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1539
 *                         self.start_mux_val = <uint8_t>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":             # <<<<<<<<<<<<<<
 *                         self.start_time_val = <uint64_t>aux[a]
 *                         new_aux[a] = aux[a]
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1542
 *                         self.start_time_val = <uint64_t>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                         continue
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_end_reason, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1542, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1543
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":
 *                         continue             # <<<<<<<<<<<<<<
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 * 
 */
          goto __pyx_L9_continue;

          /* "pyslow5.pyx":1542
 *                         self.start_time_val = <uint64_t>aux[a]
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                         continue
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 */
        }
        __pyx_L14:;
      }

      /* "pyslow5.pyx":1516
 *         if aux is not None:
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:             # <<<<<<<<<<<<<<
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 */
      __pyx_L9_continue:;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1544
 *                     elif a == "end_reason":
 *                         continue
 *             self.logger.debug("_record_type_validation: doing aux stuff...")             # <<<<<<<<<<<<<<
 * 
 *         return user_record, new_aux
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1544, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_8, __pyx_kp_u_record_type_validation_doing_au) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_record_type_validation_doing_au);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1544, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1514
 * 
 *         # check aux if given
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 */
  }

  /* "pyslow5.pyx":1546
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 * 
 *         return user_record, new_aux             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_user_record);
  __Pyx_GIVEREF(__pyx_v_user_record);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_user_record);
  __Pyx_INCREF(__pyx_v_new_aux);
  __Pyx_GIVEREF(__pyx_v_new_aux);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_new_aux);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyslow5.pyx":1478
 * 
 * 
 *     def _record_type_validation(self, user_record, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate and convert aux types before pushing to C side
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyslow5.Open._record_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_py_record_types);
  __Pyx_XDECREF(__pyx_v_py_aux_types);
  __Pyx_XDECREF(__pyx_v_new_aux);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1549
 * 
 * 
 *     def _multi_record_type_validation(self, user_record, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate and convert aux types before pushing to C side
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_57_multi_record_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_56_multi_record_type_validation[] = "\n        internal function to validate and convert aux types before pushing to C side\n        returns None, None on error\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_57_multi_record_type_validation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_user_record = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_multi_record_type_validation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_user_record,&__pyx_n_s_aux,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_user_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_multi_record_type_validation") < 0)) __PYX_ERR(0, 1549, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_user_record = values[0];
    __pyx_v_aux = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_multi_record_type_validation", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1549, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open._multi_record_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_56_multi_record_type_validation(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_user_record, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_56_multi_record_type_validation(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_user_record, PyObject *__pyx_v_aux) {
  PyObject *__pyx_v_py_record_types = NULL;
  PyObject *__pyx_v_py_aux_types = NULL;
  PyObject *__pyx_v_new_aux = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_multi_record_type_validation", 0);
  __Pyx_TraceCall("_multi_record_type_validation", __pyx_f[0], 1549, 0, __PYX_ERR(0, 1549, __pyx_L1_error));

  /* "pyslow5.pyx":1555
 *         '''
 * 
 *         py_record_types = {"read_id": type("string"),             # <<<<<<<<<<<<<<
 *                            "read_group": type(1),
 *                            "digitisation": type(1.0),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_id, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1556
 * 
 *         py_record_types = {"read_id": type("string"),
 *                            "read_group": type(1),             # <<<<<<<<<<<<<<
 *                            "digitisation": type(1.0),
 *                            "offset": type(1.0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_group, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1557
 *         py_record_types = {"read_id": type("string"),
 *                            "read_group": type(1),
 *                            "digitisation": type(1.0),             # <<<<<<<<<<<<<<
 *                            "offset": type(1.0),
 *                            "range": type(1.0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_digitisation, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1558
 *                            "read_group": type(1),
 *                            "digitisation": type(1.0),
 *                            "offset": type(1.0),             # <<<<<<<<<<<<<<
 *                            "range": type(1.0),
 *                            "sampling_rate": type(1.0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_offset, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1559
 *                            "digitisation": type(1.0),
 *                            "offset": type(1.0),
 *                            "range": type(1.0),             # <<<<<<<<<<<<<<
 *                            "sampling_rate": type(1.0),
 *                            "len_raw_signal": type(10),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_range, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1560
 *                            "offset": type(1.0),
 *                            "range": type(1.0),
 *                            "sampling_rate": type(1.0),             # <<<<<<<<<<<<<<
 *                            "len_raw_signal": type(10),
 *                            "signal": type(np.array([1, 2, 3], np.int16))}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sampling_rate, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1561
 *                            "range": type(1.0),
 *                            "sampling_rate": type(1.0),
 *                            "len_raw_signal": type(10),             # <<<<<<<<<<<<<<
 *                            "signal": type(np.array([1, 2, 3], np.int16))}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_len_raw_signal, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)

  /* "pyslow5.pyx":1562
 *                            "sampling_rate": type(1.0),
 *                            "len_raw_signal": type(10),
 *                            "signal": type(np.array([1, 2, 3], np.int16))}             # <<<<<<<<<<<<<<
 * 
 *         py_aux_types = {"channel_number": type("string"),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyList_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_int_1);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_int_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_3, 2, __pyx_int_3);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int16); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1562, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1562, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_signal, ((PyObject *)Py_TYPE(__pyx_t_2))) < 0) __PYX_ERR(0, 1555, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_py_record_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1564
 *                            "signal": type(np.array([1, 2, 3], np.int16))}
 * 
 *         py_aux_types = {"channel_number": type("string"),             # <<<<<<<<<<<<<<
 *                         "median_before": type(1.0),
 *                         "read_number": type(10),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1564, __pyx_L1_error)

  /* "pyslow5.pyx":1565
 * 
 *         py_aux_types = {"channel_number": type("string"),
 *                         "median_before": type(1.0),             # <<<<<<<<<<<<<<
 *                         "read_number": type(10),
 *                         "start_mux": type(1),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1564, __pyx_L1_error)

  /* "pyslow5.pyx":1566
 *         py_aux_types = {"channel_number": type("string"),
 *                         "median_before": type(1.0),
 *                         "read_number": type(10),             # <<<<<<<<<<<<<<
 *                         "start_mux": type(1),
 *                         "start_time": type(100),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1564, __pyx_L1_error)

  /* "pyslow5.pyx":1567
 *                         "median_before": type(1.0),
 *                         "read_number": type(10),
 *                         "start_mux": type(1),             # <<<<<<<<<<<<<<
 *                         "start_time": type(100),
 *                         "end_reason": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1564, __pyx_L1_error)

  /* "pyslow5.pyx":1568
 *                         "read_number": type(10),
 *                         "start_mux": type(1),
 *                         "start_time": type(100),             # <<<<<<<<<<<<<<
 *                         "end_reason": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, ((PyObject *)Py_TYPE(__pyx_int_100))) < 0) __PYX_ERR(0, 1564, __pyx_L1_error)

  /* "pyslow5.pyx":1569
 *                         "start_mux": type(1),
 *                         "start_time": type(100),
 *                         "end_reason": None}             # <<<<<<<<<<<<<<
 * 
 *         new_aux = {}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_end_reason, Py_None) < 0) __PYX_ERR(0, 1564, __pyx_L1_error)
  __pyx_v_py_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1571
 *                         "end_reason": None}
 * 
 *         new_aux = {}             # <<<<<<<<<<<<<<
 * 
 *         for a in user_record:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_aux = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1573
 *         new_aux = {}
 * 
 *         for a in user_record:             # <<<<<<<<<<<<<<
 *             if user_record[a] is None:
 *                 continue
 */
  if (likely(PyList_CheckExact(__pyx_v_user_record)) || PyTuple_CheckExact(__pyx_v_user_record)) {
    __pyx_t_1 = __pyx_v_user_record; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_user_record); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1573, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1573, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1573, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1573, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1573, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_10(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1573, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1574
 * 
 *         for a in user_record:
 *             if user_record[a] is None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if a not in py_record_types:
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = (__pyx_t_2 == Py_None);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_12 = (__pyx_t_11 != 0);
    if (__pyx_t_12) {

      /* "pyslow5.pyx":1575
 *         for a in user_record:
 *             if user_record[a] is None:
 *                 continue             # <<<<<<<<<<<<<<
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 */
      goto __pyx_L3_continue;

      /* "pyslow5.pyx":1574
 * 
 *         for a in user_record:
 *             if user_record[a] is None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if a not in py_record_types:
 */
    }

    /* "pyslow5.pyx":1576
 *             if user_record[a] is None:
 *                 continue
 *             if a not in py_record_types:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    __pyx_t_12 = (__Pyx_PyDict_ContainsTF(__pyx_v_a, __pyx_v_py_record_types, Py_NE)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1576, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_12 != 0);
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1577
 *                 continue
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))             # <<<<<<<<<<<<<<
 *                 return None, None
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_pri, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
        __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1577, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
        __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1577, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_13 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1577, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_7, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_7, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1577, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1577, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":1578
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None             # <<<<<<<<<<<<<<
 * 
 *             if type(user_record[a]) != py_record_types[a]:
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_tuple__6);
      __pyx_r = __pyx_tuple__6;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "pyslow5.pyx":1576
 *             if user_record[a] is None:
 *                 continue
 *             if a not in py_record_types:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    }

    /* "pyslow5.pyx":1580
 *                 return None, None
 * 
 *             if type(user_record[a]) != py_record_types[a]:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_py_record_types, __pyx_v_a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_t_2)), __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1580, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 1580, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_11) {

      /* "pyslow5.pyx":1581
 * 
 *             if type(user_record[a]) != py_record_types[a]:
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))             # <<<<<<<<<<<<<<
 *                 return None, None
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1581, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_pri_2, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1581, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_user_record, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1581, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_3 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_a, __pyx_t_13};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1581, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_a, __pyx_t_13};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1581, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1581, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (__pyx_t_3) {
          __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_13);
        PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1581, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_8 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1581, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyslow5.pyx":1582
 *             if type(user_record[a]) != py_record_types[a]:
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None             # <<<<<<<<<<<<<<
 * 
 *         # check aux if given
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_tuple__6);
      __pyx_r = __pyx_tuple__6;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "pyslow5.pyx":1580
 *                 return None, None
 * 
 *             if type(user_record[a]) != py_record_types[a]:             # <<<<<<<<<<<<<<
 *                 self.logger.error("_record_type_validation {}: {} user primary field type mismatch with pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None
 */
    }

    /* "pyslow5.pyx":1573
 *         new_aux = {}
 * 
 *         for a in user_record:             # <<<<<<<<<<<<<<
 *             if user_record[a] is None:
 *                 continue
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1585
 * 
 *         # check aux if given
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 */
  __pyx_t_11 = (__pyx_v_aux != Py_None);
  __pyx_t_12 = (__pyx_t_11 != 0);
  if (__pyx_t_12) {

    /* "pyslow5.pyx":1586
 *         # check aux if given
 *         if aux is not None:
 *             self.logger.debug("_record_type_validation: doing aux stuff...")             # <<<<<<<<<<<<<<
 *             for a in aux:
 *                 if a not in py_aux_types:
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_4, __pyx_kp_u_record_type_validation_doing_au) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_kp_u_record_type_validation_doing_au);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1587
 *         if aux is not None:
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:             # <<<<<<<<<<<<<<
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 */
    if (likely(PyList_CheckExact(__pyx_v_aux)) || PyTuple_CheckExact(__pyx_v_aux)) {
      __pyx_t_1 = __pyx_v_aux; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_aux); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1587, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1587, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_8); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1587, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1587, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_8); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1587, __pyx_L1_error)
          #else
          __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1587, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        }
      } else {
        __pyx_t_8 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_8)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1587, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyslow5.pyx":1588
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 *                 if a not in py_aux_types:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      __pyx_t_12 = (__Pyx_PyDict_ContainsTF(__pyx_v_a, __pyx_v_py_aux_types, Py_NE)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1588, __pyx_L1_error)
      __pyx_t_11 = (__pyx_t_12 != 0);
      if (__pyx_t_11) {

        /* "pyslow5.pyx":1589
 *             for a in aux:
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))             # <<<<<<<<<<<<<<
 *                     return None, None
 *                 if aux[a] is None:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1589, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_aux, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1589, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1589, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_a, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1589, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_a, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1589, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_3 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1589, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_v_a);
          __Pyx_GIVEREF(__pyx_v_a);
          PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_7, __pyx_v_a);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_7, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1589, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_8 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1589, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyslow5.pyx":1590
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None             # <<<<<<<<<<<<<<
 *                 if aux[a] is None:
 *                     continue
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_tuple__6);
        __pyx_r = __pyx_tuple__6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;

        /* "pyslow5.pyx":1588
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 *                 if a not in py_aux_types:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      }

      /* "pyslow5.pyx":1591
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 *                 if aux[a] is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:
 */
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1591, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_11 = (__pyx_t_8 == Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_12 = (__pyx_t_11 != 0);
      if (__pyx_t_12) {

        /* "pyslow5.pyx":1592
 *                     return None, None
 *                 if aux[a] is None:
 *                     continue             # <<<<<<<<<<<<<<
 *                 if type(aux[a]) != py_aux_types[a]:
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 */
        goto __pyx_L9_continue;

        /* "pyslow5.pyx":1591
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 *                 if aux[a] is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:
 */
      }

      /* "pyslow5.pyx":1593
 *                 if aux[a] is None:
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1593, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_py_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1593, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_t_8)), __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1593, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1593, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_12) {

        /* "pyslow5.pyx":1594
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))             # <<<<<<<<<<<<<<
 *                     return None, None
 *                 else:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1594, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_record_type_validation_user_aux_2, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1594, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1594, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1594, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_a, __pyx_t_3};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1594, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1594, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_a);
          __Pyx_GIVEREF(__pyx_v_a);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_7, __pyx_v_a);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_7, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1594, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1594, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1595
 *                 if type(aux[a]) != py_aux_types[a]:
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_tuple__6);
        __pyx_r = __pyx_tuple__6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;

        /* "pyslow5.pyx":1593
 *                 if aux[a] is None:
 *                     continue
 *                 if type(aux[a]) != py_aux_types[a]:             # <<<<<<<<<<<<<<
 *                     self.logger.error("_record_type_validation {}: {} user aux field type mismatch with pyslow5 aux_types".format(a, aux[a]))
 *                     return None, None
 */
      }

      /* "pyslow5.pyx":1597
 *                     return None, None
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")             # <<<<<<<<<<<<<<
 *                     if a == "channel_number":
 *                         new_aux[a] = aux[a]
 */
      /*else*/ {
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, __pyx_kp_u_record_type_validation_aux_pass) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_u_record_type_validation_aux_pass);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1598
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 *                     if a == "channel_number":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_channel_number, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1598, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1599
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 *                     if a == "channel_number":
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "median_before":
 *                         new_aux[a] = aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1599, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1599, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1598
 *                 else:
 *                     self.logger.debug("_record_type_validation: aux passed tests...")
 *                     if a == "channel_number":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1600
 *                     if a == "channel_number":
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_median_before, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1600, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1601
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "read_number":
 *                         new_aux[a] = aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1601, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1601, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1600
 *                     if a == "channel_number":
 *                         new_aux[a] = aux[a]
 *                     elif a == "median_before":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1602
 *                     elif a == "median_before":
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_read_number, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1602, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1603
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "start_mux":
 *                         new_aux[a] = aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1603, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1603, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1602
 *                     elif a == "median_before":
 *                         new_aux[a] = aux[a]
 *                     elif a == "read_number":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1604
 *                     elif a == "read_number":
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_mux, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1604, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1605
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "start_time":
 *                         new_aux[a] = aux[a]
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1605, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1605, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1604
 *                     elif a == "read_number":
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1606
 *                     elif a == "start_mux":
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_time, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1606, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1607
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":
 *                         new_aux[a] = aux[a]             # <<<<<<<<<<<<<<
 *                     elif a == "end_reason":
 *                         continue
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1607, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(PyDict_SetItem(__pyx_v_new_aux, __pyx_v_a, __pyx_t_2) < 0)) __PYX_ERR(0, 1607, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1606
 *                     elif a == "start_mux":
 *                         new_aux[a] = aux[a]
 *                     elif a == "start_time":             # <<<<<<<<<<<<<<
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":
 */
          goto __pyx_L14;
        }

        /* "pyslow5.pyx":1608
 *                     elif a == "start_time":
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_end_reason, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1608, __pyx_L1_error)
        if (__pyx_t_12) {

          /* "pyslow5.pyx":1609
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *             self.logger.debug("_record_type_validation: aux stuff done")
 */
          goto __pyx_L9_continue;

          /* "pyslow5.pyx":1608
 *                     elif a == "start_time":
 *                         new_aux[a] = aux[a]
 *                     elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        }
        __pyx_L14:;
      }

      /* "pyslow5.pyx":1587
 *         if aux is not None:
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:             # <<<<<<<<<<<<<<
 *                 if a not in py_aux_types:
 *                     self.logger.error("_record_type_validation {}: {} user aux field not in pyslow5 aux_types".format(a, aux[a]))
 */
      __pyx_L9_continue:;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1611
 *                         continue
 * 
 *             self.logger.debug("_record_type_validation: aux stuff done")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1611, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_kp_u_record_type_validation_aux_stuf) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_record_type_validation_aux_stuf);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1611, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1585
 * 
 *         # check aux if given
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             self.logger.debug("_record_type_validation: doing aux stuff...")
 *             for a in aux:
 */
  }

  /* "pyslow5.pyx":1614
 * 
 * 
 *         return user_record, new_aux             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_user_record);
  __Pyx_GIVEREF(__pyx_v_user_record);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_user_record);
  __Pyx_INCREF(__pyx_v_new_aux);
  __Pyx_GIVEREF(__pyx_v_new_aux);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_new_aux);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyslow5.pyx":1549
 * 
 * 
 *     def _multi_record_type_validation(self, user_record, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         internal function to validate and convert aux types before pushing to C side
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyslow5.Open._multi_record_type_validation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_py_record_types);
  __Pyx_XDECREF(__pyx_v_py_aux_types);
  __Pyx_XDECREF(__pyx_v_new_aux);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1617
 * 
 * 
 *     def write_header(self, header, read_group=0):             # <<<<<<<<<<<<<<
 *         '''
 *         write slow5 header to file.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_59write_header(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_58write_header[] = "\n        write slow5 header to file.\n        takes header dic for attributes, then write once.\n        Must write readgroup 0 before 1, 1 before 2, etc\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_59write_header(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_header = 0;
  PyObject *__pyx_v_read_group = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_header (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_header,&__pyx_n_s_read_group,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_header)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_group);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "write_header") < 0)) __PYX_ERR(0, 1617, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_header = values[0];
    __pyx_v_read_group = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_header", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1617, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.write_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_58write_header(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_header, __pyx_v_read_group);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_58write_header(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_header, PyObject *__pyx_v_read_group) {
  PyObject *__pyx_v_checked_header = NULL;
  int __pyx_v_errors;
  PyObject *__pyx_v_h = NULL;
  int64_t __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  char const *__pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  char const *__pyx_t_15;
  char const *__pyx_t_16;
  uint32_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_header", 0);
  __Pyx_TraceCall("write_header", __pyx_f[0], 1617, 0, __PYX_ERR(0, 1617, __pyx_L1_error));

  /* "pyslow5.pyx":1623
 *         Must write readgroup 0 before 1, 1 before 2, etc
 *         '''
 *         checked_header = self._header_type_validation(header)             # <<<<<<<<<<<<<<
 * 
 *         errors = False
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_header_type_validation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_header) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_header);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_checked_header = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1625
 *         checked_header = self._header_type_validation(header)
 * 
 *         errors = False             # <<<<<<<<<<<<<<
 *         if not self.header_add_attr_state:
 *             if read_group > 0:
 */
  __pyx_v_errors = 0;

  /* "pyslow5.pyx":1626
 * 
 *         errors = False
 *         if not self.header_add_attr_state:             # <<<<<<<<<<<<<<
 *             if read_group > 0:
 *                 self.logger.error("write_header: You must set read_group=0 first")
 */
  __pyx_t_4 = ((!(__pyx_v_self->header_add_attr_state != 0)) != 0);
  if (__pyx_t_4) {

    /* "pyslow5.pyx":1627
 *         errors = False
 *         if not self.header_add_attr_state:
 *             if read_group > 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: You must set read_group=0 first")
 *                 self.logger.error("write_header: headers failed to initialise.")
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_read_group, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1627, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1627, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_4) {

      /* "pyslow5.pyx":1628
 *         if not self.header_add_attr_state:
 *             if read_group > 0:
 *                 self.logger.error("write_header: You must set read_group=0 first")             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: headers failed to initialise.")
 *                 return -1
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1628, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_write_header_You_must_set_read_g) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_header_You_must_set_read_g);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1628, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1629
 *             if read_group > 0:
 *                 self.logger.error("write_header: You must set read_group=0 first")
 *                 self.logger.error("write_header: headers failed to initialise.")             # <<<<<<<<<<<<<<
 *                 return -1
 *             for h in checked_header:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1629, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_write_header_headers_failed_to_i) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_header_headers_failed_to_i);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1629, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1630
 *                 self.logger.error("write_header: You must set read_group=0 first")
 *                 self.logger.error("write_header: headers failed to initialise.")
 *                 return -1             # <<<<<<<<<<<<<<
 *             for h in checked_header:
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_int_neg_1);
      __pyx_r = __pyx_int_neg_1;
      goto __pyx_L0;

      /* "pyslow5.pyx":1627
 *         errors = False
 *         if not self.header_add_attr_state:
 *             if read_group > 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: You must set read_group=0 first")
 *                 self.logger.error("write_header: headers failed to initialise.")
 */
    }

    /* "pyslow5.pyx":1631
 *                 self.logger.error("write_header: headers failed to initialise.")
 *                 return -1
 *             for h in checked_header:             # <<<<<<<<<<<<<<
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)
 *                 if ret < 0:
 */
    if (likely(PyList_CheckExact(__pyx_v_checked_header)) || PyTuple_CheckExact(__pyx_v_checked_header)) {
      __pyx_t_1 = __pyx_v_checked_header; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_checked_header); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1631, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1631, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1631, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1631, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1631, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1631, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_6(__pyx_t_1);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1631, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_h, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":1632
 *                 return -1
 *             for h in checked_header:
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)             # <<<<<<<<<<<<<<
 *                 if ret < 0:
 *                     self.logger.error("write_header: slow5_hdr_add_attr {}: {} could not initialise to C s5.header struct".format(h, checked_header[h]))
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_h, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1632, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1632, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 1632, __pyx_L1_error)
      __pyx_v_ret = slow5_hdr_add_attr(__pyx_t_8, __pyx_v_self->s5->header);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":1633
 *             for h in checked_header:
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)
 *                 if ret < 0:             # <<<<<<<<<<<<<<
 *                     self.logger.error("write_header: slow5_hdr_add_attr {}: {} could not initialise to C s5.header struct".format(h, checked_header[h]))
 *                     errors = True
 */
      __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
      if (__pyx_t_4) {

        /* "pyslow5.pyx":1634
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)
 *                 if ret < 0:
 *                     self.logger.error("write_header: slow5_hdr_add_attr {}: {} could not initialise to C s5.header struct".format(h, checked_header[h]))             # <<<<<<<<<<<<<<
 *                     errors = True
 *                 else:
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1634, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_header_slow5_hdr_add_attr, __pyx_n_s_format); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1634, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_v_checked_header, __pyx_v_h); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1634, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = NULL;
        __pyx_t_12 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_12 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_v_h, __pyx_t_10};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1634, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_v_h, __pyx_t_10};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1634, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1634, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_v_h);
          __Pyx_GIVEREF(__pyx_v_h);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_v_h);
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_t_10);
          __pyx_t_10 = 0;
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_13, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1634, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1634, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1635
 *                 if ret < 0:
 *                     self.logger.error("write_header: slow5_hdr_add_attr {}: {} could not initialise to C s5.header struct".format(h, checked_header[h]))
 *                     errors = True             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.header_add_attr_state = True
 */
        __pyx_v_errors = 1;

        /* "pyslow5.pyx":1633
 *             for h in checked_header:
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)
 *                 if ret < 0:             # <<<<<<<<<<<<<<
 *                     self.logger.error("write_header: slow5_hdr_add_attr {}: {} could not initialise to C s5.header struct".format(h, checked_header[h]))
 *                     errors = True
 */
        goto __pyx_L7;
      }

      /* "pyslow5.pyx":1637
 *                     errors = True
 *                 else:
 *                     self.header_add_attr_state = True             # <<<<<<<<<<<<<<
 * 
 *         if read_group > 0 and self.header_add_attr_state:
 */
      /*else*/ {
        __pyx_v_self->header_add_attr_state = 1;
      }
      __pyx_L7:;

      /* "pyslow5.pyx":1631
 *                 self.logger.error("write_header: headers failed to initialise.")
 *                 return -1
 *             for h in checked_header:             # <<<<<<<<<<<<<<
 *                 ret = slow5_hdr_add_attr(h.encode(), self.s5.header)
 *                 if ret < 0:
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1626
 * 
 *         errors = False
 *         if not self.header_add_attr_state:             # <<<<<<<<<<<<<<
 *             if read_group > 0:
 *                 self.logger.error("write_header: You must set read_group=0 first")
 */
  }

  /* "pyslow5.pyx":1639
 *                     self.header_add_attr_state = True
 * 
 *         if read_group > 0 and self.header_add_attr_state:             # <<<<<<<<<<<<<<
 *             ret = slow5_hdr_add_rg(self.s5.header)
 *             if ret < 0:
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_read_group, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1639, __pyx_L1_error)
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 1639, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_14) {
  } else {
    __pyx_t_4 = __pyx_t_14;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_14 = (__pyx_v_self->header_add_attr_state != 0);
  __pyx_t_4 = __pyx_t_14;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_4) {

    /* "pyslow5.pyx":1640
 * 
 *         if read_group > 0 and self.header_add_attr_state:
 *             ret = slow5_hdr_add_rg(self.s5.header)             # <<<<<<<<<<<<<<
 *             if ret < 0:
 *                 self.logger.error("write_header: slow5_hdr_add_rg failed for read_group: {}".format(read_group))
 */
    __pyx_v_ret = slow5_hdr_add_rg(__pyx_v_self->s5->header);

    /* "pyslow5.pyx":1641
 *         if read_group > 0 and self.header_add_attr_state:
 *             ret = slow5_hdr_add_rg(self.s5.header)
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: slow5_hdr_add_rg failed for read_group: {}".format(read_group))
 *                 errors = True
 */
    __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
    if (__pyx_t_4) {

      /* "pyslow5.pyx":1642
 *             ret = slow5_hdr_add_rg(self.s5.header)
 *             if ret < 0:
 *                 self.logger.error("write_header: slow5_hdr_add_rg failed for read_group: {}".format(read_group))             # <<<<<<<<<<<<<<
 *                 errors = True
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_header_slow5_hdr_add_rg_fa, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_3 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_9, __pyx_v_read_group) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_read_group);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_7, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1643
 *             if ret < 0:
 *                 self.logger.error("write_header: slow5_hdr_add_rg failed for read_group: {}".format(read_group))
 *                 errors = True             # <<<<<<<<<<<<<<
 * 
 *         if errors:
 */
      __pyx_v_errors = 1;

      /* "pyslow5.pyx":1641
 *         if read_group > 0 and self.header_add_attr_state:
 *             ret = slow5_hdr_add_rg(self.s5.header)
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: slow5_hdr_add_rg failed for read_group: {}".format(read_group))
 *                 errors = True
 */
    }

    /* "pyslow5.pyx":1639
 *                     self.header_add_attr_state = True
 * 
 *         if read_group > 0 and self.header_add_attr_state:             # <<<<<<<<<<<<<<
 *             ret = slow5_hdr_add_rg(self.s5.header)
 *             if ret < 0:
 */
  }

  /* "pyslow5.pyx":1645
 *                 errors = True
 * 
 *         if errors:             # <<<<<<<<<<<<<<
 *             self.logger.error("write_header: headers failed to initialise.")
 *             return -1
 */
  __pyx_t_4 = (__pyx_v_errors != 0);
  if (__pyx_t_4) {

    /* "pyslow5.pyx":1646
 * 
 *         if errors:
 *             self.logger.error("write_header: headers failed to initialise.")             # <<<<<<<<<<<<<<
 *             return -1
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_write_header_headers_failed_to_i) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_header_headers_failed_to_i);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1647
 *         if errors:
 *             self.logger.error("write_header: headers failed to initialise.")
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         for h in checked_header:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_neg_1);
    __pyx_r = __pyx_int_neg_1;
    goto __pyx_L0;

    /* "pyslow5.pyx":1645
 *                 errors = True
 * 
 *         if errors:             # <<<<<<<<<<<<<<
 *             self.logger.error("write_header: headers failed to initialise.")
 *             return -1
 */
  }

  /* "pyslow5.pyx":1649
 *             return -1
 * 
 *         for h in checked_header:             # <<<<<<<<<<<<<<
 *             ret = slow5_hdr_set(h.encode(), checked_header[h].encode(), read_group, self.s5.header)
 *             if ret < 0:
 */
  if (likely(PyList_CheckExact(__pyx_v_checked_header)) || PyTuple_CheckExact(__pyx_v_checked_header)) {
    __pyx_t_1 = __pyx_v_checked_header; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_checked_header); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1649, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1649, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1649, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1649, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 1649, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1649, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_6(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1649, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_h, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1650
 * 
 *         for h in checked_header:
 *             ret = slow5_hdr_set(h.encode(), checked_header[h].encode(), read_group, self.s5.header)             # <<<<<<<<<<<<<<
 *             if ret < 0:
 *                 self.logger.error("write_header: slow5_hdr_set {}: {} could not set to C s5.header struct".format(h, checked_header[h]))
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_h, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_15 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 1650, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_checked_header, __pyx_v_h); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_encode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_9);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_16 = __Pyx_PyObject_AsString(__pyx_t_3); if (unlikely((!__pyx_t_16) && PyErr_Occurred())) __PYX_ERR(0, 1650, __pyx_L1_error)
    __pyx_t_17 = __Pyx_PyInt_As_uint32_t(__pyx_v_read_group); if (unlikely((__pyx_t_17 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1650, __pyx_L1_error)
    __pyx_v_ret = slow5_hdr_set(__pyx_t_15, __pyx_t_16, __pyx_t_17, __pyx_v_self->s5->header);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyslow5.pyx":1651
 *         for h in checked_header:
 *             ret = slow5_hdr_set(h.encode(), checked_header[h].encode(), read_group, self.s5.header)
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: slow5_hdr_set {}: {} could not set to C s5.header struct".format(h, checked_header[h]))
 *                 errors = True
 */
    __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
    if (__pyx_t_4) {

      /* "pyslow5.pyx":1652
 *             ret = slow5_hdr_set(h.encode(), checked_header[h].encode(), read_group, self.s5.header)
 *             if ret < 0:
 *                 self.logger.error("write_header: slow5_hdr_set {}: {} could not set to C s5.header struct".format(h, checked_header[h]))             # <<<<<<<<<<<<<<
 *                 errors = True
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1652, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_header_slow5_hdr_set_could, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1652, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_checked_header, __pyx_v_h); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1652, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_10 = NULL;
      __pyx_t_12 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_12 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_h, __pyx_t_13};
        __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1652, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_h, __pyx_t_13};
        __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1652, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1652, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (__pyx_t_10) {
          __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        }
        __Pyx_INCREF(__pyx_v_h);
        __Pyx_GIVEREF(__pyx_v_h);
        PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_12, __pyx_v_h);
        __Pyx_GIVEREF(__pyx_t_13);
        PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_12, __pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1652, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_7, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1652, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyslow5.pyx":1653
 *             if ret < 0:
 *                 self.logger.error("write_header: slow5_hdr_set {}: {} could not set to C s5.header struct".format(h, checked_header[h]))
 *                 errors = True             # <<<<<<<<<<<<<<
 * 
 *         if not errors:
 */
      __pyx_v_errors = 1;

      /* "pyslow5.pyx":1651
 *         for h in checked_header:
 *             ret = slow5_hdr_set(h.encode(), checked_header[h].encode(), read_group, self.s5.header)
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_header: slow5_hdr_set {}: {} could not set to C s5.header struct".format(h, checked_header[h]))
 *                 errors = True
 */
    }

    /* "pyslow5.pyx":1649
 *             return -1
 * 
 *         for h in checked_header:             # <<<<<<<<<<<<<<
 *             ret = slow5_hdr_set(h.encode(), checked_header[h].encode(), read_group, self.s5.header)
 *             if ret < 0:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1655
 *                 errors = True
 * 
 *         if not errors:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
  __pyx_t_4 = ((!(__pyx_v_errors != 0)) != 0);
  if (__pyx_t_4) {

    /* "pyslow5.pyx":1656
 * 
 *         if not errors:
 *             return 0             # <<<<<<<<<<<<<<
 *         else:
 *             self.logger.error("write_header: errors encountered setting up header, aborting write")
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_0);
    __pyx_r = __pyx_int_0;
    goto __pyx_L0;

    /* "pyslow5.pyx":1655
 *                 errors = True
 * 
 *         if not errors:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
  }

  /* "pyslow5.pyx":1658
 *             return 0
 *         else:
 *             self.logger.error("write_header: errors encountered setting up header, aborting write")             # <<<<<<<<<<<<<<
 *             # sys.exit(1)
 *             return -1
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1658, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_write_header_errors_encountered) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_write_header_errors_encountered);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1658, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1660
 *             self.logger.error("write_header: errors encountered setting up header, aborting write")
 *             # sys.exit(1)
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *     def write_record(self, record, aux=None):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_neg_1);
    __pyx_r = __pyx_int_neg_1;
    goto __pyx_L0;
  }

  /* "pyslow5.pyx":1617
 * 
 * 
 *     def write_header(self, header, read_group=0):             # <<<<<<<<<<<<<<
 *         '''
 *         write slow5 header to file.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyslow5.Open.write_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_checked_header);
  __Pyx_XDECREF(__pyx_v_h);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1662
 *             return -1
 * 
 *     def write_record(self, record, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         write a single record
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_61write_record(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_60write_record[] = "\n        write a single record\n        check self.header_state, if False, header needs to be written. Use aux types to do first check.\n        if the validation works, then write the header just before the first record is written.\n        This should make the user experience a lot less complicated and have less steps\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_61write_record(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_record = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_record (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_record,&__pyx_n_s_aux,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "write_record") < 0)) __PYX_ERR(0, 1662, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_record = values[0];
    __pyx_v_aux = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_record", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1662, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.write_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_60write_record(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_record, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_60write_record(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_record, PyObject *__pyx_v_aux) {
  PyObject *__pyx_v_aux_types = NULL;
  PyObject *__pyx_v_checked_record = NULL;
  PyObject *__pyx_v_checked_aux = NULL;
  int __pyx_v_error;
  PyObject *__pyx_v_slow5_aux_types = NULL;
  PyObject *__pyx_v_a = NULL;
  long __pyx_v_ret;
  PyObject *__pyx_v_start_write_copy_signal = NULL;
  PyObject *__pyx_v_memview = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_end_write_copy_signal = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  char const *__pyx_t_11;
  enum slow5_aux_type __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  char const *__pyx_t_17;
  uint32_t __pyx_t_18;
  double __pyx_t_19;
  uint64_t __pyx_t_20;
  size_t __pyx_t_21;
  int16_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_record", 0);
  __Pyx_TraceCall("write_record", __pyx_f[0], 1662, 0, __PYX_ERR(0, 1662, __pyx_L1_error));

  /* "pyslow5.pyx":1670
 *         '''
 * 
 *         aux_types = {"channel_number": type("string"),             # <<<<<<<<<<<<<<
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1670, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1670, __pyx_L1_error)

  /* "pyslow5.pyx":1671
 * 
 *         aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),             # <<<<<<<<<<<<<<
 *                      "read_number": type(10),
 *                      "start_mux": type(1),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1670, __pyx_L1_error)

  /* "pyslow5.pyx":1672
 *         aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),             # <<<<<<<<<<<<<<
 *                      "start_mux": type(1),
 *                      "start_time": type(100),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1670, __pyx_L1_error)

  /* "pyslow5.pyx":1673
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),
 *                      "start_mux": type(1),             # <<<<<<<<<<<<<<
 *                      "start_time": type(100),
 *                      "end_reason": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1670, __pyx_L1_error)

  /* "pyslow5.pyx":1674
 *                      "read_number": type(10),
 *                      "start_mux": type(1),
 *                      "start_time": type(100),             # <<<<<<<<<<<<<<
 *                      "end_reason": None}
 * 
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, ((PyObject *)Py_TYPE(__pyx_int_100))) < 0) __PYX_ERR(0, 1670, __pyx_L1_error)

  /* "pyslow5.pyx":1675
 *                      "start_mux": type(1),
 *                      "start_time": type(100),
 *                      "end_reason": None}             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("write_record: _record_type_validation running")
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_end_reason, Py_None) < 0) __PYX_ERR(0, 1670, __pyx_L1_error)
  __pyx_v_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1677
 *                      "end_reason": None}
 * 
 *         self.logger.debug("write_record: _record_type_validation running")             # <<<<<<<<<<<<<<
 *         checked_record, checked_aux = self._record_type_validation(record, aux)
 *         self.logger.debug("write_record: _record_type_validation done")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_write_record__record_type_valida) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record__record_type_valida);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1678
 * 
 *         self.logger.debug("write_record: _record_type_validation running")
 *         checked_record, checked_aux = self._record_type_validation(record, aux)             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record: _record_type_validation done")
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_record_type_validation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1678, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_record, __pyx_v_aux};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_record, __pyx_v_aux};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_record);
    __Pyx_GIVEREF(__pyx_v_record);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_record);
    __Pyx_INCREF(__pyx_v_aux);
    __Pyx_GIVEREF(__pyx_v_aux);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_aux);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1678, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_5 = __pyx_t_6(__pyx_t_3); if (unlikely(!__pyx_t_5)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_3), 2) < 0) __PYX_ERR(0, 1678, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1678, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_checked_record = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_checked_aux = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyslow5.pyx":1679
 *         self.logger.debug("write_record: _record_type_validation running")
 *         checked_record, checked_aux = self._record_type_validation(record, aux)
 *         self.logger.debug("write_record: _record_type_validation done")             # <<<<<<<<<<<<<<
 * 
 *         # check if in append state
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record__record_type_valida_2) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record__record_type_valida_2);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1683
 *         # check if in append state
 *         # if so, we can skip header setup
 *         if self.state == 2:             # <<<<<<<<<<<<<<
 *             self.header_state = True
 * 
 */
  __pyx_t_7 = ((__pyx_v_self->state == 2) != 0);
  if (__pyx_t_7) {

    /* "pyslow5.pyx":1684
 *         # if so, we can skip header setup
 *         if self.state == 2:
 *             self.header_state = True             # <<<<<<<<<<<<<<
 * 
 *         # if all checks are good, test self.header_state for 1 time write
 */
    __pyx_v_self->header_state = 1;

    /* "pyslow5.pyx":1683
 *         # check if in append state
 *         # if so, we can skip header setup
 *         if self.state == 2:             # <<<<<<<<<<<<<<
 *             self.header_state = True
 * 
 */
  }

  /* "pyslow5.pyx":1687
 * 
 *         # if all checks are good, test self.header_state for 1 time write
 *         if not self.header_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record: checking header stuff...")
 *             error = False
 */
  __pyx_t_7 = ((!(__pyx_v_self->header_state != 0)) != 0);
  if (__pyx_t_7) {

    /* "pyslow5.pyx":1688
 *         # if all checks are good, test self.header_state for 1 time write
 *         if not self.header_state:
 *             self.logger.debug("write_record: checking header stuff...")             # <<<<<<<<<<<<<<
 *             error = False
 *             if aux is not None:
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_checking_header_stu) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_checking_header_stu);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1689
 *         if not self.header_state:
 *             self.logger.debug("write_record: checking header stuff...")
 *             error = False             # <<<<<<<<<<<<<<
 *             if aux is not None:
 *                 if checked_aux is not None:
 */
    __pyx_v_error = 0;

    /* "pyslow5.pyx":1690
 *             self.logger.debug("write_record: checking header stuff...")
 *             error = False
 *             if aux is not None:             # <<<<<<<<<<<<<<
 *                 if checked_aux is not None:
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 */
    __pyx_t_7 = (__pyx_v_aux != Py_None);
    __pyx_t_8 = (__pyx_t_7 != 0);
    if (__pyx_t_8) {

      /* "pyslow5.pyx":1691
 *             error = False
 *             if aux is not None:
 *                 if checked_aux is not None:             # <<<<<<<<<<<<<<
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                     for a in slow5_aux_types:
 */
      __pyx_t_8 = (__pyx_v_checked_aux != Py_None);
      __pyx_t_7 = (__pyx_t_8 != 0);
      if (__pyx_t_7) {

        /* "pyslow5.pyx":1692
 *             if aux is not None:
 *                 if checked_aux is not None:
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)             # <<<<<<<<<<<<<<
 *                     for a in slow5_aux_types:
 *                         if slow5_aux_types[a] is None:
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_aux_header_type_validation); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1692, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_2 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_v_aux_types) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_aux_types);
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1692, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_v_slow5_aux_types = __pyx_t_1;
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1693
 *                 if checked_aux is not None:
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                     for a in slow5_aux_types:             # <<<<<<<<<<<<<<
 *                         if slow5_aux_types[a] is None:
 *                             continue
 */
        if (likely(PyList_CheckExact(__pyx_v_slow5_aux_types)) || PyTuple_CheckExact(__pyx_v_slow5_aux_types)) {
          __pyx_t_1 = __pyx_v_slow5_aux_types; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
          __pyx_t_10 = NULL;
        } else {
          __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_slow5_aux_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1693, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_10 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1693, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_10)) {
            if (likely(PyList_CheckExact(__pyx_t_1))) {
              if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_5); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1693, __pyx_L1_error)
              #else
              __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1693, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              #endif
            } else {
              if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_5); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1693, __pyx_L1_error)
              #else
              __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1693, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              #endif
            }
          } else {
            __pyx_t_5 = __pyx_t_10(__pyx_t_1);
            if (unlikely(!__pyx_t_5)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 1693, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_5);
          }
          __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_5);
          __pyx_t_5 = 0;

          /* "pyslow5.pyx":1694
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                     for a in slow5_aux_types:
 *                         if slow5_aux_types[a] is None:             # <<<<<<<<<<<<<<
 *                             continue
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 */
          __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_slow5_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1694, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_7 = (__pyx_t_5 == Py_None);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_8 = (__pyx_t_7 != 0);
          if (__pyx_t_8) {

            /* "pyslow5.pyx":1695
 *                     for a in slow5_aux_types:
 *                         if slow5_aux_types[a] is None:
 *                             continue             # <<<<<<<<<<<<<<
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                         if ret < 0:
 */
            goto __pyx_L9_continue;

            /* "pyslow5.pyx":1694
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                     for a in slow5_aux_types:
 *                         if slow5_aux_types[a] is None:             # <<<<<<<<<<<<<<
 *                             continue
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 */
          }

          /* "pyslow5.pyx":1696
 *                         if slow5_aux_types[a] is None:
 *                             continue
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)             # <<<<<<<<<<<<<<
 *                         if ret < 0:
 *                             self.logger.error("write_record: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1696, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
            if (likely(__pyx_t_3)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
              __Pyx_INCREF(__pyx_t_3);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_2, function);
            }
          }
          __pyx_t_5 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1696, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_11 = __Pyx_PyObject_AsString(__pyx_t_5); if (unlikely((!__pyx_t_11) && PyErr_Occurred())) __PYX_ERR(0, 1696, __pyx_L1_error)
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_slow5_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1696, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_12 = ((enum slow5_aux_type)__Pyx_PyInt_As_enum__slow5_aux_type(__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1696, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_ret = slow5_aux_add(__pyx_t_11, __pyx_t_12, __pyx_v_self->s5->header);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "pyslow5.pyx":1697
 *                             continue
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                         if ret < 0:             # <<<<<<<<<<<<<<
 *                             self.logger.error("write_record: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 *                             error = True
 */
          __pyx_t_8 = ((__pyx_v_ret < 0) != 0);
          if (__pyx_t_8) {

            /* "pyslow5.pyx":1698
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                         if ret < 0:
 *                             self.logger.error("write_record: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                             error = True
 *                 else:
 */
            __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1698, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_slow5_aux_add_could, __pyx_n_s_format); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1698, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1698, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_15 = NULL;
            __pyx_t_4 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_13);
              if (likely(__pyx_t_15)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
                __Pyx_INCREF(__pyx_t_15);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_13, function);
                __pyx_t_4 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_14};
              __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_13, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1698, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_14};
              __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_13, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1698, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            } else
            #endif
            {
              __pyx_t_16 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1698, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_16);
              if (__pyx_t_15) {
                __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_15); __pyx_t_15 = NULL;
              }
              __Pyx_INCREF(__pyx_v_a);
              __Pyx_GIVEREF(__pyx_v_a);
              PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_4, __pyx_v_a);
              __Pyx_GIVEREF(__pyx_t_14);
              PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_4, __pyx_t_14);
              __pyx_t_14 = 0;
              __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1698, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            }
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __pyx_t_13 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
              __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
              if (likely(__pyx_t_13)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
                __Pyx_INCREF(__pyx_t_13);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_2, function);
              }
            }
            __pyx_t_5 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_13, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1698, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "pyslow5.pyx":1699
 *                         if ret < 0:
 *                             self.logger.error("write_record: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 *                             error = True             # <<<<<<<<<<<<<<
 *                 else:
 *                     error = True
 */
            __pyx_v_error = 1;

            /* "pyslow5.pyx":1697
 *                             continue
 *                         ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                         if ret < 0:             # <<<<<<<<<<<<<<
 *                             self.logger.error("write_record: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 *                             error = True
 */
          }

          /* "pyslow5.pyx":1693
 *                 if checked_aux is not None:
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                     for a in slow5_aux_types:             # <<<<<<<<<<<<<<
 *                         if slow5_aux_types[a] is None:
 *                             continue
 */
          __pyx_L9_continue:;
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1691
 *             error = False
 *             if aux is not None:
 *                 if checked_aux is not None:             # <<<<<<<<<<<<<<
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                     for a in slow5_aux_types:
 */
        goto __pyx_L8;
      }

      /* "pyslow5.pyx":1701
 *                             error = True
 *                 else:
 *                     error = True             # <<<<<<<<<<<<<<
 * 
 *             if error:
 */
      /*else*/ {
        __pyx_v_error = 1;
      }
      __pyx_L8:;

      /* "pyslow5.pyx":1690
 *             self.logger.debug("write_record: checking header stuff...")
 *             error = False
 *             if aux is not None:             # <<<<<<<<<<<<<<
 *                 if checked_aux is not None:
 *                     slow5_aux_types = self._aux_header_type_validation(aux_types)
 */
    }

    /* "pyslow5.pyx":1703
 *                     error = True
 * 
 *             if error:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record: aux_meta fields failed to initialise")
 *                 return -1
 */
    __pyx_t_8 = (__pyx_v_error != 0);
    if (__pyx_t_8) {

      /* "pyslow5.pyx":1704
 * 
 *             if error:
 *                 self.logger.error("write_record: aux_meta fields failed to initialise")             # <<<<<<<<<<<<<<
 *                 return -1
 *             # write the header
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1704, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_aux_meta_fields_fai) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_aux_meta_fields_fai);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1704, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1705
 *             if error:
 *                 self.logger.error("write_record: aux_meta fields failed to initialise")
 *                 return -1             # <<<<<<<<<<<<<<
 *             # write the header
 *             self.logger.debug("write_record: writting header...")
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_int_neg_1);
      __pyx_r = __pyx_int_neg_1;
      goto __pyx_L0;

      /* "pyslow5.pyx":1703
 *                     error = True
 * 
 *             if error:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record: aux_meta fields failed to initialise")
 *                 return -1
 */
    }

    /* "pyslow5.pyx":1707
 *                 return -1
 *             # write the header
 *             self.logger.debug("write_record: writting header...")             # <<<<<<<<<<<<<<
 *             ret = slow5_hdr_write(self.s5)
 *             if ret < 0:
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1707, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_writting_header) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_writting_header);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1707, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1708
 *             # write the header
 *             self.logger.debug("write_record: writting header...")
 *             ret = slow5_hdr_write(self.s5)             # <<<<<<<<<<<<<<
 *             if ret < 0:
 *                 self.logger.error("write_record: slow5_hdr_write could not write header")
 */
    __pyx_v_ret = slow5_hdr_write(__pyx_v_self->s5);

    /* "pyslow5.pyx":1709
 *             self.logger.debug("write_record: writting header...")
 *             ret = slow5_hdr_write(self.s5)
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record: slow5_hdr_write could not write header")
 *                 return -1
 */
    __pyx_t_8 = ((__pyx_v_ret < 0) != 0);
    if (__pyx_t_8) {

      /* "pyslow5.pyx":1710
 *             ret = slow5_hdr_write(self.s5)
 *             if ret < 0:
 *                 self.logger.error("write_record: slow5_hdr_write could not write header")             # <<<<<<<<<<<<<<
 *                 return -1
 *             # set state true so only done once
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1710, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_slow5_hdr_write_cou) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_slow5_hdr_write_cou);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1710, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1711
 *             if ret < 0:
 *                 self.logger.error("write_record: slow5_hdr_write could not write header")
 *                 return -1             # <<<<<<<<<<<<<<
 *             # set state true so only done once
 *             self.header_state = True
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_int_neg_1);
      __pyx_r = __pyx_int_neg_1;
      goto __pyx_L0;

      /* "pyslow5.pyx":1709
 *             self.logger.debug("write_record: writting header...")
 *             ret = slow5_hdr_write(self.s5)
 *             if ret < 0:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record: slow5_hdr_write could not write header")
 *                 return -1
 */
    }

    /* "pyslow5.pyx":1713
 *                 return -1
 *             # set state true so only done once
 *             self.header_state = True             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record: header written")
 * 
 */
    __pyx_v_self->header_state = 1;

    /* "pyslow5.pyx":1714
 *             # set state true so only done once
 *             self.header_state = True
 *             self.logger.debug("write_record: header written")             # <<<<<<<<<<<<<<
 * 
 *         # Add values to read struct
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1714, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_header_written) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_header_written);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1714, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1687
 * 
 *         # if all checks are good, test self.header_state for 1 time write
 *         if not self.header_state:             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record: checking header stuff...")
 *             error = False
 */
  }

  /* "pyslow5.pyx":1717
 * 
 *         # Add values to read struct
 *         self.logger.debug("write_record: slow5_rec_init()")             # <<<<<<<<<<<<<<
 *         self.write = slow5_rec_init()
 *         if self.write == NULL:
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_slow5_rec_init) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_slow5_rec_init);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1718
 *         # Add values to read struct
 *         self.logger.debug("write_record: slow5_rec_init()")
 *         self.write = slow5_rec_init()             # <<<<<<<<<<<<<<
 *         if self.write == NULL:
 *             self.logger.error("write_record: failed to allocate space for slow5 record (self.write)")
 */
  __pyx_v_self->write = slow5_rec_init();

  /* "pyslow5.pyx":1719
 *         self.logger.debug("write_record: slow5_rec_init()")
 *         self.write = slow5_rec_init()
 *         if self.write == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.error("write_record: failed to allocate space for slow5 record (self.write)")
 *             return -1
 */
  __pyx_t_8 = ((__pyx_v_self->write == NULL) != 0);
  if (__pyx_t_8) {

    /* "pyslow5.pyx":1720
 *         self.write = slow5_rec_init()
 *         if self.write == NULL:
 *             self.logger.error("write_record: failed to allocate space for slow5 record (self.write)")             # <<<<<<<<<<<<<<
 *             return -1
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1720, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_failed_to_allocate) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_failed_to_allocate);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1720, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1721
 *         if self.write == NULL:
 *             self.logger.error("write_record: failed to allocate space for slow5 record (self.write)")
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("write_record: self.write assignments...")
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_neg_1);
    __pyx_r = __pyx_int_neg_1;
    goto __pyx_L0;

    /* "pyslow5.pyx":1719
 *         self.logger.debug("write_record: slow5_rec_init()")
 *         self.write = slow5_rec_init()
 *         if self.write == NULL:             # <<<<<<<<<<<<<<
 *             self.logger.error("write_record: failed to allocate space for slow5 record (self.write)")
 *             return -1
 */
  }

  /* "pyslow5.pyx":1723
 *             return -1
 * 
 *         self.logger.debug("write_record: self.write assignments...")             # <<<<<<<<<<<<<<
 *         checked_record["read_id"] = checked_record["read_id"].encode()
 *         self.write.read_id = strdup(checked_record["read_id"])
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_2, __pyx_kp_u_write_record_self_write_assignme) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u_write_record_self_write_assignme);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1724
 * 
 *         self.logger.debug("write_record: self.write assignments...")
 *         checked_record["read_id"] = checked_record["read_id"].encode()             # <<<<<<<<<<<<<<
 *         self.write.read_id = strdup(checked_record["read_id"])
 *         self.write.read_id_len = len(checked_record["read_id"])
 */
  __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_read_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_checked_record, __pyx_n_u_read_id, __pyx_t_1) < 0)) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1725
 *         self.logger.debug("write_record: self.write assignments...")
 *         checked_record["read_id"] = checked_record["read_id"].encode()
 *         self.write.read_id = strdup(checked_record["read_id"])             # <<<<<<<<<<<<<<
 *         self.write.read_id_len = len(checked_record["read_id"])
 *         self.write.read_group = checked_record["read_group"]
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_read_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_17 = __Pyx_PyObject_AsString(__pyx_t_1); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 1725, __pyx_L1_error)
  __pyx_v_self->write->read_id = strdup(__pyx_t_17);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1726
 *         checked_record["read_id"] = checked_record["read_id"].encode()
 *         self.write.read_id = strdup(checked_record["read_id"])
 *         self.write.read_id_len = len(checked_record["read_id"])             # <<<<<<<<<<<<<<
 *         self.write.read_group = checked_record["read_group"]
 *         self.write.digitisation = checked_record["digitisation"]
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_read_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1726, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->read_id_len = __pyx_t_9;

  /* "pyslow5.pyx":1727
 *         self.write.read_id = strdup(checked_record["read_id"])
 *         self.write.read_id_len = len(checked_record["read_id"])
 *         self.write.read_group = checked_record["read_group"]             # <<<<<<<<<<<<<<
 *         self.write.digitisation = checked_record["digitisation"]
 *         self.write.offset = checked_record["offset"]
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_read_group); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_18 = __Pyx_PyInt_As_uint32_t(__pyx_t_1); if (unlikely((__pyx_t_18 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1727, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->read_group = __pyx_t_18;

  /* "pyslow5.pyx":1728
 *         self.write.read_id_len = len(checked_record["read_id"])
 *         self.write.read_group = checked_record["read_group"]
 *         self.write.digitisation = checked_record["digitisation"]             # <<<<<<<<<<<<<<
 *         self.write.offset = checked_record["offset"]
 *         self.write.range = checked_record["range"]
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_digitisation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_19 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_19 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->digitisation = __pyx_t_19;

  /* "pyslow5.pyx":1729
 *         self.write.read_group = checked_record["read_group"]
 *         self.write.digitisation = checked_record["digitisation"]
 *         self.write.offset = checked_record["offset"]             # <<<<<<<<<<<<<<
 *         self.write.range = checked_record["range"]
 *         self.write.sampling_rate = checked_record["sampling_rate"]
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_19 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_19 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1729, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->offset = __pyx_t_19;

  /* "pyslow5.pyx":1730
 *         self.write.digitisation = checked_record["digitisation"]
 *         self.write.offset = checked_record["offset"]
 *         self.write.range = checked_record["range"]             # <<<<<<<<<<<<<<
 *         self.write.sampling_rate = checked_record["sampling_rate"]
 *         self.write.len_raw_signal = checked_record["len_raw_signal"]
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_19 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_19 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1730, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->range = __pyx_t_19;

  /* "pyslow5.pyx":1731
 *         self.write.offset = checked_record["offset"]
 *         self.write.range = checked_record["range"]
 *         self.write.sampling_rate = checked_record["sampling_rate"]             # <<<<<<<<<<<<<<
 *         self.write.len_raw_signal = checked_record["len_raw_signal"]
 *         self.logger.debug("write_record: self.write.raw_signal malloc...")
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_sampling_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_19 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_19 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->sampling_rate = __pyx_t_19;

  /* "pyslow5.pyx":1732
 *         self.write.range = checked_record["range"]
 *         self.write.sampling_rate = checked_record["sampling_rate"]
 *         self.write.len_raw_signal = checked_record["len_raw_signal"]             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record: self.write.raw_signal malloc...")
 *         self.write.raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_record["len_raw_signal"])
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_len_raw_signal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_20 = __Pyx_PyInt_As_uint64_t(__pyx_t_1); if (unlikely((__pyx_t_20 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1732, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->write->len_raw_signal = __pyx_t_20;

  /* "pyslow5.pyx":1733
 *         self.write.sampling_rate = checked_record["sampling_rate"]
 *         self.write.len_raw_signal = checked_record["len_raw_signal"]
 *         self.logger.debug("write_record: self.write.raw_signal malloc...")             # <<<<<<<<<<<<<<
 *         self.write.raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_record["len_raw_signal"])
 *         self.logger.debug("write_record: self.write.raw_signal malloc done")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_kp_u_write_record_self_write_raw_sign) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_self_write_raw_sign);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1734
 *         self.write.len_raw_signal = checked_record["len_raw_signal"]
 *         self.logger.debug("write_record: self.write.raw_signal malloc...")
 *         self.write.raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_record["len_raw_signal"])             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record: self.write.raw_signal malloc done")
 *         self.logger.debug("write_record: self.write assignments done")
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(int16_t))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_len_raw_signal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_21 = __Pyx_PyInt_As_size_t(__pyx_t_5); if (unlikely((__pyx_t_21 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1734, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_self->write->raw_signal = ((int16_t *)malloc(__pyx_t_21));

  /* "pyslow5.pyx":1735
 *         self.logger.debug("write_record: self.write.raw_signal malloc...")
 *         self.write.raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_record["len_raw_signal"])
 *         self.logger.debug("write_record: self.write.raw_signal malloc done")             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record: self.write assignments done")
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_self_write_raw_sign_2) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_self_write_raw_sign_2);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1736
 *         self.write.raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_record["len_raw_signal"])
 *         self.logger.debug("write_record: self.write.raw_signal malloc done")
 *         self.logger.debug("write_record: self.write assignments done")             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("write_record: self.write processing raw_signal")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_self_write_assignme_2) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_self_write_assignme_2);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1738
 *         self.logger.debug("write_record: self.write assignments done")
 * 
 *         self.logger.debug("write_record: self.write processing raw_signal")             # <<<<<<<<<<<<<<
 *         start_write_copy_signal = time.time()
 *         # grabs buffer of numby array so the for loop operats in C not python = super fast
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_self_write_processi) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_self_write_processi);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1739
 * 
 *         self.logger.debug("write_record: self.write processing raw_signal")
 *         start_write_copy_signal = time.time()             # <<<<<<<<<<<<<<
 *         # grabs buffer of numby array so the for loop operats in C not python = super fast
 *         memview = memoryview(checked_record["signal"])
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_start_write_copy_signal = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyslow5.pyx":1741
 *         start_write_copy_signal = time.time()
 *         # grabs buffer of numby array so the for loop operats in C not python = super fast
 *         memview = memoryview(checked_record["signal"])             # <<<<<<<<<<<<<<
 *         for i in range(checked_record["len_raw_signal"]):
 *             self.write.raw_signal[i] = memview[i]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_memoryview); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_signal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_memview = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":1742
 *         # grabs buffer of numby array so the for loop operats in C not python = super fast
 *         memview = memoryview(checked_record["signal"])
 *         for i in range(checked_record["len_raw_signal"]):             # <<<<<<<<<<<<<<
 *             self.write.raw_signal[i] = memview[i]
 *         # for i in range(checked_record["len_raw_signal"]):
 */
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_checked_record, __pyx_n_u_len_raw_signal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1742, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_1); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1742, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1742, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_1); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1742, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1742, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_10(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1742, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1743
 *         memview = memoryview(checked_record["signal"])
 *         for i in range(checked_record["len_raw_signal"]):
 *             self.write.raw_signal[i] = memview[i]             # <<<<<<<<<<<<<<
 *         # for i in range(checked_record["len_raw_signal"]):
 *         #     self.write.raw_signal[i] = checked_record["signal"][i]
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_memview, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_22 = __Pyx_PyInt_As_int16_t(__pyx_t_1); if (unlikely((__pyx_t_22 == ((int16_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1743, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_23 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_23 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1743, __pyx_L1_error)
    (__pyx_v_self->write->raw_signal[__pyx_t_23]) = __pyx_t_22;

    /* "pyslow5.pyx":1742
 *         # grabs buffer of numby array so the for loop operats in C not python = super fast
 *         memview = memoryview(checked_record["signal"])
 *         for i in range(checked_record["len_raw_signal"]):             # <<<<<<<<<<<<<<
 *             self.write.raw_signal[i] = memview[i]
 *         # for i in range(checked_record["len_raw_signal"]):
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":1746
 *         # for i in range(checked_record["len_raw_signal"]):
 *         #     self.write.raw_signal[i] = checked_record["signal"][i]
 *         end_write_copy_signal = (time.time() - start_write_copy_signal)             # <<<<<<<<<<<<<<
 *         self.total_single_write_time = self.total_single_write_time + end_write_copy_signal
 *         self.logger.debug("write_record: self.write raw_signal done")
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_time); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Subtract(__pyx_t_2, __pyx_v_start_write_copy_signal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_end_write_copy_signal = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyslow5.pyx":1747
 *         #     self.write.raw_signal[i] = checked_record["signal"][i]
 *         end_write_copy_signal = (time.time() - start_write_copy_signal)
 *         self.total_single_write_time = self.total_single_write_time + end_write_copy_signal             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record: self.write raw_signal done")
 * 
 */
  __pyx_t_5 = PyNumber_Add(__pyx_v_self->total_single_write_time, __pyx_v_end_write_copy_signal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (!(likely(PyFloat_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 1747, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->total_single_write_time);
  __Pyx_DECREF(__pyx_v_self->total_single_write_time);
  __pyx_v_self->total_single_write_time = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "pyslow5.pyx":1748
 *         end_write_copy_signal = (time.time() - start_write_copy_signal)
 *         self.total_single_write_time = self.total_single_write_time + end_write_copy_signal
 *         self.logger.debug("write_record: self.write raw_signal done")             # <<<<<<<<<<<<<<
 * 
 *         if aux:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_self_write_raw_sign_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_self_write_raw_sign_3);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1750
 *         self.logger.debug("write_record: self.write raw_signal done")
 * 
 *         if aux:             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record: aux stuff...")
 *             if checked_aux is None:
 */
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_aux); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1750, __pyx_L1_error)
  if (__pyx_t_8) {

    /* "pyslow5.pyx":1751
 * 
 *         if aux:
 *             self.logger.debug("write_record: aux stuff...")             # <<<<<<<<<<<<<<
 *             if checked_aux is None:
 *                 self.logger.error("write_record: checked_aux is None".format(a, checked_aux[a]))
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_aux_stuff) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_aux_stuff);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyslow5.pyx":1752
 *         if aux:
 *             self.logger.debug("write_record: aux stuff...")
 *             if checked_aux is None:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record: checked_aux is None".format(a, checked_aux[a]))
 *                 return -1
 */
    __pyx_t_8 = (__pyx_v_checked_aux == Py_None);
    __pyx_t_7 = (__pyx_t_8 != 0);
    if (__pyx_t_7) {

      /* "pyslow5.pyx":1753
 *             self.logger.debug("write_record: aux stuff...")
 *             if checked_aux is None:
 *                 self.logger.error("write_record: checked_aux is None".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                 return -1
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_checked_aux_is_None, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(!__pyx_v_a)) { __Pyx_RaiseUnboundLocalError("a"); __PYX_ERR(0, 1753, __pyx_L1_error) }
      if (unlikely(!__pyx_v_a)) { __Pyx_RaiseUnboundLocalError("a"); __PYX_ERR(0, 1753, __pyx_L1_error) }
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_16 = NULL;
      __pyx_t_4 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_4 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_v_a, __pyx_t_13};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1753, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_v_a, __pyx_t_13};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1753, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      {
        __pyx_t_14 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1753, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        if (__pyx_t_16) {
          __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_16); __pyx_t_16 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_4, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_13);
        PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_4, __pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1753, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_5 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyslow5.pyx":1754
 *             if checked_aux is None:
 *                 self.logger.error("write_record: checked_aux is None".format(a, checked_aux[a]))
 *                 return -1             # <<<<<<<<<<<<<<
 * 
 *             for a in checked_aux:
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_int_neg_1);
      __pyx_r = __pyx_int_neg_1;
      goto __pyx_L0;

      /* "pyslow5.pyx":1752
 *         if aux:
 *             self.logger.debug("write_record: aux stuff...")
 *             if checked_aux is None:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record: checked_aux is None".format(a, checked_aux[a]))
 *                 return -1
 */
    }

    /* "pyslow5.pyx":1756
 *                 return -1
 * 
 *             for a in checked_aux:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record: checked_aux: {}: {}".format(a, checked_aux[a]))
 *                 if checked_aux[a] is None:
 */
    if (likely(PyList_CheckExact(__pyx_v_checked_aux)) || PyTuple_CheckExact(__pyx_v_checked_aux)) {
      __pyx_t_5 = __pyx_v_checked_aux; __Pyx_INCREF(__pyx_t_5); __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_v_checked_aux); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1756, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1756, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1756, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_5, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1756, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1756, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_5, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1756, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_10(__pyx_t_5);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1756, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyslow5.pyx":1757
 * 
 *             for a in checked_aux:
 *                 self.logger.debug("write_record: checked_aux: {}: {}".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                 if checked_aux[a] is None:
 *                     continue
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1757, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_checked_aux, __pyx_n_s_format); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1757, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1757, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_16 = NULL;
      __pyx_t_4 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_14, function);
          __pyx_t_4 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_14)) {
        PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_v_a, __pyx_t_13};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1757, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
        PyObject *__pyx_temp[3] = {__pyx_t_16, __pyx_v_a, __pyx_t_13};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1757, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      } else
      #endif
      {
        __pyx_t_15 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1757, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (__pyx_t_16) {
          __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_16); __pyx_t_16 = NULL;
        }
        __Pyx_INCREF(__pyx_v_a);
        __Pyx_GIVEREF(__pyx_v_a);
        PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_4, __pyx_v_a);
        __Pyx_GIVEREF(__pyx_t_13);
        PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_4, __pyx_t_13);
        __pyx_t_13 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_15, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1757, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      }
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_14)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_2 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_14, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1757, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":1758
 *             for a in checked_aux:
 *                 self.logger.debug("write_record: checked_aux: {}: {}".format(a, checked_aux[a]))
 *                 if checked_aux[a] is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if a == "channel_number":
 */
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1758, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = (__pyx_t_2 == Py_None);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = (__pyx_t_7 != 0);
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1759
 *                 self.logger.debug("write_record: checked_aux: {}: {}".format(a, checked_aux[a]))
 *                 if checked_aux[a] is None:
 *                     continue             # <<<<<<<<<<<<<<
 *                 if a == "channel_number":
 *                     self.logger.debug("write_record: slow5_aux_set_string running...")
 */
        goto __pyx_L20_continue;

        /* "pyslow5.pyx":1758
 *             for a in checked_aux:
 *                 self.logger.debug("write_record: checked_aux: {}: {}".format(a, checked_aux[a]))
 *                 if checked_aux[a] is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if a == "channel_number":
 */
      }

      /* "pyslow5.pyx":1760
 *                 if checked_aux[a] is None:
 *                     continue
 *                 if a == "channel_number":             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record: slow5_aux_set_string running...")
 *                     self.logger.debug("write_record: slow5_aux_set_string type: {}".format(type(checked_aux[a])))
 */
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_channel_number, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1760, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1761
 *                     continue
 *                 if a == "channel_number":
 *                     self.logger.debug("write_record: slow5_aux_set_string running...")             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record: slow5_aux_set_string type: {}".format(type(checked_aux[a])))
 *                     ret = slow5_aux_set_string(self.write, self.channel_number, <const char *>self.channel_number_val, self.s5.header)
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1761, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_kp_u_write_record_slow5_aux_set_strin) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_write_record_slow5_aux_set_strin);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1761, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1762
 *                 if a == "channel_number":
 *                     self.logger.debug("write_record: slow5_aux_set_string running...")
 *                     self.logger.debug("write_record: slow5_aux_set_string type: {}".format(type(checked_aux[a])))             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set_string(self.write, self.channel_number, <const char *>self.channel_number_val, self.s5.header)
 *                     self.logger.debug("write_record: slow5_aux_set_string running done: ret = {}".format(ret))
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1762, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_slow5_aux_set_strin_2, __pyx_n_s_format); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1762, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1762, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
          }
        }
        __pyx_t_3 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_14, __pyx_t_13, ((PyObject *)Py_TYPE(__pyx_t_15))) : __Pyx_PyObject_CallOneArg(__pyx_t_14, ((PyObject *)Py_TYPE(__pyx_t_15)));
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1762, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_14)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        __pyx_t_2 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_14, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1762, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1763
 *                     self.logger.debug("write_record: slow5_aux_set_string running...")
 *                     self.logger.debug("write_record: slow5_aux_set_string type: {}".format(type(checked_aux[a])))
 *                     ret = slow5_aux_set_string(self.write, self.channel_number, <const char *>self.channel_number_val, self.s5.header)             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record: slow5_aux_set_string running done: ret = {}".format(ret))
 *                     if ret < 0:
 */
        __pyx_v_ret = slow5_aux_set_string(__pyx_v_self->write, __pyx_v_self->channel_number, ((char const *)__pyx_v_self->channel_number_val), __pyx_v_self->s5->header);

        /* "pyslow5.pyx":1764
 *                     self.logger.debug("write_record: slow5_aux_set_string type: {}".format(type(checked_aux[a])))
 *                     ret = slow5_aux_set_string(self.write, self.channel_number, <const char *>self.channel_number_val, self.s5.header)
 *                     self.logger.debug("write_record: slow5_aux_set_string running done: ret = {}".format(ret))             # <<<<<<<<<<<<<<
 *                     if ret < 0:
 *                         self.logger.error("write_record: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_aux[a]))
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_slow5_aux_set_strin_3, __pyx_n_s_format); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = __Pyx_PyInt_From_long(__pyx_v_ret); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
          }
        }
        __pyx_t_3 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_14, __pyx_t_13, __pyx_t_15) : __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_15);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_14)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        __pyx_t_2 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_14, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1764, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1765
 *                     ret = slow5_aux_set_string(self.write, self.channel_number, <const char *>self.channel_number_val, self.s5.header)
 *                     self.logger.debug("write_record: slow5_aux_set_string running done: ret = {}".format(ret))
 *                     if ret < 0:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_aux[a]))
 *                         #### We should free here
 */
        __pyx_t_8 = ((__pyx_v_ret < 0) != 0);
        if (__pyx_t_8) {

          /* "pyslow5.pyx":1766
 *                     self.logger.debug("write_record: slow5_aux_set_string running done: ret = {}".format(ret))
 *                     if ret < 0:
 *                         self.logger.error("write_record: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                         #### We should free here
 *                         return -1
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1766, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_slow5_aux_set_strin_4, __pyx_n_s_format); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1766, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1766, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_13 = NULL;
          __pyx_t_4 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_14, function);
              __pyx_t_4 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_14)) {
            PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_a, __pyx_t_15};
            __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1766, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
            PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_a, __pyx_t_15};
            __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1766, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          } else
          #endif
          {
            __pyx_t_16 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1766, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_16);
            if (__pyx_t_13) {
              __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_13); __pyx_t_13 = NULL;
            }
            __Pyx_INCREF(__pyx_v_a);
            __Pyx_GIVEREF(__pyx_v_a);
            PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_4, __pyx_v_a);
            __Pyx_GIVEREF(__pyx_t_15);
            PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_4, __pyx_t_15);
            __pyx_t_15 = 0;
            __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1766, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          }
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_14 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_14)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_14);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
            }
          }
          __pyx_t_2 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_14, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1766, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyslow5.pyx":1768
 *                         self.logger.error("write_record: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_aux[a]))
 *                         #### We should free here
 *                         return -1             # <<<<<<<<<<<<<<
 *                 elif a == "median_before":
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)
 */
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_int_neg_1);
          __pyx_r = __pyx_int_neg_1;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          goto __pyx_L0;

          /* "pyslow5.pyx":1765
 *                     ret = slow5_aux_set_string(self.write, self.channel_number, <const char *>self.channel_number_val, self.s5.header)
 *                     self.logger.debug("write_record: slow5_aux_set_string running done: ret = {}".format(ret))
 *                     if ret < 0:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_aux[a]))
 *                         #### We should free here
 */
        }

        /* "pyslow5.pyx":1760
 *                 if checked_aux[a] is None:
 *                     continue
 *                 if a == "channel_number":             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record: slow5_aux_set_string running...")
 *                     self.logger.debug("write_record: slow5_aux_set_string type: {}".format(type(checked_aux[a])))
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":1769
 *                         #### We should free here
 *                         return -1
 *                 elif a == "median_before":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)
 *                 elif a == "read_number":
 */
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_median_before, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1769, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1770
 *                         return -1
 *                 elif a == "median_before":
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)             # <<<<<<<<<<<<<<
 *                 elif a == "read_number":
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)
 */
        __pyx_v_ret = slow5_aux_set(__pyx_v_self->write, __pyx_v_self->median_before, ((void const *)(&__pyx_v_self->median_before_val)), __pyx_v_self->s5->header);

        /* "pyslow5.pyx":1769
 *                         #### We should free here
 *                         return -1
 *                 elif a == "median_before":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)
 *                 elif a == "read_number":
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":1771
 *                 elif a == "median_before":
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)
 *                 elif a == "read_number":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)
 *                 elif a == "start_mux":
 */
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_read_number, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1771, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1772
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)
 *                 elif a == "read_number":
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)             # <<<<<<<<<<<<<<
 *                 elif a == "start_mux":
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)
 */
        __pyx_v_ret = slow5_aux_set(__pyx_v_self->write, __pyx_v_self->read_number, ((void const *)(&__pyx_v_self->read_number_val)), __pyx_v_self->s5->header);

        /* "pyslow5.pyx":1771
 *                 elif a == "median_before":
 *                     ret = slow5_aux_set(self.write, self.median_before, <const void *>&self.median_before_val, self.s5.header)
 *                 elif a == "read_number":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)
 *                 elif a == "start_mux":
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":1773
 *                 elif a == "read_number":
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)
 *                 elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)
 *                 elif a == "start_time":
 */
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_mux, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1773, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1774
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)
 *                 elif a == "start_mux":
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)             # <<<<<<<<<<<<<<
 *                 elif a == "start_time":
 *                     ret = slow5_aux_set(self.write, self.start_time, <const void *>&self.start_time_val, self.s5.header)
 */
        __pyx_v_ret = slow5_aux_set(__pyx_v_self->write, __pyx_v_self->start_mux, ((void const *)(&__pyx_v_self->start_mux_val)), __pyx_v_self->s5->header);

        /* "pyslow5.pyx":1773
 *                 elif a == "read_number":
 *                     ret = slow5_aux_set(self.write, self.read_number, <const void *>&self.read_number_val, self.s5.header)
 *                 elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)
 *                 elif a == "start_time":
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":1775
 *                 elif a == "start_mux":
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)
 *                 elif a == "start_time":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.start_time, <const void *>&self.start_time_val, self.s5.header)
 *                 elif a == "end_reason":
 */
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_time, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1775, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1776
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)
 *                 elif a == "start_time":
 *                     ret = slow5_aux_set(self.write, self.start_time, <const void *>&self.start_time_val, self.s5.header)             # <<<<<<<<<<<<<<
 *                 elif a == "end_reason":
 *                     # not implemented yet becuase of variability in ONT versioning
 */
        __pyx_v_ret = slow5_aux_set(__pyx_v_self->write, __pyx_v_self->start_time, ((void const *)(&__pyx_v_self->start_time_val)), __pyx_v_self->s5->header);

        /* "pyslow5.pyx":1775
 *                 elif a == "start_mux":
 *                     ret = slow5_aux_set(self.write, self.start_mux, <const void *>&self.start_mux_val, self.s5.header)
 *                 elif a == "start_time":             # <<<<<<<<<<<<<<
 *                     ret = slow5_aux_set(self.write, self.start_time, <const void *>&self.start_time_val, self.s5.header)
 *                 elif a == "end_reason":
 */
        goto __pyx_L23;
      }

      /* "pyslow5.pyx":1777
 *                 elif a == "start_time":
 *                     ret = slow5_aux_set(self.write, self.start_time, <const void *>&self.start_time_val, self.s5.header)
 *                 elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                     # not implemented yet becuase of variability in ONT versioning
 *                     ret = 0
 */
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_end_reason, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 1777, __pyx_L1_error)
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1779
 *                 elif a == "end_reason":
 *                     # not implemented yet becuase of variability in ONT versioning
 *                     ret = 0             # <<<<<<<<<<<<<<
 *                 if ret < 0:
 *                     self.logger.error("write_record: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 */
        __pyx_v_ret = 0;

        /* "pyslow5.pyx":1777
 *                 elif a == "start_time":
 *                     ret = slow5_aux_set(self.write, self.start_time, <const void *>&self.start_time_val, self.s5.header)
 *                 elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                     # not implemented yet becuase of variability in ONT versioning
 *                     ret = 0
 */
      }
      __pyx_L23:;

      /* "pyslow5.pyx":1780
 *                     # not implemented yet becuase of variability in ONT versioning
 *                     ret = 0
 *                 if ret < 0:             # <<<<<<<<<<<<<<
 *                     self.logger.error("write_record: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 *                     return -1
 */
      __pyx_t_8 = ((__pyx_v_ret < 0) != 0);
      if (__pyx_t_8) {

        /* "pyslow5.pyx":1781
 *                     ret = 0
 *                 if ret < 0:
 *                     self.logger.error("write_record: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                     return -1
 * 
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1781, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_slow5_aux_set_could, __pyx_n_s_format); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1781, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_16 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1781, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_15 = NULL;
        __pyx_t_4 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
            __pyx_t_4 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_14)) {
          PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_16};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1781, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
          PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_16};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1781, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1781, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_15); __pyx_t_15 = NULL;
          }
          __Pyx_INCREF(__pyx_v_a);
          __Pyx_GIVEREF(__pyx_v_a);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_4, __pyx_v_a);
          __Pyx_GIVEREF(__pyx_t_16);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_4, __pyx_t_16);
          __pyx_t_16 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_13, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1781, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_14)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        __pyx_t_2 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_14, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1781, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":1782
 *                 if ret < 0:
 *                     self.logger.error("write_record: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 *                     return -1             # <<<<<<<<<<<<<<
 * 
 *             self.logger.debug("write_record: aux stuff done")
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_int_neg_1);
        __pyx_r = __pyx_int_neg_1;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        goto __pyx_L0;

        /* "pyslow5.pyx":1780
 *                     # not implemented yet becuase of variability in ONT versioning
 *                     ret = 0
 *                 if ret < 0:             # <<<<<<<<<<<<<<
 *                     self.logger.error("write_record: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 *                     return -1
 */
      }

      /* "pyslow5.pyx":1756
 *                 return -1
 * 
 *             for a in checked_aux:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record: checked_aux: {}: {}".format(a, checked_aux[a]))
 *                 if checked_aux[a] is None:
 */
      __pyx_L20_continue:;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyslow5.pyx":1784
 *                     return -1
 * 
 *             self.logger.debug("write_record: aux stuff done")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1784, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_aux_stuff_done) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_aux_stuff_done);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1784, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyslow5.pyx":1750
 *         self.logger.debug("write_record: self.write raw_signal done")
 * 
 *         if aux:             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record: aux stuff...")
 *             if checked_aux is None:
 */
  }

  /* "pyslow5.pyx":1787
 * 
 * 
 *         self.logger.debug("write_record: slow5_write()")             # <<<<<<<<<<<<<<
 *         # write the record
 *         ret = slow5_write(self.write, self.s5)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_slow5_write) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_slow5_write);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1789
 *         self.logger.debug("write_record: slow5_write()")
 *         # write the record
 *         ret = slow5_write(self.write, self.s5)             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record: slow5_write() ret: {}".format(ret))
 * 
 */
  __pyx_v_ret = slow5_write(__pyx_v_self->write, __pyx_v_self->s5);

  /* "pyslow5.pyx":1790
 *         # write the record
 *         ret = slow5_write(self.write, self.s5)
 *         self.logger.debug("write_record: slow5_write() ret: {}".format(ret))             # <<<<<<<<<<<<<<
 * 
 *         if aux is not None:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_slow5_write_ret, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_14 = __Pyx_PyInt_From_long(__pyx_v_ret); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_13, __pyx_t_14) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_14);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1792
 *         self.logger.debug("write_record: slow5_write() ret: {}".format(ret))
 * 
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             free(self.channel_number_val)
 *             self.channel_number_val = NULL
 */
  __pyx_t_8 = (__pyx_v_aux != Py_None);
  __pyx_t_7 = (__pyx_t_8 != 0);
  if (__pyx_t_7) {

    /* "pyslow5.pyx":1793
 * 
 *         if aux is not None:
 *             free(self.channel_number_val)             # <<<<<<<<<<<<<<
 *             self.channel_number_val = NULL
 *             self.median_before_val = -1.0
 */
    free(__pyx_v_self->channel_number_val);

    /* "pyslow5.pyx":1794
 *         if aux is not None:
 *             free(self.channel_number_val)
 *             self.channel_number_val = NULL             # <<<<<<<<<<<<<<
 *             self.median_before_val = -1.0
 *             self.read_number_val = -1
 */
    __pyx_v_self->channel_number_val = NULL;

    /* "pyslow5.pyx":1795
 *             free(self.channel_number_val)
 *             self.channel_number_val = NULL
 *             self.median_before_val = -1.0             # <<<<<<<<<<<<<<
 *             self.read_number_val = -1
 *             self.start_mux_val = -1
 */
    __pyx_v_self->median_before_val = -1.0;

    /* "pyslow5.pyx":1796
 *             self.channel_number_val = NULL
 *             self.median_before_val = -1.0
 *             self.read_number_val = -1             # <<<<<<<<<<<<<<
 *             self.start_mux_val = -1
 *             self.start_time_val = -1
 */
    __pyx_v_self->read_number_val = -1;

    /* "pyslow5.pyx":1797
 *             self.median_before_val = -1.0
 *             self.read_number_val = -1
 *             self.start_mux_val = -1             # <<<<<<<<<<<<<<
 *             self.start_time_val = -1
 * 
 */
    __pyx_v_self->start_mux_val = -1;

    /* "pyslow5.pyx":1798
 *             self.read_number_val = -1
 *             self.start_mux_val = -1
 *             self.start_time_val = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_self->start_time_val = -1LL;

    /* "pyslow5.pyx":1792
 *         self.logger.debug("write_record: slow5_write() ret: {}".format(ret))
 * 
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             free(self.channel_number_val)
 *             self.channel_number_val = NULL
 */
  }

  /* "pyslow5.pyx":1802
 * 
 *         # free memory
 *         self.logger.debug("write_record: slow5_rec_free()")             # <<<<<<<<<<<<<<
 *         slow5_rec_free(self.write)
 *         self.write = NULL
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_slow5_rec_free) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_slow5_rec_free);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1803
 *         # free memory
 *         self.logger.debug("write_record: slow5_rec_free()")
 *         slow5_rec_free(self.write)             # <<<<<<<<<<<<<<
 *         self.write = NULL
 * 
 */
  slow5_rec_free(__pyx_v_self->write);

  /* "pyslow5.pyx":1804
 *         self.logger.debug("write_record: slow5_rec_free()")
 *         slow5_rec_free(self.write)
 *         self.write = NULL             # <<<<<<<<<<<<<<
 * 
 *         self.logger.debug("write_record: function complete, returning 0")
 */
  __pyx_v_self->write = NULL;

  /* "pyslow5.pyx":1806
 *         self.write = NULL
 * 
 *         self.logger.debug("write_record: function complete, returning 0")             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_u_write_record_function_complete_r) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_function_complete_r);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyslow5.pyx":1807
 * 
 *         self.logger.debug("write_record: function complete, returning 0")
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_0);
  __pyx_r = __pyx_int_0;
  goto __pyx_L0;

  /* "pyslow5.pyx":1662
 *             return -1
 * 
 *     def write_record(self, record, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         write a single record
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyslow5.Open.write_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_aux_types);
  __Pyx_XDECREF(__pyx_v_checked_record);
  __Pyx_XDECREF(__pyx_v_checked_aux);
  __Pyx_XDECREF(__pyx_v_slow5_aux_types);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_start_write_copy_signal);
  __Pyx_XDECREF(__pyx_v_memview);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_end_write_copy_signal);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":1810
 * 
 * 
 *     def write_record_batch(self, records, threads=4, batchsize=4096, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         write a batch of records
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_63write_record_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7pyslow5_4Open_62write_record_batch[] = "\n        write a batch of records\n        same as write_record, but in batches\n        records = dic of read rics, where the key for each is the readID\n        if aux, aux is also a dic of dics, whhere the key for each is teh readID\n        records = {readID_0: {read_dic}, readID_1: {read_dic}}\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_63write_record_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_records = 0;
  PyObject *__pyx_v_threads = 0;
  PyObject *__pyx_v_batchsize = 0;
  PyObject *__pyx_v_aux = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_record_batch (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_records,&__pyx_n_s_threads,&__pyx_n_s_batchsize,&__pyx_n_s_aux,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)__pyx_int_4);
    values[2] = ((PyObject *)__pyx_int_4096);
    values[3] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_records)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_threads);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_batchsize);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aux);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "write_record_batch") < 0)) __PYX_ERR(0, 1810, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_records = values[0];
    __pyx_v_threads = values[1];
    __pyx_v_batchsize = values[2];
    __pyx_v_aux = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_record_batch", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1810, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyslow5.Open.write_record_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyslow5_4Open_62write_record_batch(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), __pyx_v_records, __pyx_v_threads, __pyx_v_batchsize, __pyx_v_aux);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_62write_record_batch(struct __pyx_obj_7pyslow5_Open *__pyx_v_self, PyObject *__pyx_v_records, PyObject *__pyx_v_threads, PyObject *__pyx_v_batchsize, PyObject *__pyx_v_aux) {
  PyObject *__pyx_v_start_multi_write = NULL;
  PyObject *__pyx_v_aux_types = NULL;
  Py_ssize_t __pyx_v_num_reads;
  PyObject *__pyx_v_read_list = NULL;
  PyObject *__pyx_v_batches = NULL;
  PyObject *__pyx_v_batch = NULL;
  PyObject *__pyx_v_batch_len = NULL;
  PyObject *__pyx_v_checked_records = NULL;
  PyObject *__pyx_v_checked_auxs = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_idx = NULL;
  PyObject *__pyx_v_checked_record = NULL;
  PyObject *__pyx_v_checked_aux = NULL;
  PyObject *__pyx_v_a = NULL;
  int __pyx_v_error;
  PyObject *__pyx_v_slow5_aux_types = NULL;
  long __pyx_v_ret;
  PyObject *__pyx_v_start_write_copy_signal = NULL;
  PyObject *__pyx_v_memview = NULL;
  PyObject *__pyx_v_end_write_copy_signal = NULL;
  PyObject *__pyx_v_end_multi_write = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  Py_ssize_t __pyx_t_9;
  size_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *(*__pyx_t_18)(PyObject *);
  Py_ssize_t __pyx_t_19;
  PyObject *(*__pyx_t_20)(PyObject *);
  char const *__pyx_t_21;
  Py_ssize_t __pyx_t_22;
  double __pyx_t_23;
  int32_t __pyx_t_24;
  uint8_t __pyx_t_25;
  uint64_t __pyx_t_26;
  char const *__pyx_t_27;
  enum slow5_aux_type __pyx_t_28;
  PyObject *__pyx_t_29 = NULL;
  uint32_t __pyx_t_30;
  int16_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  int __pyx_t_33;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_record_batch", 0);
  __Pyx_TraceCall("write_record_batch", __pyx_f[0], 1810, 0, __PYX_ERR(0, 1810, __pyx_L1_error));
  __Pyx_INCREF(__pyx_v_aux);

  /* "pyslow5.pyx":1818
 *         records = {readID_0: {read_dic}, readID_1: {read_dic}}
 *         '''
 *         start_multi_write = time.time()             # <<<<<<<<<<<<<<
 *         aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_start_multi_write = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1819
 *         '''
 *         start_multi_write = time.time()
 *         aux_types = {"channel_number": type("string"),             # <<<<<<<<<<<<<<
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_channel_number, ((PyObject *)Py_TYPE(__pyx_n_u_string))) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)

  /* "pyslow5.pyx":1820
 *         start_multi_write = time.time()
 *         aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),             # <<<<<<<<<<<<<<
 *                      "read_number": type(10),
 *                      "start_mux": type(1),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_median_before, ((PyObject *)Py_TYPE(__pyx_float_1_0))) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)

  /* "pyslow5.pyx":1821
 *         aux_types = {"channel_number": type("string"),
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),             # <<<<<<<<<<<<<<
 *                      "start_mux": type(1),
 *                      "start_time": type(100),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_read_number, ((PyObject *)Py_TYPE(__pyx_int_10))) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)

  /* "pyslow5.pyx":1822
 *                      "median_before": type(1.0),
 *                      "read_number": type(10),
 *                      "start_mux": type(1),             # <<<<<<<<<<<<<<
 *                      "start_time": type(100),
 *                      "end_reason": None}
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_mux, ((PyObject *)Py_TYPE(__pyx_int_1))) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)

  /* "pyslow5.pyx":1823
 *                      "read_number": type(10),
 *                      "start_mux": type(1),
 *                      "start_time": type(100),             # <<<<<<<<<<<<<<
 *                      "end_reason": None}
 *         # check an empty dic wasn't given
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_start_time, ((PyObject *)Py_TYPE(__pyx_int_100))) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)

  /* "pyslow5.pyx":1824
 *                      "start_mux": type(1),
 *                      "start_time": type(100),
 *                      "end_reason": None}             # <<<<<<<<<<<<<<
 *         # check an empty dic wasn't given
 *         if aux is not None:
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_end_reason, Py_None) < 0) __PYX_ERR(0, 1819, __pyx_L1_error)
  __pyx_v_aux_types = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyslow5.pyx":1826
 *                      "end_reason": None}
 *         # check an empty dic wasn't given
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             if len(aux) == 0:
 *                 aux = None
 */
  __pyx_t_4 = (__pyx_v_aux != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "pyslow5.pyx":1827
 *         # check an empty dic wasn't given
 *         if aux is not None:
 *             if len(aux) == 0:             # <<<<<<<<<<<<<<
 *                 aux = None
 *         self.logger.debug("write_record_batch: Setting up batching...")
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_aux); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1827, __pyx_L1_error)
    __pyx_t_5 = ((__pyx_t_6 == 0) != 0);
    if (__pyx_t_5) {

      /* "pyslow5.pyx":1828
 *         if aux is not None:
 *             if len(aux) == 0:
 *                 aux = None             # <<<<<<<<<<<<<<
 *         self.logger.debug("write_record_batch: Setting up batching...")
 *         num_reads = len(records)
 */
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_aux, Py_None);

      /* "pyslow5.pyx":1827
 *         # check an empty dic wasn't given
 *         if aux is not None:
 *             if len(aux) == 0:             # <<<<<<<<<<<<<<
 *                 aux = None
 *         self.logger.debug("write_record_batch: Setting up batching...")
 */
    }

    /* "pyslow5.pyx":1826
 *                      "end_reason": None}
 *         # check an empty dic wasn't given
 *         if aux is not None:             # <<<<<<<<<<<<<<
 *             if len(aux) == 0:
 *                 aux = None
 */
  }

  /* "pyslow5.pyx":1829
 *             if len(aux) == 0:
 *                 aux = None
 *         self.logger.debug("write_record_batch: Setting up batching...")             # <<<<<<<<<<<<<<
 *         num_reads = len(records)
 *         read_list = list(records.keys())
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1829, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_write_record_batch_Setting_up_ba) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_write_record_batch_Setting_up_ba);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1829, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1830
 *                 aux = None
 *         self.logger.debug("write_record_batch: Setting up batching...")
 *         num_reads = len(records)             # <<<<<<<<<<<<<<
 *         read_list = list(records.keys())
 *         if num_reads > batchsize:
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_records); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1830, __pyx_L1_error)
  __pyx_v_num_reads = __pyx_t_6;

  /* "pyslow5.pyx":1831
 *         self.logger.debug("write_record_batch: Setting up batching...")
 *         num_reads = len(records)
 *         read_list = list(records.keys())             # <<<<<<<<<<<<<<
 *         if num_reads > batchsize:
 *             batches = self._get_batches(read_list, size=batchsize)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_records, __pyx_n_s_keys); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_read_list = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyslow5.pyx":1832
 *         num_reads = len(records)
 *         read_list = list(records.keys())
 *         if num_reads > batchsize:             # <<<<<<<<<<<<<<
 *             batches = self._get_batches(read_list, size=batchsize)
 *         else:
 */
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_num_reads); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1832, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_3, __pyx_v_batchsize, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1832, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 1832, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "pyslow5.pyx":1833
 *         read_list = list(records.keys())
 *         if num_reads > batchsize:
 *             batches = self._get_batches(read_list, size=batchsize)             # <<<<<<<<<<<<<<
 *         else:
 *             batches = self._get_batches(read_list, size=num_reads)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_batches); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1833, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1833, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_read_list);
    __Pyx_GIVEREF(__pyx_v_read_list);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_read_list);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1833, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_size, __pyx_v_batchsize) < 0) __PYX_ERR(0, 1833, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1833, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_batches = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "pyslow5.pyx":1832
 *         num_reads = len(records)
 *         read_list = list(records.keys())
 *         if num_reads > batchsize:             # <<<<<<<<<<<<<<
 *             batches = self._get_batches(read_list, size=batchsize)
 *         else:
 */
    goto __pyx_L5;
  }

  /* "pyslow5.pyx":1835
 *             batches = self._get_batches(read_list, size=batchsize)
 *         else:
 *             batches = self._get_batches(read_list, size=num_reads)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_batches); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_read_list);
    __Pyx_GIVEREF(__pyx_v_read_list);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_read_list);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_num_reads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_size, __pyx_t_1) < 0) __PYX_ERR(0, 1835, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_batches = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L5:;

  /* "pyslow5.pyx":1838
 * 
 * 
 *         self.logger.debug("write_record_batch: batch for loop start")             # <<<<<<<<<<<<<<
 *         for batch in chain(batches):
 *             batch_len = len(batch)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_write_record_batch_batch_for_loo) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_write_record_batch_batch_for_loo);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":1839
 * 
 *         self.logger.debug("write_record_batch: batch for loop start")
 *         for batch in chain(batches):             # <<<<<<<<<<<<<<
 *             batch_len = len(batch)
 *             self.twrite = <slow5_rec_t **> malloc(sizeof(slow5_rec_t*)*batch_len)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_chain); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_batches) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_batches);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_6 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1839, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1839, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1839, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1839, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 1839, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1839, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1839, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_batch, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1840
 *         self.logger.debug("write_record_batch: batch for loop start")
 *         for batch in chain(batches):
 *             batch_len = len(batch)             # <<<<<<<<<<<<<<
 *             self.twrite = <slow5_rec_t **> malloc(sizeof(slow5_rec_t*)*batch_len)
 *             self.logger.debug("write_record_batch: _record_type_validation running")
 */
    __pyx_t_9 = PyObject_Length(__pyx_v_batch); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1840, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1840, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_batch_len, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyslow5.pyx":1841
 *         for batch in chain(batches):
 *             batch_len = len(batch)
 *             self.twrite = <slow5_rec_t **> malloc(sizeof(slow5_rec_t*)*batch_len)             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record_batch: _record_type_validation running")
 *             checked_records = {}
 */
    __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(slow5_rec_t *))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1841, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_v_batch_len); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1841, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1841, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->twrite = ((slow5_rec_t **)malloc(__pyx_t_10));

    /* "pyslow5.pyx":1842
 *             batch_len = len(batch)
 *             self.twrite = <slow5_rec_t **> malloc(sizeof(slow5_rec_t*)*batch_len)
 *             self.logger.debug("write_record_batch: _record_type_validation running")             # <<<<<<<<<<<<<<
 *             checked_records = {}
 *             checked_auxs = {}
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1842, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_7, __pyx_kp_u_write_record_batch__record_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_write_record_batch__record_type);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1842, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":1843
 *             self.twrite = <slow5_rec_t **> malloc(sizeof(slow5_rec_t*)*batch_len)
 *             self.logger.debug("write_record_batch: _record_type_validation running")
 *             checked_records = {}             # <<<<<<<<<<<<<<
 *             checked_auxs = {}
 *             self.channel_number_val_array = <char **> malloc(sizeof(char*)*batch_len)
 */
    __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1843, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_checked_records, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1844
 *             self.logger.debug("write_record_batch: _record_type_validation running")
 *             checked_records = {}
 *             checked_auxs = {}             # <<<<<<<<<<<<<<
 *             self.channel_number_val_array = <char **> malloc(sizeof(char*)*batch_len)
 *             self.median_before_val_array = <double *> malloc(sizeof(double)*batch_len)
 */
    __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_checked_auxs, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pyslow5.pyx":1845
 *             checked_records = {}
 *             checked_auxs = {}
 *             self.channel_number_val_array = <char **> malloc(sizeof(char*)*batch_len)             # <<<<<<<<<<<<<<
 *             self.median_before_val_array = <double *> malloc(sizeof(double)*batch_len)
 *             self.read_number_val_array = <int32_t *> malloc(sizeof(int32_t)*batch_len)
 */
    __pyx_t_2 = __Pyx_PyInt_FromSize_t((sizeof(char *))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1845, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Multiply(__pyx_t_2, __pyx_v_batch_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1845, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1845, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->channel_number_val_array = ((char **)malloc(__pyx_t_10));

    /* "pyslow5.pyx":1846
 *             checked_auxs = {}
 *             self.channel_number_val_array = <char **> malloc(sizeof(char*)*batch_len)
 *             self.median_before_val_array = <double *> malloc(sizeof(double)*batch_len)             # <<<<<<<<<<<<<<
 *             self.read_number_val_array = <int32_t *> malloc(sizeof(int32_t)*batch_len)
 *             self.start_mux_val_array = <uint8_t *> malloc(sizeof(uint8_t)*batch_len)
 */
    __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(double))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1846, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_v_batch_len); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1846, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1846, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->median_before_val_array = ((double *)malloc(__pyx_t_10));

    /* "pyslow5.pyx":1847
 *             self.channel_number_val_array = <char **> malloc(sizeof(char*)*batch_len)
 *             self.median_before_val_array = <double *> malloc(sizeof(double)*batch_len)
 *             self.read_number_val_array = <int32_t *> malloc(sizeof(int32_t)*batch_len)             # <<<<<<<<<<<<<<
 *             self.start_mux_val_array = <uint8_t *> malloc(sizeof(uint8_t)*batch_len)
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)
 */
    __pyx_t_2 = __Pyx_PyInt_FromSize_t((sizeof(int32_t))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Multiply(__pyx_t_2, __pyx_v_batch_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1847, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->read_number_val_array = ((int32_t *)malloc(__pyx_t_10));

    /* "pyslow5.pyx":1848
 *             self.median_before_val_array = <double *> malloc(sizeof(double)*batch_len)
 *             self.read_number_val_array = <int32_t *> malloc(sizeof(int32_t)*batch_len)
 *             self.start_mux_val_array = <uint8_t *> malloc(sizeof(uint8_t)*batch_len)             # <<<<<<<<<<<<<<
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)
 *             for i, idx in enumerate(batch):
 */
    __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(uint8_t))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1848, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_v_batch_len); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1848, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1848, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->start_mux_val_array = ((uint8_t *)malloc(__pyx_t_10));

    /* "pyslow5.pyx":1849
 *             self.read_number_val_array = <int32_t *> malloc(sizeof(int32_t)*batch_len)
 *             self.start_mux_val_array = <uint8_t *> malloc(sizeof(uint8_t)*batch_len)
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)             # <<<<<<<<<<<<<<
 *             for i, idx in enumerate(batch):
 *                 if aux is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_FromSize_t((sizeof(uint64_t))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1849, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Multiply(__pyx_t_2, __pyx_v_batch_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1849, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1849, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->start_time_val_array = ((uint64_t *)malloc(__pyx_t_10));

    /* "pyslow5.pyx":1850
 *             self.start_mux_val_array = <uint8_t *> malloc(sizeof(uint8_t)*batch_len)
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)
 *             for i, idx in enumerate(batch):             # <<<<<<<<<<<<<<
 *                 if aux is not None:
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_1 = __pyx_int_0;
    if (likely(PyList_CheckExact(__pyx_v_batch)) || PyTuple_CheckExact(__pyx_v_batch)) {
      __pyx_t_2 = __pyx_v_batch; __Pyx_INCREF(__pyx_t_2); __pyx_t_9 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_batch); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1850, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1850, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_7); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1850, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1850, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_7); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1850, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1850, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_11(__pyx_t_2);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1850, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
      __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1850, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1);
      __pyx_t_1 = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "pyslow5.pyx":1851
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)
 *             for i, idx in enumerate(batch):
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 *                     checked_records[idx] = checked_record
 */
      __pyx_t_5 = (__pyx_v_aux != Py_None);
      __pyx_t_4 = (__pyx_t_5 != 0);
      if (__pyx_t_4) {

        /* "pyslow5.pyx":1852
 *             for i, idx in enumerate(batch):
 *                 if aux is not None:
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])             # <<<<<<<<<<<<<<
 *                     checked_records[idx] = checked_record
 *                     for a in checked_aux:
 */
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_multi_record_type_validation); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1852, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_records, __pyx_v_idx); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1852, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_aux, __pyx_v_idx); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1852, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = NULL;
        __pyx_t_16 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_12);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_12, function);
            __pyx_t_16 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_12)) {
          PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_t_13, __pyx_t_14};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
          PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_t_13, __pyx_t_14};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        } else
        #endif
        {
          __pyx_t_17 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_15); __pyx_t_15 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_13);
          PyTuple_SET_ITEM(__pyx_t_17, 0+__pyx_t_16, __pyx_t_13);
          __Pyx_GIVEREF(__pyx_t_14);
          PyTuple_SET_ITEM(__pyx_t_17, 1+__pyx_t_16, __pyx_t_14);
          __pyx_t_13 = 0;
          __pyx_t_14 = 0;
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_17, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        }
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
          PyObject* sequence = __pyx_t_7;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 1852, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_17);
          #else
          __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          #endif
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_14 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1852, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_18 = Py_TYPE(__pyx_t_14)->tp_iternext;
          index = 0; __pyx_t_12 = __pyx_t_18(__pyx_t_14); if (unlikely(!__pyx_t_12)) goto __pyx_L11_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_12);
          index = 1; __pyx_t_17 = __pyx_t_18(__pyx_t_14); if (unlikely(!__pyx_t_17)) goto __pyx_L11_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_17);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_18(__pyx_t_14), 2) < 0) __PYX_ERR(0, 1852, __pyx_L1_error)
          __pyx_t_18 = NULL;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          goto __pyx_L12_unpacking_done;
          __pyx_L11_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_18 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 1852, __pyx_L1_error)
          __pyx_L12_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_checked_record, __pyx_t_12);
        __pyx_t_12 = 0;
        __Pyx_XDECREF_SET(__pyx_v_checked_aux, __pyx_t_17);
        __pyx_t_17 = 0;

        /* "pyslow5.pyx":1853
 *                 if aux is not None:
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 *                     checked_records[idx] = checked_record             # <<<<<<<<<<<<<<
 *                     for a in checked_aux:
 *                         if a == "channel_number":
 */
        if (unlikely(PyDict_SetItem(__pyx_v_checked_records, __pyx_v_idx, __pyx_v_checked_record) < 0)) __PYX_ERR(0, 1853, __pyx_L1_error)

        /* "pyslow5.pyx":1854
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 *                     checked_records[idx] = checked_record
 *                     for a in checked_aux:             # <<<<<<<<<<<<<<
 *                         if a == "channel_number":
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 */
        if (likely(PyList_CheckExact(__pyx_v_checked_aux)) || PyTuple_CheckExact(__pyx_v_checked_aux)) {
          __pyx_t_7 = __pyx_v_checked_aux; __Pyx_INCREF(__pyx_t_7); __pyx_t_19 = 0;
          __pyx_t_20 = NULL;
        } else {
          __pyx_t_19 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_v_checked_aux); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1854, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_20 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1854, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_20)) {
            if (likely(PyList_CheckExact(__pyx_t_7))) {
              if (__pyx_t_19 >= PyList_GET_SIZE(__pyx_t_7)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_17 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_19); __Pyx_INCREF(__pyx_t_17); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 1854, __pyx_L1_error)
              #else
              __pyx_t_17 = PySequence_ITEM(__pyx_t_7, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1854, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              #endif
            } else {
              if (__pyx_t_19 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_17 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_19); __Pyx_INCREF(__pyx_t_17); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 1854, __pyx_L1_error)
              #else
              __pyx_t_17 = PySequence_ITEM(__pyx_t_7, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1854, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              #endif
            }
          } else {
            __pyx_t_17 = __pyx_t_20(__pyx_t_7);
            if (unlikely(!__pyx_t_17)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 1854, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_17);
          }
          __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_17);
          __pyx_t_17 = 0;

          /* "pyslow5.pyx":1855
 *                     checked_records[idx] = checked_record
 *                     for a in checked_aux:
 *                         if a == "channel_number":             # <<<<<<<<<<<<<<
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 *                         elif a == "median_before":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_channel_number, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1855, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1856
 *                     for a in checked_aux:
 *                         if a == "channel_number":
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())             # <<<<<<<<<<<<<<
 *                         elif a == "median_before":
 *                             self.median_before_val_array[i] = <double>checked_aux[a]
 */
            __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1856, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_encode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1856, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_12 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_12)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
              }
            }
            __pyx_t_17 = (__pyx_t_12) ? __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_12) : __Pyx_PyObject_CallNoArg(__pyx_t_14);
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1856, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_21 = __Pyx_PyObject_AsString(__pyx_t_17); if (unlikely((!__pyx_t_21) && PyErr_Occurred())) __PYX_ERR(0, 1856, __pyx_L1_error)
            __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1856, __pyx_L1_error)
            (__pyx_v_self->channel_number_val_array[__pyx_t_22]) = strdup(__pyx_t_21);
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

            /* "pyslow5.pyx":1855
 *                     checked_records[idx] = checked_record
 *                     for a in checked_aux:
 *                         if a == "channel_number":             # <<<<<<<<<<<<<<
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 *                         elif a == "median_before":
 */
            goto __pyx_L15;
          }

          /* "pyslow5.pyx":1857
 *                         if a == "channel_number":
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 *                         elif a == "median_before":             # <<<<<<<<<<<<<<
 *                             self.median_before_val_array[i] = <double>checked_aux[a]
 *                         elif a == "read_number":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_median_before, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1857, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1858
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 *                         elif a == "median_before":
 *                             self.median_before_val_array[i] = <double>checked_aux[a]             # <<<<<<<<<<<<<<
 *                         elif a == "read_number":
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]
 */
            __pyx_t_17 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1858, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_23 = __pyx_PyFloat_AsDouble(__pyx_t_17); if (unlikely((__pyx_t_23 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1858, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1858, __pyx_L1_error)
            (__pyx_v_self->median_before_val_array[__pyx_t_22]) = ((double)__pyx_t_23);

            /* "pyslow5.pyx":1857
 *                         if a == "channel_number":
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 *                         elif a == "median_before":             # <<<<<<<<<<<<<<
 *                             self.median_before_val_array[i] = <double>checked_aux[a]
 *                         elif a == "read_number":
 */
            goto __pyx_L15;
          }

          /* "pyslow5.pyx":1859
 *                         elif a == "median_before":
 *                             self.median_before_val_array[i] = <double>checked_aux[a]
 *                         elif a == "read_number":             # <<<<<<<<<<<<<<
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]
 *                         elif a == "start_mux":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_read_number, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1859, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1860
 *                             self.median_before_val_array[i] = <double>checked_aux[a]
 *                         elif a == "read_number":
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]             # <<<<<<<<<<<<<<
 *                         elif a == "start_mux":
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]
 */
            __pyx_t_17 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1860, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_24 = __Pyx_PyInt_As_int32_t(__pyx_t_17); if (unlikely((__pyx_t_24 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1860, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1860, __pyx_L1_error)
            (__pyx_v_self->read_number_val_array[__pyx_t_22]) = ((int32_t)__pyx_t_24);

            /* "pyslow5.pyx":1859
 *                         elif a == "median_before":
 *                             self.median_before_val_array[i] = <double>checked_aux[a]
 *                         elif a == "read_number":             # <<<<<<<<<<<<<<
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]
 *                         elif a == "start_mux":
 */
            goto __pyx_L15;
          }

          /* "pyslow5.pyx":1861
 *                         elif a == "read_number":
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]
 *                         elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]
 *                         elif a == "start_time":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_mux, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1861, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1862
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]
 *                         elif a == "start_mux":
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]             # <<<<<<<<<<<<<<
 *                         elif a == "start_time":
 *                             self.start_time_val_array[i] = <uint64_t>checked_aux[a]
 */
            __pyx_t_17 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1862, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_25 = __Pyx_PyInt_As_uint8_t(__pyx_t_17); if (unlikely((__pyx_t_25 == ((uint8_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1862, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1862, __pyx_L1_error)
            (__pyx_v_self->start_mux_val_array[__pyx_t_22]) = ((uint8_t)__pyx_t_25);

            /* "pyslow5.pyx":1861
 *                         elif a == "read_number":
 *                             self.read_number_val_array[i] = <int32_t>checked_aux[a]
 *                         elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]
 *                         elif a == "start_time":
 */
            goto __pyx_L15;
          }

          /* "pyslow5.pyx":1863
 *                         elif a == "start_mux":
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]
 *                         elif a == "start_time":             # <<<<<<<<<<<<<<
 *                             self.start_time_val_array[i] = <uint64_t>checked_aux[a]
 *                     checked_auxs[idx] = checked_aux
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_time, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1863, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1864
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]
 *                         elif a == "start_time":
 *                             self.start_time_val_array[i] = <uint64_t>checked_aux[a]             # <<<<<<<<<<<<<<
 *                     checked_auxs[idx] = checked_aux
 *                 else:
 */
            __pyx_t_17 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1864, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_26 = __Pyx_PyInt_As_uint64_t(__pyx_t_17); if (unlikely((__pyx_t_26 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1864, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1864, __pyx_L1_error)
            (__pyx_v_self->start_time_val_array[__pyx_t_22]) = ((uint64_t)__pyx_t_26);

            /* "pyslow5.pyx":1863
 *                         elif a == "start_mux":
 *                             self.start_mux_val_array[i] = <uint8_t>checked_aux[a]
 *                         elif a == "start_time":             # <<<<<<<<<<<<<<
 *                             self.start_time_val_array[i] = <uint64_t>checked_aux[a]
 *                     checked_auxs[idx] = checked_aux
 */
          }
          __pyx_L15:;

          /* "pyslow5.pyx":1854
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 *                     checked_records[idx] = checked_record
 *                     for a in checked_aux:             # <<<<<<<<<<<<<<
 *                         if a == "channel_number":
 *                             self.channel_number_val_array[i] = strdup(checked_aux[a].encode())
 */
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyslow5.pyx":1865
 *                         elif a == "start_time":
 *                             self.start_time_val_array[i] = <uint64_t>checked_aux[a]
 *                     checked_auxs[idx] = checked_aux             # <<<<<<<<<<<<<<
 *                 else:
 *                     checked_record, checked_aux = self._record_type_validation(records[idx], aux)
 */
        if (unlikely(PyDict_SetItem(__pyx_v_checked_auxs, __pyx_v_idx, __pyx_v_checked_aux) < 0)) __PYX_ERR(0, 1865, __pyx_L1_error)

        /* "pyslow5.pyx":1851
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)
 *             for i, idx in enumerate(batch):
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 *                     checked_records[idx] = checked_record
 */
        goto __pyx_L10;
      }

      /* "pyslow5.pyx":1867
 *                     checked_auxs[idx] = checked_aux
 *                 else:
 *                     checked_record, checked_aux = self._record_type_validation(records[idx], aux)             # <<<<<<<<<<<<<<
 *                     checked_records[idx] = checked_record
 *             self.logger.debug("write_record_batch: _record_type_validation done")
 */
      /*else*/ {
        __pyx_t_17 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_record_type_validation); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1867, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_records, __pyx_v_idx); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1867, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_12 = NULL;
        __pyx_t_16 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_17))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_17);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_17, function);
            __pyx_t_16 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_17)) {
          PyObject *__pyx_temp[3] = {__pyx_t_12, __pyx_t_14, __pyx_v_aux};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_17, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_17)) {
          PyObject *__pyx_temp[3] = {__pyx_t_12, __pyx_t_14, __pyx_v_aux};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_17, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_14);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_16, __pyx_t_14);
          __Pyx_INCREF(__pyx_v_aux);
          __Pyx_GIVEREF(__pyx_v_aux);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_16, __pyx_v_aux);
          __pyx_t_14 = 0;
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_17, __pyx_t_13, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
          PyObject* sequence = __pyx_t_7;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 1867, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_17 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_13 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_17 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_13 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_17);
          __Pyx_INCREF(__pyx_t_13);
          #else
          __pyx_t_17 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_13 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          #endif
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_14 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1867, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_18 = Py_TYPE(__pyx_t_14)->tp_iternext;
          index = 0; __pyx_t_17 = __pyx_t_18(__pyx_t_14); if (unlikely(!__pyx_t_17)) goto __pyx_L16_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_17);
          index = 1; __pyx_t_13 = __pyx_t_18(__pyx_t_14); if (unlikely(!__pyx_t_13)) goto __pyx_L16_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_13);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_18(__pyx_t_14), 2) < 0) __PYX_ERR(0, 1867, __pyx_L1_error)
          __pyx_t_18 = NULL;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          goto __pyx_L17_unpacking_done;
          __pyx_L16_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_18 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 1867, __pyx_L1_error)
          __pyx_L17_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_checked_record, __pyx_t_17);
        __pyx_t_17 = 0;
        __Pyx_XDECREF_SET(__pyx_v_checked_aux, __pyx_t_13);
        __pyx_t_13 = 0;

        /* "pyslow5.pyx":1868
 *                 else:
 *                     checked_record, checked_aux = self._record_type_validation(records[idx], aux)
 *                     checked_records[idx] = checked_record             # <<<<<<<<<<<<<<
 *             self.logger.debug("write_record_batch: _record_type_validation done")
 *             if len(checked_aux) == 0:
 */
        if (unlikely(PyDict_SetItem(__pyx_v_checked_records, __pyx_v_idx, __pyx_v_checked_record) < 0)) __PYX_ERR(0, 1868, __pyx_L1_error)
      }
      __pyx_L10:;

      /* "pyslow5.pyx":1850
 *             self.start_mux_val_array = <uint8_t *> malloc(sizeof(uint8_t)*batch_len)
 *             self.start_time_val_array = <uint64_t *> malloc(sizeof(uint64_t)*batch_len)
 *             for i, idx in enumerate(batch):             # <<<<<<<<<<<<<<
 *                 if aux is not None:
 *                     checked_record, checked_aux = self._multi_record_type_validation(records[idx], aux[idx])
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1869
 *                     checked_record, checked_aux = self._record_type_validation(records[idx], aux)
 *                     checked_records[idx] = checked_record
 *             self.logger.debug("write_record_batch: _record_type_validation done")             # <<<<<<<<<<<<<<
 *             if len(checked_aux) == 0:
 *                 checked_aux = None
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1869, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_7, __pyx_kp_u_write_record_batch__record_type_2) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_batch__record_type_2);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1869, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyslow5.pyx":1870
 *                     checked_records[idx] = checked_record
 *             self.logger.debug("write_record_batch: _record_type_validation done")
 *             if len(checked_aux) == 0:             # <<<<<<<<<<<<<<
 *                 checked_aux = None
 *             for idx in range(batch_len):
 */
    if (unlikely(!__pyx_v_checked_aux)) { __Pyx_RaiseUnboundLocalError("checked_aux"); __PYX_ERR(0, 1870, __pyx_L1_error) }
    __pyx_t_9 = PyObject_Length(__pyx_v_checked_aux); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1870, __pyx_L1_error)
    __pyx_t_4 = ((__pyx_t_9 == 0) != 0);
    if (__pyx_t_4) {

      /* "pyslow5.pyx":1871
 *             self.logger.debug("write_record_batch: _record_type_validation done")
 *             if len(checked_aux) == 0:
 *                 checked_aux = None             # <<<<<<<<<<<<<<
 *             for idx in range(batch_len):
 * 
 */
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_checked_aux, Py_None);

      /* "pyslow5.pyx":1870
 *                     checked_records[idx] = checked_record
 *             self.logger.debug("write_record_batch: _record_type_validation done")
 *             if len(checked_aux) == 0:             # <<<<<<<<<<<<<<
 *                 checked_aux = None
 *             for idx in range(batch_len):
 */
    }

    /* "pyslow5.pyx":1872
 *             if len(checked_aux) == 0:
 *                 checked_aux = None
 *             for idx in range(batch_len):             # <<<<<<<<<<<<<<
 * 
 *                 # check if in append state
 */
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_batch_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1872, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_9 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1872, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_1); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1872, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1872, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_1); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1872, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1872, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_11(__pyx_t_2);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1872, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "pyslow5.pyx":1876
 *                 # check if in append state
 *                 # if so, we can skip header setup
 *                 if self.state == 2:             # <<<<<<<<<<<<<<
 *                     self.header_state = True
 * 
 */
      __pyx_t_4 = ((__pyx_v_self->state == 2) != 0);
      if (__pyx_t_4) {

        /* "pyslow5.pyx":1877
 *                 # if so, we can skip header setup
 *                 if self.state == 2:
 *                     self.header_state = True             # <<<<<<<<<<<<<<
 * 
 *                 # if all checks are good, test self.header_state for 1 time write
 */
        __pyx_v_self->header_state = 1;

        /* "pyslow5.pyx":1876
 *                 # check if in append state
 *                 # if so, we can skip header setup
 *                 if self.state == 2:             # <<<<<<<<<<<<<<
 *                     self.header_state = True
 * 
 */
      }

      /* "pyslow5.pyx":1880
 * 
 *                 # if all checks are good, test self.header_state for 1 time write
 *                 if not self.header_state:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record_batch: checking header stuff...")
 *                     error = False
 */
      __pyx_t_4 = ((!(__pyx_v_self->header_state != 0)) != 0);
      if (__pyx_t_4) {

        /* "pyslow5.pyx":1881
 *                 # if all checks are good, test self.header_state for 1 time write
 *                 if not self.header_state:
 *                     self.logger.debug("write_record_batch: checking header stuff...")             # <<<<<<<<<<<<<<
 *                     error = False
 *                     if aux is not None:
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1881, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_checking_head) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_checking_head);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1881, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1882
 *                 if not self.header_state:
 *                     self.logger.debug("write_record_batch: checking header stuff...")
 *                     error = False             # <<<<<<<<<<<<<<
 *                     if aux is not None:
 *                         if checked_aux is not None:
 */
        __pyx_v_error = 0;

        /* "pyslow5.pyx":1883
 *                     self.logger.debug("write_record_batch: checking header stuff...")
 *                     error = False
 *                     if aux is not None:             # <<<<<<<<<<<<<<
 *                         if checked_aux is not None:
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 */
        __pyx_t_4 = (__pyx_v_aux != Py_None);
        __pyx_t_5 = (__pyx_t_4 != 0);
        if (__pyx_t_5) {

          /* "pyslow5.pyx":1884
 *                     error = False
 *                     if aux is not None:
 *                         if checked_aux is not None:             # <<<<<<<<<<<<<<
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                             for a in slow5_aux_types:
 */
          if (unlikely(!__pyx_v_checked_aux)) { __Pyx_RaiseUnboundLocalError("checked_aux"); __PYX_ERR(0, 1884, __pyx_L1_error) }
          __pyx_t_5 = (__pyx_v_checked_aux != Py_None);
          __pyx_t_4 = (__pyx_t_5 != 0);
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1885
 *                     if aux is not None:
 *                         if checked_aux is not None:
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)             # <<<<<<<<<<<<<<
 *                             for a in slow5_aux_types:
 *                                 if slow5_aux_types[a] is None:
 */
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_aux_header_type_validation); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1885, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_13 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
              __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
              if (likely(__pyx_t_13)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                __Pyx_INCREF(__pyx_t_13);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_7, function);
              }
            }
            __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_v_aux_types) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_aux_types);
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1885, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XDECREF_SET(__pyx_v_slow5_aux_types, __pyx_t_1);
            __pyx_t_1 = 0;

            /* "pyslow5.pyx":1886
 *                         if checked_aux is not None:
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                             for a in slow5_aux_types:             # <<<<<<<<<<<<<<
 *                                 if slow5_aux_types[a] is None:
 *                                     continue
 */
            if (likely(PyList_CheckExact(__pyx_v_slow5_aux_types)) || PyTuple_CheckExact(__pyx_v_slow5_aux_types)) {
              __pyx_t_1 = __pyx_v_slow5_aux_types; __Pyx_INCREF(__pyx_t_1); __pyx_t_19 = 0;
              __pyx_t_20 = NULL;
            } else {
              __pyx_t_19 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_slow5_aux_types); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1886, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_20 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1886, __pyx_L1_error)
            }
            for (;;) {
              if (likely(!__pyx_t_20)) {
                if (likely(PyList_CheckExact(__pyx_t_1))) {
                  if (__pyx_t_19 >= PyList_GET_SIZE(__pyx_t_1)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_7 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_19); __Pyx_INCREF(__pyx_t_7); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 1886, __pyx_L1_error)
                  #else
                  __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1886, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_7);
                  #endif
                } else {
                  if (__pyx_t_19 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_19); __Pyx_INCREF(__pyx_t_7); __pyx_t_19++; if (unlikely(0 < 0)) __PYX_ERR(0, 1886, __pyx_L1_error)
                  #else
                  __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_19); __pyx_t_19++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1886, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_7);
                  #endif
                }
              } else {
                __pyx_t_7 = __pyx_t_20(__pyx_t_1);
                if (unlikely(!__pyx_t_7)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(0, 1886, __pyx_L1_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_7);
              }
              __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_7);
              __pyx_t_7 = 0;

              /* "pyslow5.pyx":1887
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                             for a in slow5_aux_types:
 *                                 if slow5_aux_types[a] is None:             # <<<<<<<<<<<<<<
 *                                     continue
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 */
              __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_slow5_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1887, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_4 = (__pyx_t_7 == Py_None);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_5 = (__pyx_t_4 != 0);
              if (__pyx_t_5) {

                /* "pyslow5.pyx":1888
 *                             for a in slow5_aux_types:
 *                                 if slow5_aux_types[a] is None:
 *                                     continue             # <<<<<<<<<<<<<<
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                                 if ret < 0:
 */
                goto __pyx_L25_continue;

                /* "pyslow5.pyx":1887
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                             for a in slow5_aux_types:
 *                                 if slow5_aux_types[a] is None:             # <<<<<<<<<<<<<<
 *                                     continue
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 */
              }

              /* "pyslow5.pyx":1889
 *                                 if slow5_aux_types[a] is None:
 *                                     continue
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)             # <<<<<<<<<<<<<<
 *                                 if ret < 0:
 *                                     self.logger.error("write_record_batch: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 */
              __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_n_s_encode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1889, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_13);
              __pyx_t_17 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
                __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_13);
                if (likely(__pyx_t_17)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
                  __Pyx_INCREF(__pyx_t_17);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_13, function);
                }
              }
              __pyx_t_7 = (__pyx_t_17) ? __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_17) : __Pyx_PyObject_CallNoArg(__pyx_t_13);
              __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
              if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1889, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              __pyx_t_27 = __Pyx_PyObject_AsString(__pyx_t_7); if (unlikely((!__pyx_t_27) && PyErr_Occurred())) __PYX_ERR(0, 1889, __pyx_L1_error)
              __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_slow5_aux_types, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1889, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_13);
              __pyx_t_28 = ((enum slow5_aux_type)__Pyx_PyInt_As_enum__slow5_aux_type(__pyx_t_13)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1889, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              __pyx_v_ret = slow5_aux_add(__pyx_t_27, __pyx_t_28, __pyx_v_self->s5->header);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

              /* "pyslow5.pyx":1890
 *                                     continue
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                                 if ret < 0:             # <<<<<<<<<<<<<<
 *                                     self.logger.error("write_record_batch: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 *                                     error = True
 */
              __pyx_t_5 = ((__pyx_v_ret < 0) != 0);
              if (__pyx_t_5) {

                /* "pyslow5.pyx":1891
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                                 if ret < 0:
 *                                     self.logger.error("write_record_batch: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                                     error = True
 *                         else:
 */
                __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1891, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_13);
                __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_slow5_aux_add, __pyx_n_s_format); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1891, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_14);
                if (unlikely(!__pyx_v_checked_aux)) { __Pyx_RaiseUnboundLocalError("checked_aux"); __PYX_ERR(0, 1891, __pyx_L1_error) }
                __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1891, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_12);
                __pyx_t_15 = NULL;
                __pyx_t_16 = 0;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_14, function);
                    __pyx_t_16 = 1;
                  }
                }
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_14)) {
                  PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_12};
                  __pyx_t_17 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1891, __pyx_L1_error)
                  __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                  __Pyx_GOTREF(__pyx_t_17);
                  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
                  PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_12};
                  __pyx_t_17 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1891, __pyx_L1_error)
                  __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                  __Pyx_GOTREF(__pyx_t_17);
                  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                } else
                #endif
                {
                  __pyx_t_29 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1891, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_29);
                  if (__pyx_t_15) {
                    __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_15); __pyx_t_15 = NULL;
                  }
                  __Pyx_INCREF(__pyx_v_a);
                  __Pyx_GIVEREF(__pyx_v_a);
                  PyTuple_SET_ITEM(__pyx_t_29, 0+__pyx_t_16, __pyx_v_a);
                  __Pyx_GIVEREF(__pyx_t_12);
                  PyTuple_SET_ITEM(__pyx_t_29, 1+__pyx_t_16, __pyx_t_12);
                  __pyx_t_12 = 0;
                  __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_29, NULL); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1891, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_17);
                  __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
                }
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __pyx_t_14 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
                  __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);
                  if (likely(__pyx_t_14)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
                    __Pyx_INCREF(__pyx_t_14);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_13, function);
                  }
                }
                __pyx_t_7 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_14, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_17);
                __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1891, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "pyslow5.pyx":1892
 *                                 if ret < 0:
 *                                     self.logger.error("write_record_batch: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 *                                     error = True             # <<<<<<<<<<<<<<
 *                         else:
 *                             error = True
 */
                __pyx_v_error = 1;

                /* "pyslow5.pyx":1890
 *                                     continue
 *                                 ret = slow5_aux_add(a.encode(), slow5_aux_types[a], self.s5.header)
 *                                 if ret < 0:             # <<<<<<<<<<<<<<
 *                                     self.logger.error("write_record_batch: slow5_aux_add {}: {} could not set to C s5.header.aux_meta struct".format(a, checked_aux[a]))
 *                                     error = True
 */
              }

              /* "pyslow5.pyx":1886
 *                         if checked_aux is not None:
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                             for a in slow5_aux_types:             # <<<<<<<<<<<<<<
 *                                 if slow5_aux_types[a] is None:
 *                                     continue
 */
              __pyx_L25_continue:;
            }
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyslow5.pyx":1884
 *                     error = False
 *                     if aux is not None:
 *                         if checked_aux is not None:             # <<<<<<<<<<<<<<
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 *                             for a in slow5_aux_types:
 */
            goto __pyx_L24;
          }

          /* "pyslow5.pyx":1894
 *                                     error = True
 *                         else:
 *                             error = True             # <<<<<<<<<<<<<<
 * 
 *                     if error:
 */
          /*else*/ {
            __pyx_v_error = 1;
          }
          __pyx_L24:;

          /* "pyslow5.pyx":1883
 *                     self.logger.debug("write_record_batch: checking header stuff...")
 *                     error = False
 *                     if aux is not None:             # <<<<<<<<<<<<<<
 *                         if checked_aux is not None:
 *                             slow5_aux_types = self._aux_header_type_validation(aux_types)
 */
        }

        /* "pyslow5.pyx":1896
 *                             error = True
 * 
 *                     if error:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record_batch: aux_meta fields failed to initialise")
 *                         return -1
 */
        __pyx_t_5 = (__pyx_v_error != 0);
        if (__pyx_t_5) {

          /* "pyslow5.pyx":1897
 * 
 *                     if error:
 *                         self.logger.error("write_record_batch: aux_meta fields failed to initialise")             # <<<<<<<<<<<<<<
 *                         return -1
 *                     # write the header
 */
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1897, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_13 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_aux_meta_fiel) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_aux_meta_fiel);
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1897, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pyslow5.pyx":1898
 *                     if error:
 *                         self.logger.error("write_record_batch: aux_meta fields failed to initialise")
 *                         return -1             # <<<<<<<<<<<<<<
 *                     # write the header
 *                     self.logger.debug("write_record_batch: writting header...")
 */
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_int_neg_1);
          __pyx_r = __pyx_int_neg_1;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L0;

          /* "pyslow5.pyx":1896
 *                             error = True
 * 
 *                     if error:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record_batch: aux_meta fields failed to initialise")
 *                         return -1
 */
        }

        /* "pyslow5.pyx":1900
 *                         return -1
 *                     # write the header
 *                     self.logger.debug("write_record_batch: writting header...")             # <<<<<<<<<<<<<<
 *                     ret = slow5_hdr_write(self.s5)
 *                     if ret < 0:
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1900, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_writting_head) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_writting_head);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1900, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1901
 *                     # write the header
 *                     self.logger.debug("write_record_batch: writting header...")
 *                     ret = slow5_hdr_write(self.s5)             # <<<<<<<<<<<<<<
 *                     if ret < 0:
 *                         self.logger.error("write_record_batch: slow5_hdr_write could not write header")
 */
        __pyx_v_ret = slow5_hdr_write(__pyx_v_self->s5);

        /* "pyslow5.pyx":1902
 *                     self.logger.debug("write_record_batch: writting header...")
 *                     ret = slow5_hdr_write(self.s5)
 *                     if ret < 0:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record_batch: slow5_hdr_write could not write header")
 *                         return -1
 */
        __pyx_t_5 = ((__pyx_v_ret < 0) != 0);
        if (__pyx_t_5) {

          /* "pyslow5.pyx":1903
 *                     ret = slow5_hdr_write(self.s5)
 *                     if ret < 0:
 *                         self.logger.error("write_record_batch: slow5_hdr_write could not write header")             # <<<<<<<<<<<<<<
 *                         return -1
 *                     # set state true so only done once
 */
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1903, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_13 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_slow5_hdr_wri) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_slow5_hdr_wri);
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1903, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pyslow5.pyx":1904
 *                     if ret < 0:
 *                         self.logger.error("write_record_batch: slow5_hdr_write could not write header")
 *                         return -1             # <<<<<<<<<<<<<<
 *                     # set state true so only done once
 *                     self.header_state = True
 */
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_int_neg_1);
          __pyx_r = __pyx_int_neg_1;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L0;

          /* "pyslow5.pyx":1902
 *                     self.logger.debug("write_record_batch: writting header...")
 *                     ret = slow5_hdr_write(self.s5)
 *                     if ret < 0:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record_batch: slow5_hdr_write could not write header")
 *                         return -1
 */
        }

        /* "pyslow5.pyx":1906
 *                         return -1
 *                     # set state true so only done once
 *                     self.header_state = True             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record_batch: header written")
 * 
 */
        __pyx_v_self->header_state = 1;

        /* "pyslow5.pyx":1907
 *                     # set state true so only done once
 *                     self.header_state = True
 *                     self.logger.debug("write_record_batch: header written")             # <<<<<<<<<<<<<<
 * 
 *                 # Add values to read struct
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1907, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_header_writte) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_header_writte);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1907, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1880
 * 
 *                 # if all checks are good, test self.header_state for 1 time write
 *                 if not self.header_state:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record_batch: checking header stuff...")
 *                     error = False
 */
      }

      /* "pyslow5.pyx":1910
 * 
 *                 # Add values to read struct
 *                 self.logger.debug("write_record_batch: slow5_rec_init()")             # <<<<<<<<<<<<<<
 * 
 *                 self.twrite[idx] = slow5_rec_init()
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1910, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_slow5_rec_ini) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_slow5_rec_ini);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1910, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1912
 *                 self.logger.debug("write_record_batch: slow5_rec_init()")
 * 
 *                 self.twrite[idx] = slow5_rec_init()             # <<<<<<<<<<<<<<
 *                 if self.twrite[idx] == NULL:
 *                     self.logger.error("write_record_batch: failed to allocate space for slow5 record (self.twrite[idx])")
 */
      __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1912, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_19]) = slow5_rec_init();

      /* "pyslow5.pyx":1913
 * 
 *                 self.twrite[idx] = slow5_rec_init()
 *                 if self.twrite[idx] == NULL:             # <<<<<<<<<<<<<<
 *                     self.logger.error("write_record_batch: failed to allocate space for slow5 record (self.twrite[idx])")
 *                     return -1
 */
      __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1913, __pyx_L1_error)
      __pyx_t_5 = (((__pyx_v_self->twrite[__pyx_t_19]) == NULL) != 0);
      if (__pyx_t_5) {

        /* "pyslow5.pyx":1914
 *                 self.twrite[idx] = slow5_rec_init()
 *                 if self.twrite[idx] == NULL:
 *                     self.logger.error("write_record_batch: failed to allocate space for slow5 record (self.twrite[idx])")             # <<<<<<<<<<<<<<
 *                     return -1
 * 
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1914, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_failed_to_all) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_failed_to_all);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1914, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyslow5.pyx":1915
 *                 if self.twrite[idx] == NULL:
 *                     self.logger.error("write_record_batch: failed to allocate space for slow5 record (self.twrite[idx])")
 *                     return -1             # <<<<<<<<<<<<<<
 * 
 *                 self.logger.debug("write_record_batch: self.write assignments...")
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_int_neg_1);
        __pyx_r = __pyx_int_neg_1;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L0;

        /* "pyslow5.pyx":1913
 * 
 *                 self.twrite[idx] = slow5_rec_init()
 *                 if self.twrite[idx] == NULL:             # <<<<<<<<<<<<<<
 *                     self.logger.error("write_record_batch: failed to allocate space for slow5 record (self.twrite[idx])")
 *                     return -1
 */
      }

      /* "pyslow5.pyx":1917
 *                     return -1
 * 
 *                 self.logger.debug("write_record_batch: self.write assignments...")             # <<<<<<<<<<<<<<
 *                 checked_records[batch[idx]]["read_id"] = checked_records[batch[idx]]["read_id"].encode()
 *                 self.twrite[idx].read_id = strdup(checked_records[batch[idx]]["read_id"])
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1917, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_self_write_as) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_self_write_as);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1917, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1918
 * 
 *                 self.logger.debug("write_record_batch: self.write assignments...")
 *                 checked_records[batch[idx]]["read_id"] = checked_records[batch[idx]]["read_id"].encode()             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].read_id = strdup(checked_records[batch[idx]]["read_id"])
 *                 self.twrite[idx].read_id_len = len(checked_records[batch[idx]]["read_id"])
 */
      __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_t_13, __pyx_n_u_read_id); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_encode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_13);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_13, function);
        }
      }
      __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_7, __pyx_n_u_read_id, __pyx_t_1) < 0)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1919
 *                 self.logger.debug("write_record_batch: self.write assignments...")
 *                 checked_records[batch[idx]]["read_id"] = checked_records[batch[idx]]["read_id"].encode()
 *                 self.twrite[idx].read_id = strdup(checked_records[batch[idx]]["read_id"])             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].read_id_len = len(checked_records[batch[idx]]["read_id"])
 *                 self.twrite[idx].read_group = checked_records[batch[idx]]["read_group"]
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1919, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1919, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_read_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1919, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_21 = __Pyx_PyObject_AsString(__pyx_t_1); if (unlikely((!__pyx_t_21) && PyErr_Occurred())) __PYX_ERR(0, 1919, __pyx_L1_error)
      __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1919, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_19])->read_id = strdup(__pyx_t_21);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1920
 *                 checked_records[batch[idx]]["read_id"] = checked_records[batch[idx]]["read_id"].encode()
 *                 self.twrite[idx].read_id = strdup(checked_records[batch[idx]]["read_id"])
 *                 self.twrite[idx].read_id_len = len(checked_records[batch[idx]]["read_id"])             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].read_group = checked_records[batch[idx]]["read_group"]
 *                 self.twrite[idx].digitisation = checked_records[batch[idx]]["digitisation"]
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1920, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1920, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_read_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1920, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_19 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_19 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1920, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1920, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->read_id_len = __pyx_t_19;

      /* "pyslow5.pyx":1921
 *                 self.twrite[idx].read_id = strdup(checked_records[batch[idx]]["read_id"])
 *                 self.twrite[idx].read_id_len = len(checked_records[batch[idx]]["read_id"])
 *                 self.twrite[idx].read_group = checked_records[batch[idx]]["read_group"]             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].digitisation = checked_records[batch[idx]]["digitisation"]
 *                 self.twrite[idx].offset = checked_records[batch[idx]]["offset"]
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1921, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1921, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_read_group); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1921, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_30 = __Pyx_PyInt_As_uint32_t(__pyx_t_1); if (unlikely((__pyx_t_30 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1921, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1921, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->read_group = __pyx_t_30;

      /* "pyslow5.pyx":1922
 *                 self.twrite[idx].read_id_len = len(checked_records[batch[idx]]["read_id"])
 *                 self.twrite[idx].read_group = checked_records[batch[idx]]["read_group"]
 *                 self.twrite[idx].digitisation = checked_records[batch[idx]]["digitisation"]             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].offset = checked_records[batch[idx]]["offset"]
 *                 self.twrite[idx].range = checked_records[batch[idx]]["range"]
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1922, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1922, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_digitisation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1922, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_23 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_23 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1922, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1922, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->digitisation = __pyx_t_23;

      /* "pyslow5.pyx":1923
 *                 self.twrite[idx].read_group = checked_records[batch[idx]]["read_group"]
 *                 self.twrite[idx].digitisation = checked_records[batch[idx]]["digitisation"]
 *                 self.twrite[idx].offset = checked_records[batch[idx]]["offset"]             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].range = checked_records[batch[idx]]["range"]
 *                 self.twrite[idx].sampling_rate = checked_records[batch[idx]]["sampling_rate"]
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1923, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1923, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1923, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_23 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_23 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1923, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1923, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->offset = __pyx_t_23;

      /* "pyslow5.pyx":1924
 *                 self.twrite[idx].digitisation = checked_records[batch[idx]]["digitisation"]
 *                 self.twrite[idx].offset = checked_records[batch[idx]]["offset"]
 *                 self.twrite[idx].range = checked_records[batch[idx]]["range"]             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].sampling_rate = checked_records[batch[idx]]["sampling_rate"]
 *                 self.twrite[idx].len_raw_signal = checked_records[batch[idx]]["len_raw_signal"]
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1924, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1924, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1924, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_23 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_23 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1924, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1924, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->range = __pyx_t_23;

      /* "pyslow5.pyx":1925
 *                 self.twrite[idx].offset = checked_records[batch[idx]]["offset"]
 *                 self.twrite[idx].range = checked_records[batch[idx]]["range"]
 *                 self.twrite[idx].sampling_rate = checked_records[batch[idx]]["sampling_rate"]             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].len_raw_signal = checked_records[batch[idx]]["len_raw_signal"]
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc...")
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1925, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1925, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_sampling_rate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1925, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_23 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_23 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1925, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1925, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->sampling_rate = __pyx_t_23;

      /* "pyslow5.pyx":1926
 *                 self.twrite[idx].range = checked_records[batch[idx]]["range"]
 *                 self.twrite[idx].sampling_rate = checked_records[batch[idx]]["sampling_rate"]
 *                 self.twrite[idx].len_raw_signal = checked_records[batch[idx]]["len_raw_signal"]             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc...")
 *                 self.twrite[idx].raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_records[batch[idx]]["len_raw_signal"])
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1926, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1926, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_len_raw_signal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1926, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_26 = __Pyx_PyInt_As_uint64_t(__pyx_t_1); if (unlikely((__pyx_t_26 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1926, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1926, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->len_raw_signal = __pyx_t_26;

      /* "pyslow5.pyx":1927
 *                 self.twrite[idx].sampling_rate = checked_records[batch[idx]]["sampling_rate"]
 *                 self.twrite[idx].len_raw_signal = checked_records[batch[idx]]["len_raw_signal"]
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc...")             # <<<<<<<<<<<<<<
 *                 self.twrite[idx].raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_records[batch[idx]]["len_raw_signal"])
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc done")
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1927, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_self_write_ra) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_self_write_ra);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1927, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyslow5.pyx":1928
 *                 self.twrite[idx].len_raw_signal = checked_records[batch[idx]]["len_raw_signal"]
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc...")
 *                 self.twrite[idx].raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_records[batch[idx]]["len_raw_signal"])             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc done")
 *                 self.logger.debug("write_record_batch: self.write assignments done")
 */
      __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(int16_t))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_t_13, __pyx_n_u_len_raw_signal); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = PyNumber_Multiply(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_13); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1928, __pyx_L1_error)
      (__pyx_v_self->twrite[__pyx_t_22])->raw_signal = ((int16_t *)malloc(__pyx_t_10));

      /* "pyslow5.pyx":1929
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc...")
 *                 self.twrite[idx].raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_records[batch[idx]]["len_raw_signal"])
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc done")             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record_batch: self.write assignments done")
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1929, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_13 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_kp_u_write_record_batch_self_write_ra_2) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_self_write_ra_2);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1929, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

      /* "pyslow5.pyx":1930
 *                 self.twrite[idx].raw_signal = <int16_t *> malloc(sizeof(int16_t)*checked_records[batch[idx]]["len_raw_signal"])
 *                 self.logger.debug("write_record_batch: self.write.raw_signal malloc done")
 *                 self.logger.debug("write_record_batch: self.write assignments done")             # <<<<<<<<<<<<<<
 * 
 *                 self.logger.debug("write_record_batch: self.write processing raw_signal")
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1930, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_13 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_kp_u_write_record_batch_self_write_as_2) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_self_write_as_2);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1930, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

      /* "pyslow5.pyx":1932
 *                 self.logger.debug("write_record_batch: self.write assignments done")
 * 
 *                 self.logger.debug("write_record_batch: self.write processing raw_signal")             # <<<<<<<<<<<<<<
 *                 start_write_copy_signal = time.time()
 *                 # grabs buffer of numby array so the for loop operats in C not python = super fast
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1932, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_13 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_kp_u_write_record_batch_self_write_pr) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_self_write_pr);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1932, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

      /* "pyslow5.pyx":1933
 * 
 *                 self.logger.debug("write_record_batch: self.write processing raw_signal")
 *                 start_write_copy_signal = time.time()             # <<<<<<<<<<<<<<
 *                 # grabs buffer of numby array so the for loop operats in C not python = super fast
 *                 memview = memoryview(checked_records[batch[idx]]["signal"])
 */
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1933, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1933, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_13 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1933, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_start_write_copy_signal, __pyx_t_13);
      __pyx_t_13 = 0;

      /* "pyslow5.pyx":1935
 *                 start_write_copy_signal = time.time()
 *                 # grabs buffer of numby array so the for loop operats in C not python = super fast
 *                 memview = memoryview(checked_records[batch[idx]]["signal"])             # <<<<<<<<<<<<<<
 *                 for i in range(checked_records[batch[idx]]["len_raw_signal"]):
 *                     self.twrite[idx].raw_signal[i] = memview[i]
 */
      __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_memoryview); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1935, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1935, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1935, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_n_u_signal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1935, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1935, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_memview, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "pyslow5.pyx":1936
 *                 # grabs buffer of numby array so the for loop operats in C not python = super fast
 *                 memview = memoryview(checked_records[batch[idx]]["signal"])
 *                 for i in range(checked_records[batch[idx]]["len_raw_signal"]):             # <<<<<<<<<<<<<<
 *                     self.twrite[idx].raw_signal[i] = memview[i]
 *                 # for i in range(checked_records[batch[idx]]["len_raw_signal"]):
 */
      __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_checked_records, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_len_raw_signal); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
        __pyx_t_7 = __pyx_t_1; __Pyx_INCREF(__pyx_t_7); __pyx_t_22 = 0;
        __pyx_t_20 = NULL;
      } else {
        __pyx_t_22 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1936, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_20 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1936, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      for (;;) {
        if (likely(!__pyx_t_20)) {
          if (likely(PyList_CheckExact(__pyx_t_7))) {
            if (__pyx_t_22 >= PyList_GET_SIZE(__pyx_t_7)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_1 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_22); __Pyx_INCREF(__pyx_t_1); __pyx_t_22++; if (unlikely(0 < 0)) __PYX_ERR(0, 1936, __pyx_L1_error)
            #else
            __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_22); __pyx_t_22++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1936, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            #endif
          } else {
            if (__pyx_t_22 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_22); __Pyx_INCREF(__pyx_t_1); __pyx_t_22++; if (unlikely(0 < 0)) __PYX_ERR(0, 1936, __pyx_L1_error)
            #else
            __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_22); __pyx_t_22++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1936, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            #endif
          }
        } else {
          __pyx_t_1 = __pyx_t_20(__pyx_t_7);
          if (unlikely(!__pyx_t_1)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 1936, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_1);
        }
        __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
        __pyx_t_1 = 0;

        /* "pyslow5.pyx":1937
 *                 memview = memoryview(checked_records[batch[idx]]["signal"])
 *                 for i in range(checked_records[batch[idx]]["len_raw_signal"]):
 *                     self.twrite[idx].raw_signal[i] = memview[i]             # <<<<<<<<<<<<<<
 *                 # for i in range(checked_records[batch[idx]]["len_raw_signal"]):
 *                 #     self.twrite[idx].raw_signal[i] = checked_records[batch[idx]]["signal"][i]
 */
        __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_memview, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1937, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_31 = __Pyx_PyInt_As_int16_t(__pyx_t_1); if (unlikely((__pyx_t_31 == ((int16_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1937, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1937, __pyx_L1_error)
        __pyx_t_32 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_32 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1937, __pyx_L1_error)
        ((__pyx_v_self->twrite[__pyx_t_19])->raw_signal[__pyx_t_32]) = __pyx_t_31;

        /* "pyslow5.pyx":1936
 *                 # grabs buffer of numby array so the for loop operats in C not python = super fast
 *                 memview = memoryview(checked_records[batch[idx]]["signal"])
 *                 for i in range(checked_records[batch[idx]]["len_raw_signal"]):             # <<<<<<<<<<<<<<
 *                     self.twrite[idx].raw_signal[i] = memview[i]
 *                 # for i in range(checked_records[batch[idx]]["len_raw_signal"]):
 */
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyslow5.pyx":1940
 *                 # for i in range(checked_records[batch[idx]]["len_raw_signal"]):
 *                 #     self.twrite[idx].raw_signal[i] = checked_records[batch[idx]]["signal"][i]
 *                 end_write_copy_signal = (time.time() - start_write_copy_signal)             # <<<<<<<<<<<<<<
 *                 self.total_multi_write_signal_time = self.total_multi_write_signal_time + end_write_copy_signal
 * 
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1940, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_time); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1940, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_13);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_13, function);
        }
      }
      __pyx_t_7 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1940, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = PyNumber_Subtract(__pyx_t_7, __pyx_v_start_write_copy_signal); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1940, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_end_write_copy_signal, __pyx_t_13);
      __pyx_t_13 = 0;

      /* "pyslow5.pyx":1941
 *                 #     self.twrite[idx].raw_signal[i] = checked_records[batch[idx]]["signal"][i]
 *                 end_write_copy_signal = (time.time() - start_write_copy_signal)
 *                 self.total_multi_write_signal_time = self.total_multi_write_signal_time + end_write_copy_signal             # <<<<<<<<<<<<<<
 * 
 *                 self.logger.debug("write_record_batch: self.write raw_signal done")
 */
      __pyx_t_13 = PyNumber_Add(__pyx_v_self->total_multi_write_signal_time, __pyx_v_end_write_copy_signal); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1941, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      if (!(likely(PyFloat_CheckExact(__pyx_t_13))||((__pyx_t_13) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_13)->tp_name), 0))) __PYX_ERR(0, 1941, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_13);
      __Pyx_GOTREF(__pyx_v_self->total_multi_write_signal_time);
      __Pyx_DECREF(__pyx_v_self->total_multi_write_signal_time);
      __pyx_v_self->total_multi_write_signal_time = ((PyObject*)__pyx_t_13);
      __pyx_t_13 = 0;

      /* "pyslow5.pyx":1943
 *                 self.total_multi_write_signal_time = self.total_multi_write_signal_time + end_write_copy_signal
 * 
 *                 self.logger.debug("write_record_batch: self.write raw_signal done")             # <<<<<<<<<<<<<<
 * 
 *                 if aux is not None:
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1943, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_13 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_kp_u_write_record_batch_self_write_ra_3) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_self_write_ra_3);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1943, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

      /* "pyslow5.pyx":1945
 *                 self.logger.debug("write_record_batch: self.write raw_signal done")
 * 
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record_batch: aux stuff...")
 *                     if checked_auxs[batch[idx]] is None:
 */
      __pyx_t_5 = (__pyx_v_aux != Py_None);
      __pyx_t_4 = (__pyx_t_5 != 0);
      if (__pyx_t_4) {

        /* "pyslow5.pyx":1946
 * 
 *                 if aux is not None:
 *                     self.logger.debug("write_record_batch: aux stuff...")             # <<<<<<<<<<<<<<
 *                     if checked_auxs[batch[idx]] is None:
 *                         self.logger.error("write_record_batch: checked_aux is None".format(a, checked_auxs[batch[idx]][a]))
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1946, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_13 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_kp_u_write_record_batch_aux_stuff) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_aux_stuff);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1946, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

        /* "pyslow5.pyx":1947
 *                 if aux is not None:
 *                     self.logger.debug("write_record_batch: aux stuff...")
 *                     if checked_auxs[batch[idx]] is None:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record_batch: checked_aux is None".format(a, checked_auxs[batch[idx]][a]))
 *                         return -1
 */
        __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1947, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1947, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_4 = (__pyx_t_7 == Py_None);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_5 = (__pyx_t_4 != 0);
        if (__pyx_t_5) {

          /* "pyslow5.pyx":1948
 *                     self.logger.debug("write_record_batch: aux stuff...")
 *                     if checked_auxs[batch[idx]] is None:
 *                         self.logger.error("write_record_batch: checked_aux is None".format(a, checked_auxs[batch[idx]][a]))             # <<<<<<<<<<<<<<
 *                         return -1
 * 
 */
          __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_checked_aux_i, __pyx_n_s_format); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          if (unlikely(!__pyx_v_a)) { __Pyx_RaiseUnboundLocalError("a"); __PYX_ERR(0, 1948, __pyx_L1_error) }
          __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_29 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_14); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_29);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_v_a)) { __Pyx_RaiseUnboundLocalError("a"); __PYX_ERR(0, 1948, __pyx_L1_error) }
          __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_t_29, __pyx_v_a); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
          __pyx_t_29 = NULL;
          __pyx_t_16 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_17))) {
            __pyx_t_29 = PyMethod_GET_SELF(__pyx_t_17);
            if (likely(__pyx_t_29)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
              __Pyx_INCREF(__pyx_t_29);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_17, function);
              __pyx_t_16 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_17)) {
            PyObject *__pyx_temp[3] = {__pyx_t_29, __pyx_v_a, __pyx_t_14};
            __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_17, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1948, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_17)) {
            PyObject *__pyx_temp[3] = {__pyx_t_29, __pyx_v_a, __pyx_t_14};
            __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_17, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1948, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          } else
          #endif
          {
            __pyx_t_12 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1948, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            if (__pyx_t_29) {
              __Pyx_GIVEREF(__pyx_t_29); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_29); __pyx_t_29 = NULL;
            }
            __Pyx_INCREF(__pyx_v_a);
            __Pyx_GIVEREF(__pyx_v_a);
            PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_16, __pyx_v_a);
            __Pyx_GIVEREF(__pyx_t_14);
            PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_16, __pyx_t_14);
            __pyx_t_14 = 0;
            __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_17, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1948, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          }
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          __pyx_t_17 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
            __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_13);
            if (likely(__pyx_t_17)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
              __Pyx_INCREF(__pyx_t_17);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_13, function);
            }
          }
          __pyx_t_7 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_17, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_1);
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1948, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

          /* "pyslow5.pyx":1949
 *                     if checked_auxs[batch[idx]] is None:
 *                         self.logger.error("write_record_batch: checked_aux is None".format(a, checked_auxs[batch[idx]][a]))
 *                         return -1             # <<<<<<<<<<<<<<
 * 
 *                     for a in checked_auxs[batch[idx]]:
 */
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_int_neg_1);
          __pyx_r = __pyx_int_neg_1;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L0;

          /* "pyslow5.pyx":1947
 *                 if aux is not None:
 *                     self.logger.debug("write_record_batch: aux stuff...")
 *                     if checked_auxs[batch[idx]] is None:             # <<<<<<<<<<<<<<
 *                         self.logger.error("write_record_batch: checked_aux is None".format(a, checked_auxs[batch[idx]][a]))
 *                         return -1
 */
        }

        /* "pyslow5.pyx":1951
 *                         return -1
 * 
 *                     for a in checked_auxs[batch[idx]]:             # <<<<<<<<<<<<<<
 *                         self.logger.debug("write_record_batch: checked_aux: {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                         if checked_auxs[batch[idx]][a] is None:
 */
        __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1951, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (likely(PyList_CheckExact(__pyx_t_13)) || PyTuple_CheckExact(__pyx_t_13)) {
          __pyx_t_7 = __pyx_t_13; __Pyx_INCREF(__pyx_t_7); __pyx_t_22 = 0;
          __pyx_t_20 = NULL;
        } else {
          __pyx_t_22 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_13); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1951, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_20 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1951, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        for (;;) {
          if (likely(!__pyx_t_20)) {
            if (likely(PyList_CheckExact(__pyx_t_7))) {
              if (__pyx_t_22 >= PyList_GET_SIZE(__pyx_t_7)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_13 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_22); __Pyx_INCREF(__pyx_t_13); __pyx_t_22++; if (unlikely(0 < 0)) __PYX_ERR(0, 1951, __pyx_L1_error)
              #else
              __pyx_t_13 = PySequence_ITEM(__pyx_t_7, __pyx_t_22); __pyx_t_22++; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1951, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_13);
              #endif
            } else {
              if (__pyx_t_22 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_13 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_22); __Pyx_INCREF(__pyx_t_13); __pyx_t_22++; if (unlikely(0 < 0)) __PYX_ERR(0, 1951, __pyx_L1_error)
              #else
              __pyx_t_13 = PySequence_ITEM(__pyx_t_7, __pyx_t_22); __pyx_t_22++; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1951, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_13);
              #endif
            }
          } else {
            __pyx_t_13 = __pyx_t_20(__pyx_t_7);
            if (unlikely(!__pyx_t_13)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 1951, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_13);
          }
          __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_13);
          __pyx_t_13 = 0;

          /* "pyslow5.pyx":1952
 * 
 *                     for a in checked_auxs[batch[idx]]:
 *                         self.logger.debug("write_record_batch: checked_aux: {}: {}".format(a, checked_auxs[batch[idx]][a]))             # <<<<<<<<<<<<<<
 *                         if checked_auxs[batch[idx]][a] is None:
 *                             continue
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_checked_aux, __pyx_n_s_format); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_29 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_14); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_29);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_t_29, __pyx_v_a); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
          __pyx_t_29 = NULL;
          __pyx_t_16 = 0;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_29 = PyMethod_GET_SELF(__pyx_t_12);
            if (likely(__pyx_t_29)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
              __Pyx_INCREF(__pyx_t_29);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_12, function);
              __pyx_t_16 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_12)) {
            PyObject *__pyx_temp[3] = {__pyx_t_29, __pyx_v_a, __pyx_t_14};
            __pyx_t_17 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1952, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
            PyObject *__pyx_temp[3] = {__pyx_t_29, __pyx_v_a, __pyx_t_14};
            __pyx_t_17 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1952, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          } else
          #endif
          {
            __pyx_t_15 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1952, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_15);
            if (__pyx_t_29) {
              __Pyx_GIVEREF(__pyx_t_29); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_29); __pyx_t_29 = NULL;
            }
            __Pyx_INCREF(__pyx_v_a);
            __Pyx_GIVEREF(__pyx_v_a);
            PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_16, __pyx_v_a);
            __Pyx_GIVEREF(__pyx_t_14);
            PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_16, __pyx_t_14);
            __pyx_t_14 = 0;
            __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_15, NULL); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1952, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          }
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __pyx_t_12 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_12)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_12);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
            }
          }
          __pyx_t_13 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_12, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_17);
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

          /* "pyslow5.pyx":1953
 *                     for a in checked_auxs[batch[idx]]:
 *                         self.logger.debug("write_record_batch: checked_aux: {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                         if checked_auxs[batch[idx]][a] is None:             # <<<<<<<<<<<<<<
 *                             continue
 *                         if a == "channel_number":
 */
          __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1953, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1953, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_a); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1953, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_5 = (__pyx_t_13 == Py_None);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __pyx_t_4 = (__pyx_t_5 != 0);
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1954
 *                         self.logger.debug("write_record_batch: checked_aux: {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                         if checked_auxs[batch[idx]][a] is None:
 *                             continue             # <<<<<<<<<<<<<<
 *                         if a == "channel_number":
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running...")
 */
            goto __pyx_L36_continue;

            /* "pyslow5.pyx":1953
 *                     for a in checked_auxs[batch[idx]]:
 *                         self.logger.debug("write_record_batch: checked_aux: {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                         if checked_auxs[batch[idx]][a] is None:             # <<<<<<<<<<<<<<
 *                             continue
 *                         if a == "channel_number":
 */
          }

          /* "pyslow5.pyx":1955
 *                         if checked_auxs[batch[idx]][a] is None:
 *                             continue
 *                         if a == "channel_number":             # <<<<<<<<<<<<<<
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running...")
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string type: {}".format(type(checked_auxs[batch[idx]][a])))
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_channel_number, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1955, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1956
 *                             continue
 *                         if a == "channel_number":
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running...")             # <<<<<<<<<<<<<<
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string type: {}".format(type(checked_auxs[batch[idx]][a])))
 *                             ret = slow5_aux_set_string(self.twrite[idx], self.channel_number, <const char *>self.channel_number_val_array[idx], self.s5.header)
 */
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1956, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_17 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_17)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_17);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            __pyx_t_13 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_17, __pyx_kp_u_write_record_batch_slow5_aux_set) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_write_record_batch_slow5_aux_set);
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1956, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

            /* "pyslow5.pyx":1957
 *                         if a == "channel_number":
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running...")
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string type: {}".format(type(checked_auxs[batch[idx]][a])))             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set_string(self.twrite[idx], self.channel_number, <const char *>self.channel_number_val_array[idx], self.s5.header)
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running done: ret = {}".format(ret))
 */
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_slow5_aux_set_2, __pyx_n_s_format); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_14 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_t_14, __pyx_v_a); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_15);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_14 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
              __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_12);
              if (likely(__pyx_t_14)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
                __Pyx_INCREF(__pyx_t_14);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_12, function);
              }
            }
            __pyx_t_17 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_14, ((PyObject *)Py_TYPE(__pyx_t_15))) : __Pyx_PyObject_CallOneArg(__pyx_t_12, ((PyObject *)Py_TYPE(__pyx_t_15)));
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_12 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_12)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            __pyx_t_13 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_12, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_17);
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1957, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

            /* "pyslow5.pyx":1958
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running...")
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string type: {}".format(type(checked_auxs[batch[idx]][a])))
 *                             ret = slow5_aux_set_string(self.twrite[idx], self.channel_number, <const char *>self.channel_number_val_array[idx], self.s5.header)             # <<<<<<<<<<<<<<
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running done: ret = {}".format(ret))
 *                             if ret < 0:
 */
            __pyx_t_32 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_32 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1958, __pyx_L1_error)
            __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1958, __pyx_L1_error)
            __pyx_v_ret = slow5_aux_set_string((__pyx_v_self->twrite[__pyx_t_32]), __pyx_v_self->channel_number, ((char const *)(__pyx_v_self->channel_number_val_array[__pyx_t_19])), __pyx_v_self->s5->header);

            /* "pyslow5.pyx":1959
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string type: {}".format(type(checked_auxs[batch[idx]][a])))
 *                             ret = slow5_aux_set_string(self.twrite[idx], self.channel_number, <const char *>self.channel_number_val_array[idx], self.s5.header)
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running done: ret = {}".format(ret))             # <<<<<<<<<<<<<<
 *                             if ret < 0:
 *                                 self.logger.error("write_record_batch: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_auxs[batch[idx]][a]))
 */
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1959, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_slow5_aux_set_3, __pyx_n_s_format); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1959, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_15 = __Pyx_PyInt_From_long(__pyx_v_ret); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1959, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_14 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
              __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_12);
              if (likely(__pyx_t_14)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
                __Pyx_INCREF(__pyx_t_14);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_12, function);
              }
            }
            __pyx_t_17 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_14, __pyx_t_15) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_15);
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1959, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_12 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_12)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            __pyx_t_13 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_12, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_17);
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1959, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

            /* "pyslow5.pyx":1960
 *                             ret = slow5_aux_set_string(self.twrite[idx], self.channel_number, <const char *>self.channel_number_val_array[idx], self.s5.header)
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running done: ret = {}".format(ret))
 *                             if ret < 0:             # <<<<<<<<<<<<<<
 *                                 self.logger.error("write_record_batch: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                                 #### We should free here
 */
            __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
            if (__pyx_t_4) {

              /* "pyslow5.pyx":1961
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running done: ret = {}".format(ret))
 *                             if ret < 0:
 *                                 self.logger.error("write_record_batch: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_auxs[batch[idx]][a]))             # <<<<<<<<<<<<<<
 *                                 #### We should free here
 *                                 return -1
 */
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1961, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_slow5_aux_set_4, __pyx_n_s_format); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1961, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_12);
              __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_v_batch, __pyx_v_idx); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1961, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_15);
              __pyx_t_14 = __Pyx_PyDict_GetItem(__pyx_v_checked_auxs, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1961, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_14);
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_t_14, __pyx_v_a); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1961, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_14 = NULL;
              __pyx_t_16 = 0;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
                __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_12);
                if (likely(__pyx_t_14)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
                  __Pyx_INCREF(__pyx_t_14);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_12, function);
                  __pyx_t_16 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_12)) {
                PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_v_a, __pyx_t_15};
                __pyx_t_17 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1961, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_GOTREF(__pyx_t_17);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
                PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_v_a, __pyx_t_15};
                __pyx_t_17 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1961, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_GOTREF(__pyx_t_17);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              } else
              #endif
              {
                __pyx_t_29 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1961, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_29);
                if (__pyx_t_14) {
                  __Pyx_GIVEREF(__pyx_t_14); PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_14); __pyx_t_14 = NULL;
                }
                __Pyx_INCREF(__pyx_v_a);
                __Pyx_GIVEREF(__pyx_v_a);
                PyTuple_SET_ITEM(__pyx_t_29, 0+__pyx_t_16, __pyx_v_a);
                __Pyx_GIVEREF(__pyx_t_15);
                PyTuple_SET_ITEM(__pyx_t_29, 1+__pyx_t_16, __pyx_t_15);
                __pyx_t_15 = 0;
                __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_29, NULL); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1961, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_17);
                __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
              }
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
              __pyx_t_12 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
                __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
                if (likely(__pyx_t_12)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                  __Pyx_INCREF(__pyx_t_12);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_1, function);
                }
              }
              __pyx_t_13 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_12, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_17);
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1961, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_13);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

              /* "pyslow5.pyx":1963
 *                                 self.logger.error("write_record_batch: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                                 #### We should free here
 *                                 return -1             # <<<<<<<<<<<<<<
 *                         elif a == "median_before":
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)
 */
              __Pyx_XDECREF(__pyx_r);
              __Pyx_INCREF(__pyx_int_neg_1);
              __pyx_r = __pyx_int_neg_1;
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              goto __pyx_L0;

              /* "pyslow5.pyx":1960
 *                             ret = slow5_aux_set_string(self.twrite[idx], self.channel_number, <const char *>self.channel_number_val_array[idx], self.s5.header)
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running done: ret = {}".format(ret))
 *                             if ret < 0:             # <<<<<<<<<<<<<<
 *                                 self.logger.error("write_record_batch: slow5_aux_set_string could not write aux value {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                                 #### We should free here
 */
            }

            /* "pyslow5.pyx":1955
 *                         if checked_auxs[batch[idx]][a] is None:
 *                             continue
 *                         if a == "channel_number":             # <<<<<<<<<<<<<<
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string running...")
 *                             self.logger.debug("write_record_batch: slow5_aux_set_string type: {}".format(type(checked_auxs[batch[idx]][a])))
 */
            goto __pyx_L39;
          }

          /* "pyslow5.pyx":1964
 *                                 #### We should free here
 *                                 return -1
 *                         elif a == "median_before":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)
 *                         elif a == "read_number":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_median_before, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1964, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1965
 *                                 return -1
 *                         elif a == "median_before":
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)             # <<<<<<<<<<<<<<
 *                         elif a == "read_number":
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)
 */
            __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1965, __pyx_L1_error)
            __pyx_t_32 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_32 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1965, __pyx_L1_error)
            __pyx_v_ret = slow5_aux_set((__pyx_v_self->twrite[__pyx_t_19]), __pyx_v_self->median_before, ((void const *)(&(__pyx_v_self->median_before_val_array[__pyx_t_32]))), __pyx_v_self->s5->header);

            /* "pyslow5.pyx":1964
 *                                 #### We should free here
 *                                 return -1
 *                         elif a == "median_before":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)
 *                         elif a == "read_number":
 */
            goto __pyx_L39;
          }

          /* "pyslow5.pyx":1966
 *                         elif a == "median_before":
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)
 *                         elif a == "read_number":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)
 *                         elif a == "start_mux":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_read_number, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1966, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1967
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)
 *                         elif a == "read_number":
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)             # <<<<<<<<<<<<<<
 *                         elif a == "start_mux":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)
 */
            __pyx_t_32 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_32 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1967, __pyx_L1_error)
            __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1967, __pyx_L1_error)
            __pyx_v_ret = slow5_aux_set((__pyx_v_self->twrite[__pyx_t_32]), __pyx_v_self->read_number, ((void const *)(&(__pyx_v_self->read_number_val_array[__pyx_t_19]))), __pyx_v_self->s5->header);

            /* "pyslow5.pyx":1966
 *                         elif a == "median_before":
 *                             ret = slow5_aux_set(self.twrite[idx], self.median_before, <const void *>&self.median_before_val_array[idx], self.s5.header)
 *                         elif a == "read_number":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)
 *                         elif a == "start_mux":
 */
            goto __pyx_L39;
          }

          /* "pyslow5.pyx":1968
 *                         elif a == "read_number":
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)
 *                         elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)
 *                         elif a == "start_time":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_mux, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1968, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1969
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)
 *                         elif a == "start_mux":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)             # <<<<<<<<<<<<<<
 *                         elif a == "start_time":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_time, <const void *>&self.start_time_val_array[idx], self.s5.header)
 */
            __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1969, __pyx_L1_error)
            __pyx_t_32 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_32 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1969, __pyx_L1_error)
            __pyx_v_ret = slow5_aux_set((__pyx_v_self->twrite[__pyx_t_19]), __pyx_v_self->start_mux, ((void const *)(&(__pyx_v_self->start_mux_val_array[__pyx_t_32]))), __pyx_v_self->s5->header);

            /* "pyslow5.pyx":1968
 *                         elif a == "read_number":
 *                             ret = slow5_aux_set(self.twrite[idx], self.read_number, <const void *>&self.read_number_val_array[idx], self.s5.header)
 *                         elif a == "start_mux":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)
 *                         elif a == "start_time":
 */
            goto __pyx_L39;
          }

          /* "pyslow5.pyx":1970
 *                         elif a == "start_mux":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)
 *                         elif a == "start_time":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_time, <const void *>&self.start_time_val_array[idx], self.s5.header)
 *                         elif a == "end_reason":
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_start_time, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1970, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1971
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)
 *                         elif a == "start_time":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_time, <const void *>&self.start_time_val_array[idx], self.s5.header)             # <<<<<<<<<<<<<<
 *                         elif a == "end_reason":
 *                             # not implemented yet becuase of variability in ONT versioning
 */
            __pyx_t_32 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_32 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1971, __pyx_L1_error)
            __pyx_t_19 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_19 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1971, __pyx_L1_error)
            __pyx_v_ret = slow5_aux_set((__pyx_v_self->twrite[__pyx_t_32]), __pyx_v_self->start_time, ((void const *)(&(__pyx_v_self->start_time_val_array[__pyx_t_19]))), __pyx_v_self->s5->header);

            /* "pyslow5.pyx":1970
 *                         elif a == "start_mux":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_mux, <const void *>&self.start_mux_val_array[idx], self.s5.header)
 *                         elif a == "start_time":             # <<<<<<<<<<<<<<
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_time, <const void *>&self.start_time_val_array[idx], self.s5.header)
 *                         elif a == "end_reason":
 */
            goto __pyx_L39;
          }

          /* "pyslow5.pyx":1972
 *                         elif a == "start_time":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_time, <const void *>&self.start_time_val_array[idx], self.s5.header)
 *                         elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                             # not implemented yet becuase of variability in ONT versioning
 *                             ret = 0
 */
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_a, __pyx_n_u_end_reason, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1972, __pyx_L1_error)
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1974
 *                         elif a == "end_reason":
 *                             # not implemented yet becuase of variability in ONT versioning
 *                             ret = 0             # <<<<<<<<<<<<<<
 *                         if ret < 0:
 *                             self.logger.error("write_record_batch: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 */
            __pyx_v_ret = 0;

            /* "pyslow5.pyx":1972
 *                         elif a == "start_time":
 *                             ret = slow5_aux_set(self.twrite[idx], self.start_time, <const void *>&self.start_time_val_array[idx], self.s5.header)
 *                         elif a == "end_reason":             # <<<<<<<<<<<<<<
 *                             # not implemented yet becuase of variability in ONT versioning
 *                             ret = 0
 */
          }
          __pyx_L39:;

          /* "pyslow5.pyx":1975
 *                             # not implemented yet becuase of variability in ONT versioning
 *                             ret = 0
 *                         if ret < 0:             # <<<<<<<<<<<<<<
 *                             self.logger.error("write_record_batch: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 *                             return -1
 */
          __pyx_t_4 = ((__pyx_v_ret < 0) != 0);
          if (__pyx_t_4) {

            /* "pyslow5.pyx":1976
 *                             ret = 0
 *                         if ret < 0:
 *                             self.logger.error("write_record_batch: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))             # <<<<<<<<<<<<<<
 *                             return -1
 * 
 */
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1976, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_write_record_batch_slow5_aux_set_5, __pyx_n_s_format); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1976, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            if (unlikely(!__pyx_v_checked_aux)) { __Pyx_RaiseUnboundLocalError("checked_aux"); __PYX_ERR(0, 1976, __pyx_L1_error) }
            __pyx_t_29 = __Pyx_PyObject_GetItem(__pyx_v_checked_aux, __pyx_v_a); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 1976, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_29);
            __pyx_t_15 = NULL;
            __pyx_t_16 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_12);
              if (likely(__pyx_t_15)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
                __Pyx_INCREF(__pyx_t_15);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_12, function);
                __pyx_t_16 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_12)) {
              PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_29};
              __pyx_t_17 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1976, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
              PyObject *__pyx_temp[3] = {__pyx_t_15, __pyx_v_a, __pyx_t_29};
              __pyx_t_17 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_16, 2+__pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1976, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;
            } else
            #endif
            {
              __pyx_t_14 = PyTuple_New(2+__pyx_t_16); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1976, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_14);
              if (__pyx_t_15) {
                __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_15); __pyx_t_15 = NULL;
              }
              __Pyx_INCREF(__pyx_v_a);
              __Pyx_GIVEREF(__pyx_v_a);
              PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_16, __pyx_v_a);
              __Pyx_GIVEREF(__pyx_t_29);
              PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_16, __pyx_t_29);
              __pyx_t_29 = 0;
              __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_14, NULL); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1976, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            }
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_12 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_12)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            __pyx_t_13 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_12, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_17);
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1976, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

            /* "pyslow5.pyx":1977
 *                         if ret < 0:
 *                             self.logger.error("write_record_batch: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 *                             return -1             # <<<<<<<<<<<<<<
 * 
 *                     self.logger.debug("write_record_batch: aux stuff done")
 */
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(__pyx_int_neg_1);
            __pyx_r = __pyx_int_neg_1;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            goto __pyx_L0;

            /* "pyslow5.pyx":1975
 *                             # not implemented yet becuase of variability in ONT versioning
 *                             ret = 0
 *                         if ret < 0:             # <<<<<<<<<<<<<<
 *                             self.logger.error("write_record_batch: slow5_aux_set could not write aux value {}: {}".format(a, checked_aux[a]))
 *                             return -1
 */
          }

          /* "pyslow5.pyx":1951
 *                         return -1
 * 
 *                     for a in checked_auxs[batch[idx]]:             # <<<<<<<<<<<<<<
 *                         self.logger.debug("write_record_batch: checked_aux: {}: {}".format(a, checked_auxs[batch[idx]][a]))
 *                         if checked_auxs[batch[idx]][a] is None:
 */
          __pyx_L36_continue:;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyslow5.pyx":1979
 *                             return -1
 * 
 *                     self.logger.debug("write_record_batch: aux stuff done")             # <<<<<<<<<<<<<<
 * 
 *             self.logger.debug("write_record_batch: slow5_write_batch()")
 */
        __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1979, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_13);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_13, function);
          }
        }
        __pyx_t_7 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_1, __pyx_kp_u_write_record_batch_aux_stuff_don) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_u_write_record_batch_aux_stuff_don);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1979, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyslow5.pyx":1945
 *                 self.logger.debug("write_record_batch: self.write raw_signal done")
 * 
 *                 if aux is not None:             # <<<<<<<<<<<<<<
 *                     self.logger.debug("write_record_batch: aux stuff...")
 *                     if checked_auxs[batch[idx]] is None:
 */
      }

      /* "pyslow5.pyx":1872
 *             if len(checked_aux) == 0:
 *                 checked_aux = None
 *             for idx in range(batch_len):             # <<<<<<<<<<<<<<
 * 
 *                 # check if in append state
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":1981
 *                     self.logger.debug("write_record_batch: aux stuff done")
 * 
 *             self.logger.debug("write_record_batch: slow5_write_batch()")             # <<<<<<<<<<<<<<
 * 
 *             # write the record
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_13 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_2 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_slow5_write_b) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_slow5_write_b);
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":1984
 * 
 *             # write the record
 *             if batch_len <= 0:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record_batch: batch_len 0 or less")
 *                 break
 */
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_batch_len, __pyx_int_0, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1984, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1984, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_4) {

      /* "pyslow5.pyx":1985
 *             # write the record
 *             if batch_len <= 0:
 *                 self.logger.debug("write_record_batch: batch_len 0 or less")             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1985, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_2 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_batch_len_0_o) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_batch_len_0_o);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1985, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyslow5.pyx":1986
 *             if batch_len <= 0:
 *                 self.logger.debug("write_record_batch: batch_len 0 or less")
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             ret = slow5_write_batch(self.twrite, self.s5, batch_len, threads)
 */
      goto __pyx_L7_break;

      /* "pyslow5.pyx":1984
 * 
 *             # write the record
 *             if batch_len <= 0:             # <<<<<<<<<<<<<<
 *                 self.logger.debug("write_record_batch: batch_len 0 or less")
 *                 break
 */
    }

    /* "pyslow5.pyx":1988
 *                 break
 * 
 *             ret = slow5_write_batch(self.twrite, self.s5, batch_len, threads)             # <<<<<<<<<<<<<<
 *             if ret < batch_len:
 *                 self.logger.error("write_record_batch: write failed")
 */
    __pyx_t_16 = __Pyx_PyInt_As_int(__pyx_v_batch_len); if (unlikely((__pyx_t_16 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1988, __pyx_L1_error)
    __pyx_t_33 = __Pyx_PyInt_As_int(__pyx_v_threads); if (unlikely((__pyx_t_33 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1988, __pyx_L1_error)
    __pyx_v_ret = slow5_write_batch(__pyx_v_self->twrite, __pyx_v_self->s5, __pyx_t_16, __pyx_t_33);

    /* "pyslow5.pyx":1989
 * 
 *             ret = slow5_write_batch(self.twrite, self.s5, batch_len, threads)
 *             if ret < batch_len:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record_batch: write failed")
 *                 return -1
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v_ret); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = PyObject_RichCompare(__pyx_t_2, __pyx_v_batch_len, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1989, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1989, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_4) {

      /* "pyslow5.pyx":1990
 *             ret = slow5_write_batch(self.twrite, self.s5, batch_len, threads)
 *             if ret < batch_len:
 *                 self.logger.error("write_record_batch: write failed")             # <<<<<<<<<<<<<<
 *                 return -1
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1990, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_7 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_13, __pyx_kp_u_write_record_batch_write_failed) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_batch_write_failed);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1990, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyslow5.pyx":1991
 *             if ret < batch_len:
 *                 self.logger.error("write_record_batch: write failed")
 *                 return -1             # <<<<<<<<<<<<<<
 * 
 *             self.logger.debug("write_record_batch: free()")
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_int_neg_1);
      __pyx_r = __pyx_int_neg_1;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "pyslow5.pyx":1989
 * 
 *             ret = slow5_write_batch(self.twrite, self.s5, batch_len, threads)
 *             if ret < batch_len:             # <<<<<<<<<<<<<<
 *                 self.logger.error("write_record_batch: write failed")
 *                 return -1
 */
    }

    /* "pyslow5.pyx":1993
 *                 return -1
 * 
 *             self.logger.debug("write_record_batch: free()")             # <<<<<<<<<<<<<<
 *             for i in range(batch_len):
 *                 slow5_rec_free(self.twrite[i])
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_13 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_7 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_13, __pyx_kp_u_write_record_batch_free) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_write_record_batch_free);
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyslow5.pyx":1994
 * 
 *             self.logger.debug("write_record_batch: free()")
 *             for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                 slow5_rec_free(self.twrite[i])
 *             free(self.twrite)
 */
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_batch_len); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1994, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
      __pyx_t_2 = __pyx_t_7; __Pyx_INCREF(__pyx_t_2); __pyx_t_9 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1994, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1994, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_7); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1994, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1994, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_7); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 1994, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1994, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_11(__pyx_t_2);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1994, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "pyslow5.pyx":1995
 *             self.logger.debug("write_record_batch: free()")
 *             for i in range(batch_len):
 *                 slow5_rec_free(self.twrite[i])             # <<<<<<<<<<<<<<
 *             free(self.twrite)
 * 
 */
      __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1995, __pyx_L1_error)
      slow5_rec_free((__pyx_v_self->twrite[__pyx_t_22]));

      /* "pyslow5.pyx":1994
 * 
 *             self.logger.debug("write_record_batch: free()")
 *             for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                 slow5_rec_free(self.twrite[i])
 *             free(self.twrite)
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":1996
 *             for i in range(batch_len):
 *                 slow5_rec_free(self.twrite[i])
 *             free(self.twrite)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    free(__pyx_v_self->twrite);

    /* "pyslow5.pyx":1999
 * 
 * 
 *             self.logger.debug("write_record_batch: free() aux")             # <<<<<<<<<<<<<<
 *             if aux is not None:
 *                 for i in range(batch_len):
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1999, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_13 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_2 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_13, __pyx_kp_u_write_record_batch_free_aux) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_write_record_batch_free_aux);
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1999, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyslow5.pyx":2000
 * 
 *             self.logger.debug("write_record_batch: free() aux")
 *             if aux is not None:             # <<<<<<<<<<<<<<
 *                 for i in range(batch_len):
 *                     free(self.channel_number_val_array[i])
 */
    __pyx_t_4 = (__pyx_v_aux != Py_None);
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "pyslow5.pyx":2001
 *             self.logger.debug("write_record_batch: free() aux")
 *             if aux is not None:
 *                 for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                     free(self.channel_number_val_array[i])
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_batch_len); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2001, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
        __pyx_t_7 = __pyx_t_2; __Pyx_INCREF(__pyx_t_7); __pyx_t_9 = 0;
        __pyx_t_11 = NULL;
      } else {
        __pyx_t_9 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2001, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2001, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      for (;;) {
        if (likely(!__pyx_t_11)) {
          if (likely(PyList_CheckExact(__pyx_t_7))) {
            if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_7)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 2001, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2001, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          } else {
            if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 2001, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2001, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          }
        } else {
          __pyx_t_2 = __pyx_t_11(__pyx_t_7);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 2001, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "pyslow5.pyx":2002
 *             if aux is not None:
 *                 for i in range(batch_len):
 *                     free(self.channel_number_val_array[i])             # <<<<<<<<<<<<<<
 * 
 *                 free(self.channel_number_val_array)
 */
        __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2002, __pyx_L1_error)
        free((__pyx_v_self->channel_number_val_array[__pyx_t_22]));

        /* "pyslow5.pyx":2001
 *             self.logger.debug("write_record_batch: free() aux")
 *             if aux is not None:
 *                 for i in range(batch_len):             # <<<<<<<<<<<<<<
 *                     free(self.channel_number_val_array[i])
 * 
 */
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyslow5.pyx":2004
 *                     free(self.channel_number_val_array[i])
 * 
 *                 free(self.channel_number_val_array)             # <<<<<<<<<<<<<<
 *                 free(self.median_before_val_array)
 *                 free(self.read_number_val_array)
 */
      free(__pyx_v_self->channel_number_val_array);

      /* "pyslow5.pyx":2005
 * 
 *                 free(self.channel_number_val_array)
 *                 free(self.median_before_val_array)             # <<<<<<<<<<<<<<
 *                 free(self.read_number_val_array)
 *                 free(self.start_mux_val_array)
 */
      free(__pyx_v_self->median_before_val_array);

      /* "pyslow5.pyx":2006
 *                 free(self.channel_number_val_array)
 *                 free(self.median_before_val_array)
 *                 free(self.read_number_val_array)             # <<<<<<<<<<<<<<
 *                 free(self.start_mux_val_array)
 *                 free(self.start_time_val_array)
 */
      free(__pyx_v_self->read_number_val_array);

      /* "pyslow5.pyx":2007
 *                 free(self.median_before_val_array)
 *                 free(self.read_number_val_array)
 *                 free(self.start_mux_val_array)             # <<<<<<<<<<<<<<
 *                 free(self.start_time_val_array)
 * 
 */
      free(__pyx_v_self->start_mux_val_array);

      /* "pyslow5.pyx":2008
 *                 free(self.read_number_val_array)
 *                 free(self.start_mux_val_array)
 *                 free(self.start_time_val_array)             # <<<<<<<<<<<<<<
 * 
 *         end_multi_write = time.time() - start_multi_write
 */
      free(__pyx_v_self->start_time_val_array);

      /* "pyslow5.pyx":2000
 * 
 *             self.logger.debug("write_record_batch: free() aux")
 *             if aux is not None:             # <<<<<<<<<<<<<<
 *                 for i in range(batch_len):
 *                     free(self.channel_number_val_array[i])
 */
    }

    /* "pyslow5.pyx":1839
 * 
 *         self.logger.debug("write_record_batch: batch for loop start")
 *         for batch in chain(batches):             # <<<<<<<<<<<<<<
 *             batch_len = len(batch)
 *             self.twrite = <slow5_rec_t **> malloc(sizeof(slow5_rec_t*)*batch_len)
 */
  }
  __pyx_L7_break:;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyslow5.pyx":2010
 *                 free(self.start_time_val_array)
 * 
 *         end_multi_write = time.time() - start_multi_write             # <<<<<<<<<<<<<<
 *         self.total_multi_write_time = self.total_multi_write_time + end_multi_write
 *         # free memory
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2010, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2010, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2010, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Subtract(__pyx_t_3, __pyx_v_start_multi_write); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2010, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_end_multi_write = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":2011
 * 
 *         end_multi_write = time.time() - start_multi_write
 *         self.total_multi_write_time = self.total_multi_write_time + end_multi_write             # <<<<<<<<<<<<<<
 *         # free memory
 *         # self.twrite = NULL
 */
  __pyx_t_2 = PyNumber_Add(__pyx_v_self->total_multi_write_time, __pyx_v_end_multi_write); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2011, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyFloat_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "float", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 2011, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->total_multi_write_time);
  __Pyx_DECREF(__pyx_v_self->total_multi_write_time);
  __pyx_v_self->total_multi_write_time = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyslow5.pyx":2014
 *         # free memory
 *         # self.twrite = NULL
 *         self.logger.debug("write_record_batch: function complete, returning 0")             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_kp_u_write_record_batch_function_comp) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_write_record_batch_function_comp);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":2015
 *         # self.twrite = NULL
 *         self.logger.debug("write_record_batch: function complete, returning 0")
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_0);
  __pyx_r = __pyx_int_0;
  goto __pyx_L0;

  /* "pyslow5.pyx":1810
 * 
 * 
 *     def write_record_batch(self, records, threads=4, batchsize=4096, aux=None):             # <<<<<<<<<<<<<<
 *         '''
 *         write a batch of records
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_29);
  __Pyx_AddTraceback("pyslow5.Open.write_record_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start_multi_write);
  __Pyx_XDECREF(__pyx_v_aux_types);
  __Pyx_XDECREF(__pyx_v_read_list);
  __Pyx_XDECREF(__pyx_v_batches);
  __Pyx_XDECREF(__pyx_v_batch);
  __Pyx_XDECREF(__pyx_v_batch_len);
  __Pyx_XDECREF(__pyx_v_checked_records);
  __Pyx_XDECREF(__pyx_v_checked_auxs);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_XDECREF(__pyx_v_checked_record);
  __Pyx_XDECREF(__pyx_v_checked_aux);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_slow5_aux_types);
  __Pyx_XDECREF(__pyx_v_start_write_copy_signal);
  __Pyx_XDECREF(__pyx_v_memview);
  __Pyx_XDECREF(__pyx_v_end_write_copy_signal);
  __Pyx_XDECREF(__pyx_v_end_multi_write);
  __Pyx_XDECREF(__pyx_v_aux);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyslow5.pyx":2017
 *         return 0
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         '''
 *         close file so EOF is written
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_65close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7pyslow5_4Open_64close[] = "\n        close file so EOF is written\n        ";
static PyObject *__pyx_pw_7pyslow5_4Open_65close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_64close(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_64close(struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_TraceCall("close", __pyx_f[0], 2017, 0, __PYX_ERR(0, 2017, __pyx_L1_error));

  /* "pyslow5.pyx":2021
 *         close file so EOF is written
 *         '''
 *         if self.state in [1,2]:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
  switch (__pyx_v_self->state) {
    case 1:
    case 2:

    /* "pyslow5.pyx":2022
 *         '''
 *         if self.state in [1,2]:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 */
    __pyx_t_1 = ((!(__pyx_v_self->close_state != 0)) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":2023
 *         if self.state in [1,2]:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
      __pyx_t_1 = ((__pyx_v_self->s5 != NULL) != 0);
      if (__pyx_t_1) {

        /* "pyslow5.pyx":2024
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)             # <<<<<<<<<<<<<<
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))
 */
        (void)(slow5_close(__pyx_v_self->s5));

        /* "pyslow5.pyx":2025
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 *                     self.close_state = True             # <<<<<<<<<<<<<<
 *                     self.logger.debug("{} closed".format(self.path))
 *             else:
 */
        __pyx_v_self->close_state = 1;

        /* "pyslow5.pyx":2026
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.warning("{} already closed".format(self.path))
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2026, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2026, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2026, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2026, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":2023
 *         if self.state in [1,2]:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
      }

      /* "pyslow5.pyx":2022
 *         '''
 *         if self.state in [1,2]:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 */
      goto __pyx_L3;
    }

    /* "pyslow5.pyx":2028
 *                     self.logger.debug("{} closed".format(self.path))
 *             else:
 *                 self.logger.warning("{} already closed".format(self.path))             # <<<<<<<<<<<<<<
 *         elif self.state == 0:
 *             if not self.close_state:
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2028, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_already_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2028, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2028, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2028, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L3:;

    /* "pyslow5.pyx":2021
 *         close file so EOF is written
 *         '''
 *         if self.state in [1,2]:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
    break;
    case 0:

    /* "pyslow5.pyx":2030
 *                 self.logger.warning("{} already closed".format(self.path))
 *         elif self.state == 0:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 */
    __pyx_t_1 = ((!(__pyx_v_self->close_state != 0)) != 0);
    if (__pyx_t_1) {

      /* "pyslow5.pyx":2031
 *         elif self.state == 0:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
      __pyx_t_1 = ((__pyx_v_self->s5 != NULL) != 0);
      if (__pyx_t_1) {

        /* "pyslow5.pyx":2032
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)             # <<<<<<<<<<<<<<
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))
 */
        (void)(slow5_close(__pyx_v_self->s5));

        /* "pyslow5.pyx":2033
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 *                     self.close_state = True             # <<<<<<<<<<<<<<
 *                     self.logger.debug("{} closed".format(self.path))
 *             else:
 */
        __pyx_v_self->close_state = 1;

        /* "pyslow5.pyx":2034
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 *                     self.logger.debug("{} closed".format(self.path))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.logger.warning("{} already closed".format(self.path))
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2034, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2034, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2034, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2034, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyslow5.pyx":2031
 *         elif self.state == 0:
 *             if not self.close_state:
 *                 if self.s5 is not NULL:             # <<<<<<<<<<<<<<
 *                     slow5_close(self.s5)
 *                     self.close_state = True
 */
      }

      /* "pyslow5.pyx":2030
 *                 self.logger.warning("{} already closed".format(self.path))
 *         elif self.state == 0:
 *             if not self.close_state:             # <<<<<<<<<<<<<<
 *                 if self.s5 is not NULL:
 *                     slow5_close(self.s5)
 */
      goto __pyx_L5;
    }

    /* "pyslow5.pyx":2036
 *                     self.logger.debug("{} closed".format(self.path))
 *             else:
 *                 self.logger.warning("{} already closed".format(self.path))             # <<<<<<<<<<<<<<
 *         else:
 *             self.logger.error("{} not open for writing, remake class object to refresh".format(self.path))
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_warning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2036, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_already_closed, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2036, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2036, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2036, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L5:;

    /* "pyslow5.pyx":2029
 *             else:
 *                 self.logger.warning("{} already closed".format(self.path))
 *         elif self.state == 0:             # <<<<<<<<<<<<<<
 *             if not self.close_state:
 *                 if self.s5 is not NULL:
 */
    break;
    default:

    /* "pyslow5.pyx":2038
 *                 self.logger.warning("{} already closed".format(self.path))
 *         else:
 *             self.logger.error("{} not open for writing, remake class object to refresh".format(self.path))             # <<<<<<<<<<<<<<
 *         return
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_not_open_for_writing_remake_cla, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_self->path) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->path);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    break;
  }

  /* "pyslow5.pyx":2039
 *         else:
 *             self.logger.error("{} not open for writing, remake class object to refresh".format(self.path))
 *         return             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pyslow5.pyx":2017
 *         return 0
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         '''
 *         close file so EOF is written
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyslow5.Open.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_67__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7pyslow5_4Open_67__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_66__reduce_cython__(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_66__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);
  __Pyx_TraceCall("__reduce_cython__", __pyx_f[1], 1, 0, __PYX_ERR(1, 1, __pyx_L1_error));

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyslow5.Open.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7pyslow5_4Open_69__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7pyslow5_4Open_69__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7pyslow5_4Open_68__setstate_cython__(((struct __pyx_obj_7pyslow5_Open *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyslow5_4Open_68__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7pyslow5_Open *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);
  __Pyx_TraceCall("__setstate_cython__", __pyx_f[1], 3, 0, __PYX_ERR(1, 3, __pyx_L1_error));

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyslow5.Open.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);
  __Pyx_TraceCall("PyArray_MultiIterNew1", __pyx_f[2], 735, 0, __PYX_ERR(2, 735, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":736
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);
  __Pyx_TraceCall("PyArray_MultiIterNew2", __pyx_f[2], 738, 0, __PYX_ERR(2, 738, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":739
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":738
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);
  __Pyx_TraceCall("PyArray_MultiIterNew3", __pyx_f[2], 741, 0, __PYX_ERR(2, 741, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":742
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":741
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);
  __Pyx_TraceCall("PyArray_MultiIterNew4", __pyx_f[2], 744, 0, __PYX_ERR(2, 744, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":745
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":744
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);
  __Pyx_TraceCall("PyArray_MultiIterNew5", __pyx_f[2], 747, 0, __PYX_ERR(2, 747, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":748
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":747
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);
  __Pyx_TraceCall("PyDataType_SHAPE", __pyx_f[2], 750, 0, __PYX_ERR(2, 750, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":752
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":754
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":750
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("numpy.PyDataType_SHAPE", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_array_base", 0);
  __Pyx_TraceCall("set_array_base", __pyx_f[2], 929, 0, __PYX_ERR(2, 929, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":930
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":931
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":929
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.set_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_array_base", 0);
  __Pyx_TraceCall("get_array_base", __pyx_f[2], 933, 0, __PYX_ERR(2, 933, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":934
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":936
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":935
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":937
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":933
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("numpy.get_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);
  __Pyx_TraceCall("import_array", __pyx_f[2], 941, 0, __PYX_ERR(2, 941, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":943
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 943, __pyx_L3_error)

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":944
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 944, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 945, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 945, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":942
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":941
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);
  __Pyx_TraceCall("import_umath", __pyx_f[2], 947, 0, __PYX_ERR(2, 947, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":949
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 949, __pyx_L3_error)

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":950
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 950, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 951, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 951, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":948
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":947
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);
  __Pyx_TraceCall("import_ufunc", __pyx_f[2], 953, 0, __PYX_ERR(2, 953, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":955
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 955, __pyx_L3_error)

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":956
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 956, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":957
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 957, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 957, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":954
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":953
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);
  __Pyx_TraceCall("is_timedelta64_object", __pyx_f[2], 967, 0, __PYX_ERR(2, 967, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":979
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":967
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.is_timedelta64_object", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);
  __Pyx_TraceCall("is_datetime64_object", __pyx_f[2], 982, 0, __PYX_ERR(2, 982, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":994
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":982
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.is_datetime64_object", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;
  __Pyx_TraceDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_TraceCall("get_datetime64_value", __pyx_f[2], 997, 1, __PYX_ERR(2, 997, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1004
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":997
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.get_datetime64_value", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 1);
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;
  __Pyx_TraceDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_TraceCall("get_timedelta64_value", __pyx_f[2], 1007, 1, __PYX_ERR(2, 1007, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1011
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1007
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.get_timedelta64_value", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 1);
  return __pyx_r;
}

/* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;
  __Pyx_TraceDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_TraceCall("get_datetime64_unit", __pyx_f[2], 1014, 1, __PYX_ERR(2, 1014, __pyx_L1_error));

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1018
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.get_datetime64_unit", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = (NPY_DATETIMEUNIT) 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 1);
  return __pyx_r;
}

static PyObject *__pyx_tp_new_7pyslow5_Open(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7pyslow5_Open *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7pyslow5_Open *)o);
  p->path = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->rec_press = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->sig_press = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->logger = Py_None; Py_INCREF(Py_None);
  p->aux_names = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->aux_types = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->error_codes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->slow5_press_method = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->e8 = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->total_time_slow5_get_next = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->total_time_yield_reads = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->total_single_write_time = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->total_multi_write_signal_time = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->total_multi_write_time = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7pyslow5_4Open_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7pyslow5_Open(PyObject *o) {
  struct __pyx_obj_7pyslow5_Open *p = (struct __pyx_obj_7pyslow5_Open *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_7pyslow5_4Open_5__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->path);
  Py_CLEAR(p->mode);
  Py_CLEAR(p->rec_press);
  Py_CLEAR(p->sig_press);
  Py_CLEAR(p->logger);
  Py_CLEAR(p->aux_names);
  Py_CLEAR(p->aux_types);
  Py_CLEAR(p->error_codes);
  Py_CLEAR(p->slow5_press_method);
  Py_CLEAR(p->e8);
  Py_CLEAR(p->total_time_slow5_get_next);
  Py_CLEAR(p->total_time_yield_reads);
  Py_CLEAR(p->total_single_write_time);
  Py_CLEAR(p->total_multi_write_signal_time);
  Py_CLEAR(p->total_multi_write_time);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7pyslow5_Open(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5_Open *p = (struct __pyx_obj_7pyslow5_Open *)o;
  if (p->logger) {
    e = (*v)(p->logger, a); if (e) return e;
  }
  if (p->aux_names) {
    e = (*v)(p->aux_names, a); if (e) return e;
  }
  if (p->aux_types) {
    e = (*v)(p->aux_types, a); if (e) return e;
  }
  if (p->error_codes) {
    e = (*v)(p->error_codes, a); if (e) return e;
  }
  if (p->slow5_press_method) {
    e = (*v)(p->slow5_press_method, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7pyslow5_Open(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7pyslow5_Open *p = (struct __pyx_obj_7pyslow5_Open *)o;
  tmp = ((PyObject*)p->logger);
  p->logger = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->aux_names);
  p->aux_names = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->aux_types);
  p->aux_types = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->error_codes);
  p->error_codes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->slow5_press_method);
  p->slow5_press_method = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7pyslow5_Open[] = {
  {"_convert_to_pA", (PyCFunction)__pyx_pw_7pyslow5_4Open_7_convert_to_pA, METH_O, __pyx_doc_7pyslow5_4Open_6_convert_to_pA},
  {"_get_read", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_9_get_read, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_8_get_read},
  {"_get_batches", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_11_get_batches, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_10_get_batches},
  {"_get_read_multi", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_14_get_read_multi, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_13_get_read_multi},
  {"_get_read_aux", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_17_get_read_aux, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_16_get_read_aux},
  {"_get_seq_read_aux", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_19_get_seq_read_aux, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_18_get_seq_read_aux},
  {"get_read_ids", (PyCFunction)__pyx_pw_7pyslow5_4Open_21get_read_ids, METH_NOARGS, __pyx_doc_7pyslow5_4Open_20get_read_ids},
  {"get_read", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_23get_read, METH_VARARGS|METH_KEYWORDS, 0},
  {"seq_reads", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_25seq_reads, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_24seq_reads},
  {"seq_reads_multi", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_28seq_reads_multi, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_27seq_reads_multi},
  {"get_read_list", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_31get_read_list, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_30get_read_list},
  {"get_read_list_multi", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_34get_read_list_multi, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_33get_read_list_multi},
  {"get_header_names", (PyCFunction)__pyx_pw_7pyslow5_4Open_37get_header_names, METH_NOARGS, __pyx_doc_7pyslow5_4Open_36get_header_names},
  {"get_header_value", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_39get_header_value, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_38get_header_value},
  {"get_all_headers", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_41get_all_headers, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_40get_all_headers},
  {"get_aux_names", (PyCFunction)__pyx_pw_7pyslow5_4Open_43get_aux_names, METH_NOARGS, __pyx_doc_7pyslow5_4Open_42get_aux_names},
  {"get_aux_types", (PyCFunction)__pyx_pw_7pyslow5_4Open_45get_aux_types, METH_NOARGS, __pyx_doc_7pyslow5_4Open_44get_aux_types},
  {"get_empty_header", (PyCFunction)__pyx_pw_7pyslow5_4Open_47get_empty_header, METH_NOARGS, __pyx_doc_7pyslow5_4Open_46get_empty_header},
  {"get_empty_record", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_49get_empty_record, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_48get_empty_record},
  {"_header_type_validation", (PyCFunction)__pyx_pw_7pyslow5_4Open_51_header_type_validation, METH_O, __pyx_doc_7pyslow5_4Open_50_header_type_validation},
  {"_aux_header_type_validation", (PyCFunction)__pyx_pw_7pyslow5_4Open_53_aux_header_type_validation, METH_O, __pyx_doc_7pyslow5_4Open_52_aux_header_type_validation},
  {"_record_type_validation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_55_record_type_validation, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_54_record_type_validation},
  {"_multi_record_type_validation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_57_multi_record_type_validation, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_56_multi_record_type_validation},
  {"write_header", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_59write_header, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_58write_header},
  {"write_record", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_61write_record, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_60write_record},
  {"write_record_batch", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7pyslow5_4Open_63write_record_batch, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7pyslow5_4Open_62write_record_batch},
  {"close", (PyCFunction)__pyx_pw_7pyslow5_4Open_65close, METH_NOARGS, __pyx_doc_7pyslow5_4Open_64close},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7pyslow5_4Open_67__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7pyslow5_4Open_69__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7pyslow5_Open = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.Open", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5_Open), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5_Open, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5_Open, /*tp_traverse*/
  __pyx_tp_clear_7pyslow5_Open, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7pyslow5_Open, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7pyslow5_4Open_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5_Open, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *__pyx_freelist_7pyslow5___pyx_scope_struct____cinit__[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct____cinit__ = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct____cinit__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct____cinit__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct____cinit__[--__pyx_freecount_7pyslow5___pyx_scope_struct____cinit__];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct____cinit__(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct____cinit__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct____cinit__[__pyx_freecount_7pyslow5___pyx_scope_struct____cinit__++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct____cinit__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7pyslow5___pyx_scope_struct____cinit__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__ *)o;
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_7pyslow5_Open *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct____cinit__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct____cinit__", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct____cinit__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct____cinit__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct____cinit__, /*tp_traverse*/
  __pyx_tp_clear_7pyslow5___pyx_scope_struct____cinit__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct____cinit__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *__pyx_freelist_7pyslow5___pyx_scope_struct_1_genexpr[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_1_genexpr = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_1_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_1_genexpr[--__pyx_freecount_7pyslow5___pyx_scope_struct_1_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_press);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_1_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_1_genexpr[__pyx_freecount_7pyslow5___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_press) {
    e = (*v)(p->__pyx_v_press, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *__pyx_freelist_7pyslow5___pyx_scope_struct_2__get_batches[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_2__get_batches = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_2__get_batches(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_2__get_batches > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_2__get_batches[--__pyx_freecount_7pyslow5___pyx_scope_struct_2__get_batches];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_2__get_batches(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_i);
  Py_CLEAR(p->__pyx_v_read_list);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_size);
  Py_CLEAR(p->__pyx_t_1);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_2__get_batches < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_2__get_batches[__pyx_freecount_7pyslow5___pyx_scope_struct_2__get_batches++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_2__get_batches(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches *)o;
  if (p->__pyx_v_i) {
    e = (*v)(p->__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_v_read_list) {
    e = (*v)(p->__pyx_v_read_list, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_size) {
    e = (*v)(p->__pyx_v_size, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_2__get_batches = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_2__get_batches", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_2__get_batches), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_2__get_batches, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_2__get_batches, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_2__get_batches, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *__pyx_freelist_7pyslow5___pyx_scope_struct_3__get_read_multi[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_3__get_read_multi = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_3__get_read_multi(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_3__get_read_multi > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_3__get_read_multi[--__pyx_freecount_7pyslow5___pyx_scope_struct_3__get_read_multi];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_3__get_read_multi(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aux);
  Py_CLEAR(p->__pyx_v_aux_dic);
  Py_CLEAR(p->__pyx_v_aux_set);
  Py_CLEAR(p->__pyx_v_batch);
  Py_CLEAR(p->__pyx_v_batch_len);
  Py_CLEAR(p->__pyx_v_batches);
  Py_CLEAR(p->__pyx_v_batchsize);
  Py_CLEAR(p->__pyx_v_dic);
  Py_CLEAR(p->__pyx_8genexpr1__pyx_v_i);
  Py_CLEAR(p->__pyx_8genexpr2__pyx_v_i);
  Py_CLEAR(p->__pyx_v_i);
  Py_CLEAR(p->__pyx_v_n);
  Py_CLEAR(p->__pyx_v_n_list);
  Py_CLEAR(p->__pyx_v_n_set);
  Py_CLEAR(p->__pyx_v_pA);
  Py_CLEAR(p->__pyx_v_python_parse_read_start);
  Py_CLEAR(p->__pyx_v_read_list);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_signal);
  Py_CLEAR(p->__pyx_v_t);
  Py_CLEAR(p->__pyx_v_t_list);
  Py_CLEAR(p->__pyx_v_threads);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_3__get_read_multi < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_3__get_read_multi[__pyx_freecount_7pyslow5___pyx_scope_struct_3__get_read_multi++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_3__get_read_multi(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi *)o;
  if (p->__pyx_v_aux) {
    e = (*v)(p->__pyx_v_aux, a); if (e) return e;
  }
  if (p->__pyx_v_aux_dic) {
    e = (*v)(p->__pyx_v_aux_dic, a); if (e) return e;
  }
  if (p->__pyx_v_aux_set) {
    e = (*v)(p->__pyx_v_aux_set, a); if (e) return e;
  }
  if (p->__pyx_v_batch) {
    e = (*v)(p->__pyx_v_batch, a); if (e) return e;
  }
  if (p->__pyx_v_batch_len) {
    e = (*v)(p->__pyx_v_batch_len, a); if (e) return e;
  }
  if (p->__pyx_v_batches) {
    e = (*v)(p->__pyx_v_batches, a); if (e) return e;
  }
  if (p->__pyx_v_batchsize) {
    e = (*v)(p->__pyx_v_batchsize, a); if (e) return e;
  }
  if (p->__pyx_v_dic) {
    e = (*v)(p->__pyx_v_dic, a); if (e) return e;
  }
  if (p->__pyx_8genexpr1__pyx_v_i) {
    e = (*v)(p->__pyx_8genexpr1__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_8genexpr2__pyx_v_i) {
    e = (*v)(p->__pyx_8genexpr2__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_v_i) {
    e = (*v)(p->__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_v_n) {
    e = (*v)(p->__pyx_v_n, a); if (e) return e;
  }
  if (p->__pyx_v_n_list) {
    e = (*v)(p->__pyx_v_n_list, a); if (e) return e;
  }
  if (p->__pyx_v_n_set) {
    e = (*v)(p->__pyx_v_n_set, a); if (e) return e;
  }
  if (p->__pyx_v_pA) {
    e = (*v)(p->__pyx_v_pA, a); if (e) return e;
  }
  if (p->__pyx_v_python_parse_read_start) {
    e = (*v)(p->__pyx_v_python_parse_read_start, a); if (e) return e;
  }
  if (p->__pyx_v_read_list) {
    e = (*v)(p->__pyx_v_read_list, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_signal) {
    e = (*v)(p->__pyx_v_signal, a); if (e) return e;
  }
  if (p->__pyx_v_t) {
    e = (*v)(p->__pyx_v_t, a); if (e) return e;
  }
  if (p->__pyx_v_t_list) {
    e = (*v)(p->__pyx_v_t_list, a); if (e) return e;
  }
  if (p->__pyx_v_threads) {
    e = (*v)(p->__pyx_v_threads, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_3__get_read_multi", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_3__get_read_multi), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_3__get_read_multi, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_3__get_read_multi, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_3__get_read_multi, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *__pyx_freelist_7pyslow5___pyx_scope_struct_4_seq_reads[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_4_seq_reads = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_4_seq_reads(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_4_seq_reads > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_4_seq_reads[--__pyx_freecount_7pyslow5___pyx_scope_struct_4_seq_reads];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_4_seq_reads(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aux);
  Py_CLEAR(p->__pyx_v_aux_dic);
  Py_CLEAR(p->__pyx_v_aux_set);
  Py_CLEAR(p->__pyx_v_i);
  Py_CLEAR(p->__pyx_v_n);
  Py_CLEAR(p->__pyx_v_n_list);
  Py_CLEAR(p->__pyx_v_n_set);
  Py_CLEAR(p->__pyx_v_pA);
  Py_CLEAR(p->__pyx_v_row);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_signal);
  Py_CLEAR(p->__pyx_v_start_slow5_get_next);
  Py_CLEAR(p->__pyx_v_t);
  Py_CLEAR(p->__pyx_v_t_list);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_4_seq_reads < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_4_seq_reads[__pyx_freecount_7pyslow5___pyx_scope_struct_4_seq_reads++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_4_seq_reads(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads *)o;
  if (p->__pyx_v_aux) {
    e = (*v)(p->__pyx_v_aux, a); if (e) return e;
  }
  if (p->__pyx_v_aux_dic) {
    e = (*v)(p->__pyx_v_aux_dic, a); if (e) return e;
  }
  if (p->__pyx_v_aux_set) {
    e = (*v)(p->__pyx_v_aux_set, a); if (e) return e;
  }
  if (p->__pyx_v_i) {
    e = (*v)(p->__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_v_n) {
    e = (*v)(p->__pyx_v_n, a); if (e) return e;
  }
  if (p->__pyx_v_n_list) {
    e = (*v)(p->__pyx_v_n_list, a); if (e) return e;
  }
  if (p->__pyx_v_n_set) {
    e = (*v)(p->__pyx_v_n_set, a); if (e) return e;
  }
  if (p->__pyx_v_pA) {
    e = (*v)(p->__pyx_v_pA, a); if (e) return e;
  }
  if (p->__pyx_v_row) {
    e = (*v)(p->__pyx_v_row, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_signal) {
    e = (*v)(p->__pyx_v_signal, a); if (e) return e;
  }
  if (p->__pyx_v_start_slow5_get_next) {
    e = (*v)(p->__pyx_v_start_slow5_get_next, a); if (e) return e;
  }
  if (p->__pyx_v_t) {
    e = (*v)(p->__pyx_v_t, a); if (e) return e;
  }
  if (p->__pyx_v_t_list) {
    e = (*v)(p->__pyx_v_t_list, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_4_seq_reads", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_4_seq_reads), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_4_seq_reads, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_4_seq_reads, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_4_seq_reads, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *__pyx_freelist_7pyslow5___pyx_scope_struct_5_seq_reads_multi[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_5_seq_reads_multi = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_5_seq_reads_multi(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_5_seq_reads_multi > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_5_seq_reads_multi[--__pyx_freecount_7pyslow5___pyx_scope_struct_5_seq_reads_multi];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_5_seq_reads_multi(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aux);
  Py_CLEAR(p->__pyx_v_aux_dic);
  Py_CLEAR(p->__pyx_v_aux_set);
  Py_CLEAR(p->__pyx_v_aux_time_start);
  Py_CLEAR(p->__pyx_v_batchsize);
  Py_CLEAR(p->__pyx_v_i);
  Py_CLEAR(p->__pyx_v_n);
  Py_CLEAR(p->__pyx_v_n_list);
  Py_CLEAR(p->__pyx_v_n_set);
  Py_CLEAR(p->__pyx_v_pA);
  Py_CLEAR(p->__pyx_v_pA_start_time);
  Py_CLEAR(p->__pyx_v_primary_start_time);
  Py_CLEAR(p->__pyx_v_python_parse_read_start);
  Py_CLEAR(p->__pyx_v_row);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_signal);
  Py_CLEAR(p->__pyx_v_signal_start_time);
  Py_CLEAR(p->__pyx_v_start_slow5_get_next);
  Py_CLEAR(p->__pyx_v_t);
  Py_CLEAR(p->__pyx_v_t_list);
  Py_CLEAR(p->__pyx_v_threads);
  Py_CLEAR(p->__pyx_v_timedic);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_5_seq_reads_multi < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_5_seq_reads_multi[__pyx_freecount_7pyslow5___pyx_scope_struct_5_seq_reads_multi++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_5_seq_reads_multi(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi *)o;
  if (p->__pyx_v_aux) {
    e = (*v)(p->__pyx_v_aux, a); if (e) return e;
  }
  if (p->__pyx_v_aux_dic) {
    e = (*v)(p->__pyx_v_aux_dic, a); if (e) return e;
  }
  if (p->__pyx_v_aux_set) {
    e = (*v)(p->__pyx_v_aux_set, a); if (e) return e;
  }
  if (p->__pyx_v_aux_time_start) {
    e = (*v)(p->__pyx_v_aux_time_start, a); if (e) return e;
  }
  if (p->__pyx_v_batchsize) {
    e = (*v)(p->__pyx_v_batchsize, a); if (e) return e;
  }
  if (p->__pyx_v_i) {
    e = (*v)(p->__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_v_n) {
    e = (*v)(p->__pyx_v_n, a); if (e) return e;
  }
  if (p->__pyx_v_n_list) {
    e = (*v)(p->__pyx_v_n_list, a); if (e) return e;
  }
  if (p->__pyx_v_n_set) {
    e = (*v)(p->__pyx_v_n_set, a); if (e) return e;
  }
  if (p->__pyx_v_pA) {
    e = (*v)(p->__pyx_v_pA, a); if (e) return e;
  }
  if (p->__pyx_v_pA_start_time) {
    e = (*v)(p->__pyx_v_pA_start_time, a); if (e) return e;
  }
  if (p->__pyx_v_primary_start_time) {
    e = (*v)(p->__pyx_v_primary_start_time, a); if (e) return e;
  }
  if (p->__pyx_v_python_parse_read_start) {
    e = (*v)(p->__pyx_v_python_parse_read_start, a); if (e) return e;
  }
  if (p->__pyx_v_row) {
    e = (*v)(p->__pyx_v_row, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_signal) {
    e = (*v)(p->__pyx_v_signal, a); if (e) return e;
  }
  if (p->__pyx_v_signal_start_time) {
    e = (*v)(p->__pyx_v_signal_start_time, a); if (e) return e;
  }
  if (p->__pyx_v_start_slow5_get_next) {
    e = (*v)(p->__pyx_v_start_slow5_get_next, a); if (e) return e;
  }
  if (p->__pyx_v_t) {
    e = (*v)(p->__pyx_v_t, a); if (e) return e;
  }
  if (p->__pyx_v_t_list) {
    e = (*v)(p->__pyx_v_t_list, a); if (e) return e;
  }
  if (p->__pyx_v_threads) {
    e = (*v)(p->__pyx_v_threads, a); if (e) return e;
  }
  if (p->__pyx_v_timedic) {
    e = (*v)(p->__pyx_v_timedic, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_5_seq_reads_multi", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_5_seq_reads_multi), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_5_seq_reads_multi, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_5_seq_reads_multi, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_5_seq_reads_multi, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *__pyx_freelist_7pyslow5___pyx_scope_struct_6_get_read_list[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_6_get_read_list = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_6_get_read_list(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_6_get_read_list > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_6_get_read_list[--__pyx_freecount_7pyslow5___pyx_scope_struct_6_get_read_list];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_6_get_read_list(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aux);
  Py_CLEAR(p->__pyx_v_pA);
  Py_CLEAR(p->__pyx_v_r);
  Py_CLEAR(p->__pyx_v_read_list);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_6_get_read_list < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_6_get_read_list[__pyx_freecount_7pyslow5___pyx_scope_struct_6_get_read_list++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_6_get_read_list(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list *)o;
  if (p->__pyx_v_aux) {
    e = (*v)(p->__pyx_v_aux, a); if (e) return e;
  }
  if (p->__pyx_v_pA) {
    e = (*v)(p->__pyx_v_pA, a); if (e) return e;
  }
  if (p->__pyx_v_r) {
    e = (*v)(p->__pyx_v_r, a); if (e) return e;
  }
  if (p->__pyx_v_read_list) {
    e = (*v)(p->__pyx_v_read_list, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_6_get_read_list", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_6_get_read_list), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_6_get_read_list, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_6_get_read_list, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_6_get_read_list, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *__pyx_freelist_7pyslow5___pyx_scope_struct_7_get_read_list_multi[8];
static int __pyx_freecount_7pyslow5___pyx_scope_struct_7_get_read_list_multi = 0;

static PyObject *__pyx_tp_new_7pyslow5___pyx_scope_struct_7_get_read_list_multi(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_7pyslow5___pyx_scope_struct_7_get_read_list_multi > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi)))) {
    o = (PyObject*)__pyx_freelist_7pyslow5___pyx_scope_struct_7_get_read_list_multi[--__pyx_freecount_7pyslow5___pyx_scope_struct_7_get_read_list_multi];
    memset(o, 0, sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_7_get_read_list_multi(PyObject *o) {
  struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aux);
  Py_CLEAR(p->__pyx_v_batchsize);
  Py_CLEAR(p->__pyx_v_pA);
  Py_CLEAR(p->__pyx_v_r);
  Py_CLEAR(p->__pyx_v_read_list);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_threads);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_7pyslow5___pyx_scope_struct_7_get_read_list_multi < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi)))) {
    __pyx_freelist_7pyslow5___pyx_scope_struct_7_get_read_list_multi[__pyx_freecount_7pyslow5___pyx_scope_struct_7_get_read_list_multi++] = ((struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_7pyslow5___pyx_scope_struct_7_get_read_list_multi(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *p = (struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi *)o;
  if (p->__pyx_v_aux) {
    e = (*v)(p->__pyx_v_aux, a); if (e) return e;
  }
  if (p->__pyx_v_batchsize) {
    e = (*v)(p->__pyx_v_batchsize, a); if (e) return e;
  }
  if (p->__pyx_v_pA) {
    e = (*v)(p->__pyx_v_pA, a); if (e) return e;
  }
  if (p->__pyx_v_r) {
    e = (*v)(p->__pyx_v_r, a); if (e) return e;
  }
  if (p->__pyx_v_read_list) {
    e = (*v)(p->__pyx_v_read_list, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_threads) {
    e = (*v)(p->__pyx_v_threads, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyslow5.__pyx_scope_struct_7_get_read_list_multi", /*tp_name*/
  sizeof(struct __pyx_obj_7pyslow5___pyx_scope_struct_7_get_read_list_multi), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyslow5___pyx_scope_struct_7_get_read_list_multi, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyslow5___pyx_scope_struct_7_get_read_list_multi, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyslow5___pyx_scope_struct_7_get_read_list_multi, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_pyslow5(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_pyslow5},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "pyslow5",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_kp_u_Batch, __pyx_k_Batch, sizeof(__pyx_k_Batch), 0, 1, 0, 0},
  {&__pyx_kp_u_Compression_type_rec_press_sig_p, __pyx_k_Compression_type_rec_press_sig_p, sizeof(__pyx_k_Compression_type_rec_press_sig_p), 0, 1, 0, 0},
  {&__pyx_kp_u_Creating_loading_index, __pyx_k_Creating_loading_index, sizeof(__pyx_k_Creating_loading_index), 0, 1, 0, 0},
  {&__pyx_n_s_DEBUG, __pyx_k_DEBUG, sizeof(__pyx_k_DEBUG), 0, 0, 1, 1},
  {&__pyx_kp_u_End_Of_File_reached, __pyx_k_End_Of_File_reached, sizeof(__pyx_k_End_Of_File_reached), 0, 1, 0, 0},
  {&__pyx_kp_u_FILE_mode, __pyx_k_FILE_mode, sizeof(__pyx_k_FILE_mode), 0, 1, 0, 0},
  {&__pyx_kp_u_File_could_not_be_opened_for_wri, __pyx_k_File_could_not_be_opened_for_wri, sizeof(__pyx_k_File_could_not_be_opened_for_wri), 0, 1, 0, 0},
  {&__pyx_kp_u_File_could_not_be_opened_for_wri_2, __pyx_k_File_could_not_be_opened_for_wri_2, sizeof(__pyx_k_File_could_not_be_opened_for_wri_2), 0, 1, 0, 0},
  {&__pyx_kp_u_File_unknown_open_method, __pyx_k_File_unknown_open_method, sizeof(__pyx_k_File_unknown_open_method), 0, 1, 0, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_u_NOT_IMPLEMENTED_YET_get_aux_type, __pyx_k_NOT_IMPLEMENTED_YET_get_aux_type, sizeof(__pyx_k_NOT_IMPLEMENTED_YET_get_aux_type), 0, 1, 0, 0},
  {&__pyx_kp_u_No_more_reads, __pyx_k_No_more_reads, sizeof(__pyx_k_No_more_reads), 0, 1, 0, 0},
  {&__pyx_kp_u_Not_writing_blow5_skipping_compr, __pyx_k_Not_writing_blow5_skipping_compr, sizeof(__pyx_k_Not_writing_blow5_skipping_compr), 0, 1, 0, 0},
  {&__pyx_kp_u_Number_of_read_groups, __pyx_k_Number_of_read_groups, sizeof(__pyx_k_Number_of_read_groups), 0, 1, 0, 0},
  {&__pyx_n_s_Open, __pyx_k_Open, sizeof(__pyx_k_Open), 0, 0, 1, 1},
  {&__pyx_n_s_Open__get_batches, __pyx_k_Open__get_batches, sizeof(__pyx_k_Open__get_batches), 0, 0, 1, 1},
  {&__pyx_n_s_Open__get_read_multi, __pyx_k_Open__get_read_multi, sizeof(__pyx_k_Open__get_read_multi), 0, 0, 1, 1},
  {&__pyx_n_s_Open_get_read_list, __pyx_k_Open_get_read_list, sizeof(__pyx_k_Open_get_read_list), 0, 0, 1, 1},
  {&__pyx_n_s_Open_get_read_list_multi, __pyx_k_Open_get_read_list_multi, sizeof(__pyx_k_Open_get_read_list_multi), 0, 0, 1, 1},
  {&__pyx_n_s_Open_seq_reads, __pyx_k_Open_seq_reads, sizeof(__pyx_k_Open_seq_reads), 0, 0, 1, 1},
  {&__pyx_n_s_Open_seq_reads_multi, __pyx_k_Open_seq_reads_multi, sizeof(__pyx_k_Open_seq_reads_multi), 0, 0, 1, 1},
  {&__pyx_kp_u_Please_use_only_the_following, __pyx_k_Please_use_only_the_following, sizeof(__pyx_k_Please_use_only_the_following), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_u_SLOW5_ERR_ARG, __pyx_k_SLOW5_ERR_ARG, sizeof(__pyx_k_SLOW5_ERR_ARG), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_EOF, __pyx_k_SLOW5_ERR_EOF, sizeof(__pyx_k_SLOW5_ERR_EOF), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_HDRPARSE, __pyx_k_SLOW5_ERR_HDRPARSE, sizeof(__pyx_k_SLOW5_ERR_HDRPARSE), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_IO, __pyx_k_SLOW5_ERR_IO, sizeof(__pyx_k_SLOW5_ERR_IO), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_MAGIC, __pyx_k_SLOW5_ERR_MAGIC, sizeof(__pyx_k_SLOW5_ERR_MAGIC), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_MEM, __pyx_k_SLOW5_ERR_MEM, sizeof(__pyx_k_SLOW5_ERR_MEM), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_NOAUX, __pyx_k_SLOW5_ERR_NOAUX, sizeof(__pyx_k_SLOW5_ERR_NOAUX), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_NOFLD, __pyx_k_SLOW5_ERR_NOFLD, sizeof(__pyx_k_SLOW5_ERR_NOFLD), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_NOIDX, __pyx_k_SLOW5_ERR_NOIDX, sizeof(__pyx_k_SLOW5_ERR_NOIDX), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_NOTFOUND, __pyx_k_SLOW5_ERR_NOTFOUND, sizeof(__pyx_k_SLOW5_ERR_NOTFOUND), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_OTH, __pyx_k_SLOW5_ERR_OTH, sizeof(__pyx_k_SLOW5_ERR_OTH), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_PRESS, __pyx_k_SLOW5_ERR_PRESS, sizeof(__pyx_k_SLOW5_ERR_PRESS), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_RECPARSE, __pyx_k_SLOW5_ERR_RECPARSE, sizeof(__pyx_k_SLOW5_ERR_RECPARSE), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_TRUNC, __pyx_k_SLOW5_ERR_TRUNC, sizeof(__pyx_k_SLOW5_ERR_TRUNC), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_TYPE, __pyx_k_SLOW5_ERR_TYPE, sizeof(__pyx_k_SLOW5_ERR_TYPE), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_UNK, __pyx_k_SLOW5_ERR_UNK, sizeof(__pyx_k_SLOW5_ERR_UNK), 0, 1, 0, 1},
  {&__pyx_n_u_SLOW5_ERR_VERSION, __pyx_k_SLOW5_ERR_VERSION, sizeof(__pyx_k_SLOW5_ERR_VERSION), 0, 1, 0, 1},
  {&__pyx_kp_u_Setting_up_batching, __pyx_k_Setting_up_batching, sizeof(__pyx_k_Setting_up_batching), 0, 1, 0, 0},
  {&__pyx_kp_u_Starting_rid_assignment, __pyx_k_Starting_rid_assignment, sizeof(__pyx_k_Starting_rid_assignment), 0, 1, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_u_Unable_to_set_compression, __pyx_k_Unable_to_set_compression, sizeof(__pyx_k_Unable_to_set_compression), 0, 1, 0, 0},
  {&__pyx_n_s_WARNING, __pyx_k_WARNING, sizeof(__pyx_k_WARNING), 0, 0, 1, 1},
  {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
  {&__pyx_kp_u__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 1, 0, 0},
  {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
  {&__pyx_n_u_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 1, 0, 1},
  {&__pyx_n_u_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 1, 0, 1},
  {&__pyx_kp_u_already_closed, __pyx_k_already_closed, sizeof(__pyx_k_already_closed), 0, 1, 0, 0},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_kp_u_asctime_s_levelname_s_message_s, __pyx_k_asctime_s_levelname_s_message_s, sizeof(__pyx_k_asctime_s_levelname_s_message_s), 0, 1, 0, 0},
  {&__pyx_n_u_asic_id, __pyx_k_asic_id, sizeof(__pyx_k_asic_id), 0, 1, 0, 1},
  {&__pyx_n_u_asic_id_eeprom, __pyx_k_asic_id_eeprom, sizeof(__pyx_k_asic_id_eeprom), 0, 1, 0, 1},
  {&__pyx_n_u_asic_temp, __pyx_k_asic_temp, sizeof(__pyx_k_asic_temp), 0, 1, 0, 1},
  {&__pyx_n_u_asic_version, __pyx_k_asic_version, sizeof(__pyx_k_asic_version), 0, 1, 0, 1},
  {&__pyx_n_s_astype, __pyx_k_astype, sizeof(__pyx_k_astype), 0, 0, 1, 1},
  {&__pyx_n_s_attr, __pyx_k_attr, sizeof(__pyx_k_attr), 0, 0, 1, 1},
  {&__pyx_n_u_auto_update, __pyx_k_auto_update, sizeof(__pyx_k_auto_update), 0, 1, 0, 1},
  {&__pyx_n_u_auto_update_source, __pyx_k_auto_update_source, sizeof(__pyx_k_auto_update_source), 0, 1, 0, 1},
  {&__pyx_n_s_aux, __pyx_k_aux, sizeof(__pyx_k_aux), 0, 0, 1, 1},
  {&__pyx_n_s_aux_header_type_validation, __pyx_k_aux_header_type_validation, sizeof(__pyx_k_aux_header_type_validation), 0, 0, 1, 1},
  {&__pyx_n_s_aux_names, __pyx_k_aux_names, sizeof(__pyx_k_aux_names), 0, 0, 1, 1},
  {&__pyx_n_u_aux_total_time, __pyx_k_aux_total_time, sizeof(__pyx_k_aux_total_time), 0, 1, 0, 1},
  {&__pyx_kp_u_aux_type_validation_user_aux_fi, __pyx_k_aux_type_validation_user_aux_fi, sizeof(__pyx_k_aux_type_validation_user_aux_fi), 0, 1, 0, 0},
  {&__pyx_kp_u_aux_type_validation_user_aux_fi_2, __pyx_k_aux_type_validation_user_aux_fi_2, sizeof(__pyx_k_aux_type_validation_user_aux_fi_2), 0, 1, 0, 0},
  {&__pyx_n_s_aux_types, __pyx_k_aux_types, sizeof(__pyx_k_aux_types), 0, 0, 1, 1},
  {&__pyx_kp_u_bad_argument_NULL, __pyx_k_bad_argument_NULL, sizeof(__pyx_k_bad_argument_NULL), 0, 1, 0, 0},
  {&__pyx_n_u_barcoding_enabled, __pyx_k_barcoding_enabled, sizeof(__pyx_k_barcoding_enabled), 0, 1, 0, 1},
  {&__pyx_n_s_basicConfig, __pyx_k_basicConfig, sizeof(__pyx_k_basicConfig), 0, 0, 1, 1},
  {&__pyx_kp_u_batch_for_loop_start, __pyx_k_batch_for_loop_start, sizeof(__pyx_k_batch_for_loop_start), 0, 1, 0, 0},
  {&__pyx_n_s_batchsize, __pyx_k_batchsize, sizeof(__pyx_k_batchsize), 0, 0, 1, 1},
  {&__pyx_n_u_blow5, __pyx_k_blow5, sizeof(__pyx_k_blow5), 0, 1, 0, 1},
  {&__pyx_n_u_bream_is_standard, __pyx_k_bream_is_standard, sizeof(__pyx_k_bream_is_standard), 0, 1, 0, 1},
  {&__pyx_n_s_chain, __pyx_k_chain, sizeof(__pyx_k_chain), 0, 0, 1, 1},
  {&__pyx_n_u_channel_number, __pyx_k_channel_number, sizeof(__pyx_k_channel_number), 0, 1, 0, 1},
  {&__pyx_n_s_chr, __pyx_k_chr, sizeof(__pyx_k_chr), 0, 0, 1, 1},
  {&__pyx_n_s_cinit___locals_genexpr, __pyx_k_cinit___locals_genexpr, sizeof(__pyx_k_cinit___locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_kp_u_closed, __pyx_k_closed, sizeof(__pyx_k_closed), 0, 1, 0, 0},
  {&__pyx_n_u_configuration_version, __pyx_k_configuration_version, sizeof(__pyx_k_configuration_version), 0, 1, 0, 1},
  {&__pyx_n_s_convert_to_pA, __pyx_k_convert_to_pA, sizeof(__pyx_k_convert_to_pA), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_kp_u_d_b_y_H_M_S, __pyx_k_d_b_y_H_M_S, sizeof(__pyx_k_d_b_y_H_M_S), 0, 1, 0, 0},
  {&__pyx_n_s_datefmt, __pyx_k_datefmt, sizeof(__pyx_k_datefmt), 0, 0, 1, 1},
  {&__pyx_kp_u_de_compression_failure, __pyx_k_de_compression_failure, sizeof(__pyx_k_de_compression_failure), 0, 1, 0, 0},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_deepcopy, __pyx_k_deepcopy, sizeof(__pyx_k_deepcopy), 0, 0, 1, 1},
  {&__pyx_n_u_device_id, __pyx_k_device_id, sizeof(__pyx_k_device_id), 0, 1, 0, 1},
  {&__pyx_n_u_device_type, __pyx_k_device_type, sizeof(__pyx_k_device_type), 0, 1, 0, 1},
  {&__pyx_n_s_difference, __pyx_k_difference, sizeof(__pyx_k_difference), 0, 0, 1, 1},
  {&__pyx_n_u_digitisation, __pyx_k_digitisation, sizeof(__pyx_k_digitisation), 0, 1, 0, 1},
  {&__pyx_n_u_distribution_status, __pyx_k_distribution_status, sizeof(__pyx_k_distribution_status), 0, 1, 0, 1},
  {&__pyx_n_u_distribution_version, __pyx_k_distribution_version, sizeof(__pyx_k_distribution_version), 0, 1, 0, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_u_end_reason, __pyx_k_end_reason, sizeof(__pyx_k_end_reason), 0, 1, 0, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_kp_u_error_relating_to_slow5_type, __pyx_k_error_relating_to_slow5_type, sizeof(__pyx_k_error_relating_to_slow5_type), 0, 1, 0, 0},
  {&__pyx_n_u_exp_script_name, __pyx_k_exp_script_name, sizeof(__pyx_k_exp_script_name), 0, 1, 0, 1},
  {&__pyx_n_u_exp_script_purpose, __pyx_k_exp_script_purpose, sizeof(__pyx_k_exp_script_purpose), 0, 1, 0, 1},
  {&__pyx_n_u_exp_start_time, __pyx_k_exp_start_time, sizeof(__pyx_k_exp_start_time), 0, 1, 0, 1},
  {&__pyx_n_u_experiment_duration_set, __pyx_k_experiment_duration_set, sizeof(__pyx_k_experiment_duration_set), 0, 1, 0, 1},
  {&__pyx_kp_u_field_not_found, __pyx_k_field_not_found, sizeof(__pyx_k_field_not_found), 0, 1, 0, 0},
  {&__pyx_kp_u_file_format_unknown, __pyx_k_file_format_unknown, sizeof(__pyx_k_file_format_unknown), 0, 1, 0, 0},
  {&__pyx_kp_u_file_truncated, __pyx_k_file_truncated, sizeof(__pyx_k_file_truncated), 0, 1, 0, 0},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_float32, __pyx_k_float32, sizeof(__pyx_k_float32), 0, 0, 1, 1},
  {&__pyx_n_u_flow_cell_id, __pyx_k_flow_cell_id, sizeof(__pyx_k_flow_cell_id), 0, 1, 0, 1},
  {&__pyx_n_u_flow_cell_product_code, __pyx_k_flow_cell_product_code, sizeof(__pyx_k_flow_cell_product_code), 0, 1, 0, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_getLogger, __pyx_k_getLogger, sizeof(__pyx_k_getLogger), 0, 0, 1, 1},
  {&__pyx_n_s_get_aux_names, __pyx_k_get_aux_names, sizeof(__pyx_k_get_aux_names), 0, 0, 1, 1},
  {&__pyx_kp_u_get_aux_names_aux_len, __pyx_k_get_aux_names_aux_len, sizeof(__pyx_k_get_aux_names_aux_len), 0, 1, 0, 0},
  {&__pyx_kp_u_get_aux_names_ret_is_NULL, __pyx_k_get_aux_names_ret_is_NULL, sizeof(__pyx_k_get_aux_names_ret_is_NULL), 0, 1, 0, 0},
  {&__pyx_kp_u_get_aux_names_starting, __pyx_k_get_aux_names_starting, sizeof(__pyx_k_get_aux_names_starting), 0, 1, 0, 0},
  {&__pyx_n_s_get_aux_types, __pyx_k_get_aux_types, sizeof(__pyx_k_get_aux_types), 0, 0, 1, 1},
  {&__pyx_kp_u_get_aux_types_aux_len, __pyx_k_get_aux_types_aux_len, sizeof(__pyx_k_get_aux_types_aux_len), 0, 1, 0, 0},
  {&__pyx_kp_u_get_aux_types_self_aux_get_err_i, __pyx_k_get_aux_types_self_aux_get_err_i, sizeof(__pyx_k_get_aux_types_self_aux_get_err_i), 0, 1, 0, 0},
  {&__pyx_kp_u_get_aux_types_self_s5_aux_type_i, __pyx_k_get_aux_types_self_s5_aux_type_i, sizeof(__pyx_k_get_aux_types_self_s5_aux_type_i), 0, 1, 0, 0},
  {&__pyx_kp_u_get_aux_types_starting, __pyx_k_get_aux_types_starting, sizeof(__pyx_k_get_aux_types_starting), 0, 1, 0, 0},
  {&__pyx_n_s_get_batches, __pyx_k_get_batches, sizeof(__pyx_k_get_batches), 0, 0, 1, 1},
  {&__pyx_n_s_get_header_names, __pyx_k_get_header_names, sizeof(__pyx_k_get_header_names), 0, 0, 1, 1},
  {&__pyx_n_s_get_header_value, __pyx_k_get_header_value, sizeof(__pyx_k_get_header_value), 0, 0, 1, 1},
  {&__pyx_kp_u_get_header_value_header_value_no, __pyx_k_get_header_value_header_value_no, sizeof(__pyx_k_get_header_value_header_value_no), 0, 1, 0, 0},
  {&__pyx_n_s_get_read, __pyx_k_get_read, sizeof(__pyx_k_get_read), 0, 0, 1, 1},
  {&__pyx_n_s_get_read_aux, __pyx_k_get_read_aux, sizeof(__pyx_k_get_read_aux), 0, 0, 1, 1},
  {&__pyx_kp_u_get_read_aux_atype_not_known_ski, __pyx_k_get_read_aux_atype_not_known_ski, sizeof(__pyx_k_get_read_aux_atype_not_known_ski), 0, 1, 0, 0},
  {&__pyx_kp_u_get_read_aux_type_unknown_accept, __pyx_k_get_read_aux_type_unknown_accept, sizeof(__pyx_k_get_read_aux_type_unknown_accept), 0, 1, 0, 0},
  {&__pyx_kp_u_get_read_ids_length_of_read_ids, __pyx_k_get_read_ids_length_of_read_ids, sizeof(__pyx_k_get_read_ids_length_of_read_ids), 0, 1, 0, 0},
  {&__pyx_kp_u_get_read_ids_returned_rids_is_NU, __pyx_k_get_read_ids_returned_rids_is_NU, sizeof(__pyx_k_get_read_ids_returned_rids_is_NU), 0, 1, 0, 0},
  {&__pyx_n_s_get_read_list, __pyx_k_get_read_list, sizeof(__pyx_k_get_read_list), 0, 0, 1, 1},
  {&__pyx_n_s_get_read_list_multi, __pyx_k_get_read_list_multi, sizeof(__pyx_k_get_read_list_multi), 0, 0, 1, 1},
  {&__pyx_n_s_get_read_multi, __pyx_k_get_read_multi, sizeof(__pyx_k_get_read_multi), 0, 0, 1, 1},
  {&__pyx_kp_u_get_read_multi_slow5_get_batch_r, __pyx_k_get_read_multi_slow5_get_batch_r, sizeof(__pyx_k_get_read_multi_slow5_get_batch_r), 0, 1, 0, 0},
  {&__pyx_kp_u_get_read_return_not_0, __pyx_k_get_read_return_not_0, sizeof(__pyx_k_get_read_return_not_0), 0, 1, 0, 0},
  {&__pyx_kp_u_get_read_unknown_aux_name, __pyx_k_get_read_unknown_aux_name, sizeof(__pyx_k_get_read_unknown_aux_name), 0, 1, 0, 0},
  {&__pyx_n_s_get_seq_read_aux, __pyx_k_get_seq_read_aux, sizeof(__pyx_k_get_seq_read_aux), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_u_guppy_version, __pyx_k_guppy_version, sizeof(__pyx_k_guppy_version), 0, 1, 0, 1},
  {&__pyx_n_s_header, __pyx_k_header, sizeof(__pyx_k_header), 0, 0, 1, 1},
  {&__pyx_kp_u_header_parsing_error, __pyx_k_header_parsing_error, sizeof(__pyx_k_header_parsing_error), 0, 1, 0, 0},
  {&__pyx_n_s_header_type_validation, __pyx_k_header_type_validation, sizeof(__pyx_k_header_type_validation), 0, 0, 1, 1},
  {&__pyx_kp_u_header_type_validation_conversi, __pyx_k_header_type_validation_conversi, sizeof(__pyx_k_header_type_validation_conversi), 0, 1, 0, 0},
  {&__pyx_kp_u_header_type_validation_could_no, __pyx_k_header_type_validation_could_no, sizeof(__pyx_k_header_type_validation_could_no), 0, 1, 0, 0},
  {&__pyx_kp_u_header_type_validation_is_not_a, __pyx_k_header_type_validation_is_not_a, sizeof(__pyx_k_header_type_validation_is_not_a), 0, 1, 0, 0},
  {&__pyx_n_u_heatsink_temp, __pyx_k_heatsink_temp, sizeof(__pyx_k_heatsink_temp), 0, 1, 0, 1},
  {&__pyx_n_u_hostname, __pyx_k_hostname, sizeof(__pyx_k_hostname), 0, 1, 0, 1},
  {&__pyx_n_u_hublett_board_id, __pyx_k_hublett_board_id, sizeof(__pyx_k_hublett_board_id), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_kp_u_index_not_loaded, __pyx_k_index_not_loaded, sizeof(__pyx_k_index_not_loaded), 0, 1, 0, 0},
  {&__pyx_n_u_installation_type, __pyx_k_installation_type, sizeof(__pyx_k_installation_type), 0, 1, 0, 1},
  {&__pyx_n_s_int16, __pyx_k_int16, sizeof(__pyx_k_int16), 0, 0, 1, 1},
  {&__pyx_n_s_itertools, __pyx_k_itertools, sizeof(__pyx_k_itertools), 0, 0, 1, 1},
  {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
  {&__pyx_n_u_len_raw_signal, __pyx_k_len_raw_signal, sizeof(__pyx_k_len_raw_signal), 0, 1, 0, 1},
  {&__pyx_n_s_level, __pyx_k_level, sizeof(__pyx_k_level), 0, 0, 1, 1},
  {&__pyx_n_u_local_basecalling, __pyx_k_local_basecalling, sizeof(__pyx_k_local_basecalling), 0, 1, 0, 1},
  {&__pyx_n_s_logging, __pyx_k_logging, sizeof(__pyx_k_logging), 0, 0, 1, 1},
  {&__pyx_kp_u_magic_number_invalid, __pyx_k_magic_number_invalid, sizeof(__pyx_k_magic_number_invalid), 0, 1, 0, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_u_median_before, __pyx_k_median_before, sizeof(__pyx_k_median_before), 0, 1, 0, 1},
  {&__pyx_kp_u_memory_re_allocation_error, __pyx_k_memory_re_allocation_error, sizeof(__pyx_k_memory_re_allocation_error), 0, 1, 0, 0},
  {&__pyx_n_s_memoryview, __pyx_k_memoryview, sizeof(__pyx_k_memoryview), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_multi_record_type_validation, __pyx_k_multi_record_type_validation, sizeof(__pyx_k_multi_record_type_validation), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_u_no_auxiliary_map, __pyx_k_no_auxiliary_map, sizeof(__pyx_k_no_auxiliary_map), 0, 1, 0, 0},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_u_none, __pyx_k_none, sizeof(__pyx_k_none), 0, 1, 0, 1},
  {&__pyx_kp_u_not_open_for_writing_remake_cla, __pyx_k_not_open_for_writing_remake_cla, sizeof(__pyx_k_not_open_for_writing_remake_cla), 0, 1, 0, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_num, __pyx_k_num, sizeof(__pyx_k_num), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_u_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 1, 0, 1},
  {&__pyx_n_u_operating_system, __pyx_k_operating_system, sizeof(__pyx_k_operating_system), 0, 1, 0, 1},
  {&__pyx_kp_u_other_error_big_endian_internal, __pyx_k_other_error_big_endian_internal, sizeof(__pyx_k_other_error_big_endian_internal), 0, 1, 0, 0},
  {&__pyx_kp_u_other_file_I_O_error, __pyx_k_other_file_I_O_error, sizeof(__pyx_k_other_file_I_O_error), 0, 1, 0, 0},
  {&__pyx_n_s_pA, __pyx_k_pA, sizeof(__pyx_k_pA), 0, 0, 1, 1},
  {&__pyx_n_u_pA_total_time, __pyx_k_pA_total_time, sizeof(__pyx_k_pA_total_time), 0, 1, 0, 1},
  {&__pyx_n_u_package, __pyx_k_package, sizeof(__pyx_k_package), 0, 1, 0, 1},
  {&__pyx_n_s_pathname, __pyx_k_pathname, sizeof(__pyx_k_pathname), 0, 0, 1, 1},
  {&__pyx_kp_u_pathname_2, __pyx_k_pathname_2, sizeof(__pyx_k_pathname_2), 0, 1, 0, 0},
  {&__pyx_n_u_primary_total_time, __pyx_k_primary_total_time, sizeof(__pyx_k_primary_total_time), 0, 1, 0, 1},
  {&__pyx_n_u_protocol_group_id, __pyx_k_protocol_group_id, sizeof(__pyx_k_protocol_group_id), 0, 1, 0, 1},
  {&__pyx_n_u_protocol_run_id, __pyx_k_protocol_run_id, sizeof(__pyx_k_protocol_run_id), 0, 1, 0, 1},
  {&__pyx_n_u_protocol_start_time, __pyx_k_protocol_start_time, sizeof(__pyx_k_protocol_start_time), 0, 1, 0, 1},
  {&__pyx_n_u_protocols_version, __pyx_k_protocols_version, sizeof(__pyx_k_protocols_version), 0, 1, 0, 1},
  {&__pyx_n_s_pyslow5, __pyx_k_pyslow5, sizeof(__pyx_k_pyslow5), 0, 0, 1, 1},
  {&__pyx_n_u_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 1, 0, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_u_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 1, 0, 1},
  {&__pyx_kp_u_readID_bin, __pyx_k_readID_bin, sizeof(__pyx_k_readID_bin), 0, 1, 0, 0},
  {&__pyx_n_s_read_group, __pyx_k_read_group, sizeof(__pyx_k_read_group), 0, 0, 1, 1},
  {&__pyx_n_u_read_group, __pyx_k_read_group, sizeof(__pyx_k_read_group), 0, 1, 0, 1},
  {&__pyx_n_s_read_id, __pyx_k_read_id, sizeof(__pyx_k_read_id), 0, 0, 1, 1},
  {&__pyx_n_u_read_id, __pyx_k_read_id, sizeof(__pyx_k_read_id), 0, 1, 0, 1},
  {&__pyx_kp_u_read_id_not_found, __pyx_k_read_id_not_found, sizeof(__pyx_k_read_id_not_found), 0, 1, 0, 0},
  {&__pyx_n_s_read_list, __pyx_k_read_list, sizeof(__pyx_k_read_list), 0, 0, 1, 1},
  {&__pyx_kp_u_read_list_2, __pyx_k_read_list_2, sizeof(__pyx_k_read_list_2), 0, 1, 0, 0},
  {&__pyx_n_u_read_number, __pyx_k_read_number, sizeof(__pyx_k_read_number), 0, 1, 0, 1},
  {&__pyx_n_s_rec_press, __pyx_k_rec_press, sizeof(__pyx_k_rec_press), 0, 0, 1, 1},
  {&__pyx_n_s_record, __pyx_k_record, sizeof(__pyx_k_record), 0, 0, 1, 1},
  {&__pyx_kp_u_record_parsing_error, __pyx_k_record_parsing_error, sizeof(__pyx_k_record_parsing_error), 0, 1, 0, 0},
  {&__pyx_n_s_record_type_validation, __pyx_k_record_type_validation, sizeof(__pyx_k_record_type_validation), 0, 0, 1, 1},
  {&__pyx_kp_u_record_type_validation_aux_pass, __pyx_k_record_type_validation_aux_pass, sizeof(__pyx_k_record_type_validation_aux_pass), 0, 1, 0, 0},
  {&__pyx_kp_u_record_type_validation_aux_stuf, __pyx_k_record_type_validation_aux_stuf, sizeof(__pyx_k_record_type_validation_aux_stuf), 0, 1, 0, 0},
  {&__pyx_kp_u_record_type_validation_doing_au, __pyx_k_record_type_validation_doing_au, sizeof(__pyx_k_record_type_validation_doing_au), 0, 1, 0, 0},
  {&__pyx_kp_u_record_type_validation_user_aux, __pyx_k_record_type_validation_user_aux, sizeof(__pyx_k_record_type_validation_user_aux), 0, 1, 0, 0},
  {&__pyx_kp_u_record_type_validation_user_aux_2, __pyx_k_record_type_validation_user_aux_2, sizeof(__pyx_k_record_type_validation_user_aux_2), 0, 1, 0, 0},
  {&__pyx_kp_u_record_type_validation_user_pri, __pyx_k_record_type_validation_user_pri, sizeof(__pyx_k_record_type_validation_user_pri), 0, 1, 0, 0},
  {&__pyx_kp_u_record_type_validation_user_pri_2, __pyx_k_record_type_validation_user_pri_2, sizeof(__pyx_k_record_type_validation_user_pri_2), 0, 1, 0, 0},
  {&__pyx_n_s_records, __pyx_k_records, sizeof(__pyx_k_records), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_kp_u_rid_assignment_complete, __pyx_k_rid_assignment_complete, sizeof(__pyx_k_rid_assignment_complete), 0, 1, 0, 0},
  {&__pyx_n_u_run_id, __pyx_k_run_id, sizeof(__pyx_k_run_id), 0, 1, 0, 1},
  {&__pyx_n_u_sample_frequency, __pyx_k_sample_frequency, sizeof(__pyx_k_sample_frequency), 0, 1, 0, 1},
  {&__pyx_n_u_sample_id, __pyx_k_sample_id, sizeof(__pyx_k_sample_id), 0, 1, 0, 1},
  {&__pyx_n_u_sampling_rate, __pyx_k_sampling_rate, sizeof(__pyx_k_sampling_rate), 0, 1, 0, 1},
  {&__pyx_n_u_satellite_firmware_version, __pyx_k_satellite_firmware_version, sizeof(__pyx_k_satellite_firmware_version), 0, 1, 0, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_seq_reads, __pyx_k_seq_reads, sizeof(__pyx_k_seq_reads), 0, 0, 1, 1},
  {&__pyx_n_s_seq_reads_multi, __pyx_k_seq_reads_multi, sizeof(__pyx_k_seq_reads_multi), 0, 0, 1, 1},
  {&__pyx_kp_u_seq_reads_multi_timings, __pyx_k_seq_reads_multi_timings, sizeof(__pyx_k_seq_reads_multi_timings), 0, 1, 0, 0},
  {&__pyx_n_u_sequencing_kit, __pyx_k_sequencing_kit, sizeof(__pyx_k_sequencing_kit), 0, 1, 0, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_sig_press, __pyx_k_sig_press, sizeof(__pyx_k_sig_press), 0, 0, 1, 1},
  {&__pyx_n_u_signal, __pyx_k_signal, sizeof(__pyx_k_signal), 0, 1, 0, 1},
  {&__pyx_n_u_signal_total_time, __pyx_k_signal_total_time, sizeof(__pyx_k_signal_total_time), 0, 1, 0, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_kp_u_slow5_get_batch_num_reads, __pyx_k_slow5_get_batch_num_reads, sizeof(__pyx_k_slow5_get_batch_num_reads), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_hdr_keys_head_len, __pyx_k_slow5_get_hdr_keys_head_len, sizeof(__pyx_k_slow5_get_hdr_keys_head_len), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_hdr_keys_ret_is_NULL, __pyx_k_slow5_get_hdr_keys_ret_is_NULL, sizeof(__pyx_k_slow5_get_hdr_keys_ret_is_NULL), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_error_code, __pyx_k_slow5_get_next_error_code, sizeof(__pyx_k_slow5_get_next_error_code), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_multi_aux_type_un, __pyx_k_slow5_get_next_multi_aux_type_un, sizeof(__pyx_k_slow5_get_next_multi_aux_type_un), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_multi_error_code, __pyx_k_slow5_get_next_multi_error_code, sizeof(__pyx_k_slow5_get_next_multi_error_code), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_multi_has_no_more, __pyx_k_slow5_get_next_multi_has_no_more, sizeof(__pyx_k_slow5_get_next_multi_has_no_more), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_multi_reached_end, __pyx_k_slow5_get_next_multi_reached_end, sizeof(__pyx_k_slow5_get_next_multi_reached_end), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_multi_return, __pyx_k_slow5_get_next_multi_return, sizeof(__pyx_k_slow5_get_next_multi_return), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_multi_unknown_aux, __pyx_k_slow5_get_next_multi_unknown_aux, sizeof(__pyx_k_slow5_get_next_multi_unknown_aux), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_reached_end_of_fi, __pyx_k_slow5_get_next_reached_end_of_fi, sizeof(__pyx_k_slow5_get_next_reached_end_of_fi), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_get_next_return, __pyx_k_slow5_get_next_return, sizeof(__pyx_k_slow5_get_next_return), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_idx_load_return_not_0, __pyx_k_slow5_idx_load_return_not_0, sizeof(__pyx_k_slow5_idx_load_return_not_0), 0, 1, 0, 0},
  {&__pyx_kp_u_slow5_set_press_return_not_0, __pyx_k_slow5_set_press_return_not_0, sizeof(__pyx_k_slow5_set_press_return_not_0), 0, 1, 0, 0},
  {&__pyx_n_u_start_mux, __pyx_k_start_mux, sizeof(__pyx_k_start_mux), 0, 1, 0, 1},
  {&__pyx_n_u_start_time, __pyx_k_start_time, sizeof(__pyx_k_start_time), 0, 1, 0, 1},
  {&__pyx_n_u_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 1, 0, 1},
  {&__pyx_n_u_svb_zd, __pyx_k_svb_zd, sizeof(__pyx_k_svb_zd), 0, 1, 0, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_threads, __pyx_k_threads, sizeof(__pyx_k_threads), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_kp_u_total_multi_write_signal_time_se, __pyx_k_total_multi_write_signal_time_se, sizeof(__pyx_k_total_multi_write_signal_time_se), 0, 1, 0, 0},
  {&__pyx_kp_u_total_multi_write_time_seconds, __pyx_k_total_multi_write_time_seconds, sizeof(__pyx_k_total_multi_write_time_seconds), 0, 1, 0, 0},
  {&__pyx_kp_u_total_single_write_time_seconds, __pyx_k_total_single_write_time_seconds, sizeof(__pyx_k_total_single_write_time_seconds), 0, 1, 0, 0},
  {&__pyx_kp_u_total_time_slow5_get_next_second, __pyx_k_total_time_slow5_get_next_second, sizeof(__pyx_k_total_time_slow5_get_next_second), 0, 1, 0, 0},
  {&__pyx_kp_u_total_time_yield_reads_seconds, __pyx_k_total_time_yield_reads_seconds, sizeof(__pyx_k_total_time_yield_reads_seconds), 0, 1, 0, 0},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_u_usb_config, __pyx_k_usb_config, sizeof(__pyx_k_usb_config), 0, 1, 0, 1},
  {&__pyx_n_s_user_record, __pyx_k_user_record, sizeof(__pyx_k_user_record), 0, 0, 1, 1},
  {&__pyx_n_u_version, __pyx_k_version, sizeof(__pyx_k_version), 0, 1, 0, 1},
  {&__pyx_kp_u_version_incompatible, __pyx_k_version_incompatible, sizeof(__pyx_k_version_incompatible), 0, 1, 0, 0},
  {&__pyx_n_u_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 1, 0, 1},
  {&__pyx_n_s_warning, __pyx_k_warning, sizeof(__pyx_k_warning), 0, 0, 1, 1},
  {&__pyx_kp_u_write_header_You_must_set_read_g, __pyx_k_write_header_You_must_set_read_g, sizeof(__pyx_k_write_header_You_must_set_read_g), 0, 1, 0, 0},
  {&__pyx_kp_u_write_header_errors_encountered, __pyx_k_write_header_errors_encountered, sizeof(__pyx_k_write_header_errors_encountered), 0, 1, 0, 0},
  {&__pyx_kp_u_write_header_headers_failed_to_i, __pyx_k_write_header_headers_failed_to_i, sizeof(__pyx_k_write_header_headers_failed_to_i), 0, 1, 0, 0},
  {&__pyx_kp_u_write_header_slow5_hdr_add_attr, __pyx_k_write_header_slow5_hdr_add_attr, sizeof(__pyx_k_write_header_slow5_hdr_add_attr), 0, 1, 0, 0},
  {&__pyx_kp_u_write_header_slow5_hdr_add_rg_fa, __pyx_k_write_header_slow5_hdr_add_rg_fa, sizeof(__pyx_k_write_header_slow5_hdr_add_rg_fa), 0, 1, 0, 0},
  {&__pyx_kp_u_write_header_slow5_hdr_set_could, __pyx_k_write_header_slow5_hdr_set_could, sizeof(__pyx_k_write_header_slow5_hdr_set_could), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record__record_type_valida, __pyx_k_write_record__record_type_valida, sizeof(__pyx_k_write_record__record_type_valida), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record__record_type_valida_2, __pyx_k_write_record__record_type_valida_2, sizeof(__pyx_k_write_record__record_type_valida_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_aux_meta_fields_fai, __pyx_k_write_record_aux_meta_fields_fai, sizeof(__pyx_k_write_record_aux_meta_fields_fai), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_aux_stuff, __pyx_k_write_record_aux_stuff, sizeof(__pyx_k_write_record_aux_stuff), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_aux_stuff_done, __pyx_k_write_record_aux_stuff_done, sizeof(__pyx_k_write_record_aux_stuff_done), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_Setting_up_ba, __pyx_k_write_record_batch_Setting_up_ba, sizeof(__pyx_k_write_record_batch_Setting_up_ba), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch__record_type, __pyx_k_write_record_batch__record_type, sizeof(__pyx_k_write_record_batch__record_type), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch__record_type_2, __pyx_k_write_record_batch__record_type_2, sizeof(__pyx_k_write_record_batch__record_type_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_aux_meta_fiel, __pyx_k_write_record_batch_aux_meta_fiel, sizeof(__pyx_k_write_record_batch_aux_meta_fiel), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_aux_stuff, __pyx_k_write_record_batch_aux_stuff, sizeof(__pyx_k_write_record_batch_aux_stuff), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_aux_stuff_don, __pyx_k_write_record_batch_aux_stuff_don, sizeof(__pyx_k_write_record_batch_aux_stuff_don), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_batch_for_loo, __pyx_k_write_record_batch_batch_for_loo, sizeof(__pyx_k_write_record_batch_batch_for_loo), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_batch_len_0_o, __pyx_k_write_record_batch_batch_len_0_o, sizeof(__pyx_k_write_record_batch_batch_len_0_o), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_checked_aux, __pyx_k_write_record_batch_checked_aux, sizeof(__pyx_k_write_record_batch_checked_aux), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_checked_aux_i, __pyx_k_write_record_batch_checked_aux_i, sizeof(__pyx_k_write_record_batch_checked_aux_i), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_checking_head, __pyx_k_write_record_batch_checking_head, sizeof(__pyx_k_write_record_batch_checking_head), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_failed_to_all, __pyx_k_write_record_batch_failed_to_all, sizeof(__pyx_k_write_record_batch_failed_to_all), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_free, __pyx_k_write_record_batch_free, sizeof(__pyx_k_write_record_batch_free), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_free_aux, __pyx_k_write_record_batch_free_aux, sizeof(__pyx_k_write_record_batch_free_aux), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_function_comp, __pyx_k_write_record_batch_function_comp, sizeof(__pyx_k_write_record_batch_function_comp), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_header_writte, __pyx_k_write_record_batch_header_writte, sizeof(__pyx_k_write_record_batch_header_writte), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_self_write_as, __pyx_k_write_record_batch_self_write_as, sizeof(__pyx_k_write_record_batch_self_write_as), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_self_write_as_2, __pyx_k_write_record_batch_self_write_as_2, sizeof(__pyx_k_write_record_batch_self_write_as_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_self_write_pr, __pyx_k_write_record_batch_self_write_pr, sizeof(__pyx_k_write_record_batch_self_write_pr), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_self_write_ra, __pyx_k_write_record_batch_self_write_ra, sizeof(__pyx_k_write_record_batch_self_write_ra), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_self_write_ra_2, __pyx_k_write_record_batch_self_write_ra_2, sizeof(__pyx_k_write_record_batch_self_write_ra_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_self_write_ra_3, __pyx_k_write_record_batch_self_write_ra_3, sizeof(__pyx_k_write_record_batch_self_write_ra_3), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_aux_add, __pyx_k_write_record_batch_slow5_aux_add, sizeof(__pyx_k_write_record_batch_slow5_aux_add), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_aux_set, __pyx_k_write_record_batch_slow5_aux_set, sizeof(__pyx_k_write_record_batch_slow5_aux_set), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_aux_set_2, __pyx_k_write_record_batch_slow5_aux_set_2, sizeof(__pyx_k_write_record_batch_slow5_aux_set_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_aux_set_3, __pyx_k_write_record_batch_slow5_aux_set_3, sizeof(__pyx_k_write_record_batch_slow5_aux_set_3), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_aux_set_4, __pyx_k_write_record_batch_slow5_aux_set_4, sizeof(__pyx_k_write_record_batch_slow5_aux_set_4), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_aux_set_5, __pyx_k_write_record_batch_slow5_aux_set_5, sizeof(__pyx_k_write_record_batch_slow5_aux_set_5), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_hdr_wri, __pyx_k_write_record_batch_slow5_hdr_wri, sizeof(__pyx_k_write_record_batch_slow5_hdr_wri), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_rec_ini, __pyx_k_write_record_batch_slow5_rec_ini, sizeof(__pyx_k_write_record_batch_slow5_rec_ini), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_slow5_write_b, __pyx_k_write_record_batch_slow5_write_b, sizeof(__pyx_k_write_record_batch_slow5_write_b), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_write_failed, __pyx_k_write_record_batch_write_failed, sizeof(__pyx_k_write_record_batch_write_failed), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_batch_writting_head, __pyx_k_write_record_batch_writting_head, sizeof(__pyx_k_write_record_batch_writting_head), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_checked_aux, __pyx_k_write_record_checked_aux, sizeof(__pyx_k_write_record_checked_aux), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_checked_aux_is_None, __pyx_k_write_record_checked_aux_is_None, sizeof(__pyx_k_write_record_checked_aux_is_None), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_checking_header_stu, __pyx_k_write_record_checking_header_stu, sizeof(__pyx_k_write_record_checking_header_stu), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_failed_to_allocate, __pyx_k_write_record_failed_to_allocate, sizeof(__pyx_k_write_record_failed_to_allocate), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_function_complete_r, __pyx_k_write_record_function_complete_r, sizeof(__pyx_k_write_record_function_complete_r), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_header_written, __pyx_k_write_record_header_written, sizeof(__pyx_k_write_record_header_written), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_self_write_assignme, __pyx_k_write_record_self_write_assignme, sizeof(__pyx_k_write_record_self_write_assignme), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_self_write_assignme_2, __pyx_k_write_record_self_write_assignme_2, sizeof(__pyx_k_write_record_self_write_assignme_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_self_write_processi, __pyx_k_write_record_self_write_processi, sizeof(__pyx_k_write_record_self_write_processi), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_self_write_raw_sign, __pyx_k_write_record_self_write_raw_sign, sizeof(__pyx_k_write_record_self_write_raw_sign), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_self_write_raw_sign_2, __pyx_k_write_record_self_write_raw_sign_2, sizeof(__pyx_k_write_record_self_write_raw_sign_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_self_write_raw_sign_3, __pyx_k_write_record_self_write_raw_sign_3, sizeof(__pyx_k_write_record_self_write_raw_sign_3), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_aux_add_could, __pyx_k_write_record_slow5_aux_add_could, sizeof(__pyx_k_write_record_slow5_aux_add_could), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_aux_set_could, __pyx_k_write_record_slow5_aux_set_could, sizeof(__pyx_k_write_record_slow5_aux_set_could), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_aux_set_strin, __pyx_k_write_record_slow5_aux_set_strin, sizeof(__pyx_k_write_record_slow5_aux_set_strin), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_aux_set_strin_2, __pyx_k_write_record_slow5_aux_set_strin_2, sizeof(__pyx_k_write_record_slow5_aux_set_strin_2), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_aux_set_strin_3, __pyx_k_write_record_slow5_aux_set_strin_3, sizeof(__pyx_k_write_record_slow5_aux_set_strin_3), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_aux_set_strin_4, __pyx_k_write_record_slow5_aux_set_strin_4, sizeof(__pyx_k_write_record_slow5_aux_set_strin_4), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_hdr_write_cou, __pyx_k_write_record_slow5_hdr_write_cou, sizeof(__pyx_k_write_record_slow5_hdr_write_cou), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_rec_free, __pyx_k_write_record_slow5_rec_free, sizeof(__pyx_k_write_record_slow5_rec_free), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_rec_init, __pyx_k_write_record_slow5_rec_init, sizeof(__pyx_k_write_record_slow5_rec_init), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_write, __pyx_k_write_record_slow5_write, sizeof(__pyx_k_write_record_slow5_write), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_slow5_write_ret, __pyx_k_write_record_slow5_write_ret, sizeof(__pyx_k_write_record_slow5_write_ret), 0, 1, 0, 0},
  {&__pyx_kp_u_write_record_writting_header, __pyx_k_write_record_writting_header, sizeof(__pyx_k_write_record_writting_header), 0, 1, 0, 0},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {&__pyx_n_u_zlib, __pyx_k_zlib, sizeof(__pyx_k_zlib), 0, 1, 0, 1},
  {&__pyx_n_u_zstd, __pyx_k_zstd, sizeof(__pyx_k_zstd), 0, 1, 0, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(0, 257, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 272, __pyx_L1_error)
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) __PYX_ERR(0, 383, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 446, __pyx_L1_error)
  __pyx_builtin_chr = __Pyx_GetBuiltinName(__pyx_n_s_chr); if (!__pyx_builtin_chr) __PYX_ERR(0, 782, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 1850, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 945, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pyslow5.pyx":257
 *                     if ret != 0:
 *                         self.logger.error("slow5_set_press return not 0: {}".format(ret))
 *                         raise RuntimeError("Unable to set compression")             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.logger.error("Compression type rec_press: {}, sig_press: {} could not be found.".format(self.rec_press, self.sig_press))
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_Unable_to_set_compression); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "pyslow5.pyx":1507
 *             if a not in py_record_types:
 *                 self.logger.error("_record_type_validation {}: {} user primary field not in pyslow5 record_types".format(a, user_record[a]))
 *                 return None, None             # <<<<<<<<<<<<<<
 * 
 *             if type(user_record[a]) != py_record_types[a]:
 */
  __pyx_tuple__6 = PyTuple_Pack(2, Py_None, Py_None); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 1507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":945
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(2, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":951
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(2, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_keys.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_update.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PySet_Type_difference.type = (PyObject*)&PySet_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_neg_1_0 = PyFloat_FromDouble(-1.0); if (unlikely(!__pyx_float_neg_1_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_7 = PyInt_FromLong(7); if (unlikely(!__pyx_int_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_9 = PyInt_FromLong(9); if (unlikely(!__pyx_int_9)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_10 = PyInt_FromLong(10); if (unlikely(!__pyx_int_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_11 = PyInt_FromLong(11); if (unlikely(!__pyx_int_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_12 = PyInt_FromLong(12); if (unlikely(!__pyx_int_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_13 = PyInt_FromLong(13); if (unlikely(!__pyx_int_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_14 = PyInt_FromLong(14); if (unlikely(!__pyx_int_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_15 = PyInt_FromLong(15); if (unlikely(!__pyx_int_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_17 = PyInt_FromLong(17); if (unlikely(!__pyx_int_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_18 = PyInt_FromLong(18); if (unlikely(!__pyx_int_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_19 = PyInt_FromLong(19); if (unlikely(!__pyx_int_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_20 = PyInt_FromLong(20); if (unlikely(!__pyx_int_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_21 = PyInt_FromLong(21); if (unlikely(!__pyx_int_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_22 = PyInt_FromLong(22); if (unlikely(!__pyx_int_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4096 = PyInt_FromLong(4096); if (unlikely(!__pyx_int_4096)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_2 = PyInt_FromLong(-2); if (unlikely(!__pyx_int_neg_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_3 = PyInt_FromLong(-3); if (unlikely(!__pyx_int_neg_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_4 = PyInt_FromLong(-4); if (unlikely(!__pyx_int_neg_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_5 = PyInt_FromLong(-5); if (unlikely(!__pyx_int_neg_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_6 = PyInt_FromLong(-6); if (unlikely(!__pyx_int_neg_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_7 = PyInt_FromLong(-7); if (unlikely(!__pyx_int_neg_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_8 = PyInt_FromLong(-8); if (unlikely(!__pyx_int_neg_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_9 = PyInt_FromLong(-9); if (unlikely(!__pyx_int_neg_9)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_10 = PyInt_FromLong(-10); if (unlikely(!__pyx_int_neg_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_11 = PyInt_FromLong(-11); if (unlikely(!__pyx_int_neg_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_12 = PyInt_FromLong(-12); if (unlikely(!__pyx_int_neg_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_13 = PyInt_FromLong(-13); if (unlikely(!__pyx_int_neg_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_14 = PyInt_FromLong(-14); if (unlikely(!__pyx_int_neg_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_15 = PyInt_FromLong(-15); if (unlikely(!__pyx_int_neg_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_16 = PyInt_FromLong(-16); if (unlikely(!__pyx_int_neg_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_17 = PyInt_FromLong(-17); if (unlikely(!__pyx_int_neg_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_7pyslow5_Open) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5_Open.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5_Open.tp_dictoffset && __pyx_type_7pyslow5_Open.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5_Open.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Open, (PyObject *)&__pyx_type_7pyslow5_Open) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7pyslow5_Open) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __pyx_ptype_7pyslow5_Open = &__pyx_type_7pyslow5_Open;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct____cinit__) < 0) __PYX_ERR(0, 105, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct____cinit__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct____cinit__.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct____cinit__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct____cinit__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct____cinit__ = &__pyx_type_7pyslow5___pyx_scope_struct____cinit__;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 260, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_1_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_1_genexpr.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_1_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_1_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_1_genexpr = &__pyx_type_7pyslow5___pyx_scope_struct_1_genexpr;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_2__get_batches) < 0) __PYX_ERR(0, 442, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_2__get_batches.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_2__get_batches.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_2__get_batches.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_2__get_batches.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_2__get_batches = &__pyx_type_7pyslow5___pyx_scope_struct_2__get_batches;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi) < 0) __PYX_ERR(0, 449, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_3__get_read_multi = &__pyx_type_7pyslow5___pyx_scope_struct_3__get_read_multi;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads) < 0) __PYX_ERR(0, 1048, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_4_seq_reads = &__pyx_type_7pyslow5___pyx_scope_struct_4_seq_reads;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_5_seq_reads_multi = &__pyx_type_7pyslow5___pyx_scope_struct_5_seq_reads_multi;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list) < 0) __PYX_ERR(0, 1253, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_6_get_read_list = &__pyx_type_7pyslow5___pyx_scope_struct_6_get_read_list;
  if (PyType_Ready(&__pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi) < 0) __PYX_ERR(0, 1263, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi.tp_dictoffset && __pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_7pyslow5___pyx_scope_struct_7_get_read_list_multi = &__pyx_type_7pyslow5___pyx_scope_struct_7_get_read_list_multi;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 200, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 223, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 227, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 239, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_generic) __PYX_ERR(2, 771, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType(__pyx_t_1, "numpy", "number", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_number) __PYX_ERR(2, 773, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_integer) __PYX_ERR(2, 775, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(2, 777, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(2, 779, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(2, 781, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_floating) __PYX_ERR(2, 783, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(2, 785, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(2, 787, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType(__pyx_t_1, "numpy", "character", sizeof(PyObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_character) __PYX_ERR(2, 789, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initpyslow5(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initpyslow5(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_pyslow5(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_pyslow5(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_pyslow5(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'pyslow5' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_pyslow5(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("pyslow5", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyslow5) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyslow5")) {
      if (unlikely(PyDict_SetItemString(modules, "pyslow5", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __Pyx_TraceCall("__Pyx_PyMODINIT_FUNC PyInit_pyslow5(void)", __pyx_f[0], 1, 0, __PYX_ERR(0, 1, __pyx_L1_error));

  /* "pyslow5.pyx":4
 * # cython: language_level=3
 * # cython: profile=True
 * import sys             # <<<<<<<<<<<<<<
 * import time
 * import logging
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":5
 * # cython: profile=True
 * import sys
 * import time             # <<<<<<<<<<<<<<
 * import logging
 * import copy
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_time, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":6
 * import sys
 * import time
 * import logging             # <<<<<<<<<<<<<<
 * import copy
 * from itertools import chain
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_logging, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":7
 * import time
 * import logging
 * import copy             # <<<<<<<<<<<<<<
 * from itertools import chain
 * from libc.stdlib cimport malloc, free
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_copy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_copy, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyslow5.pyx":8
 * import logging
 * import copy
 * from itertools import chain             # <<<<<<<<<<<<<<
 * from libc.stdlib cimport malloc, free
 * from libc.string cimport strdup
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_chain);
  __Pyx_GIVEREF(__pyx_n_s_chain);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_chain);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_itertools, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_chain); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chain, __pyx_t_1) < 0) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":13
 * cimport pyslow5
 * # Import the Python-level symbols of numpy
 * import numpy as np             # <<<<<<<<<<<<<<
 * # Import the C-level symbols of numpy
 * cimport numpy as np
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyslow5.pyx":18
 * # Numpy must be initialized. When using numpy from C or Cython you must
 * # _always_ do that, or you will have segfaults
 * np.import_array()             # <<<<<<<<<<<<<<
 * 
 * #
 */
  __pyx_t_3 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 18, __pyx_L1_error)

  /* "pyslow5.pyx":1
 * # distutils: language = c             # <<<<<<<<<<<<<<
 * # cython: language_level=3
 * # cython: profile=True
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../../../home/hasindu/twine/lib/python3.9/site-packages/numpy/__init__.pxd":1014
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */
  __Pyx_TraceReturn(Py_None, 0);

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyslow5", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyslow5");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* Profile */
#if CYTHON_PROFILE
static int __Pyx_TraceSetupAndCall(PyCodeObject** code,
                                   PyFrameObject** frame,
                                   PyThreadState* tstate,
                                   const char *funcname,
                                   const char *srcfile,
                                   int firstlineno) {
    PyObject *type, *value, *traceback;
    int retval;
    if (*frame == NULL || !CYTHON_PROFILE_REUSE_FRAME) {
        if (*code == NULL) {
            *code = __Pyx_createFrameCodeObject(funcname, srcfile, firstlineno);
            if (*code == NULL) return 0;
        }
        *frame = PyFrame_New(
            tstate,                          /*PyThreadState *tstate*/
            *code,                           /*PyCodeObject *code*/
            __pyx_d,                  /*PyObject *globals*/
            0                                /*PyObject *locals*/
        );
        if (*frame == NULL) return 0;
        if (CYTHON_TRACE && (*frame)->f_trace == NULL) {
            Py_INCREF(Py_None);
            (*frame)->f_trace = Py_None;
        }
#if PY_VERSION_HEX < 0x030400B1
    } else {
        (*frame)->f_tstate = tstate;
#endif
    }
    __Pyx_PyFrame_SetLineNumber(*frame, firstlineno);
    retval = 1;
    __Pyx_EnterTracing(tstate);
    __Pyx_ErrFetchInState(tstate, &type, &value, &traceback);
    #if CYTHON_TRACE
    if (tstate->c_tracefunc)
        retval = tstate->c_tracefunc(tstate->c_traceobj, *frame, PyTrace_CALL, NULL) == 0;
    if (retval && tstate->c_profilefunc)
    #endif
        retval = tstate->c_profilefunc(tstate->c_profileobj, *frame, PyTrace_CALL, NULL) == 0;
    __Pyx_LeaveTracing(tstate);
    if (retval) {
        __Pyx_ErrRestoreInState(tstate, type, value, traceback);
        return __Pyx_IsTracing(tstate, 0, 0) && retval;
    } else {
        Py_XDECREF(type);
        Py_XDECREF(value);
        Py_XDECREF(traceback);
        return -1;
    }
}
static PyCodeObject *__Pyx_createFrameCodeObject(const char *funcname, const char *srcfile, int firstlineno) {
    PyCodeObject *py_code = 0;
#if PY_MAJOR_VERSION >= 3
    py_code = PyCode_NewEmpty(srcfile, funcname, firstlineno);
    if (likely(py_code)) {
        py_code->co_flags |= CO_OPTIMIZED | CO_NEWLOCALS;
    }
#else
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    py_funcname = PyString_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    py_srcfile = PyString_FromString(srcfile);
    if (unlikely(!py_srcfile)) goto bad;
    py_code = PyCode_New(
        0,
        0,
        0,
        CO_OPTIMIZED | CO_NEWLOCALS,
        __pyx_empty_bytes,     /*PyObject *code,*/
        __pyx_empty_tuple,     /*PyObject *consts,*/
        __pyx_empty_tuple,     /*PyObject *names,*/
        __pyx_empty_tuple,     /*PyObject *varnames,*/
        __pyx_empty_tuple,     /*PyObject *freevars,*/
        __pyx_empty_tuple,     /*PyObject *cellvars,*/
        py_srcfile,       /*PyObject *filename,*/
        py_funcname,      /*PyObject *name,*/
        firstlineno,
        __pyx_empty_bytes      /*PyObject *lnotab*/
    );
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
#endif
    return py_code;
}
#endif

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* None */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_keys */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_keys, d);
    else
        return PyDict_Keys(d);
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* decode_c_string */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* None */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint32_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int8_t(int8_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int8_t neg_one = (int8_t) -1, const_zero = (int8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int8_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int8_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int8_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int16_t(int16_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int16_t neg_one = (int16_t) -1, const_zero = (int16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int16_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int16_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int16_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int16_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int16_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int16_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32_t(int32_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int32_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64_t(int64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int64_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint8_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint16_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint16_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const char neg_one = (char) -1, const_zero = (char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(char),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint64_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint64_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 2 * PyLong_SHIFT) {
                            return (uint64_t) (((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 3 * PyLong_SHIFT) {
                            return (uint64_t) (((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 4 * PyLong_SHIFT) {
                            return (uint64_t) (((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint64_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint64_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint64_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) ((((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint64_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint64_t) -1;
        }
    } else {
        uint64_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint64_t) -1;
        val = __Pyx_PyInt_As_uint64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64_t");
    return (uint64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64_t");
    return (uint64_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__slow5_aux_type(enum slow5_aux_type value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum slow5_aux_type neg_one = (enum slow5_aux_type) -1, const_zero = (enum slow5_aux_type) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum slow5_aux_type) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum slow5_aux_type) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum slow5_aux_type) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum slow5_aux_type) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum slow5_aux_type) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum slow5_aux_type),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 2 * PyLong_SHIFT) {
                            return (int32_t) (((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 3 * PyLong_SHIFT) {
                            return (int32_t) (((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 4 * PyLong_SHIFT) {
                            return (int32_t) (((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) ((((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) ((((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) ((((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int32_t) -1;
        }
    } else {
        int32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int32_t) -1;
        val = __Pyx_PyInt_As_int32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32_t");
    return (int32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32_t");
    return (int32_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint8_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint8_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 2 * PyLong_SHIFT) {
                            return (uint8_t) (((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 3 * PyLong_SHIFT) {
                            return (uint8_t) (((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 4 * PyLong_SHIFT) {
                            return (uint8_t) (((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint8_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint8_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) ((((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint8_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint8_t) -1;
        }
    } else {
        uint8_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint8_t) -1;
        val = __Pyx_PyInt_As_uint8_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8_t");
    return (uint8_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8_t");
    return (uint8_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE enum slow5_aux_type __Pyx_PyInt_As_enum__slow5_aux_type(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum slow5_aux_type neg_one = (enum slow5_aux_type) -1, const_zero = (enum slow5_aux_type) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(enum slow5_aux_type) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (enum slow5_aux_type) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enum slow5_aux_type) 0;
                case  1: __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, digit, digits[0])
                case 2:
                    if (8 * sizeof(enum slow5_aux_type) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) >= 2 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) (((((enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enum slow5_aux_type) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) >= 3 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) (((((((enum slow5_aux_type)digits[2]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enum slow5_aux_type) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) >= 4 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) (((((((((enum slow5_aux_type)digits[3]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[2]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (enum slow5_aux_type) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(enum slow5_aux_type) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum slow5_aux_type, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enum slow5_aux_type) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum slow5_aux_type, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (enum slow5_aux_type) 0;
                case -1: __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(enum slow5_aux_type,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(enum slow5_aux_type) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) - 1 > 2 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) (((enum slow5_aux_type)-1)*(((((enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(enum slow5_aux_type) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) - 1 > 2 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) ((((((enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(enum slow5_aux_type) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) - 1 > 3 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) (((enum slow5_aux_type)-1)*(((((((enum slow5_aux_type)digits[2]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(enum slow5_aux_type) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) - 1 > 3 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) ((((((((enum slow5_aux_type)digits[2]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(enum slow5_aux_type) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) - 1 > 4 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) (((enum slow5_aux_type)-1)*(((((((((enum slow5_aux_type)digits[3]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[2]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(enum slow5_aux_type) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(enum slow5_aux_type, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(enum slow5_aux_type) - 1 > 4 * PyLong_SHIFT) {
                            return (enum slow5_aux_type) ((((((((((enum slow5_aux_type)digits[3]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[2]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[1]) << PyLong_SHIFT) | (enum slow5_aux_type)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(enum slow5_aux_type) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum slow5_aux_type, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(enum slow5_aux_type) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(enum slow5_aux_type, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            enum slow5_aux_type val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (enum slow5_aux_type) -1;
        }
    } else {
        enum slow5_aux_type val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (enum slow5_aux_type) -1;
        val = __Pyx_PyInt_As_enum__slow5_aux_type(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enum slow5_aux_type");
    return (enum slow5_aux_type) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enum slow5_aux_type");
    return (enum slow5_aux_type) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int16_t __Pyx_PyInt_As_int16_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int16_t neg_one = (int16_t) -1, const_zero = (int16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int16_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int16_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int16_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int16_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int16_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int16_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) >= 2 * PyLong_SHIFT) {
                            return (int16_t) (((((int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int16_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) >= 3 * PyLong_SHIFT) {
                            return (int16_t) (((((((int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int16_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) >= 4 * PyLong_SHIFT) {
                            return (int16_t) (((((((((int16_t)digits[3]) << PyLong_SHIFT) | (int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int16_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int16_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int16_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int16_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int16_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int16_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int16_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int16_t) (((int16_t)-1)*(((((int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int16_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int16_t) ((((((int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int16_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int16_t) (((int16_t)-1)*(((((((int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int16_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int16_t) ((((((((int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int16_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int16_t) (((int16_t)-1)*(((((((((int16_t)digits[3]) << PyLong_SHIFT) | (int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int16_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int16_t) ((((((((((int16_t)digits[3]) << PyLong_SHIFT) | (int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int16_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int16_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int16_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int16_t) -1;
        }
    } else {
        int16_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int16_t) -1;
        val = __Pyx_PyInt_As_int16_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int16_t");
    return (int16_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int16_t");
    return (int16_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
#include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
